Package: bit
Title: Classes and Methods for Fast Memory-Efficient Boolean Selections
Version: 4.6.0
Authors@R: c(
    person("Michael", "Chirico", email = "MichaelChirico4@gmail.com", role = c("aut", "cre")),
    person("Jens", "Oehlschlägel", role = "aut"),
    person("Brian", "Ripley", role = "ctb")
  )
Depends: R (>= 3.4.0)
Suggests: testthat (>= 3.0.0), roxygen2, knitr, markdown, rmarkdown,
        microbenchmark, bit64 (>= 4.0.0), ff (>= 4.0.0)
Description: Provided are classes for boolean and skewed boolean vectors,
    fast boolean methods, fast unique and non-unique integer sorting,
    fast set operations on sorted and unsorted sets of integers, and
    foundations for ff (range index, compression, chunked processing).
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
Encoding: UTF-8
URL: https://github.com/r-lib/bit
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-03-05 07:18:45 UTC; michael
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Brian Ripley [ctb]
Maintainer: Michael Chirico <MichaelChirico4@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-06 10:50:01 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:59:55 UTC; unix
Archs: bit.so.dSYM

Package: bit
Title: Classes and Methods for Fast Memory-Efficient Boolean Selections
Version: 4.6.0
Authors@R: c(
    person("Michael", "Chirico", email = "MichaelChirico4@gmail.com", role = c("aut", "cre")),
    person("Jens", "Oehlschlägel", role = "aut"),
    person("Brian", "Ripley", role = "ctb")
  )
Depends: R (>= 3.4.0)
Suggests: testthat (>= 3.0.0), roxygen2, knitr, markdown, rmarkdown,
        microbenchmark, bit64 (>= 4.0.0), ff (>= 4.0.0)
Description: Provided are classes for boolean and skewed boolean vectors,
    fast boolean methods, fast unique and non-unique integer sorting,
    fast set operations on sorted and unsorted sets of integers, and
    foundations for ff (range index, compression, chunked processing).
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
Encoding: UTF-8
URL: https://github.com/r-lib/bit
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-03-05 07:18:45 UTC; michael
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Brian Ripley [ctb]
Maintainer: Michael Chirico <MichaelChirico4@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-06 10:50:01 UTC
Built: R 4.4.0; i386-portbld-midnightbsd4.0; 2026-05-12 01:11:21 UTC; unix

Package: bit
Type: Package
Title: Classes and Methods for Fast Memory-Efficient Boolean Selections
Version: 4.0.5
Date: 2022-11-13
Author: Jens Oehlschlägel [aut, cre], Brian Ripley [ctb]
Maintainer: Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
Depends: R (>= 2.9.2)
Suggests: testthat (>= 0.11.0), roxygen2, knitr, rmarkdown,
        microbenchmark, bit64 (>= 4.0.0), ff (>= 4.0.0)
Description: Provided are classes for boolean and skewed boolean vectors,
    fast boolean methods, fast unique and non-unique integer sorting,
    fast set operations on sorted and unsorted sets of integers, and
    foundations for ff (range index, compression, chunked processing).
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
Encoding: UTF-8
URL: https://github.com/truecluster/bit
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.2.0
NeedsCompilation: yes
Packaged: 2022-11-13 21:22:09 UTC; jo
Repository: CRAN
Date/Publication: 2022-11-15 21:20:16 UTC

# Generated by roxygen2: do not edit by hand

S3method("!",bit)
S3method("!",bitwhich)
S3method("!=",bit)
S3method("!=",bitwhich)
S3method("!=",booltype)
S3method("&",bit)
S3method("&",bitwhich)
S3method("&",booltype)
S3method("==",bit)
S3method("==",bitwhich)
S3method("==",booltype)
S3method("[",bit)
S3method("[",bitwhich)
S3method("[<-",bit)
S3method("[<-",bitwhich)
S3method("[[",bit)
S3method("[[",bitwhich)
S3method("[[<-",bit)
S3method("[[<-",bitwhich)
S3method("length<-",bit)
S3method("length<-",bitwhich)
S3method("physical<-",default)
S3method("virtual<-",default)
S3method("|",bit)
S3method("|",bitwhich)
S3method("|",booltype)
S3method(all,bit)
S3method(all,bitwhich)
S3method(all,booltype)
S3method(all,ri)
S3method(all,which)
S3method(any,bit)
S3method(any,bitwhich)
S3method(any,booltype)
S3method(any,ri)
S3method(any,which)
S3method(anyDuplicated,rlepack)
S3method(anyNA,bit)
S3method(anyNA,bitwhich)
S3method(anyNA,booltype)
S3method(anyNA,ri)
S3method(anyNA,which)
S3method(as.bit,"NULL")
S3method(as.bit,bit)
S3method(as.bit,bitwhich)
S3method(as.bit,double)
S3method(as.bit,integer)
S3method(as.bit,logical)
S3method(as.bit,ri)
S3method(as.bit,which)
S3method(as.bitwhich,"NULL")
S3method(as.bitwhich,bit)
S3method(as.bitwhich,bitwhich)
S3method(as.bitwhich,double)
S3method(as.bitwhich,integer)
S3method(as.bitwhich,logical)
S3method(as.bitwhich,ri)
S3method(as.bitwhich,which)
S3method(as.booltype,default)
S3method(as.character,bit)
S3method(as.character,bitwhich)
S3method(as.double,bit)
S3method(as.double,bitwhich)
S3method(as.double,ri)
S3method(as.integer,bit)
S3method(as.integer,bitwhich)
S3method(as.integer,ri)
S3method(as.logical,bit)
S3method(as.logical,bitwhich)
S3method(as.logical,ri)
S3method(as.logical,which)
S3method(as.ri,default)
S3method(as.ri,ri)
S3method(as.which,"NULL")
S3method(as.which,bit)
S3method(as.which,bitwhich)
S3method(as.which,integer)
S3method(as.which,logical)
S3method(as.which,numeric)
S3method(as.which,ri)
S3method(as.which,which)
S3method(c,bit)
S3method(c,bitwhich)
S3method(c,booltype)
S3method(chunk,default)
S3method(clone,default)
S3method(is.na,bitwhich)
S3method(length,bit)
S3method(length,bitwhich)
S3method(length,ri)
S3method(max,bit)
S3method(max,bitwhich)
S3method(max,booltype)
S3method(max,ri)
S3method(max,which)
S3method(maxindex,bit)
S3method(maxindex,bitwhich)
S3method(maxindex,default)
S3method(maxindex,logical)
S3method(maxindex,ri)
S3method(maxindex,which)
S3method(min,bit)
S3method(min,bitwhich)
S3method(min,booltype)
S3method(min,ri)
S3method(min,which)
S3method(physical,default)
S3method(poslength,bit)
S3method(poslength,bitwhich)
S3method(poslength,default)
S3method(poslength,logical)
S3method(poslength,ri)
S3method(poslength,which)
S3method(print,bit)
S3method(print,bitwhich)
S3method(print,physical)
S3method(print,ri)
S3method(print,virtual)
S3method(range,bit)
S3method(range,bitwhich)
S3method(range,booltype)
S3method(range,ri)
S3method(range,which)
S3method(rep,bit)
S3method(rep,bitwhich)
S3method(rev,bit)
S3method(rev,bitwhich)
S3method(rev,rlepack)
S3method(rlepack,integer)
S3method(rleunpack,rlepack)
S3method(str,bit)
S3method(str,bitwhich)
S3method(sum,bit)
S3method(sum,bitwhich)
S3method(sum,booltype)
S3method(sum,ri)
S3method(sum,which)
S3method(summary,bit)
S3method(summary,bitwhich)
S3method(summary,booltype)
S3method(summary,ri)
S3method(summary,which)
S3method(unique,rlepack)
S3method(virtual,default)
S3method(xor,bit)
S3method(xor,bitwhich)
S3method(xor,booltype)
S3method(xor,default)
S3method(xor,logical)
export("!=.booltype")
export("&.booltype")
export("==.booltype")
export("is.sorted<-")
export("na.count<-")
export("nties<-")
export("nunique<-")
export("physical<-")
export("repfromto<-")
export("virtual<-")
export("|.booltype")
export(.BITS)
export(all.booltype)
export(any.booltype)
export(anyNA.booltype)
export(as.bit)
export(as.bitwhich)
export(as.booltype)
export(as.ri)
export(as.which)
export(bbatch)
export(bit)
export(bit_anyDuplicated)
export(bit_done)
export(bit_duplicated)
export(bit_in)
export(bit_init)
export(bit_intersect)
export(bit_rangediff)
export(bit_setdiff)
export(bit_setequal)
export(bit_sort)
export(bit_sort_unique)
export(bit_sumDuplicated)
export(bit_symdiff)
export(bit_union)
export(bit_unique)
export(bitsort)
export(bitwhich)
export(bitwhich_representation)
export(booltype)
export(booltypes)
export(c.booltype)
export(chunk)
export(chunks)
export(clone)
export(copy_vector)
export(countsort)
export(firstNA)
export(get_length)
export(getsetattr)
export(in.bitwhich)
export(intisasc)
export(intisdesc)
export(intrle)
export(is.bit)
export(is.bitwhich)
export(is.booltype)
export(is.hi)
export(is.ri)
export(is.sorted)
export(is.which)
export(keyorder)
export(keysort)
export(keysortorder)
export(max.booltype)
export(maxindex)
export(merge_anyDuplicated)
export(merge_duplicated)
export(merge_first)
export(merge_firstin)
export(merge_firstnotin)
export(merge_in)
export(merge_intersect)
export(merge_last)
export(merge_lastin)
export(merge_lastnotin)
export(merge_match)
export(merge_notin)
export(merge_rangediff)
export(merge_rangein)
export(merge_rangenotin)
export(merge_rangesect)
export(merge_rev)
export(merge_setdiff)
export(merge_setequal)
export(merge_sumDuplicated)
export(merge_symdiff)
export(merge_union)
export(merge_unique)
export(mergeorder)
export(mergesort)
export(mergesortorder)
export(min.booltype)
export(na.count)
export(nties)
export(nunique)
export(nvalid)
export(physical)
export(poslength)
export(quickorder)
export(quicksort)
export(quicksort2)
export(quicksort3)
export(quicksortorder)
export(radixorder)
export(radixsort)
export(radixsortorder)
export(ramorder)
export(ramsort)
export(ramsortorder)
export(range.booltype)
export(range_na)
export(range_nanozero)
export(range_sortna)
export(repeat.time)
export(repfromto)
export(reverse_vector)
export(ri)
export(rlepack)
export(rleunpack)
export(setattr)
export(setattributes)
export(shellorder)
export(shellsort)
export(shellsortorder)
export(still.identical)
export(sum.booltype)
export(summary.booltype)
export(symdiff)
export(unattr)
export(vecseq)
export(virtual)
export(xor)
export(xor.booltype)
importFrom(utils,packageDescription)
importFrom(utils,strOptions)
useDynLib(bit, .registration = TRUE, .fixes = "C_")

## ----echo = FALSE, results = "hide", message = FALSE--------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(bit)
library(microbenchmark)
# rmarkdown::render("vignettes/bit-performance.Rmd")
# these are the real settings for the performance vignette
times <- 5
Domain <- c(small=1e3, big=1e6)
Sample <- c(small=1e3, big=1e6)
# these are the settings to keep the cost of CRAN low
#times <- 5
#Domain <- c(small=1e1, big=1e3)
#Sample <- c(small=1e1, big=1e3)

pagebreak <- function() {
  if (knitr::is_latex_output())
    "\\newpage"
  else
    '<div style="page-break-before: always;" />'
}


## ----echo=TRUE, results='asis'------------------------------------------------
a <- 1L
b <- 1e7L
i <- sample(a:b, 1e3)
x <- c(
  R = median(microbenchmark((a:b)[-i], times=times)$time),
  bit = median(microbenchmark(bit_rangediff(c(a, b), i), times=times)$time),
  merge = median(microbenchmark(merge_rangediff(c(a, b), bit_sort(i)), times=times)$time)
)
knitr::kable(as.data.frame(as.list(x / x["R"] * 100)), caption="% of time relative to R", digits=1)

## ----echo=FALSE, results='asis', echo=3:5-------------------------------------
# TODO(r-lib/lintr#773): nolint as a chunk option.
# nolint start: strings_as_factors_linter.
knitr::kable(
  data.frame(
    coin="random 50%",
    often="random 99%",
    rare="random 1%",
    chunk="contiguous chunk of 5%"
  ),
  caption="selection characteristic"
)
# nolint end: strings_as_factors_linter.

## ----echo=FALSE, results='asis'-----------------------------------------------
B <- booltypes[c("logical", "bit", "bitwhich", "which", "ri")]
M <-
  c("size", "[]", "[which]", "[which]<-TRUE", "[]<-logical", "!", "&", "|", "==", "!=", "summary")
G <- list(
  coin = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.5, 0.5)),
  often = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.01, 0.99)),
  rare = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.99, 0.01)),
  chunk = function(n) ri(n %/% 20, 2L * n %/% 20, n)
)
X <- vector("list", length(B) * length(G))
dim(X) <- c(booltype=length(B), data=length(G))
dimnames(X) <- list(booltype=names(B), data=names(G))
tim <- array(NA,
  dim=c(booltype=length(B), metric=length(M), data=length(G)),
  dimnames=list(booltype=names(B), metric=M, data=names(G))
)
for (g in names(G)) {
  x <- G[[g]](Sample[["big"]])
  if (g %in% c("coin", "often", "rare"))
    w <- as.which(as.logical(x))
  for (b in B) {
    if (booltypes[[b]] < 'ri' || (b == 'ri' && g == 'chunk')) {
      X[[b, g]] <- as.booltype(x, b)
      if (g %in% c("coin", "often", "rare") && b %in% c("logical", "bit", "bitwhich")) {
        l <- as.booltype(logical(Sample[["big"]]), b)
        tim[b, "[which]", g] <- median(microbenchmark(l[w], times=times)$time)
        tim[b, "[which]<-TRUE", g] <- median(microbenchmark(l[w] <- TRUE, times=times)$time)
        tim[b, "[]", g] <- median(microbenchmark(l[], times=times)$time)
        tim[b, "[]<-logical", g] <- median(microbenchmark(l[] <- x, times=times)$time)
      }
      tim[b, "size", g] <- object.size(X[[b, g]])
    }
  }
}
for (g in names(G)) {
  for (b in c("logical", "bit", "bitwhich")) {
    x <- X[[b, g]]
    if (!is.null(x)) {
      tim[b, "!", g] <- median(microbenchmark(!x, times=times)$time)
      tim[b, "&", g] <- median(microbenchmark(x & x, times=times)$time)
      tim[b, "|", g] <- median(microbenchmark(x | x, times=times)$time)
      tim[b, "==", g] <- median(microbenchmark(x == x, times=times)$time)
      tim[b, "!=", g] <- median(microbenchmark(x != x, times=times)$time)
      tim[b, "summary", g] <- median(microbenchmark(summary.booltype(x), times=times)$time)
    }
  }
}
i <- match("size", M)
for (b in rev(names(B))) { # logical was in first position, so we do this last!
  tim[b, i, ] <- 100 * tim[b, i, ] / tim["logical", i, ]
  tim[b, -i, ] <- 100 * tim[b, -i, ] / max(tim["logical", -i, ], na.rm=TRUE)
}
#rm(X)

## ----echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'rare' scenario"----
x <- tim[1:3, , "rare"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'rare' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)

## ----echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'often' scenario"----
x <- tim[1:3, , "often"]
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'often' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)

## ----echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'coin' scenario"----
x <- tim[1:3, , "coin"]
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'coin' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "size", ], 1), caption="% bytes of logical")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "[]", ], 1), caption="% time of logical")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "[]<-logical", ], 1), caption="% time of logical")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "[which]", ], 1), caption="% time of logical")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "[which]<-TRUE", ], 1), caption="% time of logical")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "!", ], 1), caption="% time for Boolean NOT")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "&", ], 1), caption="% time for Boolean &")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "|", ], 1), caption="% time for Boolean |")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "==", ], 1), caption="% time for Boolean ==")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "!=", ], 1), caption="% time for Boolean !=")

## ----echo=FALSE, results='asis'-----------------------------------------------
knitr::kable(round(tim[, "summary", ][1:2, 1:2], 1), caption="% time for Boolean summary")

## ----echo=FALSE, results='asis'-----------------------------------------------
binaryDomain <- list(
  smallsmall = rep(Domain["small"], 2L),
  smallbig=Domain,
  bigsmall=rev(Domain),
  bigbig=rep(Domain["big"], 2L)
)
binarySample <- list(
  smallsmall = rep(Sample["small"], 2L),
  smallbig=Sample,
  bigsmall=rev(Sample),
  bigbig=rep(Sample["big"], 2L)
)

M <- c("R", "bit", "merge")
G <- c("sort", "sortunique")
D <- c("unsorted", "sorted")

sortM <- vector("list", length(M) * length(G))
dim(sortM) <- c(method=length(M), goal=length(G))
dimnames(sortM) <- list(method=M, goal=G)
sortM[["R", "sort"]] <- sort
sortM[["R", "sortunique"]] <- function(x) sort(unique(x))
sortM[["bit", "sort"]] <- bit_sort
sortM[["bit", "sortunique"]] <- bit_sort_unique

timsort <- array(NA_integer_,
  dim=c(M=2, G=length(G), D=length(D), N=length(Domain)),
  dimnames=list(M=M[1:2], G=G, D=D, N=names(Domain))
)
for (n in names(Domain)) {
  x <- sample(Domain[[n]], Sample[[n]], replace = TRUE)
  d <- "unsorted"
  for (m in c("R", "bit")) {
    for (g in G) {
      timsort[m, g, d, n] <- median(microbenchmark(sortM[[m, g]](x), times=times)$time)
    }
  }
  x <- bit_sort(x)
  d <- "sorted"
  for (m in 1:2) {
    for (g in G) {
      timsort[m, g, d, n] <- median(microbenchmark(sortM[[m, g]](x), times=times)$time)
    }
  }
}


binaryU <-
  c("match", "in", "notin", "union", "intersect", "setdiff", "symdiff", "setequal", "setearly")
binaryM <- vector("list", length(M) * length(binaryU))
dim(binaryM) <- c(method=length(M), task=length(binaryU))
dimnames(binaryM) <- list(method=M, task=binaryU)
binaryM[["R", "match"]] <- match
binaryM[["merge", "match"]] <- merge_match

binaryM[["R", "in"]] <- get("%in%")
binaryM[["bit", "in"]] <- bit_in
binaryM[["merge", "in"]] <- merge_in

binaryM[["R", "notin"]] <- function(x, y) !(x %in% y)
binaryM[["bit", "notin"]] <- function(x, y) !bit_in(x, y)
binaryM[["merge", "notin"]] <- merge_notin

binaryM[["R", "union"]] <- union
binaryM[["bit", "union"]] <- bit_union
binaryM[["merge", "union"]] <- merge_union

binaryM[["R", "intersect"]] <- intersect
binaryM[["bit", "intersect"]] <- bit_intersect
binaryM[["merge", "intersect"]] <- merge_intersect

binaryM[["R", "setdiff"]] <- setdiff
binaryM[["bit", "setdiff"]] <- bit_setdiff
binaryM[["merge", "setdiff"]] <- merge_setdiff

binaryM[["R", "symdiff"]] <- function(x, y) union(setdiff(x, y), setdiff(y, x))
binaryM[["bit", "symdiff"]] <- bit_symdiff
binaryM[["merge", "symdiff"]] <- merge_symdiff

binaryM[["R", "setequal"]] <-
  function(x, y) setequal(x, x) # we compare x to x which avoids early termination
binaryM[["bit", "setequal"]] <- function(x, y) bit_setequal(x, x)
binaryM[["merge", "setequal"]] <- function(x, y) merge_setequal(x, x)

binaryM[["R", "setearly"]] <-
  function(x, y) setequal(x, y) # we compare x to x which avoids early termination
binaryM[["bit", "setearly"]] <- function(x, y) bit_setequal(x, y)
binaryM[["merge", "setearly"]] <- function(x, y) merge_setequal(x, y)

unaryU <- c("unique", "duplicated", "anyDuplicated", "sumDuplicated")
unaryM <- vector("list", length(M) * length(unaryU))
dim(unaryM) <- c(method=length(M), task=length(unaryU))
dimnames(unaryM) <- list(method=M, task=unaryU)
unaryM[["R", "unique"]] <- unique
unaryM[["bit", "unique"]] <- bit_unique
unaryM[["merge", "unique"]] <- merge_unique
unaryM[["R", "duplicated"]] <- duplicated
unaryM[["bit", "duplicated"]] <- bit_duplicated
unaryM[["merge", "duplicated"]] <- merge_duplicated
unaryM[["R", "anyDuplicated"]] <- anyDuplicated
unaryM[["bit", "anyDuplicated"]] <- bit_anyDuplicated
unaryM[["merge", "anyDuplicated"]] <- merge_anyDuplicated
unaryM[["R", "sumDuplicated"]] <- function(x) sum(duplicated(x))
unaryM[["bit", "sumDuplicated"]] <- bit_sumDuplicated
unaryM[["merge", "sumDuplicated"]] <- merge_sumDuplicated

tim <- array(NA_integer_,
  dim=c(M=length(M), U=length(unaryU) + length(binaryU), N=length(binaryDomain), D=length(D)),
  dimnames=list(M=M, U=c(unaryU, binaryU), N=names(binaryDomain), D=D)
)

for (n in names(binaryDomain)) {
  xnam <- names(binaryDomain[[n]])[1]
  ynam <- names(binaryDomain[[n]])[2]
  x <- sample(binaryDomain[[n]][1], binarySample[[n]][1], replace = FALSE)
  y <- sample(binaryDomain[[n]][2], binarySample[[n]][2], replace = FALSE)
  d <- "unsorted"
  if (length(x) == length(y)) {
    for (u in unaryU) {
      for (m in setdiff(M, "merge")) {
        f <- unaryM[[m, u]]
        if (!is.null(f))
          tim[m, u, n, d] <- median(microbenchmark(f(x), times=times)$time)
      }
    }
  }
  for (u in binaryU) {
    for (m in setdiff(M, "merge")) {
      f <- binaryM[[m, u]]
      if (!is.null(f))
        tim[m, u, n, d] <- median(microbenchmark(f(x, y), times=times)$time)
    }
  }
  x <- bit_sort(x)
  y <- bit_sort(y)
  d <- "sorted"
  if (length(x) == length(y)) {
    for (u in unaryU) {
      for (m in M) {
        f <- unaryM[[m, u]]
        if (!is.null(f)) {
          tim[m, u, n, d] <- median(microbenchmark(f(x), times=times)$time)
          # now plug-in measures for unsorted merge
          if (m == "merge") {
            tim["merge", u, n, "unsorted"] <-
              timsort["bit", "sort", "unsorted", xnam] +  tim["merge", u, n, "sorted"]
          }
        }
      }
    }
  }
  for (u in binaryU) {
    for (m in M) {
      f <- binaryM[[m, u]]
      if (!is.null(f)) {
        tim[m, u, n, d] <- median(microbenchmark(f(x, y), times=times)$time)
        # now plug-in measures for unsorted merge
        if (m == "merge") {
          tim["merge", u, n, "unsorted"] =
            timsort["bit", "sort", "unsorted", xnam] +
            timsort["bit", "sort", "unsorted", ynam] +
            tim["merge", u, n, "sorted"]
        }
      }
    }
  }
}

## ----echo=FALSE, fig.cap = "Execution time for R (R) and bit (b)"-------------
y <- timsort[, , , "big"]
y <- 100 * y / max(y["R", , ], na.rm=TRUE)
oldpar <- par(mfrow=c(2, 1), mar=c(5, 8, 2, 1))
x <- y[, , "unsorted"]
dotchart(x,
  xlim=c(0, max(100, max(y))),
  labels="",
  pch=c("R", "b"),
  xlab="execution time",
  main="unsorted",
  col=c("red", "blue")
)
x <- y[, , "sorted"]
dotchart(x,
  xlim=c(0, max(100, max(y))),
  labels="",
  pch=c("R", "b"),
  xlab="execution time",
  main="sorted",
  col=c("red", "blue")
)
par(oldpar)

## ----echo=FALSE, results='hide'-----------------------------------------------
tim2 <- tim
for (n in names(binaryDomain)) {
  for (d in D) {
    tim2[, , n, d] <- 100 * tim[, , n, d] / max(tim["R", , n, d], na.rm=TRUE)
  }
}

## ----echo=FALSE, fig.cap = "Execution time for R, bit and merge relative to most expensive R in 'unsorted bigbig' scenario"----
y <- tim2[, , "bigbig", ]
y <- 100 * y / max(y["R", , ], na.rm=TRUE)
x <- y[, , "unsorted"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x,
  xlim=c(0, max(100, max(y, na.rm=TRUE))),
  labels=m,
  pch=c("R", "b", "m"),
  col=c("red", "blue", "black"),
  main="Timings in 'unsorted bigbig' scenario",
  sub="R='hash'   b='bit'   m='merge'"
)

## ----echo=FALSE, fig.cap = "Execution time for R, bit and merge in 'sorted bigbig' scenario"----
x <- y[, , "sorted"]
dotchart(x,
  xlim=c(0, max(y, na.rm=TRUE)),
  labels=m,
  pch=c("R", "b", "m"),
  col=c("red", "blue", "black"),
  main="Timings in 'sorted bigbig' scenario",
  sub="R='hash'   b='bit'   m='merge'"
)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- 100 * timsort["bit", , , ] / timsort["R", , , ]
s <- "sorted"
knitr::kable(x[, s, ], caption=paste(s, "data relative to R's sort"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, s, ], caption=paste(s, "data relative to R's sort"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
f <- function(u) {
  n <- c("smallsmall", "bigbig")
  x <- tim[c("bit", "merge", "merge"), u, n, ]
  dimnames(x)$M[3] <- "sort"
  dimnames(x)$N <- c("small", "big")
  x["sort", , "unsorted"] <- timsort["bit", "sort", "unsorted", ]
  x["sort", , "sorted"] <- 0
  for (m in dimnames(x)$M) {
    x[m, , ] <- x[m, , ] / tim["R", u, n, ] * 100
  }
  x
}
x <- f("unique")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("duplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("anyDuplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("sumDuplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
f <- function(u) {
  x <- tim[c("bit", "merge", "merge"), u, , ]
  dimnames(x)$M[3] <- "sort"
  s <- timsort["bit", "sort", "unsorted", ]
  x["sort", , "unsorted"] <- rep(s, c(2, 2)) + c(s, s)
  x["sort", , "sorted"] <- 0
  for (m in dimnames(x)$M) {
    x[m, , ] <- x[m, , ] / tim["R", u, , ] * 100
  }
  x
}
x <- f("match")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("in")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("notin")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("union")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("intersect")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("setdiff")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("symdiff")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("setequal")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
x <- f("setearly")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

## ----echo=FALSE, results='asis'-----------------------------------------------
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)

# ff tests that require being highly intentional about load/attach order, e.g. #3

library(bit)

if (isNamespaceLoaded("ff")) {
  cat("ff must be unloaded initially for this test to WAI\n")
  q("n")
}
chunk(1)

if (!requireNamespace("ff")) {
  cat("ff failed to load\n")
  q("n")
}
chunk(1)

# nolint next: undesirable_function_linter, unused_import_linter. It is used, by chunk().
if (!require("ff")) {
  cat("ff failed to attach\n")
  q("n")
}
chunk(1)

# Package intialization
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk


#' @useDynLib bit, .registration = TRUE, .fixes = "C_"
#' @importFrom utils packageDescription
.onLoad <- function(lib, pkg) {
  ##library.dynam("bit", pkg, lib) use useDynLib(bit, .registration = TRUE, .fixes = "C_") in NAMESPACE instead
  ##packageStartupMessage("Loading package bit ", packageDescription("bit", fields="Version"))
  bit_init()
}

# .onAttach <- function(libname, pkgname){
#   packageStartupMessage("Attaching package bit")
  # packageStartupMessage("package:bit (c) 2008-2017 Jens Oehlschlaegel (GPL-2)")
  # packageStartupMessage("creators: bit bitwhich")
  # packageStartupMessage("coercion: as.logical as.integer as.bit as.bitwhich which")
  # packageStartupMessage("operator: ! & | xor != ==")
  # packageStartupMessage("querying: print length any all min max range sum summary")
  # packageStartupMessage("bit access: length<- [ [<- [[ [[<-")
  # packageStartupMessage("for more help type ?bit")
# }

.onUnload <- function(libpath){
   #packageStartupMessage("Unloading package bit")
   bit_done()
   library.dynam.unload("bit", libpath)
}

# Functions for boolean vectors
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

# source("C:/mwp/eanalysis/bit/R/bit.R")

# Configuration: set this to 32L or 64L and keep in sync with BITS in bit.c
#' @rdname bit_init
#' @export
.BITS <- 32L

#' Initializing bit masks
#' 
#' Functions to allocate (and de-allocate) bit masks
#' 
#' The C-code operates with bit masks.  The memory for these is allocated
#' dynamically.  \code{bit_init} is called by \code{\link{.First.lib}} and
#' \code{bit_done} is called by \code{\link{.Last.lib}}.  You don't need to
#' care about these under normal circumstances.
#' 
#' @return NULL
#' @author Jens Oehlschlägel
#' @seealso \code{\link{bit}}
#' @keywords classes logic
#' @examples
#' 
#'   bit_done()
#'   bit_init()
#' 
#' @export
bit_init <- function()
  .Call(C_R_bit_init, .BITS)

#' @rdname bit_init
#' @export
bit_done <- function()
  .Call(C_R_bit_done)




#' Create empty bit vector 
#'
#' Bit vectors are a boolean type wihout \code{NA} that requires by factor 32 less RAM than \code{\link{logical}}.
#' For details on usage see the \href{../doc/bit-usage.html}{usage-vignette} and for details on performance see \href{../doc/bit-performance.html}{performance-vignette}
#'
#' @param length length in bits
#' @return \code{bit} returns a vector of integer sufficiently long to store 'length' bits
#' @seealso  \code{\link{booltype}}, \code{\link{bitwhich}}, \code{\link{logical}}
#' @keywords classes logic
#' @examples
#' bit(12)
#' !bit(12)
#' str(bit(128))
#' @export
bit <- function(length=0L){
  length <- as.integer(length)
  if (length %% .BITS)
    n <- length %/% .BITS + 1L
  else
    n <- length %/% .BITS
  if (.BITS==64L)
    x <- integer(2L*n)
  else
    x <- integer(n)
  #physical(x) <- list(vmode="boolean")
  #virtual(x)  <- list(Length=length)
  #class(x) <- "bit"
  # tuning
  p <- list()
  v <- list()
  attributes(p) <- list(vmode="boolean", class="physical")
  attributes(v) <- list(Length=length, class="virtual")
  setattributes(x, list(physical=p, virtual=v, class=c("booltype","bit")))
  x
}



#' Print method for bit 
#'
#' @param x a bit vector
#' @param ... passed to print
#' @return a character vector showing first and last elements of the bit vector
#' @examples
#' print(bit(120))
#' @export
print.bit <- function(x, ...){
  n <- length(x)
  cat("bit length=", n, " occupying only ", length(unclass(x)), " int32\n", sep="")
  if (n>16){
    y <- c(x[1:8], "..", x[(n-7L):n])
    names(y) <- c(1:8, "", (n-7L):n)
    print(y, quote=FALSE, ...)
  }else if(n){
    y <- c(x[])
    names(y) <- c(1:n)
    print(y, quote=FALSE, ...)
  }
}

#' Coerce bit to character
#'
#' @param x a \code{\link{bit}} vector
#' @param ... ignored
#' @return a character vector of zeroes and ones
#' @examples
#' as.character(bit(12))
#' @export
as.character.bit <- function(x, ...){
  c("0","1")[1+as.logical(x)]
}


#' Str method for bit
#'
#' To actually view the internal structure use \code{str(unclass(bit))}
#'
#' @inheritParams utils::str
#' @return \code{\link{invisible}}
#' @importFrom utils strOptions
#' @examples
#' str(bit(120))
#' @export
str.bit <- function(object
, vec.len  = strO$vec.len
, give.head = TRUE
, give.length = give.head
, ...
)
  {
  strO <- strOptions()
  vec.len <- 8*vec.len
  n <- length(object)
  if (n>vec.len)
    object <- as.bit(object[seq_len(vec.len)])
  cat(if (give.head)paste("bit ", if (give.length && n>1) paste(" [1:",n,"] ",sep=""), sep=""), paste(as.character(object), collapse=" ")," \n", sep="")
  invisible()
}


#' Coerce bitwhich to character
#'
#' @param x a \code{\link{bitwhich}} vector
#' @param ... ignored
#' @return a character vector of zeroes and ones
#' @examples
#' as.character(bitwhich(12))
#' @export
as.character.bitwhich <- function(x, ...)c("0","1")[1+as.logical(x)]


#' Str method for bitwhich
#'
#' To actually view the internal structure use \code{str(unclass(bitwhich))}
#'
#' @inheritParams utils::str
#' @return \code{\link{invisible}}
#' @examples
#' str(bitwhich(120))
#' @export
str.bitwhich <- function(object
                    , vec.len  = strO$vec.len
                    , give.head = TRUE
                    , give.length = give.head
                    , ...
){
  strO <- strOptions()
  vec.len <- 8*vec.len
  n <- length(object)
  if (n>vec.len)
    object <- as.bitwhich(object[seq_len(vec.len)])
  cat(if (give.head)paste("bitwhich ", if (give.length && n>1) paste(" [1:",n,"] ",sep=""), sep=""), paste(as.character(object), collapse=" ")," \n", sep="")
  invisible()
}



#' Create bitwhich vector (skewed boolean)
#'
#' A bitwhich object represents a boolean filter like a \code{\link{bit}} object (NAs are not allowed)
#' but uses a sparse representation suitable for very skewed (asymmetric) selections. 
#' Three extreme cases are represented with logical values, no length via logical(), 
#' all TRUE with TRUE and all FALSE with FALSE. All other selections are represented with 
#' positive or negative integers, whatever is shorter. 
#' This needs less RAM compared to \code{\link{logical}} (and often less than \code{\link{bit}} or \code{\link[=as.which]{which}}).
#' Logical operations are fast if the selection is asymetric (only few or almost all selected).
#'
#' @param maxindex length of the vector
#' @param x Information about which positions are FALSE or TRUE: either \code{logical()} or \code{TRUE} or \code{FALSE} or a integer vector of positive or of negative subscripts. 
#' @param xempty what to assume about parameter \code{x} if \code{x=integer(0)}, typically \code{TRUE} or \code{FALSE}. 
#' @param poslength tuning: \code{poslength} is calculated automatically, you can give \code{poslength} explicitely, in this case it must be correct and \code{x} must be sorted and not have duplicates.
#' @param is.unsorted tuning: FALSE implies that \code{x} is already sorted and sorting is skipped
#' @param has.dup tuning: FALSE implies that \code{x} has no duplicates 
#' @return an object of class 'bitwhich' carrying two attributes
#' \describe{
#'   \item{maxindex}{ see above }
#'   \item{poslength}{ see above }
#' }
#' @seealso \code{\link{bitwhich_representation}},  \code{\link{as.bitwhich}}, \code{\link{bit}}
#' @examples
#' bitwhich()
#' bitwhich(12)
#' bitwhich(12, x=TRUE)
#' bitwhich(12, x=3)
#' bitwhich(12, x=-3)
#' bitwhich(12, x=integer())
#' bitwhich(12, x=integer(), xempty=TRUE)
#' @export
bitwhich <- function(maxindex=0L, x=NULL, xempty=FALSE, poslength=NULL, is.unsorted=TRUE, has.dup=TRUE){
  maxindex <- as.integer(maxindex)
  if (maxindex==0L){
    if ((!is.null(poslength) && poslength) || (length(x) && (!is.logical(x) || x[[1]]==TRUE)))
      stop("maxindex=0 given with poslength or x")
    poslength <- 0L
    ret <- logical()
  }else{
    stopifnot(maxindex>0L)
    if (length(x)){
      if (is.logical(x)){
        if (length(x)!=1L || is.na(x)){
          stop("logical x should be scalar FALSE or TRUE")
        }else if (x){
          if (is.null(poslength))
            poslength <- maxindex
          else if (poslength!=maxindex)
            stop("x==TRUE implies poslength==maxindex")
          ret <- copy_vector(TRUE)
        }else{
          if (is.null(poslength))
            poslength <- 0L
          else if (poslength!=0L)
            stop("x==FALSE implies poslength==0")
          ret <- copy_vector(FALSE)
        }
      }else{
        x <- as.integer(x)
        if (is.null(poslength)){
          ret <- range_nanozero(x)
          r <- getsetattr(ret, "range_na", NULL)
          if (r[3]>0L)
            stop("NA positions not allowed (neither positive nor negative)")
          if (r[1]<0L){
            if (r[2]>0L)
              stop("mixed negative and positive subscripts not allowed")
            if (-r[1] > maxindex)
              stop("index value outside -maxindex..-1")
          }else{
            if (r[2] > maxindex)
              stop("index value outside 1..maxindex")
          }
          if (is.unsorted)
            ret <- bit_sort_unique(ret, na.last=NA, range_na=r)
          else if (has.dup)
            ret <- bit_unique(ret, na.rm = FALSE, range_na=r)
          if (ret[1]<0){
            poslength <- maxindex - length(ret)
            if (poslength){
              if (poslength <= maxindex%/%2L)
                ret <- merge_rangediff(c(1L,maxindex), ret, revx=FALSE, revy=TRUE)
            }else{
              ret <- copy_vector(FALSE)
            }
          }else{
            poslength <- length(ret)
            if (poslength < maxindex){
              if (poslength > maxindex%/%2L)
                ret <- merge_rangediff(c(1L,maxindex), ret, revx=TRUE, revy=TRUE)
            }else{
              ret <- copy_vector(TRUE)
            }
          }
        }else{
          poslength <- as.integer(poslength)
          if (poslength==0L)
            ret <- copy_vector(FALSE)
          else if (poslength==maxindex)
            ret <- copy_vector(TRUE)
          else{
            if (length(x) > 2 && x[1] >= x[2])
              stop("x is not sorted unique")
            if ( x[1]<0L ){
              if ( poslength != maxindex - length(x) ) 
                stop("wrong poslength")
              if (poslength <= maxindex%/%2L)
                ret <- merge_rangediff(c(1L,maxindex), x, revx=FALSE, revy=TRUE)
              else
                ret <- copy_vector(x)
            }else{
              if ( poslength != length(x) )
                stop("wrong poslength")
              if (poslength > maxindex%/%2L)
                ret <- merge_rangediff(c(1L,maxindex), x, revx=TRUE, revy=TRUE)
              else
                ret <- copy_vector(x)
            }
            
          }
        }
      }
    }else{
      if (is.null(poslength)){
          if (!is.logical(xempty) || length(xempty)!=1 || is.na(xempty))
            stop("xempty must be FALSE or TRUE")
          if (xempty)
            poslength <- maxindex
          else
            poslength <- 0L
          ret <- copy_vector(xempty)
      }else{
        poslength <- as.integer(poslength)
        if (poslength==0)
          ret <- copy_vector(FALSE)
        else if (poslength==maxindex)
          ret <- copy_vector(TRUE)
        else
          stop("need x with extreme poslength")
      } 
    }
  }
  setattributes(ret, list("maxindex" = maxindex, "poslength" = poslength, "class" = c("booltype","bitwhich")))
  ret
}



#' Diagnose representation of bitwhich
#'
#' @param x a \code{\link{bitwhich}} object
#' @return a scalar, one of \code{logical()}, \code{FALSE}, \code{TRUE}, \code{-1} or \code{1}  
#' @examples
#' bitwhich_representation(bitwhich())
#' bitwhich_representation(bitwhich(12,FALSE))
#' bitwhich_representation(bitwhich(12,TRUE))
#' bitwhich_representation(bitwhich(12, -3))
#' bitwhich_representation(bitwhich(12, 3))
#' @export
bitwhich_representation <- function(x)
{
  .Call(C_R_bitwhich_representation, x)
}



#' Print method for bitwhich
#'
#' @param x a \code{\link{bitwhich}} object
#' @param ... ignored
#' @export
print.bitwhich <- function(x, ...){
  n <- length(x)
  cat("bitwhich: ", sum(x), "/", n, " occupying only ", length(unclass(x)), " int32 in ", bitwhich_representation(x), " representation\n", sep="")
  if (n>16){
    y <- c(x[1:8], "..", x[(n-7L):n])
    names(y) <- c(1:8, "", (n-7L):n)
    print(y, quote=FALSE, ...)
  }else if(n){
    y <- c(x[])
    names(y) <- c(1:n)
    print(y, quote=FALSE, ...)
  }
}


#' Boolean types
#' 
#' The \code{\link{ordered}} factor \code{booltypes} ranks the boolean types. 
#' 
#' There are currently six boolean types, \code{booltypes} is an \code{\link{ordered}} vector with the following ordinal \code{\link{levels}} \describe{
#' \item{nobool}{non-boolean types}
#' \item{\code{\link{logical}}}{for representing any boolean data including \code{NA} }
#' \item{\code{\link{bit}}}{for representing dense boolean data }
#' \item{\code{\link{bitwhich}}}{for representing sparse (skewed) boolean data  }
#' \item{\code{\link{which}}}{for representing sparse boolean data with few \code{TRUE}}
## \item{\code{\link[ff]{hi}}}{hybrid-indexing, implemented in package \code{\link[ff]{ff}} }
#' \item{\code{\link{ri}}}{range-indexing, for representing sparse boolean data with a single range of \code{TRUE} }
#' }
#' \code{booltypes} has a \code{\link{names}} attribute such that elements can be selected by name. 
#'
#' @note do not rely on the internal integer codes of these levels, we might add-in \code{\link[ff]{hi}} later
#' @seealso \code{\link{booltype}}, \code{\link{is.booltype}}, \code{\link{as.booltype}}
#' @export
booltypes <- c("nobool","logical","bit","bitwhich","which","ri")
booltypes <- ordered(booltypes, levels=booltypes)
names(booltypes) <- booltypes


#' Diagnosing boolean types
#' 
#' Specific methods for \code{booltype} are required, where non-unary methods can combine multiple bollean types, particularly boolean binary operators.
#' 
#' Function \code{booltype} returns the boolean type of its argument.
#' There are currently six boolean types, \code{booltypes} is an \code{\link{ordered}} vector with the following ordinal \code{\link{levels}} \describe{
#' \item{nobool}{non-boolean types}
#' \item{\code{\link{logical}}}{for representing any boolean data including \code{NA} }
#' \item{\code{\link{bit}}}{for representing dense boolean data }
#' \item{\code{\link{bitwhich}}}{for representing sparse (skewed) boolean data  }
#' \item{\code{\link{which}}}{for representing sparse boolean data with few \code{TRUE}}
## \item{\code{\link[ff]{hi}}}{hybrid-indexing, implemented in package \code{\link[ff]{ff}} }
#' \item{\code{\link{ri}}}{range-indexing, for representing sparse boolean data with a single range of \code{TRUE} }
#' }
#' @param x an R object
#' 
#' @return one scalar element of \code{\link{booltypes}} in case of 'nobool' it carries a name attribute with the data type.
#' @note do not rely on the internal integer codes of these levels, we might add-in \code{\link[ff]{hi}} later
#' @seealso \code{\link{booltypes}}, \code{\link{is.booltype}}, \code{\link{as.booltype}}
#'
#' @examples
#' unname(booltypes)
#' str(booltypes)
#' sapply(list(double(),integer(),logical(),bit(),bitwhich(),as.which(),ri(1,2,3)), booltype)
#' @export
booltype <- function(x){
  if (is.ri(x))
    booltypes[["ri"]]
  else if (is.hi(x))
    booltypes[["hi"]]
  else if (is.which(x)) 
    booltypes[["which"]]
  else if (is.bitwhich(x)) 
    booltypes[["bitwhich"]]
  else if (is.bit(x)) 
    booltypes[["bit"]]
  else if (is.logical(x)) 
    booltypes[["logical"]]
  else {
    ret <- booltypes[["nobool"]]
    names(ret) <- typeof(x)
    ret
  }
}




#' Testing for boolean types
#'
#' All \code{\link{booltypes}} including  \code{\link{logical}} except 'nobool' types are considered 'is.booltype'. 
#'
#' @param x an R object
#'
#' @return logical scalar
#' @seealso \code{\link{booltypes}}, \code{\link{booltype}}, \code{\link{as.booltype}}
#'
#' @examples
#' sapply(list(double(),integer(),logical(),bit(),bitwhich(),as.which(),ri(1,2,3)), is.booltype)
#' @export
is.booltype <- function(x){
  inherits(x, "booltype") || is.logical(x)
}

#' @describeIn is.booltype tests for \code{\link{bit}}
#' @export
is.bit <- function(x)
  inherits(x, "bit")

#' @describeIn is.booltype tests for \code{\link{bitwhich}}
#' @export
is.bitwhich <- function(x)
  inherits(x, "bitwhich")

#' @describeIn is.booltype tests for \code{\link[=as.which]{which}}
#' @export
is.which <- function(x)
  inherits(x, "which")

#' @describeIn is.booltype tests for \code{\link[ff]{hi}}
#' @export
is.hi <- function(x)
  inherits(x, "hi")

#' @describeIn is.booltype tests for \code{\link{ri}}
#' @export
is.ri <- function(x)
  inherits(x, "ri")


#' @describeIn as.booltype default method for as.booltype
#' @export
as.booltype.default <- function(x, booltype="logical", ...){
  bt <- match.arg(as.character(booltype), as.character(booltypes))
  do.call(switch(bt
    , logical = "as.logical"
    , bit     = "as.bit"
    , bitwhich= "as.bitwhich"
    , which= "as.which"
    , hi= stop("not implemented for booltype hi")
    , ri= "as.ri"
    ), c(list(x, ...)))
}


#' @describeIn as.ri method to coerce \code{\link{ri}} to \code{\link{ri}}
#' @export
as.ri.ri <- function(x, ...)x

#' @describeIn as.ri default method to coerce to \code{\link{ri}}
#' @export
as.ri.default <- function(x, ...){
  r <- range.booltype(x)
  n <- maxindex(x)
  ri(r[[1]], r[[2]], n)
}



#' @describeIn maxindex default method for \code{maxindex}
#' @export
maxindex.default <- 
  function (x, ...) 
  {
    mi <- attr(x, "maxindex")
    if (is.null(mi)) 
      NA_integer_
    else mi
  }

#' @describeIn maxindex default method for \code{poslength}
#' @export
poslength.default <- 
  function (x, ...) 
  {
    pl <- attr(x, "poslength")
    if (is.null(pl))  
      NA_integer_
    else pl
  }


#' @describeIn maxindex \code{maxindex} method for class \code{\link{logical}}
#' @export
maxindex.logical <- function(x, ...){
  length(x)
}

#' @describeIn maxindex \code{poslength} method for class \code{\link{logical}}
#' @export
poslength.logical <- function(x, ...){
  sum(x)
}

#' @describeIn maxindex \code{maxindex} method for class \code{\link{bit}}
#' @export
maxindex.bit <- function(x, ...){
  length(x)
}

#' @describeIn maxindex \code{poslength} method for class \code{\link{bit}}
#' @export
poslength.bit <- function(x, ...)
  sum(x, ...)

#' @describeIn maxindex \code{maxindex} method for class \code{\link{bitwhich}}
#' @export
maxindex.bitwhich <- function(x, ...)
  length(x, ...)

#' @describeIn maxindex \code{poslength} method for class \code{\link{bitwhich}}
#' @export
poslength.bitwhich <- function(x, ...)
  sum(x, ...)

#' @describeIn maxindex \code{maxindex} method for class \code{\link[=as.which]{which}}
#' @export
maxindex.which <- function(x, ...){
  attr(x, "maxindex")
}

#' @describeIn maxindex \code{poslength} method for class \code{\link[=as.which]{which}}
#' @export
poslength.which <- function(x, ...){
  length(x)
}

#' @describeIn maxindex \code{maxindex} method for class \code{\link{ri}}
#' @export
maxindex.ri <- function(x, ...){
  x[[3]]
}

#' @describeIn maxindex \code{poslength} method for class \code{\link{ri}}
#' @export
poslength.ri <- function(x, ...){
  x[[2]] - x[[1]] + 1L
}



#' Getting and setting length of bit, bitwhich and ri objects
#' 
#' Query the number of bits in a \code{\link{bit}} vector or change the number
#' of bits in a bit vector. \cr Query the number of bits in a
#' \code{\link{bitwhich}} vector or change the number of bits in a bit vector.
#' \cr
#' 
#' NOTE that the length does NOT reflect the number of selected (\code{TRUE})
#' bits, it reflects the sum of both, \code{TRUE} and \code{FALSE} bits.
#' Increasing the length of a \code{\link{bit}} object will set new bits to
#' \code{FALSE}.  The behaviour of increasing the length of a
#' \code{\link{bitwhich}} object is different and depends on the content of the
#' object: \itemize{ 
#' \item TRUE -- all included, new bits are set to \code{TRUE}
#' \item positive integers -- some included, new bits are set to \code{FALSE}
#' \item negative integers -- some excluded, new bits are set to \code{TRUE}
#' \item FALSE -- all excluded:, new bits are set to \code{FALSE} } Decreasing the
#' length of bit or bitwhich removes any previous information about the status
#' bits above the new length.
#' 
#' @name length.bit
#' @param x a \code{\link{bit}}, \code{\link{bitwhich}} or \code{\link{ri}}
#' object
#' @param value the new number of bits
#' @return the length A bit vector with the new length
#' @author Jens Oehlschlägel
#' @seealso \code{\link{length}}, \code{\link[=sum.bit]{sum}},
#' \code{\link{poslength}}, \code{\link{maxindex}}
#' @keywords classes logic
#' @examples
#' 
#'   stopifnot(length(ri(1, 1, 32))==32)
#' 
#'   x <- as.bit(ri(32, 32, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==0)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==0)
#' 
#'   x <- as.bit(ri(1, 1, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#' 
#'   x <- as.bitwhich(bit(32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==0)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==0)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==0)
#' 
#'   x <- as.bitwhich(!bit(32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==32)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==16)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==32)
#' 
#'   x <- as.bitwhich(ri(32, 32, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==0)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==0)
#' 
#'   x <- as.bitwhich(ri(2, 32, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==31)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==15)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==31)
#' 
#'   x <- as.bitwhich(ri(1, 1, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==1)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==1)
#' 
#'   x <- as.bitwhich(ri(1, 31, 32))
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==31)
#'   message("NOTE the change from 'some excluded' to 'all excluded' here")
#'   length(x) <- 16
#'   stopifnot(length(x)==16)
#'   stopifnot(sum(x)==16)
#'   length(x) <- 32
#'   stopifnot(length(x)==32)
#'   stopifnot(sum(x)==32)
#' 
#' @export
length.bit <- function(x)
  virtual(x)$Length

#' @rdname length.bit
#' @export
"length<-.bit" <- function(x, value){
  value <- as.integer(value)
  vattr <- attr(x, "virtual")
  oldvalue <- attr(vattr, "Length")
  if (value!=oldvalue){
    pattr <- attr(x, "physical")
    cl <- oldClass(x)
    oldn <- get_length(x)
    dn <- value %% .BITS
    if (dn){
      n <- value %/% .BITS + 1L
    }else{
      n <- value %/% .BITS
    }
    if (oldn<n){
      ret <- integer(n)
      ret[seq_len(oldn)] <- x
    }else if (n<oldn){
      ret <- unclass(x)[seq_len(n)]
    }else{
      ret <- copy_vector(x)
    }
    if (dn && value<oldvalue){
      .Call(C_R_bit_set_logical, ret, FALSE, c(value+1L, n*.BITS))
    }
    attr(vattr, "Length") <- value
    setattributes(ret, list("physical" = pattr, "virtual" = vattr, "class" = cl))
    ret
  }else
    x
}


#' @rdname length.bit
#' @export
length.bitwhich <- function(x)
  attr(x, "maxindex")

#' @rdname length.bit
#' @export
"length<-.bitwhich" <- function(x, value){
  if (value!=length(x)){
    value <- as.integer(value)
    a <- attributes(x)
    if (value){
      if (is.integer(x)){
          oldClass(x) <- NULL
          if (x[1]>0){
            ret <- x[x <= value]
            l <- length(ret)
            if (l==0)
              ret <- copy_vector(FALSE)
            else if (l==value)
              ret <- copy_vector(TRUE)
            else if (l>(value%/%2L))
              ret <- merge_rangediff(c(-value,-1L), ret, revy=TRUE)
          }else{
            ret <- x[x >= -value]
            l <- length(ret)
            if (l==0)
              ret <- copy_vector(TRUE)
            else if (l==value)
              ret <- copy_vector(FALSE)
            else if (!((value-l)>(value%/%2L)))
              ret <- merge_rangediff(c(1L,value), ret, revy=TRUE)
            l <- value - l
          }
        }else{
          if (length(x) && x){
            ret <- bitwhich(value, x=TRUE, poslength=value)
            l <- value
          }else{
            ret <- bitwhich(value, x=FALSE, poslength=0L)
            l <- 0L
          }
        }
    }else{
      ret <- bitwhich()
      l <- 0L
    }
  }
  a$maxindex <- value
  a$poslength <- l
  setattributes(ret, a)
  ret
}



#' Concatenating booltype vectors
#' 
#' Creating new boolean vectors by concatenating boolean vectors
#' 
#' @param \dots \code{\link{booltype}} vectors
#' @return a vector with the lowest input \code{\link{booltype}} (but not lower than\code{\link{logical}})
#' @author Jens Oehlschlägel
#' @seealso \code{\link{c}}, \code{\link{bit}} , \code{\link{bitwhich}},  , \code{\link{which}}
#' @keywords classes logic
#' @examples
#'  c(bit(4), !bit(4))
#'  c(bit(4), !bitwhich(4))
#'  c(bitwhich(4), !bit(4))
#'  c(ri(1,2,4), !bit(4))
#'  c(bit(4), !logical(4))
#'  message("logical in first argument does not dispatch: c(logical(4), bit(4))")
#'  c.booltype(logical(4), !bit(4))
#' 
#' @export c.booltype
#' @export
c.booltype <- function(...){
  l <- list(...)
  bt <- sapply(l, booltype)
  # xx TEMPORARY WORKAROND: work around a bug in sapply which destroys ordered levels
  class(bt) <- c("ordered", "factor")
  bt <- max(booltypes[["logical"]], min(bt, booltypes[["bitwhich"]]))
  bt <- as.character(bt)
  f <- list(logical=as.logical, bit=as.bit, bitwhich=as.bitwhich, which=as.which)[[bt]]
  l <- lapply(l, f)
  do.call(switch(bt
 , logical="c"
 , bit="c.bit"
 , bitwhich="c.bitwhich"
  ),  l)
}

#' @rdname c.booltype
#' @export
c.bit <- function(...){
  l <- list(...)
  nl <- length(l)
  nold <- sapply(l, length)
  nnew <- sum(nold)
  ncum <- cumsum(nold)
  offsets <- c(0L, ncum[-length(ncum)])
  x <- bit(nnew)
  for (i in seq_len(nl)){
    b <- as.bit(l[[i]])
    .Call(C_R_bit_shiftcopy, bsource_=b, btarget_=x, otarget_=offsets[i], n_=nold[i])
  }
  x
}

#' @rdname c.booltype
#' @export
c.bitwhich <- function(...){
  l <- list(...)
  if (length(l)==1)
    l[[1]]
  else
    as.bitwhich(do.call("c", lapply(l, as.bit)))
}


#' Reversing bit and bitwhich vectors
#' 
#' Creating new bit or bitwhich by reversing such vectors
#' 
#' @name rev.booltype
#' @param x bit or bitwhich object
#' @return An object of class 'bit' or 'bitwhich'
#' @author Jens Oehlschlägel
#' @seealso \code{\link{rev}}, \code{\link{bit}} , \code{\link{bitwhich}}
#' @keywords classes logic
#' @examples
#' 
#'  rev(as.bit(c(FALSE,TRUE)))
#'  rev(as.bitwhich(c(FALSE,TRUE)))
NULL

#' @rdname rev.booltype
#' @export
rev.bit <- function(x){
  if (length(x)){
    x <- .Call(C_R_bit_reverse, x, bit(length(x)))
  }
  x
}

#' @rdname rev.booltype
#' @export
rev.bitwhich <- function(x){
  n <- length(x)
  if (is.logical(x)){
    ret <- bitwhich(n, copy_vector(x), poslength=sum(x))
  }else{
    y <- bitwhich_representation(x)
    if (n < .Machine$integer.max){
      if (y[1]<0)
        ret <- bitwhich(n, -(n+1L)-reverse_vector(x), poslength=sum(x))
      else
        ret <- bitwhich(n, (n+1L)-reverse_vector(x), poslength=sum(x))
    }else{
      if (y[1]<0)
        ret <- bitwhich(n, -n-reverse_vector(x)-1L, poslength=sum(x))
      else
        ret <- bitwhich(n, n-reverse_vector(x)+1L, poslength=sum(x))
    }
  }
  ret
}


#' Replicating bit and bitwhich vectors
#' 
#' Creating new bit or bitwhich by recycling such vectors
#' 
#' @name rep.booltype
#' @param x bit or bitwhich object
#' @param times number of replications
#' @param length.out final length of replicated vector (dominates times)
#' @param \dots not used
#' @return An object of class 'bit' or 'bitwhich'
#' @author Jens Oehlschlägel
#' @seealso \code{\link{rep}}, \code{\link{bit}} , \code{\link{bitwhich}}
#' @keywords classes logic
#' @examples
#' 
#'  rep(as.bit(c(FALSE,TRUE)), 2)
#'  rep(as.bit(c(FALSE,TRUE)), length.out=7)
#'  rep(as.bitwhich(c(FALSE,TRUE)), 2)
#'  rep(as.bitwhich(c(FALSE,TRUE)), length.out=1)
NULL

#' @rdname rep.booltype
#' @export
rep.bit <- function(x, times = 1L, length.out = NA, ...){
  if (length(times)>1L)
    stop("only scalar times supported")
  if (is.na(length.out))
    length.out <- length(x)*as.integer(times)
  else
    length.out <- as.integer(length.out)
  ret <- bit(length.out)
  .Call(C_R_bit_recycle, ret, x)
}

#' @rdname rep.booltype
#' @export
rep.bitwhich <- function(x, times = 1L, length.out = NA, ...){
  as.bitwhich(rep(as.bit(x), times=times, length.out=as.integer(length.out), ...))
}


#' @describeIn as.bit method to coerce to \code{\link{bit}} (zero length) from \code{\link{NULL}}
#' @export
as.bit.NULL <- function(x, ...){
  bit(0L)
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from \code{\link{bit}}
#' @export
as.bit.bit <- function(x, ...)
  x

#' @describeIn as.bit method to coerce to \code{\link{bit}} from \code{\link{logical}}
#' @export
as.bit.logical <- function(x, ...){
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_logical, b, x, c(1L, n))
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from
#'   \code{\link{integer}} (\code{0L} and \code{NA} become \code{FALSE},
#'   everthing else becomes \code{TRUE})
#' @examples as.bit(c(0L,1L,2L,-2L,NA))
#' @export
as.bit.integer <- function(x, ...){
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_integer, b, x, c(1L, n))
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from
#'   \code{\link{double}} (\code{0} and \code{NA} become \code{FALSE}, everthing
#'   else becomes \code{TRUE})
#' @examples as.bit(c(0,1,2,-2,NA))
#' @export
as.bit.double <- function(x, ...){
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_integer, b, as.integer(x), c(1L, n))
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from \code{\link{bitwhich}}
#' @export
as.bit.bitwhich <- function(x, ...){
  if (length(x)){
    b <- bit(length(x))
    if (is.logical(x)){
      if (unclass(x))
        !b
      else
        b
    }else{
      .Call(C_R_bit_replace, b, x, TRUE)
    }
  }else{
    bit()
  }
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from \code{\link[=as.which]{which}}
#' @export
as.bit.which <- function(x, length=attr(x, "maxindex"), ...){
  if (is.na(length))
    stop("cannot coerce to bit from which object with unknown maxindex")
  b <- bit(length)
  .Call(C_R_bit_replace, b, x, TRUE)
}

#' @describeIn as.bit method to coerce to \code{\link{bit}} from \code{\link{ri}}
#' @export
as.bit.ri <- function(x, ...){
  if (is.na(x[3]))
    stop("cannot coerce to bit from ri object with unknown maxindex")
  b <- bit(x[3])
  .Call(C_R_bit_set_logical, b, TRUE, x)
}



#' Coercion from bit, bitwhich, which and ri to logical, integer, double
#' 
#' Coercion from bit is quite fast because we use a double loop that fixes each
#' word in a processor register.
#' 
#' @name CoercionToStandard
#' @param x an object of class \code{\link{bit}}, \code{\link{bitwhich}} or
#' \code{\link{ri}}
#' @param length length of the boolean vector (required for \code{as.logical.which})
#' @param \dots ignored
#' @return \code{\link{as.logical}} returns a vector of \code{FALSE, TRUE},
#' \code{\link{as.integer}} and \code{\link{as.double}} return a vector of
#' \code{0,1}.
#' @author Jens Oehlschlägel
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
#' , \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
#' @keywords classes logic
#' @examples
#' 
#'   x <- ri(2, 5, 10)
#'   y <- as.logical(x)
#'   y
#'   stopifnot(identical(y, as.logical(as.bit(x))))
#'   stopifnot(identical(y, as.logical(as.bitwhich(x))))
#' 
#'   y <- as.integer(x)
#'   y
#'   stopifnot(identical(y, as.integer(as.logical(x))))
#'   stopifnot(identical(y, as.integer(as.bit(x))))
#'   stopifnot(identical(y, as.integer(as.bitwhich(x))))
#' 
#'   y <- as.double(x)
#'   y
#'   stopifnot(identical(y, as.double(as.logical(x))))
#'   stopifnot(identical(y, as.double(as.bit(x))))
#'   stopifnot(identical(y, as.double(as.bitwhich(x))))
NULL

#' @rdname CoercionToStandard
#' @export 
as.logical.bit <- function(x, ...){
  l <- logical(length(x))
  .Call(C_R_bit_get_logical, x, l, c(1L, length(x)))
}

#' @rdname CoercionToStandard
#' @export 
as.integer.bit <- function(x, ...){
  l <- integer(length(x))
  .Call(C_R_bit_get_integer, x, l, c(1L, length(x)))
}

#' @rdname CoercionToStandard
#' @export 
as.double.bit <- function(x, ...){
  l <- integer(length(x))
  as.double(.Call(C_R_bit_get_integer, x, l, c(1L, length(x))))
}

#' @rdname CoercionToStandard
#' @export 
as.integer.bitwhich <- function(x, ...){
  n <- length(x)
  if (is.logical(x)){
    if (sum(x)==n)
      rep(1L, n)
    else
      rep(0L, n)
  }else{
    ret <- integer(n)
    ret[x] <- 1L
    ret
  }
}

#' @rdname CoercionToStandard
#' @export 
as.double.bitwhich <- function(x, ...){
  n <- length(x)
  if (is.logical(x)){
    if (sum(x)==n)
      rep(1, n)
    else
      rep(0, n)
  }else{
    ret <- double(n)
    ret[x] <- 1
    ret
  }
}


#' @rdname CoercionToStandard
#' @export 
as.logical.bitwhich <- function(x, ...){
  n <- length(x)
  p <- sum(x)
  if (p==0){
    rep(FALSE, length(x))
  }else if (p==n){
    rep(TRUE, length(x))
  }else{
    ret <- logical(length(x))
    ret[x] <- TRUE
    ret
  }
}

#' @rdname CoercionToStandard
#' @export 
as.logical.ri <- function(x, ...){
  if (is.na(x[3]))
    stop("cannot coerce to logical from ri object with unknown maxindex")
  ret <- logical(x[3])
  ret[x[1]:x[2]] <- TRUE
  ret
}

#' @rdname CoercionToStandard
#' @export 
as.integer.ri <- function(x, ...){
  if (is.na(x[3]))
    stop("cannot coerce to integer from ri object with unknown maxindex")
  ret <- integer(x[3])
  ret[x[1]:x[2]] <- 1L
  ret
}

#' @rdname CoercionToStandard
#' @export 
as.double.ri <- function(x, ...){
  if (is.na(x[3]))
    stop("cannot coerce to integer from ri object with unknown maxindex")
  ret <- double(x[3])
  ret[x[1]:x[2]] <- 1
  ret
}


#' @rdname CoercionToStandard
#' @export 
as.logical.which <- function(x, length=attr(x, "maxindex"), ...){
  if (is.na(length))
    stop("cannot coerce to logical from which object with unknown maxindex")
  l <- logical(length)
  l[x] <- TRUE
  l
}


#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link[=as.which]{which}}
#' @export
as.which.which <- function(x, maxindex=NA_integer_, ...)x

#' @describeIn as.which method to coerce to zero length \code{\link[=as.which]{which}} from \code{\link{NULL}}
#' @export
as.which.NULL <- function(x, ...)structure(integer(), maxindex=0L, class=c("booltype", "which"))

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{numeric}}
#' @export
as.which.numeric <- function(x, maxindex=NA_integer_, ...){
  as.which(as.integer(x), maxindex=maxindex, ...)
}

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{integer}}
#' @export
as.which.integer <- function(x, maxindex=NA_integer_, is.unsorted=TRUE, has.dup=TRUE, ...){
  ret <- range_nanozero(as.integer(x))
  r <- getsetattr(ret, "range_na", NULL)
  if (length(ret)){
    if (r[3]>0L)
      stop("NA positions not allowed (neither positive nor negative)")
    if (r[1]<0L){
      if (r[2]>0L)
        stop("mixed negative and positive subscripts not allowed")
      if (is.na(maxindex))
        stop("need maxindex with negative subscripts")
      else if (-r[1] > maxindex)
        stop("index value outside -maxindex..-1")
    }else{
      if (!is.na(maxindex) && r[2] > maxindex)
        stop("index value outside 1..maxindex")
    }
    if (is.unsorted)
      ret <- bit_sort_unique(ret, na.last=NA, range_na=r, has.dup=has.dup)
    else if (has.dup)
      ret <- bit_unique(ret, na.rm = FALSE, range_na=r)
    if (r[1]<0L)
      ret <- merge_rangediff(c(1L,maxindex), ret, revx=FALSE, revy=TRUE)
  }
  setattributes(ret, list("maxindex" = maxindex, "class" = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{logical}}
#' @export
as.which.logical <- function(x, ...){
  ret <- which(x)
  setattributes(ret, list("maxindex" = as.integer(length(x)), "class" = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{ri}}
#' @export
as.which.ri <- function(x, ...){
  ret <- x[1]:x[2]
  setattributes(ret, list("maxindex" = as.integer(x[3]), "class" = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{bit}}
#' @export
as.which.bit <- function(x, range=NULL, ...){
  maxindex <- length(x)
  if (is.null(range))
    range <- c(1L, maxindex)
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>maxindex)
      stop("illegal range")
  }
  s <- sum(x, range=range)
  n <- range[2] - range[1] + 1L
  if (s==0L){
    ret <- integer()
  }else if (s==n){
    #ret <- as.integer(seq.int(from=range[1], to=range[2], by=1))
    ret <- merge_rangediff(range, integer())
  }else
    ret <- .Call(C_R_bit_which, x, s, range, negative=FALSE)
  setattributes(ret, list("maxindex" = as.integer(maxindex), "class" = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to \code{\link[=as.which]{which}} from \code{\link{bitwhich}}
#' @export
as.which.bitwhich <- function(x, ...){
  maxindex <- length(x)
  if (is.logical(x)){
    if (maxindex && unclass(x))
      ret <- seq_len(maxindex)
    else
      ret <- integer()
  }else{
    if (unclass(x)[[1]]<0)
      ret <- merge_rangediff(c(1L,maxindex), x, revx=FALSE, revy=TRUE)
    else
      ret <- copy_vector(x)
  }
  setattributes(ret, list("maxindex" = as.integer(maxindex), "class" = c("booltype", "which")))
  ret
}


#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} (zero length) from \code{\link{NULL}}
#' @export
as.bitwhich.NULL <- function(x, ...){
  bitwhich()
}

#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from \code{\link{bitwhich}}
#' @export
as.bitwhich.bitwhich <- function(x, ...){
  x
}

#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from \code{\link[=as.which]{which}}
#' @export
as.bitwhich.which <- function(x, maxindex=attr(x, "maxindex"), ...){
  if (is.na(maxindex))
    stop("need maxindex")
  if (maxindex==0)
    bitwhich()
  else{
    poslength <- length(x)
    if (poslength==0)
      bitwhich(maxindex, FALSE, poslength)
    else if (poslength==maxindex)
      bitwhich(maxindex, TRUE, poslength)
    else if (poslength>(maxindex%/%2L)){
      bitwhich(maxindex, merge_rangediff(c(1L,maxindex), x, revx=TRUE, revy=TRUE), poslength=poslength)
    }else{
      bitwhich(maxindex, x, poslength=poslength)
    }
  } 
}

#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from \code{\link{ri}}
#' @export
as.bitwhich.ri <- function(x, ...){
  maxindex <- length(x)
  if (is.na(maxindex))
    stop("you must provide maxindex with ri() in as.bitwhich.ri()")
  # ri selects at least one element, 
  # hence maxindex>0 and poslength>0
  poslength <- sum(x)
  if (poslength==maxindex)
    bitwhich(maxindex, TRUE, poslength=poslength)
  else if (poslength>(maxindex%/%2L)){
    if (x[1]>1L) b <- (-x[1]+1L):(-1) else b <- integer()
    if (x[2]<maxindex) a <- (-maxindex):(-x[2]-1L) else a <- integer()
    bitwhich(maxindex, c(a,b), poslength=poslength)
  }else{
    bitwhich(maxindex, x[1]:x[2], poslength=poslength)
  }
}



#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from
#'   \code{\link{integer}} (\code{0} and \code{NA} become \code{FALSE}, everthing
#'   else becomes \code{TRUE})
#' @examples as.bitwhich(c(0L,1L,2L,-2L,NA))
#' @export
as.bitwhich.integer <- function(x, poslength=NULL, ...)
  as.bitwhich(as.logical(x), poslength=poslength, ...)


#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from
#'   \code{\link{double}} (\code{0} and \code{NA} become \code{FALSE}, everthing
#'   else becomes \code{TRUE})
#' @examples as.bitwhich(c(0,1,2,-2,NA))
#' @export
as.bitwhich.double <- as.bitwhich.integer 


#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from \code{\link{logical}}
#' @export
as.bitwhich.logical <- function(x, poslength=NULL, ...){
  maxindex <- length(x)
  if (maxindex==0)
    bitwhich()
  else{
    if (is.null(poslength))
      poslength <- sum(x, na.rm=TRUE)
    else
      if(poslength>maxindex)
        stop("poslength > maxindex")
    if (poslength==0)
      bitwhich(maxindex, FALSE, poslength=poslength)
    else if (poslength==maxindex)
      bitwhich(maxindex, TRUE, poslength=poslength)
    else if (poslength>(maxindex%/%2L)){
      as.bitwhich(as.bit(x), poslength=poslength)
    }else{
      bitwhich(maxindex, which(x), poslength=poslength)
    }
  } 
}


#' @describeIn as.bitwhich method to coerce to \code{\link{bitwhich}} from \code{\link{bit}}
#' @export
as.bitwhich.bit <- function(x, range=NULL, poslength=NULL, ...){
  maxindex <- length(x)
  if (maxindex){
    if (is.null(range))
      range <- c(1L, maxindex)
    else{
      range <- as.integer(range[1:2])
      if (range[1]<1L || range[2]>maxindex)
        stop("illegal range")
    }
    if (is.null(poslength))
      poslength <- sum(x, range=range, na.rm=TRUE)
    else
      if(poslength>maxindex)
        stop("poslength > maxindex")
    if (poslength==0)
      bitwhich(maxindex, FALSE, poslength=poslength)
    else if (poslength==maxindex)
      bitwhich(maxindex, TRUE, poslength=poslength)
    else{
      if (poslength>(maxindex%/%2L)){
        bitwhich(maxindex, .Call(C_R_bit_which, x, maxindex - poslength, range=range, negative=TRUE), poslength=poslength)
      }else{
        bitwhich(maxindex, .Call(C_R_bit_which, x, poslength, range=range, negative=FALSE), poslength=poslength)
      }
    }
  }else bitwhich()
}


#' Test for NA in bit and bitwhich
#'
#' @name is.na.bit
#' @param x a \code{\link{bit}} or  \code{\link{bitwhich}} vector
#'
#' @return vector of same type with all elements \code{FALSE}
#' @seealso \code{\link{is.na}}
#'
#' @examples
#' is.na(bit(6))
#' is.na(bitwhich(6))
is.na.bit <- function(x)bit(length(x))

#' @describeIn is.na.bit method for \code{\link{is.na}} from \code{\link{bitwhich}}
#' @export
is.na.bitwhich <- function(x)bitwhich(length(x))


#' @describeIn xor default method for \code{\link{xor}}
#' @export
xor.default <- function (x, y) 
{
  cat("default\n")
  (x | y) & !(x & y)
}

#' @describeIn xor \code{\link{logical}} method for \code{\link{xor}}
#' @export
xor.logical <- function(x,y){
    as.logical(x) != as.logical(y)
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{!}}
#' @export
"!.bit" <- function(x){
  if (length(x)){
    ret <- copy_vector(x)
    setattributes(ret, attributes(x))
    .Call(C_R_bit_not, ret)
  }else{
    x
  }
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{&}}
#' @export
"&.bit" <- function(e1, e2){
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bit())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_and, e1, e2, ret)
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{|}}
#' @export
"|.bit" <- function(e1, e2){
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bit())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_or, e1, e2, ret)
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{==}}
#' @export
"==.bit" <- function(e1, e2){
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bit())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_equal, e1, e2, ret)
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{!=}}
#' @export
"!=.bit" <- function(e1, e2){
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bit())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_xor, e1, e2, ret)
}

#' @describeIn xor \code{\link{bit}} method for \code{\link{xor}}
#' @export
"xor.bit" <- function(x, y){
  n <- c(length(x), length(y))
  if (any(n==0L))
    return(bit())
  if(n[1]!=n[2])
    stop("length(x) != length()")
  x <- as.bit(x)
  y <- as.bit(y)
  ret <- bit(n[1])
  .Call(C_R_bit_xor, x, y, ret)
}


#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{!}}
#' @export
"!.bitwhich" <- function(x){
  n <- length(x)
  p <- sum(x)
  if (is.logical(x)){
    if (n==0)
      bitwhich()
    else if (p==n){
      bitwhich(maxindex=n, FALSE, poslength=0L)
    }else{
      bitwhich(maxindex=n, TRUE, poslength=n)
    }
  }else{
    #bitwhich(maxindex=n, -rev(unclass(x)), poslength=n-p, is.unsorted = FALSE, has.dup=FALSE)
    bitwhich(maxindex=n, copy_vector(x, revx=TRUE), poslength=n-p, is.unsorted = FALSE, has.dup=FALSE)
  }
}

#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{&}}
#' @export
"&.bitwhich" <- function(e1, e2){
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bitwhich())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1]==0 || p[2]==0)
    return(bitwhich(n[1], FALSE, 0L))
  if (p[1]==n[1])
    return(e2)
  if (p[2]==n[2])
    return(e1)
  #negative <- p>(n%/%2L)
  negative <- c(unclass(e1)[1]<0, unclass(e2)[1]<0)
  if (negative[1]){
    if (negative[2]){
      ret <- merge_union(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret)) 
    }else{
      ret <- merge_setdiff(e2, e1, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }else{
    if (negative[2]){
      ret <- merge_setdiff(e1, e2, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }else{
      ret <- merge_intersect(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }
}


#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{|}}
#' @export
"|.bitwhich" <- function(e1, e2){
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bitwhich())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1]==n[1] || p[2]==n[2])
    return(bitwhich(n[1], TRUE, n[1]))
  if (p[1]==0)
    return(e2)
  if (p[2]==0)
    return(e1)
  #negative <- p>(n%/%2L)
  negative <- c(unclass(e1)[1]<0, unclass(e2)[1]<0)
  if (negative[1]){
    if (negative[2]){
      ret <- merge_intersect(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }else{
      ret <- merge_setdiff(e1, e2, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }
  }else{
    if (negative[2]){
      ret <- merge_setdiff(e2, e1, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }else{
      ret <- merge_union(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret)) 
    }
  }
}

#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{==}}
#' @export
"==.bitwhich" <- function(e1, e2){
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bitwhich())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1]==0)
    return(!e2)
  if (p[1]==n[1])
    return(e2)
  if (p[2]==0)
    return(!e1)
  if (p[2]==n[2])
    return(e1)
  #negative <- p>(n%/%2L)
  negative <- c(unclass(e1)[1]<0, unclass(e2)[1]<0)
  if (negative[1]){
    if (negative[2]){
      ret <- merge_symdiff(e1, e2, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }else{
      ret <- merge_symdiff(e1, e2, revx=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }else{
    if (negative[2]){
      ret <- merge_symdiff(e1, e2, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }else{
      ret <- merge_symdiff(e1, e2, revx=TRUE, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }
  }
}

#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{!=}}
#' @export
"!=.bitwhich" <- function(e1, e2){
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n==0L))
    return(bitwhich())
  if(n[1]!=n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1]==0)
    return(e2)
  if (p[1]==n[1])
    return(!e2)
  if (p[2]==0)
    return(e1)
  if (p[2]==n[2])
    return(!e1)
  #negative <- p>(n%/%2L)
  negative <- c(unclass(e1)[1]<0, unclass(e2)[1]<0)
  if (negative[1]){
    if (negative[2]){
      ret <- merge_symdiff(e1, e2, revx=TRUE, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }else{
      ret <- merge_symdiff(e1, e2, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }
  }else{
    if (negative[2]){
      ret <- merge_symdiff(e1, e2, revx=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1]-length(ret))
    }else{
      ret <- merge_symdiff(e1, e2, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }
}

#' @describeIn xor \code{\link{bitwhich}} method for \code{\link{xor}}
#' @export
"xor.bitwhich" <- function(x, y) x != y

#' @describeIn xor \code{\link{booltype}} method for \code{\link{&}}
#' @export &.booltype
#' @export
"&.booltype" <- function(e1, e2){
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1,b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2){
    if (n1 < n2){
      if (n2%%n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    }else if (n2 < n1){
      if (n1%%n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(  as.character(b)
         , "logical" = e1 & e2
         , "bit" = "&.bit"(e1, e2)
         , "bitwhich" = "&.bitwhich"(e1, e2)
  )
}

#' @describeIn xor \code{\link{booltype}} method for \code{\link{|}}
#' @export |.booltype
#' @export
"|.booltype" <- function(e1, e2){
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1,b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2){
    if (n1 < n2){
      if (n2%%n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    }else if (n2 < n1){
      if (n1%%n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(  as.character(b)
           , "logical" = e1 | e2
           , "bit" = "|.bit"(e1, e2)
           , "bitwhich" = "|.bitwhich"(e1, e2)
  )
}

#' @describeIn xor \code{\link{booltype}} method for \code{\link{==}}
#' @export ==.booltype
#' @export
"==.booltype" <- function(e1, e2){
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1,b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2){
    if (n1 < n2){
      if (n2%%n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    }else if (n2 < n1){
      if (n1%%n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(  as.character(b)
           , "logical" = e1 == e2
           , "bit" = "==.bit"(e1, e2)
           , "bitwhich" = "==.bitwhich"(e1, e2)
  )
}

#' @describeIn xor \code{\link{booltype}} method for \code{\link{!=}}
#' @export !=.booltype
#' @export
"!=.booltype" <- function(e1, e2){
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1,b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2){
    if (n1 < n2){
      if (n2%%n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    }else if (n2 < n1){
      if (n1%%n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(  as.character(b)
           , "logical" = e1 != e2
           , "bit" = "!=.bit"(e1, e2)
           , "bitwhich" = "!=.bitwhich"(e1, e2)
  )
}

#' @describeIn xor \code{\link{booltype}} method for \code{\link{xor}}
#' @export xor.booltype
#' @export
"xor.booltype" <- function(x, y){
  x != y
}



#' Summaries of boolean vectors
#' 
#' Fast aggregation functions for \code{\link{booltype}} vectors. namely \code{\link{bit}}, \code{\link{all}}, \code{\link{any}}, \code{\link{anyNA}},
#' \code{\link{min}}, \code{\link{max}}, \code{\link{range}},
#' \code{\link{sum}} and \code{\link{summary}}.
#' Now all boolean summaries (except for \code{anyNA} because the generic does not allow it) have an optional \code{range} argument to restrict the range of evalution.
#' Note that the boolean summaries have meaning and return values differing from logical aggregation functions: they treat \code{NA} as \code{FALSE}, 
#' \code{min}, \code{max} and \code{range} give the minimum and maximum positions of \code{TRUE}, \code{summary} returns counts of \code{FALSE},  \code{TRUE} and the \code{range}.
#' Note that you can force the boolean interpretation by calling the booltype method explicitely on any \code{\link{booltypes}} input, e.g. \code{min.booltype()}, see the examples.
#' 
#' Summaries of \code{\link{bit}} vectors are quite fast because we use a double loop that fixes each
#' word in a processor register.  Furthermore we break out of looping as soon
#' as possible. Summaries of \code{\link{bitwhich}} vectors are even faster, if the selection is very skewed.
#' 
#' @name Summaries
#' @param x an object of class bit or bitwhich
#' @param object an object of class bit
#' @param range a \code{\link{ri}} or an integer vector of length==2 giving a
#' range restriction for chunked processing
#' @param recursive formally required but not used
#' @param \dots formally required but not used
#' @return as expected
#' @author Jens Oehlschlägel
#' @seealso \code{\link{length}}
#' @keywords classes logic
#' @examples
#' 
#'   l <- c(NA, FALSE, TRUE)
#'   b <- as.bit(l)
#'   
#'   all(l)
#'   all(b)
#'   all(b, range=c(3,3))
#'   all.booltype(l, range=c(3,3))
#'   
#'   min(l)
#'   min(b)
#'   
#'   sum(l)
#'   sum(b)
#'   
#'   summary(l)
#'   summary(b)
#'   summary.booltype(l)
NULL

# xx MEMO: R CMD check --no-tests  --no-manual --no-vignettes bit

#' @rdname Summaries
#' @export 
all.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_all, x, range)
}

#' @rdname Summaries
#' @export
any.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_any, x, range)
}

#' @rdname Summaries
#' @export
anyNA.bit <- function(x
                      #, range=NULL
                      , recursive = FALSE)FALSE

#' @rdname Summaries
#' @export
sum.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_sum, x, range)
}

#' @rdname Summaries
#' @export
min.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_min, x, range)
}

#' @rdname Summaries
#' @export
max.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_max, x, range)
}

#' @rdname Summaries
#' @export
range.bit <- function(x, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(x))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(x))
      stop("illegal range")
  }
  ret <- integer(2)
  ret[1] <- .Call(C_R_bit_min, x, range)
  if (is.na(ret[1]))
    ret[2] <- NA_integer_
  else
    ret[2] <- .Call(C_R_bit_max, x, range)
  ret
}

#' @rdname Summaries
#' @export
summary.bit <- function(object, range=NULL, ...){
  if (is.null(range))
    range <- c(1L, length(object))
  else{
    range <- as.integer(range[1:2])
    if (range[1]<1L || range[2]>length(object))
      stop("illegal range")
  }
  s <- sum(object, range=range)
  r <- range(object, range=range)
  c("FALSE"=range[2]-range[1]+1L-s, "TRUE"=s, "Min."=r[1], "Max."=r[2])
}



#' @rdname Summaries
#' @export
all.bitwhich <- function(x, range=NULL, ...){
  if (is.null(range))
    attr(x, "poslength") == attr(x, "maxindex")
  else{
    y <- bitwhich_representation(x)
    range <- as.integer(range)
    if (is.logical(y)){
      if (y) 
        TRUE
      else
        FALSE
    }else{
      if (y<0){
        all(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
      }else{
        all(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
      }
    }
  }
}

#' @rdname Summaries
#' @export
any.bitwhich <- function(x, range=NULL, ...){
  if (is.null(range))
    attr(x, "poslength") > 0L
  else{
    y <- bitwhich_representation(x)
    range <- as.integer(range)
    if (is.logical(y)){
      if (y) 
        TRUE
      else
        FALSE
    }else{
      if (y<0){
        any(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
      }else{
        any(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
      }
    }
  }
}

#' @rdname Summaries
#' @export
anyNA.bitwhich <- function(x
                           #, range=NULL
                           , recursive = FALSE)FALSE

#' @rdname Summaries
#' @export
sum.bitwhich <- function(x, range=NULL, ...){
  if (is.null(range))
    attr(x, "poslength")
  else{
    y <- bitwhich_representation(x)
    range <- as.integer(range)
    if (is.logical(y)){
      if (y) 
        range[2] - range[1] + 1L
      else
        0L
    }else{
      if (y<0){
        sum(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
      }else{
        sum(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
      }
    }
  }
}

#' @rdname Summaries
#' @export
min.bitwhich <- function(x, range=NULL, ...){
  y <- bitwhich_representation(x)
  if (is.logical(y)){
    if (length(y) && y)
      1L
    else
      NA_integer_
  }else{
    if (is.null(range)){
      if (y<0L){
        merge_firstnotin(c(1L,length(x)), x, revy=TRUE)
      }else{
        merge_first(x)
      }
    }else{
      range <- as.integer(range)
      if (y<0L){
        merge_firstnotin(range, x, revy=TRUE)
      }else{
        merge_firstin(range, x)
      }
    }
  }  
}

#' @rdname Summaries
#' @export
max.bitwhich <- function(x, range=NULL, ...){
  y <- bitwhich_representation(x)
  if (is.logical(y)){
    if (length(y) && y)
      length(x)
    else
      NA_integer_
  }else{
    if (is.null(range)){
      if (y<0L){
        merge_lastnotin(c(1L,length(x)), x, revy=TRUE)
      }else{
        merge_last(x)
      }
    }else{
      range <- as.integer(range)
      if (y<0L){
        merge_lastnotin(range, x, revy=TRUE)
      }else{
        merge_lastin(range, x)
      }
    }
  }  
}

#' @rdname Summaries
#' @export
range.bitwhich <- function(x, range=NULL, ...){
  c(min(x, range=range, ...), max(x, range=range, ...))
}

#' @rdname Summaries
#' @export
summary.bitwhich <- function(object, range=NULL, ...){
  n <- attr(object, "maxindex")
  p <- attr(object, "poslength")
  r <- range(object)
  c("FALSE"=n-p, "TRUE"=p, "Min."=r[1], "Max."=r[2])
}




#' @rdname Summaries
#' @export
all.which <- function(x, range=NULL, ...){
  if (is.null(range))
    length(x) == attr(x, "maxindex")
  else{
    range <- as.integer(range)
    all(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
any.which <- function(x, range=NULL, ...){
  if (is.null(range))
    length(x) > 0L
  else{
    range <- as.integer(range)
    any(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
anyNA.which <- function(x
                        #, range=NULL
                        , recursive = FALSE)FALSE

#' @rdname Summaries
#' @export
sum.which <- function(x, range=NULL, ...){
  if (is.null(range))
    length(x)
  else{
    sum(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
min.which <- function(x, range=NULL, ...){
  if (is.null(range)){
    merge_first(x)
  }else{
    range <- as.integer(range)
    merge_firstin(range, x)
  }
}

#' @rdname Summaries
#' @export
max.which <- function(x, range=NULL, ...){
  if (is.null(range)){
    merge_last(x)
  }else{
    range <- as.integer(range)
    merge_lastin(range, x)
  }
}

#' @rdname Summaries
#' @export
range.which <- function(x, range=NULL, ...){
  c(min(x, range=range, ...), max(x, range=range, ...))
}

#' @rdname Summaries
#' @export
summary.which <- function(object, range=NULL, ...){
  n <- attr(object, "maxindex")
  p <- attr(object, "poslength")
  r <- range(object)
  c("FALSE"=n-p, "TRUE"=p, "Min."=r[1], "Max."=r[2])
}

#' @rdname Summaries
#' @export all.booltype
#' @export
all.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=all.bit(as.bit(x), range=range, ...)
         , logical=all.bit(as.bit(x), range=range, ...)
         , bit=all.bit(x, range=range, ...)
         , bitwhich=all.bitwhich(x, range=range, ...)
         , which=all.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=all.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export any.booltype
#' @export
any.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=any.bit(as.bit(x), range=range, ...)
         , logical=any.bit(as.bit(x), range=range, ...)
         , bit=any.bit(x, range=range, ...)
         , bitwhich=any.bitwhich(x, range=range, ...)
         , which=any.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=any.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export anyNA.booltype
#' @export
anyNA.booltype <- function(x
                           #, range=NULL
                           , ...){
  switch(as.character(booltype(x))
         , nobool=anyNA.bit(as.bit(x)
                            #, range=range
                            , ...)
         , logical=anyNA.bit(as.bit(x)
                             #, range=range
                             , ...)
         , bit=anyNA.bit(x
                         #, range=range
                         , ...)
         , bitwhich=anyNA.bitwhich(x
                                   #, range=range
                                   , ...)
         , which=anyNA.bit(as.bit(x)
                           #, range=range
                           , ...)
         , hi=stop("not implemented")
         , ri=anyNA.ri(x
                       #, range=range
                       , ...)
  )
}


#' @rdname Summaries
#' @export sum.booltype
#' @export
sum.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=sum.bit(as.bit(x), range=range, ...)
         , logical=sum.bit(as.bit(x), range=range, ...)
         , bit=sum.bit(x, range=range, ...)
         , bitwhich=sum.bitwhich(x, range=range, ...)
         , which=sum.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=sum.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export min.booltype
#' @export
min.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=min.bit(as.bit(x), range=range, ...)
         , logical=min.bit(as.bit(x), range=range, ...)
         , bit=min.bit(x, range=range, ...)
         , bitwhich=min.bitwhich(x, range=range, ...)
         , which=min.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=min.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export max.booltype
#' @export
max.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=max.bit(as.bit(x), range=range, ...)
         , logical=max.bit(as.bit(x), range=range, ...)
         , bit=max.bit(x, range=range, ...)
         , bitwhich=max.bitwhich(x, range=range, ...)
         , which=max.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=max.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export range.booltype
#' @export
range.booltype <- function(x, range=NULL, ...){
  switch(as.character(booltype(x))
         , nobool=range.bit(as.bit(x), range=range, ...)
         , logical=range.bit(as.bit(x), range=range, ...)
         , bit=range.bit(x, range=range, ...)
         , bitwhich=range.bitwhich(x, range=range, ...)
         , which=range.bit(as.bit(x), range=range, ...)
         , hi=stop("not implemented")
         , ri=range.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export summary.booltype
#' @export
summary.booltype <- function(object, range=NULL, ...){
  switch(as.character(booltype(object))
         , nobool=summary.bit(as.bit(object), range=range, ...)
         , logical=summary.bit(as.bit(object), range=range, ...)
         , bit=summary.bit(object, range=range, ...)
         , bitwhich=summary.bitwhich(object, range=range, ...)
         , which=summary.bit(as.bit(object), range=range, ...)
         , hi=stop("not implemented")
         , ri=summary.ri(object, range=range, ...)
  )
}


#' Extract or replace part of an boolean vector
#' 
#' Operators acting on \code{\link{bit}} or \code{\link{bitwhich}} objects to extract or replace parts.
#' 
#' The typical usecase for for '[' and '[<-' is subscripting with positive integers, 
#' negative integers are allowed but slower, 
#' as logical subscripts only scalars are allowed. 
#' The subscript can be given as a \code{\link{bitwhich}} object.
#' Also \code{\link{ri}} can be used as subscript.
#' 
#' Extracting from \code{\link{bit}} and \code{\link{bitwhich}} is faster than from  \code{\link{logical}} if positive subscripts are used.
#' integer subscripts make sense.  Negative subscripts are converted to
#' positive ones, beware the RAM consumption.  
#' 
#' @name Extract
#' @param x a \code{\link{bit}} or \code{\link{bitwhich}} object
#' @param i preferrably a positive integer subscript or a \code{\link{ri}}, see text
#' @param value new logical or integer values
#' @return The extractors \code{[[} and \code{[} return a logical scalar or
#' vector.  The replacment functions return an object of \code{class(x)}. 
#' @author Jens Oehlschlägel
#' @seealso \code{\link{bit}}, \code{\link[base]{Extract}}
#' @keywords classes logic
#' @examples
#' 
#'   x <- as.bit(c(FALSE, NA, TRUE))
#'   x[] <- c(FALSE, NA, TRUE)
#'   x[1:2]
#'   x[-3]
#'   x[ri(1,2)]
#'   x[as.bitwhich(c(TRUE,TRUE,FALSE))]
#'   x[[1]]
#'   x[] <- TRUE
#'   x[1:2] <- FALSE
#'   x[[1]] <- TRUE
#' 
NULL

#' @rdname Extract
#' @export
"[[.bit" <- function(x, i){
  if (length(i)!=1L)
    stop("subscript length not 1")
  if (is.numeric(i)){
    i <- as.integer(i)
    if (is.na(i) || i<1L || i>length(x))
      stop("subscript must be positive integer (or double) within length")
    ret <- .Call(C_R_bit_extract, x, i)
    setattr(ret, "vmode", "boolean")
    ret
  }else
    stop("subscript must be positive integer (or double) within length")
}

#' @rdname Extract
#' @export
"[[<-.bit" <- function(x, i, value){
  if (length(i)!=1L)
    stop("subscript length not 1")
  if (length(value)!=1)
    stop("value length not 1")
  if (is.numeric(i)){
    i <- as.integer(i)
    if (is.na(i) || i<1L)
      stop("subscript must be positive integer (or double)")
    if ((mi <- max(i))>length(x))
      length(x) <- mi
    value <- as.logical(value)
    .Call(C_R_bit_replace, x, i, value)
  }else
    stop("subscript must be positive integer (or double) within length")
}


#' @rdname Extract
#' @export
"[.bit" <- function(x, i){
  nx <- length(x)
  if ( missing(i) ){
    ret <- logical(nx)
    .Call(C_R_bit_get_logical, x, ret, range=c(1L, nx))
  }else{
    if (inherits(i, "bit")){
      i <- as.bitwhich(i)
    }
    if (inherits(i, "bitwhich")){
      i <- unclass(i)
    }
    if(is.numeric(i)){
      if (inherits(i, "ri")){
        if (i[1]<1L || i[2]>nx )
          stop("illegal range index 'ri'")
        ret <- logical(i[2]-i[1]+1L)
        .Call(C_R_bit_get_logical, x, ret, range=i)
      }else{
        i <- as.integer(i)
        r <- range_na(i)
        if (is.na(r[1])){
          ret <- logical()
        }else if (r[1]<0L){
          # check for positive or NA mixed with negative
          if (r[2]>0L || r[3]>0L)
            stop("only 0's may be mixed with negative subscripts")
          isasc <- intisasc(i, "none") # NAs checked already, early terminate on FALSE
          if (!isasc){
            if((length(i) / (r[2]-r[1])) < 0.05)
              i <- sort.int(i, method="quick")
            else
              i <- bit_sort_unique(i)
          }
        } # is positive, hence no sorting
        ret <- .Call(C_R_bit_extract, x, i) 
      }
    }else if(is.logical(i)){
      if (poslength(i)==0L){
         ret <- logical()
      }else{
        if (inherits(i, "bitwhich")){
          i <- unclass(i)
        }else{
          if (length(i)!=1 || is.na(i))
            stop("only scalar TRUE or FALSE allowed")
        }
        if (i){
          ret <- logical(nx)
          .Call(C_R_bit_get_logical, x, ret, range=c(1L, nx))
        }else{
          ret <- logical()
        }
      }
    }else
      stop("subscript must be ri or integer (or double) or  TRUE (or missing) or FALSE")
  }
  setattr(ret, "vmode", "boolean")
  ret
}


#' @rdname Extract
#' @export
"[<-.bit" <- function(x, i, value){
  nx <- length(x)
  value <- as.logical(value)
  nv <- length(value)
  if (missing(i))
    i <- TRUE
  if (inherits(i, "bit")){
    i <- as.bitwhich(i)
  }
  if (inherits(i, "bitwhich")){
    i <- unclass(i)
  }
  if (is.logical(i)){
    if (length(i)!=1L || is.na(i))
      stop("logical only scalar TRUE or FALSE allowed")
    if (i){
      if (nv==0L)
        stop("replacement has length zero")
      if (nx %% nv)
        warning("number of items to replace is not a multiple of replacement length")
      .Call(C_R_bit_set_logical, x, value, range=c(1L, nx))
    }else{
      x
    }
  }else if(is.numeric(i)){
    if (inherits(i, "ri")){
      if (i[1]<1L)
        stop("illegal range index 'ri'")
      if (i[2]>nx)
        length(x) <- i[2]
      ni <- i[2] - i[1] + 1L
      if (nv==0L)
        stop("replacement has length zero")
      if (ni %% nv)
        warning("number of items to replace is not a multiple of replacement length")
      .Call(C_R_bit_set_logical, x, value, range=i)
    }else{
      if (inherits(i, "which")){
				ni <- length(i)
        if (ni && i[ni] > nx)
          length(x) <- i[ni]
			}else{
        i <- range_nanozero(as.integer(i))
        r <- getsetattr(i, "range_na", NULL)
        ni <- length(i)
				if (ni){
          if (r[3]>0L)
             stop("NAs are not allowed in subscripted assignments")
          if (r[1]>0L){
            if (r[2] > nx)
              length(x) <- r[2]
          }else{
            if (r[2] > 0L)
              stop("only 0's may be mixed with negative subscripts")
            # R_bit_replace expects sorted i if i is negative
            i <- bit_sort_unique(i, range_na = r)
            ni <- nx - length(i)
          }
				}
			}
      if (nv != ni){
        if (nv==0L)
          stop("replacement has length zero")
        if (ni %% nv)
          warning("number of items to replace is not a multiple of replacement length")
        
      }
      .Call(C_R_bit_replace, x, i, value)
    }
  }else  stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
}


#' Check existence of integers in table
#' 
#' If the table is sorted, this can be much faster than \code{\link{\%in\%}}
#'
#' @param x a vector of integer
#' @param table a \code{\link{bitwhich}} object or a vector of integer
#' @param is.unsorted logical telling the function whether the table is (un)sorted. With the defautl \code{NULL} \code{FALSE} is assumed for \code{\link{bitwhich}} tables, otherwise \code{TRUE}
#'
#' @return logical vector
#' @seealso \code{\link{\%in\%}}
#'
#' @examples
#' x <- bitwhich(100)
#' x[3] <- TRUE
#' in.bitwhich(c(NA,2,3), x)
#' @export
in.bitwhich <- function(x, table, is.unsorted=NULL){
  x <- as.integer(x)
  if (is.null(is.unsorted))
    is.unsorted <- !is.bitwhich(table)
  if (is.logical(table)){
    if (length(table) && table){
      1L <= x & x <= length(table)
    }else{
      rep(FALSE, length(x))
    }
  }else{
    y <- bitwhich_representation(table)
    if (length(x)>1L && is.unsorted){
      if (y[1]>0L)
        !is.na(match(x,table))
      else
        is.na(match(-x,table))
      
    }else{
      if (y[1]>0L)
        merge_in(x,table)
      else
        merge_notin(x,table, revy=TRUE)
    }  
  }
}


#' @rdname Extract
#' @export
"[[.bitwhich" <- function(x, i){
  if (length(i)!=1L)
    stop("subscript length not 1")
  if (is.numeric(i)){
    i <- as.integer(i)
    if (is.na(i) || i<1L || i>length(x))
      stop("subscript must be positive integer (or double) within length")
    y <- bitwhich_representation(x)
    if (is.logical(y))
      ret <- as.vector(x)
    else{
      ret <- in.bitwhich(i, x)
    }
    setattr(ret, "vmode", "boolean")
    ret
  }else
    stop("subscript must be positive integer (or double) within length")
}



#' @rdname Extract
#' @export
"[[<-.bitwhich" <- function(x, i, value){
  if (length(i)!=1L)
    stop("subscript length not 1")
  if (length(value)!=1L)
    stop("value length not 1")
  value <- as.logical(value)
  if (is.na(value))
    value <- FALSE
  n <- length(x)
  if (i>n)
    warning("increasing length of bitwhich, which has non-standard semantics")
  if (is.numeric(i)){
    i <- as.integer(i)
    if (is.na(i) || i<1L || i>.Machine$integer.max)
      stop("subscript must be positive integer (or double)")
    y <- bitwhich_representation(x)
    if (is.logical(y)){
      if (length(y)){
        if (value == y){
          if (i>n)
            length(x) <- i
          return(x)
        }else if (value)
          ret <- bitwhich(max(n,i), i, poslength=1L)
        else
          ret <- bitwhich(max(n,i), -i, poslength=n-1L)
      }else{
        if (value)
          ret <- bitwhich(i, i, poslength=1L)
        else
          ret <- bitwhich(i, -i, poslength=n-1L)
      }
    }else{
      if (i>n){
        n <- i
        length(x) <- i
      }
      oldvalue <- in.bitwhich(i, x)
      if (value == oldvalue)
        return(x)
      else{
        if (value == (y>0)){
          ret <- bitwhich(n, merge_union(x, y*i, method = "all"), poslength=attr(x, "poslength")+y)
        }else{
          ret <- bitwhich(n, merge_setdiff(x, y*i, method = "exact"), poslength=attr(x, "poslength")-y)
        }
      }
    }
  }else
    stop("subscript must be positive integer (or double) within length")
  a <- attributes(x)
  a$poslength <- attr(ret, "poslength")
  setattributes(ret, a)
  ret
}


#' @rdname Extract
#' @export
"[.bitwhich" <- function(x, i){
  nx <- length(x)
  if ( missing(i) ){
    ret <- as.logical(x)
  }else{
    if (inherits(i, "bit"))
      stop("please use as.which(bit) for subscripting with bit")
    if (inherits(i, "bitwhich"))
      stop("please use unclass(bitwhich) or as.which(bitwhich) to clarify what you want")
    if (length(i)==0){
      ret <- logical()
    }else{
      if(is.logical(i)){
        if (length(i)!=1L || is.na(i))
          stop("only scalar TRUE or FALSE allowed")
        if (i){
          ret <- as.logical(x)
        }else{
          ret <- logical()
        }
      }else if(is.numeric(i)){
        if (inherits(i, "ri")){
          if (i[1]<1L || i[2]>nx )
            stop("illegal range index 'ri'")
          if (is.logical(x)){
            if (length(x))
              ret <- rep(copy_vector(x), i[2]-i[1]+1L)
            else
              ret <- rep(NA, i[2]-i[1]+1L)
          }else{
            #y <- unclass(x)
            y <- bitwhich_representation(x)
            if (y[1]>0L){
              #ret <- rep(FALSE, i[2]-i[1]+1L)
              #ret[y[i[1]<=y & y<=i[2]] - i[1] + 1L] <- TRUE
              ret <- merge_rangein(c(i[1], i[2]), x)
            }else{
              # ret <- rep(TRUE, i[2]-i[1]+1L)
              # ret[-y[(-i[1])>=y & y>=(-i[2])] - i[1] + 1L] <- FALSE
              ret <- merge_rangenotin(c(i[1], i[2]), x, revy=TRUE)
            }
          }
        }else{
          i <- range_nanozero(as.integer(i))
          r <- getsetattr(i, "range_na", NULL)
          n <- length(i)
          if (r[3]==n) # if allNA
            ret <- rep(NA, n)
          else{
            if (r[1] < 0L && r[2] > 0L)
              stop("only 0's may be mixed with negative subscripts")
            ret <- as.bit(x)[i]
          }
        } 
      }else stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
    }
  }
  setattr(ret, "vmode", "boolean")
  ret
}


#' @rdname Extract
#' @export
"[<-.bitwhich" <- function(x, i, value){
  nx <- length(x)
  value <- as.logical(value)
  if (anyNA(value))
    value[is.na(value)] <- FALSE
  nv <- length(value)
  if (missing(i))
    i <- TRUE
  if (inherits(i, "bit"))
    stop("please use as.which(bit) for subscripting with bit")
  if (inherits(i, "bitwhich"))
    stop("please use unclass(bitwhich) or as.which(bitwhich) to clarify what you want")
  if (length(i)){
    if (is.logical(i)){
      if (length(i)!=1L || is.na(i))
        stop("logical only scalar TRUE or FALSE allowed")
      if (i){
        if (nv==1L){
          ret <- bitwhich(nx, value)
        }else{
          b <- as.bit(value)
          if (nv==nx){
            ret <- as.bitwhich(b)
          }else{
            if (nv==0L)
              stop("replacement has length zero")
            if (nx%%nv)
              warning("number of items to replace is not a multiple of replacement length")
            ret <- as.bitwhich(rep(b, length.out=nx))
          }
        } 
      }else{
        return(x)
      }
    }else if(is.numeric(i)){
      if (nv>1L){
        b <- as.bit(x) 
        b[i] <- value
        ret <- as.bitwhich(b)
      }else{
        if (inherits(i, "ri")){
          if (i[1]<1L)
            stop("illegal range index 'ri'")
          biggest_mentioned_index <- max(abs(i[1:2]))
          i <- i[1]:i[2]
          ni <- length(i)
        }else{
          if (inherits(i, "which")){
            ni <- length(i)
            biggest_mentioned_index <- i[length(i)]
          }else{
            i <- range_nanozero(as.integer(i))
            r <- getsetattr(i, "range_na", NULL)
            if (length(i)){
              if (r[3]>0L)
                stop("NAs are not allowed in subscripted assignments")
              if (r[1]>0L){
                # since value is a scalar removing duplicates does not harm and speeds up
                i <- bit_sort_unique(i, range_na = r)
                ni <- length(i)
              }else{
                if (r[2] > 0L)
                  stop("only 0's may be mixed with negative subscripts")
                i <- bit_sort_unique(i, range_na = r)
                ni <- nx - length(i)
              }
              # since value is a scalar removing duplicates does not harm and speeds up
              biggest_mentioned_index <- max(abs(i[1:2]), na.rm=TRUE)
            }else{
              ni <- 0L
              biggest_mentioned_index <- 0L
            }
          }
        }
        if (!ni){
          return(x)
        }else{
          if (nv==0L)
            stop("replacement has length zero")
          if (biggest_mentioned_index>nx){
            length(x) <- biggest_mentioned_index
            nx <- biggest_mentioned_index
          }
          y <- bitwhich_representation(x)
          if (is.logical(y)){
            if (value == y){
              # assignment doesn't change anything
              return(x)
            }else{
              if (value){
                # assignment has first inclusions
                if (i[1]<0){
                  # assignment enumerates those not included
                  ret <- bitwhich(nx, i, poslength=nx-length(i))
                }else{
                  # assignment enumerates those included
                  ret <- bitwhich(nx, i, poslength=length(i))
                }
              }else{
                # assignment has first exclusions
                if (i[1]<0){
                  # assignment enumerates those not excluded
                  ret <- bitwhich(nx, copy_vector(i, revx=TRUE), poslength=length(i))
                }else{
                  # assignment enumerates those excluded
                  ret <- bitwhich(nx, copy_vector(i, revx=TRUE), poslength=nx-length(i))
                }
              }
            }
          }else{
            if (y<0){
              # object maintains exclusions
              if (value){
                # assignment has inclusions
                if (i[1]<0){
                  # assignment enumerates those not included
                  # w2 <- w <- bitwhich(12, -(1:3)); w2[-(3:5)] <- TRUE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_intersect(x,i, method='exact'), xempty=TRUE, is.unsorted = FALSE, has.dup = FALSE) #done
                }else{
                  # assignment enumerates those included
                  # w2 <- w <- bitwhich(12, -(1:3)); w2[(3:5)] <- TRUE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_setdiff(x,i,revy=TRUE, method='exact'), xempty=TRUE, is.unsorted = FALSE, has.dup = FALSE)  #done
                }
              }else{
                # assignment has exclusions
                if (i[1]<0){
                  # assignment enumerates those not excluded
                  # w2 <- w <- bitwhich(12, -(1:3)); w2[-(3:5)] <- FALSE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_setdiff(i, x, revx=TRUE, revy=TRUE, method='exact'), xempty=FALSE, is.unsorted = FALSE, has.dup = FALSE) #done
                }else{
                  # assignment enumerates those excluded
                  # w2 <- w <- bitwhich(12, -(1:3)); w2[(3:5)] <- FALSE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_union(x,i,revy=TRUE, method='exact'), is.unsorted = FALSE, has.dup = FALSE) #done
                }
              }
            }else{
              # object maintains inclusions
              if (value){
                # assignment has inclusions
                if (i[1]<0){
                  # assignment enumerates those not included
                  # w2 <- w <- bitwhich(12, (1:3)); w2[-(3:5)] <- TRUE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_setdiff(i,x, revy = TRUE, method='exact'), xempty=TRUE, is.unsorted = FALSE, has.dup = FALSE) #done
                }else{
                  # assignment enumerates those included
                  # w2 <- w <- bitwhich(12, (1:3)); w2[(3:5)] <- TRUE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_union(x,i, method='exact'), is.unsorted = FALSE, has.dup = FALSE) #done
                }
              }else{
                # assignment has exclusions
                if (i[1]<0){
                  # assignment enumerates those not excluded
                  # w2 <- w <- bitwhich(12, (1:3)); w2[-(3:5)] <- FALSE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_intersect(x, i, revy=TRUE, method='exact'), xempty=FALSE, is.unsorted = FALSE, has.dup = FALSE) #done
                }else{
                  # assignment enumerates those excluded
                  # w2 <- w <- bitwhich(12, (1:3)); w2[(3:5)] <- FALSE; cbind(as.logical(w), as.logical(w2))
                  ret <- bitwhich(nx, merge_setdiff(x,i, method='exact'), xempty=FALSE, is.unsorted = FALSE, has.dup = FALSE)
                }
              }
            }
          }
        }
      }
    }else  stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
    a <- attributes(x)
    a$poslength <- sum(ret)
    setattributes(ret, a)
    ret
  }else 
    x
}



#' Range index
#' 
#' A range index can be used to extract or replace a continuous ascending part
#' of the data
#' 
#' @param from first position
#' @param to last posistion
#' @param x an object of class 'ri'
#' @param maxindex the maximal length of the object-to-be-subscripted (if
#' known)
#' @param \dots further arguments
#' @return A two element integer vector with class 'ri'
#' @author Jens Oehlschlägel
#' @seealso \code{\link[ff]{as.hi}}
#' @keywords classes logic
#' @examples
#' 
#'  bit(12)[ri(1,6)]
#' 
#' @export ri
ri <- function(from, to=NULL, maxindex=NA){
  if (is.null(to)){
    x <- as.integer(c(from, maxindex))
  }else{
    x <- as.integer(c(from, to, maxindex))
  }
  maxindex = maxindex
  if (length(x)!=3 )
    stop("range must have exactly three elements")
  if (x[[1]]<1L)
    stop("range must at least select one element")
  if (x[[1]]>x[[2]])
    stop("lower bound must be smaller or equal than upper bound")
  if (!is.na(x[[3]]) && x[[2]]>x[[3]])
    stop("lower and upper bound must be smaller or equal to maxindex")
  oldClass(x) <- c("booltype","ri")
  x
}

#' @rdname ri
#' @export
print.ri <- function(x, ...)
  cat("range index (ri) from", x[[1]], "to", x[[2]], "maxindex",  x[[3]], "\n")

#' @rdname length.bit
#' @export
length.ri <- function(x)x[[3]]


#' @rdname Summaries
#' @export
all.ri <- function(x, range=NULL, ...){
  if (is.null(range)){
    range[[1]] <- 1L
    range[[2]] <- x[[3]]
  }
  x[[1]]<=range[[1]] && x[[2]]>=range[[2]]
}

#' @rdname Summaries
#' @export
any.ri <- function(x, range=NULL, ...){
  if (is.null(range)){
    range[[1]] <- 1L
    range[[2]] <- x[[3]]
  }
  range[[1]]<=x[[1]] && range[[2]]>=x[[2]]
}

#' @rdname Summaries
#' @export
anyNA.ri <- function(x
                     #, range=NULL
                     , recursive = FALSE)FALSE

#' @rdname Summaries
#' @export
sum.ri <- function(x, ...){
  if (any(names(match.call(expand.dots = TRUE))=="range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[2]] - x[[1]] + 1L
}

#' @rdname Summaries
#' @export
min.ri <- function(x, ...){
  if (any(names(match.call(expand.dots = TRUE))=="range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[1]]
}

#' @rdname Summaries
#' @export
max.ri <- function(x, ...){
  if (any(names(match.call(expand.dots = TRUE))=="range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[2]]
}

#' @rdname Summaries
#' @export
range.ri <- function(x, ...){
  if (any(names(match.call(expand.dots = TRUE))=="range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[1:2]
}

#' @rdname Summaries
#' @export
summary.ri <- function(object, ...){
  if (any(names(match.call(expand.dots = TRUE))=="range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  s <- object[[2]] - object[[1]] + 1L
   c(`FALSE` = object[[3]] - s, `TRUE` = s, Min. = object[[1]], Max. = object[[2]])
}



# this version without vmode() will be overwritte by the version in package ff
#' @rdname PhysVirt
#' @export
physical.default <- function(x){
  p <- attributes(attr(x, "physical"))
  p <- p[is.na(match(names(p), "class"))]
  p
}
#' @rdname PhysVirt
#' @export
"physical<-.default" <- function(x, value){
  attributes(attr(x, "physical")) <- c(value, list(class="physical"))
  x
}


#' @rdname PhysVirt
#' @export
virtual.default <- function(x){
  v <- attributes(attr(x, "virtual"))
  v[is.na(match(names(v), "class"))]
}
#' @rdname PhysVirt
#' @export
"virtual<-.default" <- function(x, value){
  attributes(attr(x, "virtual")) <- c(value, list(class="virtual"))
  x
}


#' @rdname PhysVirt
#' @export
print.physical <- function(x, ...){
  cat("(hidden, use physical(x) to access the physical attributes and vmode(x) for accessing vmode)\n")
  invisible()
}

#' @rdname PhysVirt
#' @export
print.virtual <- function(x, ...){
  cat("(hidden, use virtual(x) to access the virtual attributes)\n")
  invisible()
}




# not exported - just here to avoid cross calling the dll from ff
R_bit_as_hi <- function(x, range, offset)
.Call(C_R_bit_as_hi, x, range, offset)

test_that("Can create zero length bit objects", {
  expect_identical(bit(), bit(0))
  expect_identical(as.bit(), bit(0))
  expect_identical(as.bit(NULL), bit(0))
  expect_identical(as.bit(logical()), bit(0))
  expect_identical(as.bit(integer()), bit(0))
  expect_error(as.bit(factor()))
})

test_that("length<-.bit does set unused bits to FALSE", {
  b <- !bit(.BITS)
  length(b) <- 7
  b2 <- !bit(7)
  expect_identical(b, b2)
  length(b) <- .BITS
  length(b2) <- .BITS
  expect_identical(b, b2)
  b <- !bit(2 * .BITS)
  length(b) <- 7
  b2 <- !bit(7)
  expect_identical(b, b2)
  length(b) <- 2 * .BITS
  length(b2) <- 2 * .BITS
  expect_identical(b, b2)
})

test_that("length<-.bit does set new bits to FALSE", {
  b <- !bit(1)
  length(b) <- .BITS
  b2 <- bit(.BITS)
  b2[1] <- TRUE
  expect_identical(b, b2)
  b <- !bit(1)
  length(b) <- 2 * .BITS
  b2 <- bit(2 * .BITS)
  b2[1] <- TRUE
  expect_identical(b, b2)
})

test_that("c.bit does set unused bits to FALSE", {
  b <- !bit(.BITS - 1)
  b <- c(b, b)
  b2 <- !bit(2 * .BITS - 2)
  expect_identical(b, b2)
})

test_that("Can create zero length bitwhich objects", {
  expect_identical(bitwhich(), bitwhich(0))
  expect_identical(as.bitwhich(), bitwhich(0))
  expect_identical(as.bitwhich(NULL), bitwhich(0))
  expect_identical(as.bitwhich(logical()), bitwhich(0))
  expect_identical(as.bitwhich(integer()), bitwhich(0))
  expect_error(as.bitwhich(factor()))
})

# nolint start: undesirable_function_linter. structure() seems OK here.
test_that("bitwhich creates correctly", {
  # to check whether we properly obtain integer
  n <- 12
  x <- rep(3:10, 2)
  y <- c(-12L, -11L, -2L, -1L)
  attr(y, "maxindex") = 12L
  attr(y, "poslength") = 8L
  class(y) = c("booltype", "bitwhich")
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(
    expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, c(-12L, -11L, -2L, -1L), poslength=8), y),
    list(n=n, x=x, y=y)
  ))

  x <- -rev(x)
  y <- c(1L, 2L, 11L, 12L)
  attr(y, "maxindex") = 12L
  attr(y, "poslength") = 4L
  class(y) = c("booltype", "bitwhich")
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(
    expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, c(1L, 2L, 11L, 12L), poslength=4), y),
    list(n=n, x=x, y=y)
  ))

  x <- rep(5:6, 2)
  y <- structure(5:6, maxindex = 12L, poslength = 2L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(
    expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(expect_identical(bitwhich(n, 5:6, poslength=2), y), list(n=n, x=x, y=y)))

  x <- -rev(x)
  y <- structure(-6:-5, maxindex = 12L, poslength = 10L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(
    expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(
    expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y),
    list(n=n, x=x, y=y)
  ))
  eval(substitute(expect_identical(bitwhich(n, -6:-5, poslength=10), y), list(n=n, x=x, y=y)))

  y <- structure(TRUE, maxindex = 12L, poslength = 12L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, TRUE), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, integer(), poslength=n), y), list(n=n, y=y)))

  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, FALSE), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, integer(), poslength=0), y), list(n=n, y=y)))

  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))

  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))

  y <- structure(logical(0), maxindex = 0L, poslength = 0L, class = c("booltype", "bitwhich"))
  eval(substitute(expect_identical(bitwhich(0), y), list(y=y)))
  eval(substitute(expect_identical(bitwhich(0, poslength=0), y), list(y=y)))
})
# nolint end: undesirable_function_linter.

test_that("length<-.bitwhich does set new bits according to the rules given in details", {
  w <- bitwhich(0)
  length(w) <- 2
  w2 <- bitwhich(2, FALSE)
  expect_identical(w, w2)

  w <- bitwhich(1, FALSE)
  length(w) <- 2
  w2 <- bitwhich(2, FALSE)
  expect_identical(w, w2)

  w <- bitwhich(1, TRUE)
  length(w) <- 2
  w2 <- bitwhich(2, TRUE)
  expect_identical(w, w2)

  w <- bitwhich(6, 1:2)
  length(w) <- 12
  w2 <- bitwhich(12, 1:2)
  expect_identical(w, w2)

  w <- bitwhich(6, -(2:1))
  length(w) <- 12
  w2 <- bitwhich(12, -(2:1))
  expect_identical(w, w2)
})

test_that("c() works", {
  l <- b <- w <- list()
  for (k in 1:12) {
    l[[k]] <- rep_len(c(FALSE, TRUE), k)
    b[[k]] <- as.bit(l[[k]])
    w[[k]] <- as.bitwhich(l[[k]])
  }
  l <- do.call("c", l)
  b <- as.logical(do.call("c", b))
  w <- as.logical(do.call("c", w))
  expect_identical(b, l)
  expect_identical(w, l)
})

# nolint start: rep_len_linter. Specifically testing rep().
test_that("rep() works", {
  l <- c(FALSE, TRUE)
  b <- as.bit(l)
  w <- as.bitwhich(l)
  for (k in 1:(3 * .BITS)) {
    expect_identical(as.logical(rep(b, length.out=k)), rep(l, length.out=k))
    expect_identical(as.logical(rep(w, length.out=k)), rep(l, length.out=k))
  }
  for (k in 1:(2 * .BITS)) {
    expect_identical(as.logical(rep(b, k)), rep(l, k))
    expect_identical(as.logical(rep(w, k)), rep(l, k))
  }
})
# nolint end: rep_len_linter.

test_that("NAs are coerced to FALSE", {
  expect_identical(as.logical(as.bit(c(NA, FALSE, TRUE))), c(FALSE, FALSE, TRUE))
  expect_identical(as.logical(as.bitwhich(c(NA, FALSE, TRUE))), c(FALSE, FALSE, TRUE))
})

test_that("coercions work", {
  for (i in c(FALSE, TRUE)) {
    for (j in c(FALSE, TRUE)) {
      for (k in c(FALSE, TRUE)) {
        l <- c(i, j, k)
        expect_identical(as.logical(as.bit(l)), l)
        expect_identical(as.logical(as.bitwhich(l)), l)
        expect_identical(as.logical(as.bit(as.bit(l))), l)
        expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
        expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
        expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
        set.seed(1)
        for (m in 1:ifelse(sum(l) %in% c(0L, 3L), 1, 24)) {
          l <- sample(l, 3 * .BITS, TRUE)
          expect_identical(as.logical(as.bit(l)), l)
          expect_identical(as.logical(as.bitwhich(l)), l)
          expect_identical(as.logical(as.bit(as.bit(l))), l)
          expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
          expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
          expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
        }
      }
    }
  }

  set.seed(1)
  for (k in 0:(3 * .BITS)) {
    l <- sample(c(FALSE, TRUE), k, TRUE)
    expect_identical(as.logical(as.bit(l)), l)
    expect_identical(as.logical(as.bit(as.bit(l))), l)
    expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
    expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
    expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
  }
  set.seed(1)
  for (k in 0:(3 * .BITS)) {
    l <- sample(c(NA, FALSE, TRUE), k, TRUE)
    b <- as.bit(l)
    expect_identical(as.bit(as.bit(l)), b)
    expect_identical(as.bit(as.bit(as.bit(l))), b)
    expect_identical(as.bit(as.bit(as.bitwhich(l))), b)
    expect_identical(as.bit(as.bitwhich(as.bit(l))), b)
    expect_identical(as.bit(as.bitwhich(as.bitwhich(l))), b)
  }
  set.seed(1)
  for (k in 0:(3 * .BITS)) {
    i <- sample(-2:2, k, TRUE)
    expect_identical(as.logical(as.bit(i)), as.logical(i))
    expect_identical(as.logical(as.bitwhich(i)), as.logical(i))
    i <- as.double(i)
    expect_identical(as.logical(as.bit(i)), as.logical(i))
    expect_identical(as.logical(as.bitwhich(i)), as.logical(i))
  }

}
)


test_that("boolean operations work", {
  N <- c(1L, .BITS - 1L, .BITS, .BITS + 1L)
  X <- c(rev(-N), N)
  N <- c(0L, N)
  fx <- function(x, n) as.integer(sign(x)) * sample(n, abs(x))
  for (n in N) {
    for (x1 in X[abs(X) <= abs(n)]) {
      for (x2 in X[abs(X) <= abs(n)]) {
        set.seed(1)
        w1 <- bitwhich(n, fx(x1, n))
        w2 <- bitwhich(n, fx(x2, n))
        l1 <- as.logical(w1)
        l2 <- as.logical(w2)
        b1 <- as.bit(l1)
        b2 <- as.bit(l2)
        fun <- function(x1, x2, f) {
          eval(substitute(
            expect_identical(f(!x1), !f(x1)),
            list(x1=x1, f=f)
          ))
          eval(substitute(
            expect_identical(f(x1 & x2), f(x1) & f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(f(x1 | x2), f(x1) | f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(f(x1 == x2), f(x1) == f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(f(x1 != x2), f(x1) != f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(f(xor(x1, x2)), xor(f(x1), f(x2))),
            list(x1=x1, x2=x2, f=f)
          ))
        }
        fun(w1, w2, as.logical)
        fun(b1, b2, as.logical)
        fun <- function(x1, x2, f) {
          eval(substitute(
            expect_identical(x1 & x2, f(x1) & f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(x1 | x2, f(x1) | f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(x1 == x2, f(x1) == f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(x1 != x2, f(x1) != f(x2)),
            list(x1=x1, x2=x2, f=f)
          ))
          eval(substitute(
            expect_identical(xor(x1, x2), xor(f(x1), f(x2))),
            list(x1=x1, x2=x2, f=f)
          ))
        }
        fun(b1, w2, as.bit)
        fun(w1, b2, as.bit)
        fun(b1, l2, as.logical)
        fun(l1, b2, as.logical)
        fun(l1, w2, as.logical)
        fun(w1, l2, as.logical)
      }
    }
  }
})


test_that("promotion is correct in boolean operations and concatenation", {
  N2 <- N1 <- c(2L, 4L)
  T2 <- T1 <- c("logical", "bit", "bitwhich")
  FUN <- list(logical=logical, bit=bit, bitwhich=bitwhich)
  for (t1 in T1) {
    for (t2 in T2) {
      for (n1 in N1) {
        for (n2 in N2) {
          x1 <- FUN[[t1]](n1)
          x2 <- FUN[[t2]](n2)
          eval(substitute(
            expect_identical(booltype(x1 & x2), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
          eval(substitute(
            expect_identical(booltype(x1 | x2), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
          eval(substitute(
            expect_identical(booltype(x1 == x2), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
          eval(substitute(
            expect_identical(booltype(x1 != x2), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
          eval(substitute(
            expect_identical(booltype(xor(x1, x2)), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
          if (t1 != "logical") { # c with first argument logical does not dispatch
            eval(substitute(
              expect_identical(booltype(c(x1, x2)), min(booltypes[[t1]], booltypes[[t2]])),
              list(x1=x1, x2=x2, t1=t1, t2=t2)
            ))
          }
          eval(substitute(
            expect_identical(booltype(c.booltype(x1, x2)), min(booltypes[[t1]], booltypes[[t2]])),
            list(x1=x1, x2=x2, t1=t1, t2=t2)
          ))
        }
      }
    }
  }
})


test_that("subscript operations work", {
  N <- c(1L, .BITS - 1L, .BITS, .BITS + 1L)
  X <- c(rev(-N), N)
  J <- c(rev(-N), 0L, N)
  N <- c(0L, N)
  R <- 1
  fx <- function(x, n) as.integer(sign(x)) * sample(n, abs(x), TRUE)
  fi <- function(x, n) as.integer(sign(x)) * sample(0:n, abs(x), FALSE)

  for (r in 1:R) {
    for (n in N) {
      for (x in X[abs(X) <= abs(n)]) {
        set.seed(r)
        w <- bitwhich(n, fx(x, n))
        l <- as.logical(w)
        b <- as.bit(l)
        I <- J[abs(J) <= n]
        I <- lapply(I, fi, n=n)
        I <- c(list(FALSE, TRUE), I, lapply(I, as.which, maxindex=n))
        for (i in I) {
          v <- l[i]
          eval(substitute(expect_identical(b[i], v, ignore_attr="vmode"), list(b=b, i=i, v=v)))
          eval(substitute(expect_identical(w[i], v, ignore_attr="vmode"), list(w=w, i=i, v=v)))
          l2 <- l
          l2[i] <- !v
          eval(substitute(env = list(b=b, i=i, v=v, l2=l2), {
            b2 <- b
            b2[i] <- !v
            expect_identical(b2, as.bit(l2))
          }))
          eval(substitute(env = list(w=w, i=i, v=v, l2=l2), {
            w2 <- w
            w2[i] <- !v
            expect_identical(w2, as.bitwhich(l2))
          }))
          if (length(v) > 1L && (length(v) %% 2L) == 0) {
            v2 <- !v[seq_len(ceiling(length(v) / 2))]
            l2 <- l
            l2[i] <- v2
            eval(substitute(env = list(b=b, i=i, v=v, l2=l2), {
              b2 <- b
              b2[i] <- v2
              expect_identical(b2, as.bit(l2))
            }))
            eval(substitute(env = list(w=w, i=i, v=v, l2=l2), {
              w2 <- w
              w2[i] <- v2
              expect_identical(w2, as.bitwhich(l2))
            }))
          }
        }
        I <- lapply(1:1, function(x) {
          i <- quicksort2(sample(n, 2, TRUE))
          ri(i[1], i[2], n)
        })
        for (i in I) {
          i2 <- i[1]:i[2]
          v <- l[i2]
          eval(substitute(expect_identical(b[i], v, ignore_attr="vmode"), list(b=b, i=i, v=v)))
          eval(substitute(expect_identical(w[i], v, ignore_attr="vmode"), list(w=w, i=i, v=v)))
          # debugonce(get("[.bitwhich"))
          l2 <- l
          l2[i2] <- !v
          eval(substitute(env = list(b=b, i=i, v=v, l2=l2), {
            b2 <- b
            b2[i] <- !v
            expect_identical(b2, as.bit(l2))
          }))
          eval(substitute(env = list(w=w, i=i, v=v, l2=l2), {
            w2 <- w
            w2[i] <- !v
            expect_identical(w2, as.bitwhich(l2))
          }))
          if (length(v) > 1L && (length(v) %% 2L) == 0) {
            v2 <- !v[seq_len(ceiling(length(v) / 2))]
            l2[i2] <- v2
            eval(substitute(env = list(b=b, i=i, v2=v2, l2=l2), {
              b2 <- b
              b2[i] <- v2
              expect_identical(b2, as.bit(l2))
            }))
            eval(substitute(env = list(w=w, i=i, v2=v2, l2=l2), {
              w2 <- w
              w2[i] <- v2
              expect_identical(w2, as.bitwhich(l2))
            }))
          }
        }
        i <- sample(n, 1)
        v <- l[[i]]
        eval(substitute(expect_identical(b[[i]], v, ignore_attr="vmode"), list(b=b, i=i, v=v)))
        eval(substitute(expect_identical(w[[i]], v, ignore_attr="vmode"), list(w=w, i=i, v=v)))
        l2 <- l
        l2[[i]] <- !v
        eval(substitute(env = list(b=b, i=i, v=v, l2=l2), {
          b2 = b
          b2[[i]] = !v
          expect_identical(b2, as.bit(l2))
        }))
        eval(substitute(env = list(w=w, i=i, v=v, l2=l2), {
          w2 = w
          w2[[i]] = !v
          expect_identical(w2, as.bitwhich(l2))
        }))

      }
    }
  }

})


test_that("aggregation functions work", {
  D <- list(
    full_range=ri(1, 128, 128),
    begin_range=ri(1, 5, 128),
    end_range=ri(99, 128, 128),
    begin_scalar=ri(1, 1, 128),
    end_scalar=ri(128, 128, 128)
  )
  R <- list(
    norange=NULL,
    begin_range=ri(1, 64, 128),
    end_range=ri(65, 128, 128),
    mid_range=ri(32, 96, 128),
    full_range=ri(1, 128, 128)
  )
  I <- list(id=function(x) x, not=function(x) !x)
  A <- list(
    logical=as.logical,
    bit=as.bit,
    bitwhich=as.bitwhich,
    which=as.which,
    ri=function(x) x
  )
  S1 <- list(any=any, all=all, sum=sum)
  S2 <- list(
    min=function(x) {
      if (booltype(x) != "logical") return(min(x))
      if (!any(x)) return(NA_integer_)
      which.max(x)
    },
    max=function(x) {
      if (booltype(x) != "logical") return(max(x))
      if (!any(x)) return(NA_integer_)
      length(x) - which.max(rev(x)) + 1L
    },
    range=function(x) if (booltype(x) == "logical") range.booltype(x) else range(x),
    summary=function(x) if (booltype(x) == "logical") summary.booltype(x) else summary(x)
  )
  S3 <- list(
    sum=sum.booltype,
    min=min.booltype,
    max=max.booltype,
    range=range.booltype,
    summary=summary.booltype
  )
  for (d in names(D)) {
    for (i in names(I)) {
      x <- I[[i]](as.logical(D[[d]]))
      for (a in names(A)) if (! (i == "not" && a %in% c("which", "ri"))) {
        y <- I[[i]](A[[a]](D[[d]]))
        for (s in names(S1)) {
          expect_identical(S1[[s]](x), S1[[s]](y))
        }
        if (a != "which") for (s in names(S2)) {
          expect_identical(S2[[s]](x), S2[[s]](y))
        }
        if (a != "which") for (s in names(S3)) {
          expect_identical(S3[[s]](x), S3[[s]](y))
        }
      }
    }
  }

})

test_that("rev() methods work", {
  x = c(FALSE, TRUE)
  expect_identical(rev(as.bit(x)), as.bit(rev(x)))
  expect_identical(rev(as.bitwhich(x)), as.bitwhich(rev(x)))
})

test_that("as.integer method for bit works", {
  expect_identical(as.integer(as.bit(0:1)), 0:1)
})

library("testthat")
library("bit")

# expect_identical <- function(x, y, ...){
#   stopifnot(identical(x,y))
# }

context("bit")

test_that("Can create zero length bit objects", {
  expect_equal(bit(), bit(0))
  expect_equal(as.bit(), bit(0))
  expect_equal(as.bit(NULL), bit(0))
  expect_equal(as.bit(c()), bit(0))
  expect_equal(as.bit(logical()), bit(0))
  expect_equal(as.bit(integer()), bit(0))
  expect_error(as.bit(factor()))
})

test_that("length<-.bit does set unused bits to FALSE", {
  b <- !bit(bit:::.BITS)
  length(b) <- 7
  b2 <- !bit(7) 
  expect_identical(b,b2)
  length(b) <- bit:::.BITS
  length(b2) <- bit:::.BITS
  expect_identical(b,b2)
  b <- !bit(2*bit:::.BITS)
  length(b) <- 7
  b2 <- !bit(7) 
  expect_identical(b,b2)
  length(b) <- 2*bit:::.BITS
  length(b2) <- 2*bit:::.BITS
  expect_identical(b,b2)
})

test_that("length<-.bit does set new bits to FALSE", {
  b <- !bit(1)
  length(b) <- bit:::.BITS
  b2 <- bit(bit:::.BITS)
  b2[1] <- TRUE
  expect_identical(b,b2)
  b <- !bit(1)
  length(b) <- 2*bit:::.BITS
  b2 <- bit(2*bit:::.BITS)
  b2[1] <- TRUE
  expect_identical(b,b2)
})

test_that("c.bit does set unused bits to FALSE", {
  b <- !bit(bit:::.BITS-1)
  b <- c(b,b)
  b2 <- !bit(2*bit:::.BITS-2)
  expect_identical(b,b2)
})

context("bitwhich")

test_that("Can create zero length bitwhich objects", {
  expect_equal(bitwhich(), bitwhich(0))
  expect_equal(as.bitwhich(), bitwhich(0))
  expect_equal(as.bitwhich(NULL), bitwhich(0))
  expect_equal(as.bitwhich(c()), bitwhich(0))
  expect_equal(as.bitwhich(logical()), bitwhich(0))
  expect_equal(as.bitwhich(integer()), bitwhich(0))
  expect_error(as.bitwhich(factor()))
})

test_that("bitwhich creates correctly", {
  # to check whether we properly obtain integer
  n <- 12
  x <- rep(3:10, 2)
  y <- structure(c(-12L, -11L, -2L, -1L), maxindex = 12L, poslength = 8L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, c(-12L, -11L, -2L, -1L), poslength=8), y), list(n=n, x=x, y=y)))
  
  x <- -rev(x)
  y <- structure(c(1L, 2L, 11L, 12L), maxindex = 12L, poslength = 4L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, c(1L, 2L, 11L, 12L), poslength=4), y), list(n=n, x=x, y=y)))
  
  x <- rep(5:6, 2)
  y <- structure(5:6, maxindex = 12L, poslength = 2L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, 5:6, poslength=2), y), list(n=n, x=x, y=y)))
  
  x <- -rev(x)
  y <- structure(-6:-5, maxindex = 12L, poslength = 10L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, x), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, unique(x), has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(x), is.unsorted = FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, sort(unique(x)), is.unsorted = FALSE, has.dup=FALSE), y), list(n=n, x=x, y=y)))
  eval(substitute(expect_identical(bitwhich(n, -6:-5, poslength=10), y), list(n=n, x=x, y=y)))
  
  y <- structure(TRUE, maxindex = 12L, poslength = 12L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, TRUE), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, integer(), poslength=n), y), list(n=n, y=y)))
  
  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n, FALSE), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, integer(), poslength=0), y), list(n=n, y=y)))
  
  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))
  
  y <- structure(FALSE, maxindex = 12L, poslength = 0L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n), y), list(n=n, y=y)))
  eval(substitute(expect_identical(bitwhich(n, poslength=0), y), list(n=n, y=y)))
  
  y <- structure(logical(0), maxindex = 0L, poslength = 0L, class = c("booltype","bitwhich"))
  eval(substitute(expect_identical(bitwhich(0), y), list(y=y)))
  eval(substitute(expect_identical(bitwhich(0, poslength=0), y), list(y=y)))
})


test_that("length<-.bitwhich does set new bits according to the rules given in details", {
  w <- bitwhich(0)
  length(w) <- 2
  w2 <- bitwhich(2, FALSE)
  expect_identical(w,w2)
  
  w <- bitwhich(1, FALSE)
  length(w) <- 2
  w2 <- bitwhich(2, FALSE)
  expect_identical(w,w2)
  
  w <- bitwhich(1, TRUE)
  length(w) <- 2
  w2 <- bitwhich(2, TRUE)
  expect_identical(w,w2)
  
  w <- bitwhich(6, 1:2)
  length(w) <- 12
  w2 <- bitwhich(12, 1:2)
  expect_identical(w,w2)
  
  w <- bitwhich(6, -(2:1))
  length(w) <- 12
  w2 <- bitwhich(12, -(2:1))
  expect_identical(w,w2)
})


context("bit and bitwhich")

test_that("c() works", {
  l <- b <- w <- list()
  for(k in 1:12){
    l[[k]] <- rep(c(FALSE,TRUE), length.out=k)
    b[[k]] <- as.bit(l[[k]])
    w[[k]] <- as.bitwhich(l[[k]])
  }
  l <- do.call("c", l)
  b <- as.logical(do.call("c", b))
  w <- as.logical(do.call("c", w))
  expect_identical(b,l)
  expect_identical(w,l)
})

test_that("rep() works", {
  l <- c(FALSE, TRUE)
  b <- as.bit(l)
  w <- as.bitwhich(l)
  for(k in 1:(3*bit:::.BITS)){
    expect_identical(as.logical(rep(b, length.out=k)),rep(l, length.out=k))
    expect_identical(as.logical(rep(w, length.out=k)),rep(l, length.out=k))
  }
  for(k in 1:(2*bit:::.BITS)){
    expect_identical(as.logical(rep(b, k)),rep(l, k))
    expect_identical(as.logical(rep(w, k)),rep(l, k))
  }
})
    

test_that("NAs are coerced to FALSE", {
  expect_identical(as.logical(as.bit(c(NA, FALSE, TRUE))), c(FALSE, FALSE, TRUE))
  expect_identical(as.logical(as.bitwhich(c(NA, FALSE, TRUE))), c(FALSE, FALSE, TRUE))
})

test_that("coercions work", {
  for(i in c(FALSE, TRUE))
    for(j in c(FALSE, TRUE))
      for(k in c(FALSE, TRUE))
      {
        l <- c(i,j,k)
        expect_identical(as.logical(as.bit(l)), l)
        expect_identical(as.logical(as.bitwhich(l)), l)
        expect_identical(as.logical(as.bit(as.bit(l))), l)
        expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
        expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
        expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
        set.seed(1)
        for (m in 1:ifelse(sum(l) %in% c(0L,3L), 1, 24)){
          l <- sample(l, 3*bit:::.BITS, TRUE)
          expect_identical(as.logical(as.bit(l)), l)
          expect_identical(as.logical(as.bitwhich(l)), l)
          expect_identical(as.logical(as.bit(as.bit(l))), l)
          expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
          expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
          expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
        }
      }
  
  set.seed(1)
  for (k in 0:(3*bit:::.BITS)){
    l <- sample(c(FALSE, TRUE), k, TRUE)
    expect_identical(as.logical(as.bit(l)), l)
    expect_identical(as.logical(as.bit(as.bit(l))), l)
    expect_identical(as.logical(as.bit(as.bitwhich(l))), l)
    expect_identical(as.logical(as.bitwhich(as.bit(l))), l)
    expect_identical(as.logical(as.bitwhich(as.bitwhich(l))), l)
  }
  set.seed(1)
  for (k in 0:(3*bit:::.BITS)){
    l <- sample(c(NA, FALSE, TRUE), k, TRUE)
    b <- as.bit(l)
    expect_identical(as.bit(as.bit(l)), b)
    expect_identical(as.bit(as.bit(as.bit(l))), b)
    expect_identical(as.bit(as.bit(as.bitwhich(l))), b)
    expect_identical(as.bit(as.bitwhich(as.bit(l))), b)
    expect_identical(as.bit(as.bitwhich(as.bitwhich(l))), b)
  }
  set.seed(1)
  for (k in 0:(3*bit:::.BITS)){
    i <- sample(c(-2:2), k, TRUE)
    expect_identical(as.logical(as.bit(i)), as.logical(i))
    expect_identical(as.logical(as.bitwhich(i)), as.logical(i))
    i <- as.double(i)
    expect_identical(as.logical(as.bit(i)), as.logical(i))
    expect_identical(as.logical(as.bitwhich(i)), as.logical(i))
  }
  
}
)


test_that("boolean operations work", {
  #N <- c(1L,bit:::.BITS/2L-1L,bit:::.BITS/2L,bit:::.BITS/2L+1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L,2L*bit:::.BITS-1L,2L*bit:::.BITS,2L*bit:::.BITS+1L)
  #N <- c(1L,bit:::.BITS/2L-1L,bit:::.BITS/2L,bit:::.BITS/2L+1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L)
  N <- c(1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L)
  X <- c(rev(-N), N)
  N <- c(0L, N)
  fx <- function(x,n)as.integer(sign(x))*sample(n,abs(x))
  for (n in N){
    for (x1 in X[abs(X)<=abs(n)]){
      for (x2 in X[abs(X)<=abs(n)]){
        set.seed(1)
        w1 <- bitwhich(n,fx(x1,n))
        w2 <- bitwhich(n,fx(x2,n))
        l1 <- as.logical(w1)
        l2 <- as.logical(w2)
        b1 <- as.bit(l1)
        b2 <- as.bit(l2)
        fun <- function(x1,x2,f){
          eval(substitute(expect_identical( f(!x1), !f(x1) ), list(x1=x1, f=f)))
          eval(substitute(expect_identical( f(x1 & x2), f(x1) & f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( f(x1 | x2), f(x1) | f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( f(x1 == x2), f(x1) == f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( f(x1 != x2), f(x1) != f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( f(xor(x1, x2)),xor( f(x1), f(x2)) ), list(x1=x1, x2=x2, f=f)))
        }
        fun(w1,w2,as.logical)
        fun(b1,b2,as.logical)
        fun <- function(x1,x2,f){
          eval(substitute(expect_identical( x1 & x2, f(x1) & f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( x1 | x2, f(x1) | f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( x1 == x2, f(x1) == f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( x1 != x2, f(x1) != f(x2) ), list(x1=x1, x2=x2, f=f)))
          eval(substitute(expect_identical( xor(x1, x2), xor( f(x1), f(x2)) ), list(x1=x1, x2=x2, f=f)))
        }
        fun(b1,w2,as.bit)
        fun(w1,b2,as.bit)
        fun(b1,l2,as.logical)
        fun(l1,b2,as.logical)
        fun(l1,w2,as.logical)
        fun(w1,l2,as.logical)
      }
    }
  }
})


test_that("promotion is correct in boolean operations and concatenation", {
  N2 <- N1 <- c(2L,4L)
  T2 <- T1 <- c("logical","bit","bitwhich")
  F <- list(logical=logical, bit=bit, bitwhich=bitwhich)
  for (t1 in T1)
    for (t2 in T2)
      for (n1 in N1)
        for (n2 in N2){
          x1 <- F[[t1]](n1)
          x2 <- F[[t2]](n2)
          eval(substitute(expect_identical( booltype(x1 & x2), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          eval(substitute(expect_identical( booltype(x1 | x2), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          eval(substitute(expect_identical( booltype(x1 == x2), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          eval(substitute(expect_identical( booltype(x1 != x2), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          eval(substitute(expect_identical( booltype(xor(x1,x2)), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          if (t1!="logical")  # c with first argument logical does not dispatch
            eval(substitute(expect_identical( booltype(c(x1,x2)), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
          eval(substitute(expect_identical( booltype(c.booltype(x1,x2)), min(booltypes[[t1]],booltypes[[t2]]) ), list(x1=x1, x2=x2, t1=t1, t2=t2)))
        }
})


test_that("subscript operations work", {
  #N <- c(1L,bit:::.BITS/2L-1L,bit:::.BITS/2L,bit:::.BITS/2L+1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L,2L*bit:::.BITS-1L,2L*bit:::.BITS,2L*bit:::.BITS+1L)
  #N <- c(1L,bit:::.BITS/2L-1L,bit:::.BITS/2L,bit:::.BITS/2L+1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L)
  N <- c(1L,bit:::.BITS-1L,bit:::.BITS,bit:::.BITS+1L)
  X <- c(rev(-N), N)
  J <- c(rev(-N), 0L, N)
  N <- c(0L, N)
  R <- 1
  fx <- function(x,n)as.integer(sign(x))*sample(n,abs(x), TRUE)
  fi <- function(x,n)as.integer(sign(x))*sample(0:n,abs(x), FALSE)
  
  for (r in 1:R){
    for (n in N){
      for (x in X[abs(X)<=abs(n)]){
        set.seed(r)
        w <- bitwhich(n, fx(x,n))
        l <- as.logical(w)
        b <- as.bit(l)
        I <- J[abs(J)<=n]
        I <- lapply(I,fi,n=n)
        I <- c(
          list(
            FALSE
          , TRUE
          )
          , I
          , lapply(I, as.which, maxindex=n)
        )
        for (i in I){
          v <- l[i]
          eval(substitute(expect_equivalent( b[i], v ), list(b=b, i=i, v=v)))
          eval(substitute(expect_equivalent( w[i], v ), list(w=w, i=i, v=v)))
          l2 <- l
          l2[i] <- !v
          eval(substitute(expect_equivalent( {b2 <- b; b2[i] <- !v; b2}, as.bit(l2) ), list(b=b, i=i, v=v, l2=l2)))
          eval(substitute(expect_equivalent( {w2 <- w; w2[i] <- !v; w2}, as.bitwhich(l2) ), list(w=w, i=i, v=v, l2=l2)))
          if (length(v)>1 && (length(v)%%2L) == 0){
            v2 <- !v[seq_len(ceiling(length(v)/2))]
            l2 <- l
            l2[i] <- v2
            eval(substitute(expect_equivalent( {b2 <- b; b2[i] <- v2; b2}, as.bit(l2) ), list(b=b, i=i, v=v, l2=l2)))
            eval(substitute(expect_equivalent( {w2 <- w; w2[i] <- v2; w2}, as.bitwhich(l2) ), list(w=w, i=i, v=v, l2=l2)))
          }
        }
        I <- lapply(1:1, function(x){
          i <- quicksort2(sample(n, 2, TRUE))
          ri(i[1], i[2], n)
        })
        for (i in I){
          i2 <- i[1]:i[2]
          v <- l[i2]
          eval(substitute(expect_equivalent( b[i], v ), list(b=b, i=i, v=v)))
          eval(substitute(expect_equivalent( w[i], v ), list(w=w, i=i, v=v)))
          # debugonce(get("[.bitwhich"))
          l2 <- l
          l2[i2] <- !v
          eval(substitute(expect_equivalent( {b2 <- b; b2[i] <- !v; b2}, as.bit(l2) ), list(b=b, i=i, v=v, l2=l2)))
          eval(substitute(expect_equivalent( {w2 <- w; w2[i] <- !v; w2}, as.bitwhich(l2) ), list(w=w, i=i, v=v, l2=l2)))
          if (length(v)>1 && (length(v)%%2L) == 0){
            v2 <- !v[seq_len(ceiling(length(v)/2))]
            l2[i2] <- v2
            eval(substitute(expect_equivalent( {b2 <- b; b2[i] <- v2; b2}, as.bit(l2) ), list(b=b, i=i, v2=v2, l2=l2)))
            eval(substitute(expect_equivalent( {w2 <- w; w2[i] <- v2; w2}, as.bitwhich(l2) ), list(w=w, i=i, v2=v2, l2=l2)))
          }
        }
        i <- sample(n, 1)
          v <- l[[i]]
          eval(substitute(expect_equivalent( b[[i]], v ), list(b=b, i=i, v=v)))
          eval(substitute(expect_equivalent( w[[i]], v ), list(w=w, i=i, v=v)))
          l2 <- l
          l2[[i]] <- !v
          eval(substitute(expect_equivalent( {b2 <- b; b2[[i]] <- !v; b2}, as.bit(l2) ), list(b=b, i=i, v=v, l2=l2)))
          eval(substitute(expect_equivalent( {w2 <- w; w2[[i]] <- !v; w2}, as.bitwhich(l2) ), list(w=w, i=i, v=v, l2=l2)))

      }
    }
  }
    
})


test_that("aggregation functions work", {
  D <- list(
    full_range=ri(1,128,128)
  , begin_range=ri(1,5,128)
  , end_range=ri(99,128,128)
  , begin_scalar=ri(1,1,128)
  , end_scalar=ri(128,128,128)
  )
  R <- list(
    norange=NULL
  , begin_range=ri(1,64,128)
  , end_range=ri(65,128,128)
  , mid_range=ri(32, 96, 128)
  , full_range=ri(1,128,128)
  )
  I <- list(
      id=function(x)x
    , not=function(x)!x
  )
  A <- list(
      logical=as.logical
    , bit=as.bit
    , bitwhich=as.bitwhich
    , which=as.which
    , ri=function(x)x
  )
  S1 <- list(
      any=any
    , all=all
    , sum=sum
  )
  S2 <- list(
      min=function(x)if (booltype(x)=="logical")  {if (any(x)) which.max(x) else NA_integer_} else min(x)
    , max=function(x)if (booltype(x)=="logical")  {if (any(x)) length(x)-which.max(rev(x))+1L else NA_integer_} else max(x)
    , range=function(x)if (booltype(x)=="logical") range.booltype(x) else range(x)
    , summary=function(x)if (booltype(x)=="logical") summary.booltype(x) else summary(x)
  )
  S3 <- list(
      sum=sum.booltype
    , min=min.booltype
    , max=max.booltype
    , range=range.booltype
    , summary=summary.booltype
  )
  for(d in names(D)){
    for (i in names(I)){
      x <- I[[i]](as.logical(D[[d]]))
      for (a in names(A))if ( ! (i=="not" && a %in% c("which","ri") )){
        y <- I[[i]](A[[a]](D[[d]]))
        for (s in names(S1)){
          expect_identical(S1[[s]](x), S1[[s]](y))
        }
        if (a != "which")for (s in names(S2)){
            expect_identical(S2[[s]](x), S2[[s]](y))
          }
        if (a != "which")for (s in names(S3)){
          expect_identical(S3[[s]](x), S3[[s]](y))
        }
      }
    }
  }
    
})

# Fast methods for integers leveraging bit vectors
# (c) 2016-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

#' Get range and number of NAs
#'
#' @param x an integer vector
#'
#' @return an integer vector with three elements \describe{
#'   \item{ 1 }{ min integer }
#'   \item{ 2 }{ max integer }
#'   \item{ 3 }{ number of NAs }
#'   }
#' @seealso \code{\link{range_nanozero}} and \code{\link{range_sortna}}
#'
#' @examples
#' range_na(c(0L,1L,2L,NA))
#' @export
range_na <- function(x){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  .Call(C_R_range_na, x)
}

#' Remove zeros and get range and number of NAs 
#'
#' @param x an integer vector
#'
#' @return an integer vector without zeros and with an attribute \code{\link{range_na}}  with three elements \describe{
#'   \item{ 1 }{ min integer }
#'   \item{ 2 }{ max integer }
#'   \item{ 3 }{ number of NAs }
#'   }
#' @seealso \code{\link{range_na}} and \code{\link{range_sortna}}
#'
#' @examples
#' range_nanozero(c(0L,1L,2L,NA))
#' @export
range_nanozero <- function(x){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  .Call(C_R_range_nanozero, x)
}

#' Prepare for sorting and get range, number of NAs and unsortedness
#' 
#' In one pass over the vector \code{NA}s are treated according to parameter
#' \code{na.last} exactly like \code{\link{sort}} does, the \code{\link{range}},
#' number of \code{NA}s and unsortedness is determined.
#'
#' @param x an integer vector
#' @inheritParams bit_sort
#'
#' @return an integer vector with \code{NA}s are treated and an \code{attribute} \code{\link{range_na}}  with four elements \describe{
#'   \item{ 1 }{ min integer }
#'   \item{ 2 }{ max integer }
#'   \item{ 3 }{ number of NAs }
#'   \item{ 3 }{ 0 for sorted vector and 1 for \code{\link{is.unsorted}}  }
#'   }
#' @seealso \code{\link{range_na}} and \code{\link{range_nanozero}}
#'
#' @examples
#' range_sortna(c(0L,1L,NA,2L))
#' range_sortna(c(2L,NA,1L,0L))
#' range_sortna(c(0L,1L,NA,2L), na.last=TRUE)
#' range_sortna(c(2L,NA,1L,0L), na.last=TRUE)
#' range_sortna(c(0L,1L,NA,2L), na.last=FALSE)
#' range_sortna(c(2L,NA,1L,0L), na.last=FALSE)
#' @export
range_sortna <- function(x, decreasing = FALSE, na.last=NA){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  .Call(C_R_range_sortna, x, as.logical(decreasing), as.logical(na.last))
}


#' Low-level sorting: binary quicksort
#' 
#' In one pass over the vector \code{NA}s are handled according to parameter
#' \code{na.last} by \code{\link{range_sortna}}, then, if the vector is unsorted, 
#' binary quicksort is invoked.
#' 
#' @param x an integer vector
#' @inheritParams bit_sort
#'
#' @return a sorted vector
#'
#' @examples
#' quicksort2(c(2L,0L,1L,NA,2L))
#' quicksort2(c(2L,0L,1L,NA,2L), na.last=TRUE)
#' quicksort2(c(2L,0L,1L,NA,2L), na.last=FALSE)
#' @export
quicksort2 <- function(x, na.last=NA){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  x <- range_sortna(x, na.last=na.last)
  range_sortna <- getsetattr(x, "range_sortna", NULL)
  if (is.na(range_sortna[1]))  # no numbers to sort
    return(rep(NA_integer_, range_sortna[3]))  
  if (range_sortna[4])  # is.unsorted
    .Call(C_R_int_quicksort2, x, range_sortna, as.logical(na.last))
  else
    x 
}

#' Low-level sorting: threeway quicksort
#' 
#' In one pass over the vector \code{NA}s are handled according to parameter
#' \code{na.last} by \code{\link{range_sortna}}, then, if the vector is unsorted, 
#' threeway quicksort is invoked.
#' 
#' @param x an integer vector
#' @inheritParams bit_sort
#'
#' @return a sorted vector
#'
#' @examples
#' countsort(c(2L,0L,1L,NA,2L))
#' countsort(c(2L,0L,1L,NA,2L), na.last=TRUE)
#' countsort(c(2L,0L,1L,NA,2L), na.last=FALSE)
#' @export
quicksort3 <- function(x, na.last=NA){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  x <- range_sortna(x, na.last=na.last)
  range_sortna <- getsetattr(x, "range_sortna", NULL)
  if (is.na(range_sortna[1]))  # no numbers to sort
    return(rep(NA_integer_, range_sortna[3]))  
  if (range_sortna[4])  # is.unsorted
    .Call(C_R_int_quicksort3, x, range_sortna, as.logical(na.last))
  else
    x
}


#' Low-level sorting: counting sort
#' 
#' In one pass over the vector \code{NA}s are handled according to parameter
#' \code{na.last} by \code{\link{range_sortna}}, then, if the vector is unsorted, 
#' counting sort is invoked.
#' 
#' @param x an integer vector
#' @inheritParams bit_sort
#'
#' @return a sorted vector
#'
#' @examples
#' countsort(c(2L,0L,1L,NA,2L))
#' countsort(c(2L,0L,1L,NA,2L), na.last=TRUE)
#' countsort(c(2L,0L,1L,NA,2L), na.last=FALSE)
#' @export
countsort <- function(x, na.last=NA){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  x <- range_sortna(x, na.last=na.last)
  range_sortna <- getsetattr(x, "range_sortna", NULL)
  if (is.na(range_sortna[1]))  # no numbers to sort
    return(rep(NA_integer_, range_sortna[3]))  
  if (range_sortna[4])  # is.unsorted
    .Call(C_R_int_countsort, x, range_sortna, as.logical(na.last))
  else
    x
}

#' Low-level sorting: bit sort
#' 
#' In one pass over the vector \code{NA}s are handled according to parameter
#' \code{na.last} by \code{\link{range_sortna}}, then, if the vector is unsorted, 
#' bit sort is invoked.
#' 
#' @param x an integer vector
#' @param depth an integer scalar giving the number of bit-passed before switching to quicksort
#' @inheritParams bit_sort
#'
#' @return a sorted vector
#'
#' @examples
#' bitsort(c(2L,0L,1L,NA,2L))
#' bitsort(c(2L,0L,1L,NA,2L), na.last=TRUE)
#' bitsort(c(2L,0L,1L,NA,2L), na.last=FALSE)
#' @export
bitsort <- function(x, na.last=NA, depth=1){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  x <- range_sortna(x, na.last=na.last)
  range_sortna <- getsetattr(x, "range_sortna", NULL)
  if (is.na(range_sortna[1]))  # no numbers to sort
    return(rep(NA_integer_, range_sortna[3]))  
  if (!range_sortna[4])  # already sorted
    return(x)
  n <- as.double(range_sortna[2])-as.double(range_sortna[1])+1
  .Call(C_R_bit_sort, tmp=bit(n), x, range_sortna, as.logical(na.last), depth=as.integer(depth))
}



# if (FALSE){
#   # time measure relative speeds (radixsort is not yet integrated)
#   require(microbenchmark)
#   require(bit)
#   require(ff)
#   x <- sample(2,1e8,T)
#   system.time(bitsort(x))
#   x <- sample(1e8,1e8,T)
#   system.time(bitsort(x))
#   
#   D <- as.integer(2^(1:27))
#   N <- as.integer(2^(1:27))
#   M <- c("b","c","q") #,"r")
#   tim <- array(NA, dim=c(D=length(D), N=length(N), M=length(M)), dimnames=list(D=D, N=N, M=M))
#   for (ni in seq_along(N)){
#     n <- N[ni]
#     for (di in seq_along(D)){
#       d <- D[di]    
#       x <- sample(d,n,T)
#       x[1] <- d
#       if (di > ni-5 && di < ni+10){
#         tim[di,ni,"b"] <- microbenchmark(bitsort(x), times = 1)$time
#         tim[di,ni,"c"] <- microbenchmark(countsort(x), times = 1)$time
#         tim[di,ni,"q"] <- microbenchmark(quicksort3(x), times = 1)$time
#         #tim[di,ni,"r"] <- microbenchmark(radixsort(copy_vector(x)), times = 1)$time
#       }
#     }
#     print(apply(tim[,1:ni,,drop=FALSE], 1:2, function(x){i <- which.min(x); if (length(i))M[i] else "."}), quote=FALSE)
#   }
#   
#   round( tim[,,"b"] / tim[,,"c"], 1 )
#   round( tim[,,"b"] / tim[,,"q"], 1 )
#   round( tim[,,"c"] / tim[,,"q"], 1 )
#   round( tim[,,"c"] / tim[,,"r"], 1 )
#   round( tim[,,"q"] / tim[,,"r"], 1 )
#   
#   M <- dimnames(tim)[[3]]
#   r <- as.integer(rownames(tim[,,1]))[row(tim[,,1])]
#   n <- as.integer(colnames(tim[,,1]))[col(tim[,,1])]
#   d <- n/r
#   #x <- ifelse(d>=1, "c", "q")
#   x <- ifelse(d>=0.5,"c",ifelse(n<=1024,"q",ifelse(d<0.0625,"q","b")))
#   x <- ifelse(n<=1024, ifelse(d<0.5, "q", "c"), ifelse(d<1, ifelse(d<0.0625, "q", "b"), "c"))
#   dim(x) <- dim(tim[,,1])
#   dimnames(x) <- dimnames(tim[,,1])
#   y <- tim[cbind(as.vector(row(x)), as.vector(col(x)), match(as.vector(x), M))]
#   attributes(y) <- attributes(x)
#   print(apply(tim, 1:2, function(x){i <- which.min(x); if (length(i))M[i] else "."}), quote=FALSE)
#   print(x, quote=F)
#   round(y / apply(tim[,,-4], 1:2, min, na.rm=T), 1)
#   round(y / tim[,,1], 1)
#   
#   f <- function(n, d=1, k=1){
#     x <- dbinom(1:log2(n), n, 1/(d*n))*n
#     r <- rev(cumsum(rev(x)))
#     rr <- c(r[-1], 0)
#     c <- r*log2(r)
#     b <- n*k+r+rr*log2(rr)
#     w <- as.integer(!is.na(b) & b<c)
#     #print(round(data.frame(x=x, r=r, rr=rr, c=c, b=b, w=w)))
#     sum(w)
#   }
#   sapply(2^(1:30), f, d=8, k=1)
#   sapply(2^(1:30), f, k=2)  
#   sapply(2^(1:30), f, k=4)  
#   sapply(2^(1:30), f, k=8)  
#   sapply(2^(1:30), f, k=16)  
#   sapply(2^(1:30), f, k=32)  
#   
#   require(microbenchmark)
#   require(bit)
#   n <- as.integer(2^27)
#   x <- sample(n*4, n, T)
#   times <- 1
#   microbenchmark(countsort(x), times=times)
#   microbenchmark(quicksort3(x), times=times)
#   microbenchmark(bitsort(x, depth=1), times=times)
#   microbenchmark(bitsort(x, depth=2), times=times)
#   microbenchmark(bitsort(x, depth=3), times=times)
#   microbenchmark(bitsort(x, depth=4), times=times)
#   microbenchmark(bitsort(x, depth=5), times=times)
#   
#   microbenchmark(quicksort2(x), times=times)
#   microbenchmark(bit_sort_unique(x), times=times)
#   
#   # ergo: ab 2^16 zwischen 1/4 und 1/32
#   
# }


#' bit sort
#' 
#' fast sorting of integers
#' 
#' determines the range of the integers and checks if the density justifies use 
#' of a bit vector; if yes, sorts the first occurences of each integer in the
#' range using a bit vector, sorts the rest and merges; if no, falls back to quicksort.
#' 
#' @param x an integer vector
#' @param decreasing  (currently only \code{FALSE} is supported)
#' @param na.last \code{NA} removes NAs, \code{FALSE} puts NAs at the beginning,
#'   \code{TRUE} puts NAs at the end
#' @param has.dup TRUE (the default) assumes that \code{x} might have 
#'   duplicates, set to \code{FALSE} if duplicates are impossible
#'   
#' @return a sorted vector
#' @seealso \code{\link{sort}}, \code{\link{ramsort}}, 
#'   \code{\link{bit_sort_unique}}
#' 
#' @examples
#' bit_sort(c(2L,1L,NA,NA,1L,2L))
#' bit_sort(c(2L,1L,NA,NA,1L,2L), na.last=FALSE)
#' bit_sort(c(2L,1L,NA,NA,1L,2L), na.last=TRUE)
#' 
#' \dontrun{
#' x <- sample(1e7, replace=TRUE)
#' system.time(bit_sort(x))
#' system.time(sort(x))
#' }
#' @export
bit_sort <- function(x, decreasing = FALSE, na.last=NA, has.dup = TRUE){
  if (decreasing)
    stop("decreasing=TRUE not implemented")
  if (!has.dup)
    return(bit_sort_unique(x, decreasing = decreasing, na.last=na.last, has.dup=FALSE))
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  x <- range_sortna(x, decreasing = decreasing, na.last=na.last)
  range_sortna <- getsetattr(x, "range_sortna", NULL)
  if (is.na(range_sortna[1]))  # no numbers to sort
    return(rep(NA_integer_, range_sortna[3]))  
  if (!range_sortna[4])  # already sorted
    return(x)
  nr <- as.double(range_sortna[2])-as.double(range_sortna[1])+1
  d <- length(x) / nr
  if (nr<=65536){
    if (d<0.5)
      ret <- .Call(C_R_int_quicksort3, x, range_sortna, as.logical(na.last))
    else
      ret <- .Call(C_R_int_countsort, x, range_sortna, as.logical(na.last))
  }else{
    if (d < 0.03125 || nr > .Machine$integer.max ){
      ret <- .Call(C_R_int_quicksort3, x, range_sortna, as.logical(na.last))
    }else if (d <= 0.25){
      ret <- .Call(C_R_bit_sort, tmp=bit(nr), x, range_sortna, as.logical(na.last), depth=1L)
    }else{
      ret <- .Call(C_R_int_countsort, x, range_sortna, as.logical(na.last))
    }
  }
  ret
}


#' bit sort unique
#' 
#' fast combination of \code{\link{sort}} and \code{\link{unique}} for integers
#' 
#' determines the range of the integers and checks if the density justifies use 
#' of a bit vector; if yes, creates the result using a bit vector; if no, falls back to \code{sort(unique())}
#'
#' @param x an integer vector
#' @param decreasing \code{FALSE} (ascending) or \code{TRUE} (descending)
#' @param na.last \code{NA} removes NAs, \code{FALSE} puts NAs at the beginning, \code{TRUE} puts NAs at the end  
#' @param has.dup TRUE (the default) assumes that \code{x} might have duplicates, set to \code{FALSE} if duplicates are impossible
#' @param range_na \code{NULL} calls \code{\link{range_na}}, optionally the result of \code{\link{range_na}} can be given here to avoid calling it again
#' 
#' @return a sorted unique integer vector 
#' @seealso \code{\link{sort}}, \code{\link{unique}}, 
#'   \code{\link{bit_sort}}, \code{\link{bit_unique}}
#'
#' @examples
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L))
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L), na.last=FALSE)
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L), na.last=TRUE)
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE)
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE, na.last=FALSE)
#' bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE, na.last=TRUE)
#' 
#' \dontrun{
#' x <- sample(1e7, replace=TRUE)
#' system.time(bit_sort_unique(x))
#' system.time(sort(unique(x)))
#' x <- sample(1e7)
#' system.time(bit_sort_unique(x))
#' system.time(sort(x))
#' } 
#' @export
bit_sort_unique <- function(x, decreasing = FALSE, na.last=NA, has.dup=TRUE, range_na=NULL){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (is.null(range_na))
    range_na <- range_na(x)
  else 
    range_na <- as.integer(range_na)
  if (is.na(range_na[1])){
    if (is.na(na.last) || length(range_na)<3L || range_na[3]==0)
      return(integer())
    else 
      return(NA_integer_)
  }
  nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- length(x) / nr
  if (nr > .Machine$integer.max || d < 0 ){
    if (has.dup){
      ret <- quicksort3(x, na.last = xor(na.last, decreasing))
      ret <- merge_unique(ret)
    }else{
      ret <- quicksort2(x, na.last = xor(na.last, decreasing))
    }
    if (decreasing)
      reverse_vector(ret)
    else
      ret
  }else
    .Call(C_R_bit_sort_unique, tmp=bit(nr), x, range_na, as.logical(na.last), as.logical(decreasing))
} 


#' bit \%in\%
#' 
#' fast \code{\link{\%in\%}} for integers
#' 
#' determines the range of the integers and checks if the density justifies use 
#' of a bit vector; if yes, maps \code{x} or \code{table} -- whatever is smaller
#' -- into a bit vector and searches the other of \code{table} or \code{x} in
#' the it vector; if no, falls back to \code{\link{\%in\%}}
#' 
#' @param x an integer vector of values to be looked-up
#' @param table an integer vector used as lookup-table
#' @param retFUN a function that coerces \code{\link{bit}} and \code{\link{logical}} vectors
#'   
#' @return a boolean vector coerced to \code{retFUN}
#' @seealso \code{\link{\%in\%}}
#' @examples
#' bit_in(1:2, 2:3)
#' bit_in(1:2, 2:3, retFUN=as.logical)
#' @export
bit_in <- function(x, table, retFUN=as.bit){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(table) || is.factor(table)))
    stop("table must be integer (or factor)")
  nx <- length(x)
  nt <- length(table)
  reverse <- nx<nt
  if (reverse){
    range_na <- range_na(x)
    n <- nx
  }else{
    range_na <- range_na(table)
    n <- nt
  }
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+nt) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/64 ){
    ret <- !is.na(match(x, table))
  }else{
    if (reverse){
      ret <- .Call(C_R_bit_table_in, tmp=bit(nr), x, table, range_na, ret=bit(length(x)))
    }else{
      ret <- .Call(C_R_bit_in_table, tmp=bit(nr), x, table, range_na, ret=bit(length(x)))
    }
  }
  retFUN(ret)
}

#' bit unique and duplicated
#' 
#' Fast versions of \code{\link{unique}}, \code{\link{duplicated}} ,
#' \code{\link{anyDuplicated}} and \code{sum(duplicated(x))} for integers.
#' 
#' determines the range of the integers and checks if the density justifies use 
#' of a bit vector; if yes, uses a bit vector for finding duplicates; if no, 
#' falls back to \code{\link{unique}}, \code{\link{duplicated}}, \code{\link{anyDuplicated}} and \code{sum(duplicated(x))}
#' 
#' @name bit_unidup
#' @param x an integer vector
#' @param na.rm \code{NA} treats NAs like other integers, \code{TRUE} treats 
#'   \emph{all} NAs as duplicates, \code{FALSE} treats \emph{no} NAs as 
#'   duplicates
#' @inheritParams bit_in
#' @inheritParams bit_sort_unique
#'   
#' @return 
#'   \code{bit_unique} returns a vector of unique integers, \cr
#'   \code{bit_duplicated} returns a boolean vector coerced to \code{retFUN}, \cr
#'   \code{bit_anyDuplicated} returns the position of the first duplicate (or zero if no duplicates) \cr
#'   \code{bit_sumDuplicated} returns the number of duplicated values (as.integer)
#' @seealso \code{\link{bit_sort_unique}}
#' 
#' @examples
#' bit_unique(c(2L,1L,NA,NA,1L,2L))
#' bit_unique(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
#' bit_unique(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)
#' 
#' bit_duplicated(c(2L,1L,NA,NA,1L,2L))
#' bit_duplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
#' bit_duplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)
#'
#' bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L))
#' bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
#' bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)
#'
#' bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L))
#' bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
#' bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)
NULL

#' @describeIn bit_unidup extracts unique elements
#' @export
bit_unique <- function(x, na.rm = NA, range_na=NULL){
  na.rm <- as.logical(na.rm)
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(x)
  if (is.null(range_na))
    range_na <- range_na(x)
  else 
    range_na <- as.integer(range_na)
  if (is.na(range_na[1])){
    if (is.na(na.rm)){
      return(NA_integer_)
    }else if (na.rm){
      return(integer())
    }else{
      return(rep(NA_integer_, range_na[3]))
    }
  }
  nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- length(x) / nr
  if (nr > .Machine$integer.max || d < 1/64 ){
    if (is.na(na.rm)){
      unique(x, incomparables = FALSE)
    }else if (na.rm){
      x <- unique(x, incomparables = FALSE)
      x <- x[!is.na(x)]
      x
    }else{
      unique(x, incomparables = NA)
    }
  }else{
    .Call(C_R_bit_unique, tmp=bit(nr), x, range_na, na.rm)
  }
}

#' @describeIn bit_unidup determines duplicate elements
#' @export
bit_duplicated <- function(x, na.rm = NA, range_na=NULL, retFUN=as.bit){
  na.rm <- as.logical(na.rm)
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(retFUN(NULL))
  if (is.null(range_na))
    range_na <- range_na(x)
  else 
    range_na <- as.integer(range_na)
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- length(x) / nr
  if (nr > .Machine$integer.max || d < 1/64 ){
    if (is.na(na.rm)){
      ret <- duplicated(x, incomparables = FALSE)
    }else if (na.rm){
      ret <- duplicated(x, incomparables = FALSE) | is.na(x)
    }else{
      ret <- duplicated(x, incomparables = NA)
    }
  }else{
    ret <- .Call(C_R_bit_duplicated, tmp=bit(nr), x, range_na, ret=bit(length(x)), as.logical(na.rm))
  }
  retFUN(ret)
}

#' @describeIn bit_unidup checks for existence of duplicate elements
#' @export
bit_anyDuplicated <- function(x, na.rm = NA, range_na=NULL){
  na.rm <- as.logical(na.rm)
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(0L)
  if (is.null(range_na))
    range_na <- range_na(x)
  else 
    range_na <- as.integer(range_na)
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- length(x) / nr
  if (nr > .Machine$integer.max || d < 1/64 ){
    if (is.na(na.rm)){
      anyDuplicated(x, incomparables = FALSE)
    }else if (na.rm){
      max(0L, min(anyDuplicated(x, incomparables = NA), firstNA(x)))
    }else{
      anyDuplicated(x, incomparables = NA)
    }
  }else{
    .Call(C_R_bit_anyDuplicated, bit(nr), x, range_na=range_na, as.logical(na.rm))
  }
}

#' @describeIn bit_unidup counts duplicate elements
#' @export
bit_sumDuplicated <- function(x, na.rm = NA, range_na=NULL){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!length(x))
    return(0L)
  if (is.null(range_na))
    range_na <- range_na(x)
  else 
    range_na <- as.integer(range_na)
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- length(x) / nr
  if (nr > .Machine$integer.max || d < 1/64 ){
    if (is.na(na.rm)){
      sum(duplicated(x, incomparables = FALSE))
    }else if (na.rm){
      sum(duplicated(x, incomparables = NA)) + sum(is.na(x))
    }else{
      sum(duplicated(x, incomparables = NA))
    }
  }else{
    .Call(C_R_bit_sumDuplicated, bit(nr), x, range_na=range_na, as.logical(na.rm))
  }
}




#' bit set operations
#' 
#' Fast versions of \code{\link{union}}, \code{\link{intersect}}, 
#' \code{\link{setdiff}}, symmetric difference and \code{\link{setequal}}
#' for integers.
#' 
#' determines the range of the integers and checks if the density justifies use 
#' of a bit vector; if yes, uses a bit vector for finding duplicates; if no, 
#' falls back to \code{\link{union}}, \code{\link{intersect}},
#' \code{\link{setdiff}}, \code{union(setdiff(x,y),setdiff(y,x))} and \code{\link{setequal}}
#' 
#' @name bit_setops
#' @param x an integer vector
#' @param y an integer vector
#'   
#' @return an integer vector
#' @seealso \code{\link{bit_in}}, \code{\link{bit_rangediff}}
#' 
#' @examples
#' bit_union(1:2, 2:3)
#' bit_intersect(1:2, 2:3)
#' bit_setdiff(1:2, 2:3)
#' bit_symdiff(1:2, 2:3)
#' bit_setequal(1:2, 2:3)
#' bit_setequal(1:2, 2:1)
NULL

#' @describeIn bit_setops union
#' @export
bit_union <- function(x, y){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(y) || is.factor(y)))
    stop("y must be integer (or factor)")
  nx <- length(x)
  ny <- length(y)
  rx <- range_na(x)
  ry <- range_na(y)
  range_na <- c(min(rx[1],ry[1]), max(rx[2],ry[2]), rx[3]+ry[3])
  if (max(rx[1],ry[1])>min(rx[2],ry[2])){
    if ( range_na[3]>0 && (rx[3]>0) == (rx[3]>0) )
      return(bit_unique(c(x,y), range_na=range_na))
    else
      return(c(bit_unique(x, range_na=rx), bit_unique(y, range_na=ry)))
  }
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+ny) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/64 ){
    ret <- union(x,y)
  }else{
    ret <- .Call(C_R_bit_union, tmp=bit(nr), x, y, range_na)
  }
  ret
}

#' @describeIn bit_setops intersection
#' @export
bit_intersect <- function(x, y){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(y) || is.factor(y)))
    stop("y must be integer (or factor)")
  nx <- length(x)
  ny <- length(y)
  rx <- range_na(x)
  ry <- range_na(y)
  range_na <- c(max(rx[1],ry[1]), min(rx[2],ry[2]), rx[3]+ry[3])
  if (range_na[1]>range_na[2])
    return(integer())
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+ny) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/64 ){
    ret <- intersect(x,y)
  }else{
    ret <- .Call(C_R_bit_intersect, tmp=bit(nr), x, y, range_na)
  }
  ret
}

#' @describeIn bit_setops asymmetric difference
#' @export
bit_setdiff <- function(x, y){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(y) || is.factor(y)))
    stop("y must be integer (or factor)")
  nx <- length(x)
  ny <- length(y)
  range_na <- range_na(x)
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+ny) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/64 ){
    ret <- setdiff(x,y)
  }else{
    ret <- .Call(C_R_bit_setdiff, tmp=bit(nr), x, y, range_na)
  }
  ret
}

#' @describeIn bit_setops symmetricx difference
#' @export
bit_symdiff <- function(x, y){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(y) || is.factor(y)))
    stop("y must be integer (or factor)")
  nx <- length(x)
  ny <- length(y)
  rx <- range_na(x)
  ry <- range_na(y)
  range_na <- c(min(rx[1],ry[1]), max(rx[2],ry[2]), rx[3]+ry[3])
  if (max(rx[1],ry[1])>min(rx[2],ry[2])){
    if ( range_na[3]>0 && (rx[3]>0) == (rx[3]>0) )
      return(bit_unique(c(x,y), range_na=range_na))
    else
      return(c(bit_unique(x, range_na=rx), bit_unique(y, range_na=ry)))
  }
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+ny) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/64 ){
    ret <- union(setdiff(x,y),setdiff(y,x))
  }else{
    ret <- .Call(C_R_bit_symdiff, bit(nr), bit(nr), x, y, range_na, rx[3]>0, ry[3]>0)
  }
  ret
}

#' @describeIn bit_setops equality
#' @export
bit_setequal <- function(x, y){
  if (!(is.integer(x) || is.factor(x)))
    stop("x must be integer (or factor)")
  if (!(is.integer(y) || is.factor(y)))
    stop("y must be integer (or factor)")
  nx <- length(x)
  ny <- length(y)
  rx <- range_na(x)
  ry <- range_na(y)
  rx[3] <- as.logical(rx[3])  # map NA count to 0/1
  ry[3] <- as.logical(ry[3])  # map NA count to 0/1
  if (!identical(rx,ry))
    return(FALSE)
  range_na <- rx
  if (is.na(range_na[1]))
    nr <- 0L  # no numbers, no temp bit vector
  else
    nr <- as.double(range_na[2])-as.double(range_na[1])+1
  d <- (nx+ny) / nr
  if (nr==0L || nr > .Machine$integer.max || d < 1/256 ){
    ret <- setequal(x,y)
  }else{
    ret <- .Call(C_R_bit_setequal, bit(nr), bit(nr), x, y, range_na)
  }
  ret
}


#' bit range difference
#' 
#' Fast version of \code{setdiff(rx[1]:rx[2], y)}.
#' 
#' determines the range of the integers \code{y} and checks if the density justifies use 
#' of a bit vector; if yes, uses a bit vector for the set operation; if no, 
#' falls back to a quicksort and \code{\link{merge_rangediff}}
#' 
#' @param y an integer vector of elements to exclude
#' @param revx \code{FALSE} as is, \code{TRUE} to reverse the direction and sign of \code{rx}
#' @param revy \code{FALSE} as is, \code{TRUE} to reverse the direction and sign of \code{y}
#' @inheritParams merge_rev
#'
#' @return an integer vector
#' @seealso \code{\link{bit_setdiff}}, \code{\link{merge_rangediff}}
#'
#' @examples
#' bit_rangediff(c(1L,6L), c(3L,4L))
#' bit_rangediff(c(6L,1L), c(3L,4L))
#' bit_rangediff(c(6L,1L), c(3L,4L), revx=TRUE)
#' bit_rangediff(c(6L,1L), c(3L,4L), revx=TRUE, revy=TRUE)
#' @export
bit_rangediff <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.integer(y))
    stop("y must be integer")
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (rx[1]>rx[2])
    ox <- as.double(rev(rx))
  else
    ox <- as.double(rx)
  n <- ox[2]-ox[1]+1L
  d <- length(y) / n
  if (n > .Machine$integer.max || d < 1/64 ){
    ox <- as.integer(ox)
    y <- .Call(C_R_int_quicksort2, copy_vector(y), c(ox, 0L), na.last=NA)
    ret <- merge_rangediff(ox, y, revx=revx, revy=revy)
    if (rx[1]>rx[2])
      ret <- rev(ret)
    ret
  }else{
    .Call(C_R_bit_rangediff, tmp=bit(n), as.integer(rx), y, as.logical(revx), as.logical(revy))
  }
}


# /* some experiments - just ignore
# bit_sample1 <- function (x, size, replace = FALSE, prob = NULL){
#   if (length(x)==1L){
#     x <- seq_len(x)
#   } else{
#     x <- as.integer(x)
#   }  
#   size <- as.integer(size)
#   if (replace)
#     stop("replace not implemented")
#   else{
#     stopifnot(size <= length(x))
#     .Call(C_R_bit_sample, x, bit(length(x)), size)
#   }
# }
# bit_sample <- function (x, size=length(x), replace = FALSE, prob = NULL){
#   if (length(x)==1L){
#     x <- seq_len(x)
#   } else{
#     x <- as.integer(x)
#   }  
#   size <- as.integer(size)
#   replace <- as.logical(replace)
#   if (!replace && size>length(x))
#     stop("size > length(x) without replace")
#   .Call(C_R_bit_sample, x, size, replace)
# }
# */

test_that("bitsort function work on several inputs", {
  nums <- list(
    integer(),
    as.integer(9:0),
    as.integer(0:9),
    as.integer(0:(-9)),
    as.integer((-9):0),
    as.integer((-4):4),
    as.integer(4:(-4))
  )
  nas <- list(
    integer(),
    NA_integer_,
    c(NA_integer_, NA_integer_)
  )
  set.seed(1)
  for (num in nums) {
    for (na in nas) {
      x <- c(num, na)
      if (length(x) > 1)
        x <- sample(x)

      range_result <- suppressWarnings(as.integer(c(range(x, na.rm=TRUE), sum(is.na(x)))))
      expect_identical(
        range_na(x),
        range_result,
        label=paste0("range_na(c(", toString(x), "))")
      )

      range_result <- suppressWarnings(as.integer(c(
        range(x[is.na(x) | x != 0], na.rm=TRUE),
        sum(is.na(x))
      )))
      expect_identical(
        attr(range_nanozero(x), "range_na"),
        range_result,
        label=paste0("range_na(c(", toString(x), ")")
      )

      for (na.last in c(NA, FALSE, TRUE)) {
        expected = range_sortna(x, na.last=na.last)
        expected[!is.na(expected)] = 1L

        actual = sort.int(x, na.last=na.last, method='quick')
        r = range_na(actual)
        r[4L] = is.unsorted(x, na.rm=TRUE)
        actual[!is.na(actual)] = 1L
        attr(actual, "range_sortna") = r
        expect_identical(
          expected, actual,
          label=sprintf("range_sortna(c(%s), na.last=%s)", toString(x), na.last)
        )

        expect_identical(
          bit_sort_unique(x, na.last=na.last, has.dup = TRUE),
          sort.int(unique(x), na.last=na.last, method="quick"),
          label=sprintf("bit_sort_unique(c(%s), na.last=%s, has.dup=TRUE)", toString(x), na.last)
        )

        y <- unique(x)
        expect_identical(
          bit_sort_unique(y, na.last=na.last, has.dup = FALSE),
          sort.int(y, na.last=na.last, method="quick"),
          label=sprintf("bit_sort_unique(c(%s), na.last=%s, has.dup=FALSE)", toString(x), na.last)
        )

        expect_identical(
          bit_sort(x, na.last=na.last),
          sort.int(x, na.last=na.last, method="quick"),
          label=paste0("bit_sort(c(", toString(x), "), na.last=", na.last, ")")
        )
      }

      expect_identical(
        bit_unique(x, na.rm=NA),
        unique(x, incomparables=FALSE),
        label=paste0("bit_unique(c(", toString(x), "), na.rm=", NA, ")")
      )
      expect_identical(
        bit_unique(x, na.rm=FALSE),
        unique(x, incomparables=NA),
        label=paste0("bit_unique(c(", toString(x), "), na.rm=", FALSE, ")")
      )
      expect_identical(
        bit_unique(x, na.rm=TRUE),
        unique(x[!is.na(x)]),
        label=paste0("bit_unique(c(", toString(x), "), na.rm=", TRUE, ")")
      )

      expect_identical(
        bit_duplicated(x, na.rm=NA),
        as.bit(duplicated(x, incomparables=FALSE)),
        label=paste0("bit_duplicated(c(", toString(x), "), na.rm=", NA, ")")
      )
      expect_identical(
        bit_duplicated(x, na.rm=FALSE),
        as.bit(duplicated(x, incomparables=NA)),
        label=paste0("bit_duplicated(c(", toString(x), "), na.rm=", FALSE, ")")
      )
      expect_identical(
        bit_duplicated(x, na.rm=TRUE),
        as.bit(duplicated(x) | is.na(x)),
        label=paste0("bit_duplicated(c(", toString(x), "), na.rm=", TRUE, ")")
      )

      expect_identical(
        bit_anyDuplicated(x, na.rm=NA),
        anyDuplicated(x, incomparables=FALSE),
        label=paste0("bit_anyDuplicated(c(", toString(x), "), na.rm=", NA, ")")
      )
      expect_identical(
        bit_anyDuplicated(x, na.rm=FALSE),
        anyDuplicated(x, incomparables=NA),
        label=paste0("bit_anyDuplicated(c(", toString(x), "), na.rm=", FALSE, ")")
      )
      expect_identical(
        bit_anyDuplicated(x, na.rm=TRUE),
        max(0L, head(which(is.na(x) | duplicated(x)), 1)),
        label=paste0("bit_anyDuplicated(c(", toString(x), "), na.rm=", TRUE, ")")
      )

      expect_identical(
        bit_sumDuplicated(x, na.rm=NA),
        sum(duplicated(x, incomparables=FALSE)),
        label=paste0("bit_sumDuplicated(c(", toString(x), "))")
      )
      expect_identical(
        bit_sumDuplicated(x, na.rm=FALSE),
        sum(duplicated(x, incomparables=NA)),
        label=paste0("bit_sumDuplicated(c(", toString(x), "))")
      )
      expect_identical(
        bit_sumDuplicated(x, na.rm=TRUE),
        sum(duplicated(x, incomparables=NA)) + sum(is.na(x)),
        label=paste0("bit_sumDuplicated(c(", toString(x), "))")
      )

    }
  }

})


test_that("bit_sort and bit_sort_unique are OK", {
  n <- 6L
  for (s in 1:10) {
    set.seed(s)
    x <- 2L * sample(n, replace=TRUE)
    for (napos in c(-1L, 0L, 1L)) {
      if (napos == -1) {
        y <- c(NA, x)
      } else if (napos == 1) {
        y <- c(x, NA)
      } else {
        y <- c(x[1:(n %/% 2)], NA, x[(n %/% 2 + 1L):n])
      }
      for (has.dup in c(TRUE, FALSE)) {
        if (!has.dup)
          y <- unique(y)
        for (na.last in c(NA, FALSE, TRUE)) {
          eval(substitute(
            expect_identical(
              bit_sort(y, na.last=na.last, has.dup = has.dup),
              sort(y, na.last = na.last)
            ),
            list(
              y=y,
              na.last=na.last,
              has.dup = has.dup
            )
          ))
          for (decreasing in c(FALSE, TRUE)) {
            eval(substitute(
              expect_identical(
                bit_sort_unique(y, decreasing=decreasing, na.last=na.last, has.dup = has.dup),
                sort(unique(y), decreasing=decreasing, na.last = na.last)
              ),
              list(
                y=y,
                decreasing=decreasing,
                na.last=na.last,
                has.dup = has.dup
              )
            ))
            range_na = range_na(y)
            env = list(
              y=y,
              decreasing=decreasing,
              na.last=na.last,
              has.dup = has.dup,
              range_na = range_na
            )
            eval(substitute(env = env, {
              expect_identical(
                bit_sort_unique(y,
                  decreasing=decreasing,
                  na.last=na.last,
                  has.dup=has.dup,
                  range_na=range_na
                ),
                sort(unique(y),
                  decreasing=decreasing,
                  na.last=na.last
                )
              )
            }))
          }
        }
      }
    }
  }
})

test_that("bit_setops work", {
  x = 1:5
  y = 2:6
  x_na = c(x, NA_integer_)
  y_na = c(y, NA_integer_)

  expect_identical(bit_setdiff(x, y), 1L)
  expect_identical(bit_setdiff(x_na, y), c(1L, NA_integer_))
  expect_identical(bit_setdiff(x, y_na), 1L)
  expect_identical(bit_setdiff(x_na, y_na), 1L)

  expect_identical(bit_symdiff(x, y), c(1L, 6L))
  expect_identical(bit_symdiff(x_na, y), c(1L, NA_integer_, 6L))
  expect_identical(bit_symdiff(x, y_na), c(1L, 6L, NA_integer_))
  expect_identical(bit_symdiff(x_na, y_na), c(1L, 6L))

  expect_identical(bit_intersect(x, y), 2:5)
  expect_identical(bit_intersect(x_na, y), 2:5)
  expect_identical(bit_intersect(x, y_na), 2:5)
  expect_identical(bit_intersect(x_na, y_na), c(2:5, NA_integer_))

  expect_identical(bit_union(x, y), 1:6)
  expect_identical(bit_union(x_na, y), c(1:5, NA_integer_, 6L))
  expect_identical(bit_union(x, y_na), c(1:6, NA_integer_))
  expect_identical(bit_union(x_na, y_na), c(1:5, NA_integer_, 6L))

  expect_false(bit_setequal(x, y))
  expect_false(bit_setequal(x_na, y))
  expect_false(bit_setequal(x, y_na))
  expect_false(bit_setequal(x_na, y_na))
  expect_true(bit_setequal(x, x))
  expect_true(bit_setequal(x_na, x_na))

  expect_identical(bit_rangediff(c(1L, 6L), c(3L, 4L)), c(1:2, 5:6))
  expect_identical(bit_rangediff(c(1L, 6L), c(3L, 4L), revx=TRUE), -(6:1))
  expect_identical(bit_rangediff(c(1L, 6L), c(3L, 4L), revy=TRUE), 1:6)
  expect_identical(bit_rangediff(c(1L, 6L), c(3L, 4L), revx=TRUE, revy=TRUE), -c(6:5, 2:1))
  expect_identical(bit_rangediff(c(6L, 1L), c(3L, 4L), revx=TRUE), -(1:6))
  expect_identical(bit_rangediff(c(6L, 1L), c(3L, 4L), revy=TRUE), 6:1)
  expect_identical(bit_rangediff(c(6L, 1L), c(3L, 4L), revx=TRUE, revy=TRUE), -c(1:2, 5:6))
})

test_that("bitsort works", {
  expect_identical(bitsort(c(2L, 0L, 1L, NA, 2L)), c(0:2, 2L))
  expect_identical(bitsort(c(2L, 0L, 1L, NA, 2L), na.last=TRUE), c(0:2, 2L, NA))
})

test_that("bit_in works", {
  x = 1:5
  y = 3:6
  x_na = c(x, NA)
  y_na = c(y, NA)

  expect_identical(bit_in(x, y), as.bit(x %in% y))
  expect_identical(bit_in(y, x), as.bit(y %in% x))
  expect_identical(bit_in(x_na, y), as.bit(x_na %in% y))
  expect_identical(bit_in(y, x), as.bit(y %in% x_na))
  expect_identical(bit_in(x, y_na), as.bit(x %in% y_na))
  expect_identical(bit_in(y_na, x), as.bit(y_na %in% x))
  expect_identical(bit_in(x_na, y_na), as.bit(x_na %in% y_na))
  expect_identical(bit_in(y_na, x_na), as.bit(y_na %in% x_na))
})

library("testthat")
library("bit")

context("bitsort")

# expect_identical <- function(x, y, ...){
#  stopifnot(identical(x,y))
# }

test_that("bitsort function work on several inputs", {
  nums <- list(
      integer()
    , as.integer(9:0)
    , as.integer(0:9)
    , as.integer(0:(-9))
    , as.integer((-9):0)
    , as.integer((-4):4)
    , as.integer(4:(-4))
  )
  nas <- list(
      integer()
    , NA_integer_
    , c(NA_integer_,NA_integer_)
  )
  set.seed(1)
  for (num in nums){
    for (na in nas){
      x <- c(num,na)
      if (length(x)>1)
        x <- sample(x)
      #dput(x)
      
      range_result <- suppressWarnings( as.integer(c( range(x, na.rm=TRUE), sum(is.na(x)) )) )
      expect_identical(
        range_na(x)
      , range_result
      , label=paste("range_na(c(", paste(x, collapse=","), "))", sep="")
      )
      
      range_result <- suppressWarnings( as.integer(c( range(x[is.na(x) | x!=0], na.rm=TRUE), sum(is.na(x)) )) )
      expect_identical(
        attr(range_nanozero(x),"range_na")
      , range_result
      , label=paste("range_na(c(", paste(x, collapse=","), ")", sep=""))
      
      for (na.last in c(NA,FALSE,TRUE)){
        expect_identical(
            {y <- range_sortna(x, na.last=na.last); y[!is.na(y)] <- 1L; y}
          , {y <- sort.int(x, na.last=na.last, method="quick"); r <- range_na(y); r[4] <- is.unsorted(x, na.rm=TRUE); y[!is.na(y)] <- 1L; attr(y, "range_sortna") <- r; y}
          , label=paste("range_sortna(c(", paste(x, collapse=","), "), na.last=", na.last,")", sep="")
        )

        expect_identical(
           bit_sort_unique(x, na.last=na.last, has.dup = TRUE)
           , sort.int(unique(x), na.last=na.last, method="quick")
           , label=paste("bit_sort_unique(c(", paste(x, collapse=","), "), na.last=", na.last,", has.dup=TRUE)", sep="")
        )
        
        y <- unique(x)
        expect_identical(
            bit_sort_unique(y, na.last=na.last, has.dup = FALSE)
          , sort.int(y, na.last=na.last, method="quick")
          , label=paste("bit_sort_unique(c(", paste(x, collapse=","), "), na.last=", na.last,", has.dup=FALSE)", sep="")
        )
        
        expect_identical(
          bit_sort(x, na.last=na.last)
        , sort.int(x, na.last=na.last, method="quick")
        , label=paste("bit_sort(c(", paste(x, collapse=","), "), na.last=", na.last,")", sep="")
        )
      }

      expect_identical(  
        bit_unique(x, na.rm=NA)
      , unique(x, incomparables=FALSE)
      , label=paste("bit_unique(c(", paste(x, collapse=","), "), na.rm=", NA, ")", sep="")
      )
      expect_identical(  
        bit_unique(x, na.rm=FALSE)
      , unique(x, incomparables=NA)
      , label=paste("bit_unique(c(", paste(x, collapse=","), "), na.rm=", FALSE, ")", sep="")
      )
      expect_identical(  
        bit_unique(x, na.rm=TRUE)
      , unique(x[!is.na(x)])
      , label=paste("bit_unique(c(", paste(x, collapse=","), "), na.rm=", TRUE, ")", sep="")
      )

      expect_identical(  
        bit_duplicated(x, na.rm=NA)
      , as.bit(duplicated(x, incomparables=FALSE))
      , label=paste("bit_duplicated(c(", paste(x, collapse=","), "), na.rm=", NA, ")", sep="")
      )
      expect_identical(  
        bit_duplicated(x, na.rm=FALSE)
      , as.bit(duplicated(x, incomparables=NA))
      , label=paste("bit_duplicated(c(", paste(x, collapse=","), "), na.rm=", FALSE, ")", sep="")
      )
      expect_identical(  
        bit_duplicated(x, na.rm=TRUE)
      , as.bit(duplicated(x) | is.na(x))
      , label=paste("bit_duplicated(c(", paste(x, collapse=","), "), na.rm=", TRUE, ")", sep="")
      )
      
      expect_identical(
        bit_anyDuplicated(x, na.rm=NA)
      , anyDuplicated(x, incomparables=FALSE)
      , label=paste("bit_anyDuplicated(c(", paste(x, collapse=","), "), na.rm=", NA, ")", sep="")
        )
      expect_identical(
        bit_anyDuplicated(x, na.rm=FALSE)
      , anyDuplicated(x, incomparables=NA)
      , label=paste("bit_anyDuplicated(c(", paste(x, collapse=","), "), na.rm=", FALSE, ")", sep="")
      )
      expect_identical(
        bit_anyDuplicated(x, na.rm=TRUE)
      , max(0L, head(which(is.na(x)|duplicated(x)), 1))
      , label=paste("bit_anyDuplicated(c(", paste(x, collapse=","), "), na.rm=", TRUE, ")", sep="")
      )

      expect_identical(
        bit_sumDuplicated(x, na.rm=NA)
      , sum(duplicated(x, incomparables=FALSE))
      , label=paste("bit_sumDuplicated(c(", paste(x, collapse=","), "))", sep="")
      )
      expect_identical(
        bit_sumDuplicated(x, na.rm=FALSE)
      , sum(duplicated(x, incomparables=NA))
      , label=paste("bit_sumDuplicated(c(", paste(x, collapse=","), "))", sep="")
      )
      expect_identical(
        bit_sumDuplicated(x, na.rm=TRUE)
      , sum(duplicated(x, incomparables=NA)) + sum(is.na(x))
      , label=paste("bit_sumDuplicated(c(", paste(x, collapse=","), "))", sep="")
      )
      
    }
  }

})
  

test_that("bit_sort and bit_sort_unique are OK", {
    n <- 6L
    for (s in 1:10){
      set.seed(s)
      x <- 2L*sample(n, replace=TRUE)
      for (napos in c(-1L,0L,1L)){
        if (napos == -1){
          y <- c(NA,x)
        } else if (napos == 1){
          y <- c(x,NA)
        } else {
          y <- c(x[1:(n%/%2)],NA,x[(n%/%2+1L):n])
        }
        for (has.dup in c(TRUE,FALSE)){
          if (!has.dup)
            y <- unique(y)
        for (na.last in c(NA, FALSE, TRUE)){
          eval(
            substitute(
              expect_identical(
                bit_sort(y, na.last=na.last, has.dup = has.dup)
              , sort(y, na.last = na.last)
              )
              , list(
                  y=y
                , na.last=na.last
                , has.dup = has.dup
              )
            )
          )
         for (decreasing in c(FALSE, TRUE)){
              eval(
              substitute(
                expect_identical(
                    bit_sort_unique(y, decreasing=decreasing, na.last=na.last, has.dup = has.dup)
                  , sort(unique(y), decreasing=decreasing, na.last = na.last)
                )
                , list(
                    y=y
                  , decreasing=decreasing
                  , na.last=na.last
                  , has.dup = has.dup
                )
              )
            )
            range_na <- range_na(y)
            eval(
              substitute(
                expect_identical(
                  bit_sort_unique(y, decreasing=decreasing, na.last=na.last, has.dup = has.dup, range_na=range_na)
                  , sort(unique(y), decreasing=decreasing, na.last = na.last)
                )
                , list(
                  y=y
                  , decreasing=decreasing
                  , na.last=na.last
                  , has.dup = has.dup
                  , range_na = range_na
                )
              )
            )
          }
        }
      }
      }
    }
  })

# Package intialization
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

#' @useDynLib bit, .registration = TRUE, .fixes = "C_"
#' @importFrom utils packageDescription
.onLoad <- function(lib, pkg) {
  bit_init()
}

.onUnload <- function(libpath) {
  bit_done()
  library.dynam.unload("bit", libpath)
}

# Fast methods for sorted integers
# (c) 2016-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk


# Attention: 
#   must not use as.integer() on x and y here, otherwise we cannot pass bitwhich objects
#   (as.integer) would turn positions into 0/1

#' Fast functions for sorted sets of integer
#' 
#' The \code{merge_} functions allow unary and binary operations 
#' on (ascending) sorted vectors of \code{link{integer}}. 
#' \code{merge_rev(x)} will do in one scan what costs two scans in \code{-\link{rev}(x)}, see also \code{\link{reverse_vector}(x)}.
#' Many of these \code{merge_} can optionally scan their input in reverse order (and switch the sign), 
#' which again saves extra scans for calling \code{merge_rev(x)} first. 
#' 
#' @details These are low-level functions and hence do not check whether the set is actually sorted. 
#' Note that the `merge_*` and `merge_range*` functions have no special treatment for `NA`. 
#' If vectors with `NA` are sorted ith `NA` in the first positions (`na.last=FALSE`) and arguments `revx=` or `revy=` have not been used, 
#' then `NAs` are treated like ordinary integers. 
#' `NA` sorted elsewhere or using `revx=` or `revy=` can cause unexpected results 
#' (note for example that `revx=` switches the sign on all integers but `NAs`). 
#' \cr
#' \cr
#' The *binary* `merge_*` functions have a `method="exact"` 
#' which in both sets treats consecutive occurrences of the same value as if they were different values,
#' more precisely they are handled as if the identity of ties were tuples of \code{ties, rank(ties)}. 
#' \code{method="exact"} delivers unique output if the input is unique, and in this case works faster than \code{method="unique"}.
#' 
#' @note xx OPTIMIZATION OPPORTUNITY These are low-level functions could be optimized with initial binary search (not findInterval, which coerces to double).
#'
#' @param x a sorted set
#' @param rx range of integers given as \code{\link{ri}} or as a two-element \code{\link{integer}}
#' @param y a sorted set
#' @param revx default \code{FALSE}, set to \code{TRUE} to reverse scan parameter 'x'
#' @param revy default \code{FALSE}, set to \code{TRUE} to reverse scan parameter 'y'
#' @param nomatch integer value returned for non-matched elements, see \code{\link{match}}
#' @param method one of "unique", "exact" (or "all") which governs how to treat ties, see the function descriptions 
#'
#' @return \code{merge_rev(x)} returns \code{-\link{rev}(x)} for \code{\link{integer}} and \code{\link{double}} and \code{!\link{rev}(x)} for \code{\link{logical}}
#'
#' @examples
#' merge_rev(1:9)
#' 
#' merge_match(1:7, 3:9)
#' #' merge_match(merge_rev(1:7), 3:9)
#' merge_match(merge_rev(1:7), 3:9, revx=TRUE)
#' merge_match(merge_rev(1:7), 3:9, revy=TRUE)
#' merge_match(merge_rev(1:7), merge_rev(3:9))
#' 
#' merge_in(1:7, 3:9)
#' merge_notin(1:7, 3:9)
#' 
#' merge_anyDuplicated(c(1L,1L,2L,3L))
#' merge_duplicated(c(1L,1L,2L,3L))
#' merge_unique(c(1L,1L,2L,3L))
#' 
#' merge_union(c(1L,2L,2L,2L), c(2L,2L,3L))
#' merge_union(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
#' merge_union(c(1L,2L,2L,2L), c(2L,2L,3L), method="all")
#' 
#' merge_setdiff(c(1L,2L,2L,2L), c(2L,2L,3L))
#' merge_setdiff(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
#' merge_setdiff(c(1L,2L,2L), c(2L,2L,2L,3L), method="exact")
#' 
#' merge_symdiff(c(1L,2L,2L,2L), c(2L,2L,3L))
#' merge_symdiff(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
#' merge_symdiff(c(1L,2L,2L), c(2L,2L,2L,3L), method="exact")
#' 
#' merge_intersect(c(1L,2L,2L,2L), c(2L,2L,3L))
#' merge_intersect(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
#' 
#' merge_setequal(c(1L,2L,2L), c(1L,2L))
#' merge_setequal(c(1L,2L,2L), c(1L,2L,2L))
#' merge_setequal(c(1L,2L,2L), c(1L,2L), method="exact")
#' merge_setequal(c(1L,2L,2L), c(1L,2L,2L), method="exact")
#' 
#' @export

merge_rev <- function(x){
  .Call(C_R_merge_rev, x)
}

#' @describeIn merge_rev returns integer positions of sorted set x in sorted set y, see \code{\link{match}(x, y, ...)}
#' @export
merge_match <- function(x, y, revx=FALSE, revy=FALSE, nomatch = NA_integer_){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_match, x, y, as.logical(revx), as.logical(revy), nomatch=as.integer(nomatch))
}

# xx OPTIMIZATION OPPORTUNITY this could be optimized with proper binary search (not findInterval, which coerces to double)
#' @describeIn merge_rev returns logical existence of sorted set x in sorted set y, see \code{x \link{\%in\%} y}
#' @export
merge_in <- function(x, y, revx=FALSE, revy=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_in, x, y, as.logical(revx), as.logical(revy))
}

# xx OPTIMIZATION OPPORTUNITY this could be optimized with proper binary search (not findInterval, which coerces to double)
#' @describeIn merge_rev returns logical in-existence of sorted set x in sorted set y, see \code{!(x \link{\%in\%} y)}
#' @export
merge_notin <- function(x, y, revx=FALSE, revy=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_notin, x, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns the duplicated status of a sorted set x, see \code{\link{duplicated}}
#' @export
merge_duplicated <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_duplicated, x, as.logical(revx))
}

#' @describeIn merge_rev returns the anyDuplicated status of a sorted set x, see \code{\link{anyDuplicated}}
#' @export
merge_anyDuplicated <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_anyDuplicated, x, as.logical(revx))
}

#' @describeIn merge_rev returns the sumDuplicated status of a sorted set x, see \code{\link{bit_sumDuplicated}}
#' @export
merge_sumDuplicated <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_sumDuplicated, x, as.logical(revx))
}
#' @describeIn merge_rev returns unique elements of sorted set x, see \code{\link{unique}}
#' @export
merge_unique <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_unique, x, as.logical(revx))
}

#' @describeIn merge_rev returns union of two sorted sets.
#' Default \code{method='unique'} returns a unique sorted set, see \code{\link{union}}; 
#' \code{method='exact'} returns a sorted set with the maximum number of ties in either input set;
#' \code{method='all'} returns a sorted set with the sum of ties in both input sets.
#' @export
merge_union <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique","exact","all")){
  method <- match.arg(method)
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_union, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns sorted set x minus sorted set y
#' Default \code{method='unique'} returns a unique sorted set, see \code{\link{setdiff}}; 
#' \code{ethod='exact'} returns a sorted set with sum(x ties) minus sum(y ties);
#' @export
merge_setdiff <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique","exact")){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_setdiff, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns those elements that are in sorted set \code{y} \code{\link{xor}} in sorted set \code{y}
#' Default \code{method='unique'} returns the sorted unique set complement, see \code{\link{symdiff}}; 
#' \code{method='exact'} returns a sorted set set complement with abs(sum(x ties) minus sum(y ties));
#' @export
merge_symdiff <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique","exact")){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_symdiff, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns the intersection of two sorted sets x and y
#' Default \code{method='unique'} returns the sorted unique intersect, see \code{\link{intersect}}; 
#' \code{method='exact'} returns the intersect with the minium number of ties in either set;
#' @export
merge_intersect <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique","exact")){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_intersect, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns \code{TRUE} for equal sorted sets and \code{FALSE} otherwise
#' Default \code{method='unique'} compares the sets after removing ties, see \code{\link{setequal}};  
#' \code{method='exact'} compares the sets without removing ties;
#' @export
merge_setequal <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique","exact")){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_setequal, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns logical existence of range rx in sorted set y, see \code{\link{merge_in}}
#' @export
merge_rangein <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangein, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns logical in-existence of range rx in sorted set y, see \code{\link{merge_notin}}
#' @export
merge_rangenotin <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangenotin, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns the intersection of range rx and sorted set y, see \code{\link{merge_intersect}}
#' @export
merge_rangesect <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangesect, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns range rx minus sorted set y, see \code{\link{merge_setdiff}}
#' @export
merge_rangediff <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangediff, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev quickly returns the first element of a sorted set x (or \code{NA} if x is empty), hence \code{x[1]} or \code{merge_rev(x)[1]}
#' @export
merge_first <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_first, x, as.logical(revx))
}
#' @describeIn merge_rev quickly returns the last element of a sorted set x, (or \code{NA} if x is empty), hence \code{x[n]} or \code{merge_rev(x)[n]}
#' @export
merge_last <- function(x, revx=FALSE){
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_last, x, as.logical(revx))
}

#' @describeIn merge_rev quickly returns the first common element of a range rx and a sorted set y, (or \code{NA} if the intersection is empty), hence \code{merge_first(merge_rangesect(rx,y))}
#' @export
merge_firstin <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_firstin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the last common element of a range rx and a sorted set y, (or \code{NA} if the intersection is empty), hence \code{merge_last(merge_rangesect(rx,y))}
#' @export
merge_lastin <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_lastin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the first element of a range rx which is not in a sorted set y (or \code{NA} if all rx are in y), hence \code{merge_first(merge_rangediff(rx,y))}
#' @export
merge_firstnotin <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_firstnotin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the last element of a range rx which is not in a sorted set y (or \code{NA} if all rx are in y), hence \code{merge_last(merge_rangediff(rx,y))}
#' @export
merge_lastnotin <- function(rx, y, revx=FALSE, revy=FALSE){
  if (!is.ri(rx)){
    stopifnot(length(rx)==2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_lastnotin, rx, y, as.logical(revx), as.logical(revy))
}



#' Symmetric set complement
#'
#' @param x a vector
#' @param y a vector
#' @return \code{union(setdiff(x,y), setdiff(y,x))}
#' @seealso \code{\link{merge_symdiff}} and \code{\link{xor}}
#' @note that \code{symdiff(x,y)} is not \code{\link{identical}} 
#' as \code{symdiff(y,x)} without applying \code{\link{sort}} to the result
#' @examples
#' symdiff(c(1L,2L,2L), c(2L,3L))
#' symdiff(c(2L,3L), c(1L,2L,2L))
#' @export
symdiff <- function(x,y){
  union(setdiff(x,y), setdiff(y,x))  
}

# map NAs to position 1
NAtab <- function(x, nbins=max(1L, x, na.rm = TRUE)) {
  force(nbins)
  x <- x + 1L
  x[is.na(x)] <- 1L
  tabulate(x, nbins + 1L)
}

test_that("positive merging is OK with NAs", {
  xs <- list(
    rep(3:7, rep(1L, 5L)),
    rep(3:7, rep(2L, 5L)),
    rep(c(3L, 5L, 7L), rep(1L, 3L)),
    rep(c(3L, 5L, 7L), rep(2L, 3L))
  )
  ys <- list(
    rep(1:9, rep(1L, 9L)),
    rep(1:9, rep(2L, 9L)),
    rep(2:8, rep(1L, 7L)),
    rep(2:8, rep(2L, 7L)),
    rep(3:7, rep(1L, 5L)),
    rep(3:7, rep(2L, 5L)),
    rep(c(3L, 5L, 7L), rep(1L, 3L)),
    rep(c(3L, 5L, 7L), rep(2L, 3L)),
    rep(4:6, rep(1L, 3L)),
    rep(4:6, rep(2L, 3L)),
    rep(5L, 1L),
    rep(5L, 2L),
    rep(5L, 3L),
    rep(1L, 1L),
    rep(1L, 1L),
    rep(1L, 2L),
    rep(1:2, rep(1L, 2L)),
    rep(1:2, rep(2L, 2L)),
    rep(1:3, rep(1L, 3L)),
    rep(1:3, rep(2L, 3L)),
    rep(1:4, rep(1L, 4L)),
    rep(1:4, rep(2L, 4L)),
    rep(1:5, rep(1L, 5L)),
    rep(1:5, rep(2L, 5L)),
    rep(5:9, rep(1L, 5L)),
    rep(5:9, rep(2L, 5L)),
    rep(6:9, rep(1L, 4L)),
    rep(6:9, rep(2L, 4L)),
    rep(7:9, rep(1L, 3L)),
    rep(7:9, rep(2L, 3L)),
    rep(8:9, rep(1L, 2L)),
    rep(8:9, rep(2L, 2L)),
    rep(9L, 1L),
    rep(9L, 1L),
    rep(9L, 2L)
  )
  for (xi in seq_along(xs)) {
    for (yi in seq_along(ys)) {
      x <- sort.int(xs[[xi]], na.last=FALSE, method="quick")
      y <- sort.int(ys[[yi]], na.last=FALSE, method="quick")
      env = list(x=x, y=y)
      eval(substitute(env=env, expect_identical(
        merge_union(x, y, method="all"),
        sort.int(c(x, y), na.last=FALSE, method="quick")
      )))
      eval(substitute(env=env, expect_identical(
        merge_union(x, y, method="exact"),
        rep(c(NA, 1:9), pmax(NAtab(x, 9), NAtab(y, 9)))
      )))
      eval(substitute(env=env, expect_identical(
        merge_intersect(x, y, method="exact"),
        rep(c(NA, 1:9), pmin(NAtab(x, 9), NAtab(y, 9)))
      )))
      eval(substitute(env=env, expect_identical(
        merge_setdiff(x, y, method="exact"),
        rep(c(NA, 1:9), pmax(0L, NAtab(x, 9) - NAtab(y, 9)))
      )))
      eval(substitute(env=env, expect_identical(
        merge_symdiff(x, y, method="exact"),
        rep(c(NA, 1:9), abs(NAtab(x, 9) - NAtab(y, 9)))
      )))
      eval(substitute(env=env, expect_identical(
        merge_union(x, y),
        sort.int(union(x, y), na.last=FALSE, method="quick")
      )))
      eval(substitute(env=env, expect_identical(
        merge_intersect(x, y),
        sort.int(intersect(x, y), na.last=FALSE, method="quick")
      )))
      eval(substitute(env=env, expect_identical(
        merge_setdiff(x, y),
        sort.int(setdiff(x, y), na.last=FALSE, method="quick")
      )))
      eval(substitute(env=env, expect_identical(
        merge_symdiff(x, y),
        sort.int(union(setdiff(x, y), setdiff(y, x)), na.last=FALSE, method="quick")
      )))
    }
  }
})


test_that("reversed merging is OK (without NAs)", {
  set.seed(1)
  for (i in 1:24) {
    x <- sort.int(sample(9, replace=TRUE), na.last=FALSE, method="quick")
    y <- sort.int(sample(9, replace=TRUE), na.last=FALSE, method="quick")
    for (revx in c(FALSE, TRUE)) {
      if (revx)
        rx <- rev(-x)
      else
        rx <- x
      for (revy in c(FALSE, TRUE)) {
        if (revy)
          ry <- rev(-y)
        else
          ry <- y
        eval(substitute(expect_identical(
          merge_union(rx, ry, revx=revx, revy=revy, method="all")
          , sort.int(c(x, y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_union(rx, ry, revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmax(NAtab(x, 9), NAtab(y, 9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_intersect(rx, ry, revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmin(NAtab(x, 9), NAtab(y, 9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_setdiff(rx, ry, revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmax(0L, NAtab(x, 9) - NAtab(y, 9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_symdiff(rx, ry, revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), abs(NAtab(x, 9) - NAtab(y, 9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_union(rx, ry, revx=revx, revy=revy)
          , sort.int(union(x, y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_intersect(rx, ry, revx=revx, revy=revy)
          , sort.int(intersect(x, y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_setdiff(rx, ry, revx=revx, revy=revy)
          , sort.int(setdiff(x, y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_symdiff(rx, ry, revx=revx, revy=revy)
          , sort.int(union(setdiff(x, y), setdiff(y, x)), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))

      }
    }
  }

})


test_that("for-looped merging is OK (without NAs)", {
  nx <- 9
  x <- 1:nx
  set.seed(1)
  for (i in 1:12) {
    y <- sort.int(sample(nx, replace=TRUE), na.last=FALSE, method="quick")
    for (revx in c(FALSE, TRUE)) {
      if (revx) {
        rx <- rev(-x)
        rnx <- c(-nx, -1L)
      } else {
        rx <- x
        rnx <- c(1L, nx)
      }
      for (revy in c(FALSE, TRUE)) {
        if (revy) {
          ry <- rev(-y)
        } else {
          ry <- y
        }
        eval(substitute(expect_identical(
          merge_rangesect(rnx, ry, revx=revx, revy=revy)
          , merge_intersect(rx, ry, revx=revx, revy=revy)
        ), list(rnx=rnx, rx=rx, ry=ry, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangediff(rnx, ry, revx=revx, revy=revy)
          , merge_setdiff(rx, ry, revx=revx, revy=revy)
        ), list(rnx=rnx, rx=rx, ry=ry, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangein(rnx, ry, revx=revx, revy=revy)
          , copy_vector(rx, revx=revx) %in% copy_vector(ry, revx=revy)
        ), list(rnx=rnx, rx=rx, ry=ry, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangenotin(rnx, ry, revx=revx, revy=revy)
          , !(copy_vector(rx, revx=revx) %in% copy_vector(ry, revx=revy))
        ), list(rnx=rnx, rx=rx, ry=ry, revx=revx, revy=revy)))
      }
    }
  }

})

library("testthat")
library("bit")

context("merge")

# expect_identical <- function(x, y, ...){
#  stopifnot(identical(x,y))
# }

# map NAs to position 1
NAtab <- function(x, nbins=max(1L, x, na.rm = TRUE)){
  force(nbins)
  x <- x + 1L
  x[is.na(x)] <- 1L
  tabulate(x, nbins+1L)
}

test_that("positive merging is OK with NAs", {
  xs <- lapply(list(
      rep(3:7, rep(1,5))
    , rep(3:7, rep(2,5))
    , rep(c(3,5,7), rep(1,3))
    , rep(c(3,5,7), rep(2,3))
  ), as.integer)
  ys <- lapply(list(
      rep(1:9, rep(1,9))
    , rep(1:9, rep(2,9))
    , rep(2:8, rep(1,7))
    , rep(2:8, rep(2,7))
    , rep(3:7, rep(1,5))
    , rep(3:7, rep(2,5))
    , rep(c(3,5,7), rep(1,3))
    , rep(c(3,5,7), rep(2,3))
    , rep(4:6, rep(1,3))
    , rep(4:6, rep(2,3))
    , rep(5L, 1)
    , rep(5L, 2)
    , rep(5L, 3)
    , rep(1L, 1)
    , rep(1L, 1)
    , rep(1L, 2)
    , rep(1:2, rep(1,2))
    , rep(1:2, rep(2,2))
    , rep(1:3, rep(1,3))
    , rep(1:3, rep(2,3))
    , rep(1:4, rep(1,4))
    , rep(1:4, rep(2,4))
    , rep(1:5, rep(1,5))
    , rep(1:5, rep(2,5))
    , rep(5:9, rep(1,5))
    , rep(5:9, rep(2,5))
    , rep(6:9, rep(1,4))
    , rep(6:9, rep(2,4))
    , rep(7:9, rep(1,3))
    , rep(7:9, rep(2,3))
    , rep(8:9, rep(1,2))
    , rep(8:9, rep(2,2))
    , rep(9L, 1)
    , rep(9L, 1)
    , rep(9L, 2)
  ), as.integer)
  for (xi in seq_along(xs))
    for (yi in seq_along(ys)){
        x <- sort.int(xs[[xi]], na.last=FALSE, method="quick")
        y <- sort.int(ys[[yi]], na.last=FALSE, method="quick")
        eval(substitute(expect_identical(
            merge_union(x,y, method="all")
          , sort.int(c(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_union(x,y, method="exact")
        , rep(c(NA, 1:9), pmax(NAtab(x,9), NAtab(y,9)))
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_intersect(x,y, method="exact")
          , rep(c(NA, 1:9), pmin(NAtab(x,9), NAtab(y,9)))
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_setdiff(x,y, method="exact")
          , rep(c(NA, 1:9), pmax(0L, NAtab(x,9) - NAtab(y,9)))
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_symdiff(x,y, method="exact")
          , rep(c(NA, 1:9), abs(NAtab(x,9) - NAtab(y,9)))
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_union(x,y)
          , sort.int(union(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_intersect(x,y)
          , sort.int(intersect(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_setdiff(x,y)
          , sort.int(setdiff(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y)))
        eval(substitute(expect_identical(
          merge_symdiff(x,y)
          , sort.int(union(setdiff(x,y),setdiff(y,x)), na.last=FALSE, method="quick")
        ), list(x=x, y=y)))
    }
})


test_that("reversed merging is OK (without NAs)", {
  set.seed(1)
  for (i in 1:24){
    x <- sort.int(sample(9, replace=TRUE), na.last=FALSE, method="quick")
    y <- sort.int(sample(9, replace=TRUE), na.last=FALSE, method="quick")
    for (revx in c(FALSE,TRUE)){
      if (revx)
        rx <- rev(-x)
      else
        rx <- x
      for (revy in c(FALSE,TRUE)){
        if (revy)
          ry <- rev(-y)
        else
          ry <- y
        eval(substitute(expect_identical(
          merge_union(rx,ry,revx=revx, revy=revy, method="all")
          , sort.int(c(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_union(rx,ry,revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmax(NAtab(x,9), NAtab(y,9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_intersect(rx,ry,revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmin(NAtab(x,9), NAtab(y,9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_setdiff(rx,ry,revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), pmax(0L, NAtab(x,9) - NAtab(y,9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_symdiff(rx,ry,revx=revx, revy=revy, method="exact")
          , rep(c(NA, 1:9), abs(NAtab(x,9) - NAtab(y,9)))
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_union(rx,ry,revx=revx, revy=revy)
          , sort.int(union(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_intersect(rx,ry,revx=revx, revy=revy)
          , sort.int(intersect(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_setdiff(rx,ry,revx=revx, revy=revy)
          , sort.int(setdiff(x,y), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_symdiff(rx,ry,revx=revx, revy=revy)
          , sort.int(union(setdiff(x,y),setdiff(y,x)), na.last=FALSE, method="quick")
        ), list(x=x, y=y, revx=revx, revy=revy)))
        
      }
    }
  }
  
})


test_that("for-looped merging is OK (without NAs)", {
  nx <- 9
  x <- 1:nx
  set.seed(1)
  for (i in 1:12){
    y <- sort.int(sample(nx, replace=TRUE), na.last=FALSE, method="quick")
    for (revx in c(FALSE,TRUE)){
      if (revx){
        rx <- rev(-x)
        rnx <- c(-nx, -1L)
      }else{
        rx <- x
        rnx <- c(1L,nx)
      }
      for (revy in c(FALSE,TRUE)){
        if (revy){
          ry <- rev(-y)
        }else{
          ry <- y
        }
        eval(substitute(expect_identical(
          merge_rangesect(rnx,ry,revx=revx, revy=revy)
          , merge_intersect(rx,ry,revx=revx, revy=revy)
        ), list(rnx=rnx, rx=rx, ry=ry,revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangediff(rnx,ry,revx=revx, revy=revy)
          , merge_setdiff(rx,ry,revx=revx, revy=revy)
        ), list(rnx=rnx, rx=rx, ry=ry,revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangein(rnx,ry,revx=revx, revy=revy)
          , copy_vector(rx,revx=revx) %in% copy_vector(ry, revx=revy)
        ), list(rnx=rnx, rx=rx, ry=ry,revx=revx, revy=revy)))
        eval(substitute(expect_identical(
          merge_rangenotin(rnx,ry,revx=revx, revy=revy)
          , ! ( copy_vector(rx,revx=revx) %in% copy_vector(ry, revx=revy) )
        ), list(rnx=rnx, rx=rx, ry=ry,revx=revx, revy=revy)))
      }
    }
  }
  
})

# Functions for boolean vectors
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

# source("C:/mwp/eanalysis/bit/R/bit.R")

# Configuration: set this to 32L or 64L and keep in sync with BITS in bit.c
#' @rdname bit_init
#' @export
.BITS <- 32L

#' Initializing bit masks
#'
#' Functions to allocate (and de-allocate) bit masks
#'
#' The C-code operates with bit masks.  The memory for these is allocated
#' dynamically.  `bit_init` is called by [.First.lib()] and
#' `bit_done` is called by [.Last.lib()].  You don't need to
#' care about these under normal circumstances.
#'
#' @return NULL
#' @author Jens Oehlschlägel
#' @seealso [bit()]
#' @keywords classes logic
#' @examples
#'
#'   bit_done()
#'   bit_init()
#'
#' @export
bit_init <- function() .Call(C_R_bit_init, .BITS)

#' @rdname bit_init
#' @export
bit_done <- function() .Call(C_R_bit_done)

#' Create empty bit vector
#'
#' Bit vectors are a boolean type wihout `NA` that requires by factor 32 less
#'   RAM than [logical()].
#' For details on usage see `vignette("bit-usage")` and for details on
#'   performance see `vignette("bit-performance")`.
#'
#' @param length length in bits
#' @return `bit` returns a vector of integer sufficiently long to store 'length' bits
#' @seealso  [booltype()], [bitwhich()], [logical()]
#' @keywords classes logic
#' @examples
#' bit(12)
#' !bit(12)
#' str(bit(128))
#' @export
bit <- function(length=0L) {
  length <- as.integer(length)
  if (length %% .BITS)
    n <- length %/% .BITS + 1L
  else
    n <- length %/% .BITS
  if (.BITS == 64L)
    x <- integer(2L * n)
  else
    x <- integer(n)
  #physical(x) <- list(vmode="boolean")
  #virtual(x)  <- list(Length=length)
  #class(x) <- "bit"
  # tuning
  p <- list()
  v <- list()
  attributes(p) <- list(vmode="boolean", class="physical")
  attributes(v) <- list(Length=length, class="virtual")
  setattributes(x, list(physical=p, virtual=v, class=c("booltype", "bit")))
  x
}



#' Print method for bit
#'
#' @param x a bit vector
#' @param ... passed to print
#' @return a character vector showing first and last elements of the bit vector
#' @examples
#' print(bit(120))
#' @export
print.bit <- function(x, ...) {
  n <- length(x)
  cat("bit length=", n, " occupying only ", length(unclass(x)), " int32\n", sep="")
  if (n > 16) {
    y <- c(x[1:8], "..", x[(n - 7L):n])
    names(y) <- c(1:8, "", (n - 7L):n)
    print(y, quote=FALSE, ...)
  } else if (n) {
    y <- c(x[])
    names(y) <- 1:n
    print(y, quote=FALSE, ...)
  }
}

#' Coerce bit to character
#'
#' @param x a [bit()] vector
#' @param ... ignored
#' @return a character vector of zeroes and ones
#' @examples
#' as.character(bit(12))
#' @export
as.character.bit <- function(x, ...) {
  c("0", "1")[1 + as.logical(x)]
}


#' Str method for bit
#'
#' To actually view the internal structure use `str(unclass(bit))`
#'
#' @inheritParams utils::str
#' @return [invisible()]
#' @importFrom utils strOptions
#' @examples
#' str(bit(120))
#' @export
str.bit <- function(
  object,
  vec.len  = strO$vec.len,
  give.head = TRUE,
  give.length = give.head,
  ...
) {
  strO <- strOptions()
  vec.len <- 8 * vec.len
  n <- length(object)
  if (n > vec.len)
    object <- as.bit(object[seq_len(vec.len)])
  if (give.head) {
    cat("bit ")
    if (give.length && n > 1L) cat(sprintf(" [1:%d] ", n))
  }
  cat(paste(as.character(object), collapse=" "))
  cat("\n")
  invisible()
}


#' Coerce bitwhich to character
#'
#' @param x a [bitwhich()] vector
#' @param ... ignored
#' @return a character vector of zeroes and ones
#' @examples
#' as.character(bitwhich(12))
#' @export
as.character.bitwhich <- function(x, ...) c("0", "1")[1 + as.logical(x)]


#' Str method for bitwhich
#'
#' To actually view the internal structure use `str(unclass(bitwhich))`
#'
#' @inheritParams utils::str
#' @return [invisible()]
#' @examples
#' str(bitwhich(120))
#' @export
str.bitwhich <- function(
  object,
  vec.len  = strO$vec.len,
  give.head = TRUE,
  give.length = give.head,
  ...
) {
  strO <- strOptions()
  vec.len <- 8 * vec.len
  n <- length(object)
  if (n > vec.len)
    object <- as.bitwhich(object[seq_len(vec.len)])
  if (give.head) {
    cat("bitwhich ")
    if (give.length && n > 1L) cat(sprintf(" [1:%d] ", n))
  }
  cat(paste(as.character(object), collapse = " "))
  cat("\n")
  invisible()
}



#' Create bitwhich vector (skewed boolean)
#'
#' A bitwhich object represents a boolean filter like a [bit()] object (NAs are not
#'   allowed) but uses a sparse representation suitable for very skewed (asymmetric)
#'   selections. Three extreme cases are represented with logical values, no length via
#'   `logical()`, all `TRUE` with `TRUE` and all `FALSE` with `FALSE`. All other
#'   selections are represented with positive or negative integers, whatever is shorter.
#' This needs less RAM compared to [logical()] (and often less than [bit()] or
#'   [`which()`][as.which]). Logical operations are fast if the selection is asymmetric
#'   (only few or almost all selected).
#'
#' @param maxindex length of the vector
#' @param x Information about which positions are `FALSE` or `TRUE`: either `logical()` or
#'   `TRUE` or `FALSE` or a integer vector of positive or of negative subscripts.
#' @param xempty what to assume about parameter `x` if `x=integer(0)`, typically `TRUE`
#'   or `FALSE`.
#' @param poslength tuning: `poslength` is calculated automatically, you can give
#'   `poslength` explicitly, in this case it must be correct and `x` must be sorted and
#'   not have duplicates.
#' @param is.unsorted tuning: FALSE implies that `x` is already sorted and sorting
#'   is skipped
#' @param has.dup tuning: FALSE implies that `x` has no duplicates
#' @return an object of class 'bitwhich' carrying two attributes
#'    - maxindex: see above
#'    - poslength: see above
#' @seealso [bitwhich_representation()],  [as.bitwhich()], [bit()]
#' @examples
#' bitwhich()
#' bitwhich(12)
#' bitwhich(12, x=TRUE)
#' bitwhich(12, x=3)
#' bitwhich(12, x=-3)
#' bitwhich(12, x=integer())
#' bitwhich(12, x=integer(), xempty=TRUE)
#' @export
bitwhich <- function(
  maxindex=0L,
  x=NULL,
  xempty=FALSE,
  poslength=NULL,
  is.unsorted=TRUE,
  has.dup=TRUE
) {
  maxindex <- as.integer(maxindex)
  if (maxindex == 0L) {
    if ((!is.null(poslength) && poslength) || (length(x) && (!is.logical(x) || x[[1]])))
      stop("maxindex=0 given with poslength or x")
    poslength <- 0L
    ret <- logical()
  } else {
    stopifnot(maxindex > 0L)
    if (length(x)) {
      if (is.logical(x)) {
        if (length(x) != 1L || is.na(x)) {
          stop("logical x should be scalar FALSE or TRUE")
        } else if (x) {
          if (is.null(poslength))
            poslength <- maxindex
          else if (poslength != maxindex)
            stop("x == TRUE implies poslength == maxindex")
          ret <- copy_vector(TRUE)
        } else {
          if (is.null(poslength))
            poslength <- 0L
          else if (poslength != 0L)
            stop("x == FALSE implies poslength == 0")
          ret <- copy_vector(FALSE)
        }
      } else {
        x <- as.integer(x)
        if (is.null(poslength)) {
          ret <- range_nanozero(x)
          r <- getsetattr(ret, "range_na", NULL)
          if (r[3] > 0L)
            stop("NA positions not allowed (neither positive nor negative)")
          if (r[1] < 0L) {
            if (r[2] > 0L)
              stop("mixed negative and positive subscripts not allowed")
            if (-r[1] > maxindex)
              stop("index value outside -maxindex..-1")
          } else if (r[2] > maxindex) {
            stop("index value outside 1..maxindex")
          }
          if (is.unsorted)
            ret <- bit_sort_unique(ret, na.last=NA, range_na=r)
          else if (has.dup)
            ret <- bit_unique(ret, na.rm = FALSE, range_na=r)
          if (ret[1] < 0) {
            poslength <- maxindex - length(ret)
            if (poslength) {
              if (poslength <= maxindex %/% 2L)
                ret <- merge_rangediff(c(1L, maxindex), ret, revx=FALSE, revy=TRUE)
            } else {
              ret <- copy_vector(FALSE)
            }
          } else {
            poslength <- length(ret)
            if (poslength < maxindex) {
              if (poslength > maxindex %/% 2L)
                ret <- merge_rangediff(c(1L, maxindex), ret, revx=TRUE, revy=TRUE)
            } else {
              ret <- copy_vector(TRUE)
            }
          }
        } else {
          poslength <- as.integer(poslength)
          if (poslength == 0L) {
            ret <- copy_vector(FALSE)
          } else if (poslength == maxindex) {
            ret <- copy_vector(TRUE)
          } else {
            if (length(x) > 2 && x[1] >= x[2])
              stop("x is not sorted unique")
            if (x[1] < 0L) {
              if (poslength != maxindex - length(x))
                stop("wrong poslength")
              if (poslength <= maxindex %/% 2L)
                ret <- merge_rangediff(c(1L, maxindex), x, revx=FALSE, revy=TRUE)
              else
                ret <- copy_vector(x)
            } else {
              if (poslength != length(x))
                stop("wrong poslength")
              if (poslength > maxindex %/% 2L)
                ret <- merge_rangediff(c(1L, maxindex), x, revx=TRUE, revy=TRUE)
              else
                ret <- copy_vector(x)
            }

          }
        }
      }
    } else if (is.null(poslength)) {
      if (!is.logical(xempty) || length(xempty) != 1 || is.na(xempty))
        stop("xempty must be FALSE or TRUE")
      if (xempty)
        poslength <- maxindex
      else
        poslength <- 0L
      ret <- copy_vector(xempty)
    } else {
      poslength <- as.integer(poslength)
      if (poslength == 0)
        ret <- copy_vector(FALSE)
      else if (poslength == maxindex)
        ret <- copy_vector(TRUE)
      else
        stop("need x with extreme poslength")
    }
  }
  setattributes(ret,
    list(maxindex = maxindex, poslength = poslength, class = c("booltype", "bitwhich"))
  )
  ret
}

#' Diagnose representation of bitwhich
#'
#' @param x a [bitwhich()] object
#' @return a scalar, one of `logical()`, `FALSE`, `TRUE`, `-1` or `1`
#' @examples
#' bitwhich_representation(bitwhich())
#' bitwhich_representation(bitwhich(12, FALSE))
#' bitwhich_representation(bitwhich(12, TRUE))
#' bitwhich_representation(bitwhich(12, -3))
#' bitwhich_representation(bitwhich(12, 3))
#' @export
bitwhich_representation <- function(x) .Call(C_R_bitwhich_representation, x)

#' Print method for bitwhich
#'
#' @param x a [bitwhich()] object
#' @param ... ignored
#' @export
print.bitwhich <- function(x, ...) {
  n <- length(x)
  cat(sprintf(
    "bitwhich: %d/ %d occupying only % d int32 in %s representation\n",
    sum(x), n, length(unclass(x)), bitwhich_representation(x)
  ))
  if (n > 16) {
    y <- c(x[1:8], "..", x[(n - 7L):n])
    names(y) <- c(1:8, "", (n - 7L):n)
    print(y, quote=FALSE, ...)
  } else if (n) {
    y <- c(x[])
    names(y) <- 1:n
    print(y, quote=FALSE, ...)
  }
}


#' Boolean types
#'
#' The [ordered()] factor `booltypes` ranks the boolean types.
#'
#' There are currently six boolean types, `booltypes` is an [ordered()] vector with the
#'   following ordinal [levels()]:
#'
#'  - nobool: non-boolean type
#'  - [logical()]: for representing any boolean data including `NA`
#'  - [bit()]: for representing dense boolean data
#'  - [bitwhich()]: for representing sparse (skewed) boolean data
#'  - [which()]: for representing sparse boolean data with few `TRUE
#   - [`hi`][ff::hi]: hybrid-indexing, implemented in package [`ff`][ff::ff]
#'  - [ri()]: range-indexing, for representing sparse boolean data with a single range of
#'    `TRUE`
#'
#' `booltypes` has a [names()] attribute such that elements can be selected by name.
#'
#' @note do not rely on the internal integer codes of these levels, we might add-in
#'   [`hi`][ff::hi] later
#' @seealso [booltype()], [is.booltype()], [as.booltype()]
#' @export
booltypes <- c("nobool", "logical", "bit", "bitwhich", "which", "ri")
booltypes <- ordered(booltypes, levels=booltypes)
names(booltypes) <- booltypes


#' Diagnosing boolean types
#'
#' Specific methods for `booltype` are required, where non-unary methods can combine
#'   multiple bollean types, particularly boolean binary operators.
#'
#' Function `booltype` returns the boolean type of its argument.
#' There are currently six boolean types, `booltypes` is an [ordered()] vector with the
#'   following ordinal [levels()]:
#'
#'  - nobool: non-boolean type
#'  - [logical()]: for representing any boolean data including `NA`
#'  - [bit()]: for representing dense boolean data
#'  - [bitwhich()]: for representing sparse (skewed) boolean data
#'  - [which()]: for representing sparse boolean data with few `TRUE
#   - [`hi`][ff::hi]: hybrid-indexing, implemented in package [`ff`][ff::ff]
#'  - [ri()]: range-indexing, for representing sparse boolean data with a single range of
#'    `TRUE`
#'
#' @param x an R object
#'
#' @return one scalar element of [booltypes()] in case of 'nobool' it carries a name
#'   attribute with the data type.
#' @note do not rely on the internal integer codes of these levels, we might add-in
#'   [`hi`][ff::hi] later
#' @seealso [booltypes()], [is.booltype()], [as.booltype()]
#'
#' @examples
#' unname(booltypes)
#' str(booltypes)
#' sapply(
#'   list(double(), integer(), logical(), bit(), bitwhich(), as.which(), ri(1, 2, 3)),
#'   booltype
#' )
#' @export
booltype <- function(x) {
  if (is.ri(x))
    booltypes[["ri"]]
  else if (is.hi(x))
    booltypes[["hi"]]
  else if (is.which(x))
    booltypes[["which"]]
  else if (is.bitwhich(x))
    booltypes[["bitwhich"]]
  else if (is.bit(x))
    booltypes[["bit"]]
  else if (is.logical(x))
    booltypes[["logical"]]
  else { # nolint: brace_linter.
    ret <- booltypes[["nobool"]]
    names(ret) <- typeof(x)
    ret
  }
}

#' Testing for boolean types
#'
#' All [booltypes()] including  [logical()] except 'nobool' types are considered
#'   'is.booltype'.
#'
#' @param x an R object
#'
#' @return logical scalar
#' @seealso [booltypes()], [booltype()], [as.booltype()]
#'
#' @examples
#' sapply(
#'   list(double(), integer(), logical(), bit(), bitwhich(), as.which(), ri(1, 2, 3)),
#'   is.booltype
#' )
#' @export
is.booltype <- function(x) {
  inherits(x, "booltype") || is.logical(x)
}

#' @describeIn is.booltype tests for [bit()]
#' @export
is.bit <- function(x) inherits(x, "bit")

#' @describeIn is.booltype tests for [bitwhich()]
#' @export
is.bitwhich <- function(x) inherits(x, "bitwhich")

#' @describeIn is.booltype tests for [`which()`][as.which]
#' @export
is.which <- function(x) inherits(x, "which")

#' @describeIn is.booltype tests for [`hi`][ff::hi]
#' @export
is.hi <- function(x) inherits(x, "hi")

#' @describeIn is.booltype tests for [ri()]
#' @export
is.ri <- function(x) inherits(x, "ri")


#' @describeIn as.booltype default method for as.booltype
#' @export
as.booltype.default <- function(x, booltype="logical", ...) {
  bt <- match.arg(as.character(booltype), as.character(booltypes))
  f = switch(bt,
    logical="as.logical",
    bit="as.bit",
    bitwhich="as.bitwhich",
    which="as.which",
    hi=stop("not implemented for booltype hi"),
    ri="as.ri"
  )
  do.call(f, c(list(x, ...)))
}


#' @describeIn as.ri method to coerce [ri()] to [ri()]
#' @export
as.ri.ri <- function(x, ...) x

#' @describeIn as.ri default method to coerce to [ri()]
#' @export
as.ri.default <- function(x, ...) {
  r <- range.booltype(x)
  n <- maxindex(x)
  ri(r[[1]], r[[2]], n)
}

#' @describeIn maxindex default method for `maxindex`
#' @export
maxindex.default <- function(x, ...) {
  mi <- attr(x, "maxindex")
  if (is.null(mi))
    NA_integer_
  else mi
}

#' @describeIn maxindex default method for `poslength`
#' @export
poslength.default <- function(x, ...) {
  pl <- attr(x, "poslength")
  if (is.null(pl))
    NA_integer_
  else pl
}

#' @describeIn maxindex `maxindex` method for class [logical()]
#' @export
maxindex.logical <- function(x, ...) length(x)

#' @describeIn maxindex `poslength` method for class [logical()]
#' @export
poslength.logical <- function(x, ...) sum(x)

#' @describeIn maxindex `maxindex` method for class [bit()]
#' @export
maxindex.bit <- function(x, ...) length(x)

#' @describeIn maxindex `poslength` method for class [bit()]
#' @export
poslength.bit <- function(x, ...) sum(x, ...)

#' @describeIn maxindex `maxindex` method for class [bitwhich()]
#' @export
maxindex.bitwhich <- function(x, ...) length(x, ...)

#' @describeIn maxindex `poslength` method for class [bitwhich()]
#' @export
poslength.bitwhich <- function(x, ...) sum(x, ...)

#' @describeIn maxindex `maxindex` method for class [`which()`][as.which]
#' @export
maxindex.which <- function(x, ...) attr(x, "maxindex")

#' @describeIn maxindex `poslength` method for class [`which()`][as.which]
#' @export
poslength.which <- function(x, ...) length(x)

#' @describeIn maxindex `maxindex` method for class [ri()]
#' @export
maxindex.ri <- function(x, ...) x[[3L]]

#' @describeIn maxindex `poslength` method for class [ri()]
#' @export
poslength.ri <- function(x, ...) x[[2L]] - x[[1L]] + 1L

#' Getting and setting length of bit, bitwhich and ri objects
#'
#' Query the number of bits in a [bit()] vector or change the number
#'   of bits in a bit vector.
#' Query the number of bits in a bitwhich()] vector or change the number of bits in a bit
#'   vector.
#'
#' NOTE that the length does NOT reflect the number of selected (`TRUE`)
#' bits, it reflects the sum of both, `TRUE` and `FALSE` bits.
#' Increasing the length of a [bit()] object will set new bits to
#' `FALSE`.  The behaviour of increasing the length of a
#' [bitwhich()] object is different and depends on the content of the
#' object:
#'  - TRUE -- all included, new bits are set to `TRUE`
#'  - positive integers -- some included, new bits are set to `FALSE`
#'  - negative integers -- some excluded, new bits are set to `TRUE`
#'  - FALSE -- all excluded:, new bits are set to `FALSE`
#'
#' Decreasing the length of bit or bitwhich removes any previous information
#' about the status bits above the new length.
#'
#' @name length.bit
#' @param x a [bit()], [bitwhich()] or [ri()]
#' object
#' @param value the new number of bits
#' @return the length A bit vector with the new length
#' @author Jens Oehlschlägel
#' @seealso [length()], [`sum()`][sum.bit],
#' [poslength()], [maxindex()]
#' @keywords classes logic
#' @examples
#'
#'   stopifnot(length(ri(1, 1, 32)) == 32)
#'
#'   x <- as.bit(ri(32, 32, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 0)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 0)
#'
#'   x <- as.bit(ri(1, 1, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'
#'   x <- as.bitwhich(bit(32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 0)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 0)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 0)
#'
#'   x <- as.bitwhich(!bit(32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 32)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 16)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 32)
#'
#'   x <- as.bitwhich(ri(32, 32, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 0)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 0)
#'
#'   x <- as.bitwhich(ri(2, 32, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 31)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 15)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 31)
#'
#'   x <- as.bitwhich(ri(1, 1, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 1)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 1)
#'
#'   x <- as.bitwhich(ri(1, 31, 32))
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 31)
#'   message("NOTE the change from 'some excluded' to 'all excluded' here")
#'   length(x) <- 16
#'   stopifnot(length(x) == 16)
#'   stopifnot(sum(x) == 16)
#'   length(x) <- 32
#'   stopifnot(length(x) == 32)
#'   stopifnot(sum(x) == 32)
#'
#' @export
length.bit <- function(x) virtual(x)$Length

#' @rdname length.bit
#' @export
`length<-.bit` <- function(x, value) {
  value <- as.integer(value)
  vattr <- attr(x, "virtual")
  oldvalue <- attr(vattr, "Length")
  if (value == oldvalue)
    return(x)
  pattr <- attr(x, "physical")
  cl <- oldClass(x)
  oldn <- get_length(x)
  dn <- value %% .BITS
  if (dn) {
    n <- value %/% .BITS + 1L
  } else {
    n <- value %/% .BITS
  }
  if (oldn < n) {
    ret <- integer(n)
    ret[seq_len(oldn)] <- x
  } else if (n < oldn) {
    ret <- unclass(x)[seq_len(n)]
  } else {
    ret <- copy_vector(x)
  }
  if (dn && value < oldvalue) {
    .Call(C_R_bit_set_logical, ret, FALSE, c(value + 1L, n * .BITS))
  }
  attr(vattr, "Length") <- value
  setattributes(ret, list(physical = pattr, virtual = vattr, class = cl))
  ret
}

#' @rdname length.bit
#' @export
length.bitwhich <- function(x) attr(x, "maxindex")

#' @rdname length.bit
#' @export
`length<-.bitwhich` <- function(x, value) {
  if (value != length(x)) {
    value <- as.integer(value)
    a <- attributes(x)
    if (value) {
      if (is.integer(x)) {
        oldClass(x) <- NULL
        if (x[1] > 0) {
          ret <- x[x <= value]
          l <- length(ret)
          if (l == 0)
            ret <- copy_vector(FALSE)
          else if (l == value)
            ret <- copy_vector(TRUE)
          else if (l > value %/% 2L)
            ret <- merge_rangediff(c(-value, -1L), ret, revy=TRUE)
        } else {
          ret <- x[x >= -value]
          l <- length(ret)
          if (l == 0)
            ret <- copy_vector(TRUE)
          else if (l == value)
            ret <- copy_vector(FALSE)
          else if (value - l <= value %/% 2L)
            ret <- merge_rangediff(c(1L, value), ret, revy=TRUE)
          l <- value - l
        }
      } else if (length(x) && x) {
        ret <- bitwhich(value, x=TRUE, poslength=value)
        l <- value
      } else {
        ret <- bitwhich(value, x=FALSE, poslength=0L)
        l <- 0L
      }
    } else {
      ret <- bitwhich()
      l <- 0L
    }
  }
  a$maxindex <- value
  a$poslength <- l
  setattributes(ret, a)
  ret
}



#' Concatenating booltype vectors
#'
#' Creating new boolean vectors by concatenating boolean vectors
#'
#' @param ... [booltype()] vectors
#' @return a vector with the lowest input [booltype()] (but not lower than[logical()])
#' @author Jens Oehlschlägel
#' @seealso [c()], [bit()] , [bitwhich()],  , [which()]
#' @keywords classes logic
#' @examples
#'  c(bit(4), !bit(4))
#'  c(bit(4), !bitwhich(4))
#'  c(bitwhich(4), !bit(4))
#'  c(ri(1, 2, 4), !bit(4))
#'  c(bit(4), !logical(4))
#'  message("logical in first argument does not dispatch: c(logical(4), bit(4))")
#'  c.booltype(logical(4), !bit(4))
#'
#' @export c.booltype
#' @export
c.booltype <- function(...) {
  l <- list(...)
  bt <- sapply(l, booltype)
  # xx TEMPORARY WORKAROND: work around a bug in sapply which destroys ordered levels
  class(bt) <- c("ordered", "factor")
  bt <- max(booltypes[["logical"]], min(bt, booltypes[["bitwhich"]]))
  bt <- as.character(bt)
  f <- list(logical=as.logical, bit=as.bit, bitwhich=as.bitwhich, which=as.which)[[bt]]
  l <- lapply(l, f)
  do.call(switch(bt, logical="c", bit="c.bit", bitwhich="c.bitwhich"),  l)
}

#' @rdname c.booltype
#' @export
c.bit <- function(...) {
  l <- list(...)
  nl <- length(l)
  nold <- lengths(l)
  nnew <- sum(nold)
  ncum <- cumsum(nold)
  offsets <- c(0L, ncum[-length(ncum)])
  x <- bit(nnew)
  for (i in seq_len(nl)) {
    b <- as.bit(l[[i]])
    .Call(C_R_bit_shiftcopy, bsource_=b, btarget_=x, otarget_=offsets[i], n_=nold[i])
  }
  x
}

#' @rdname c.booltype
#' @export
c.bitwhich <- function(...) {
  l <- list(...)
  if (length(l) == 1)
    l[[1]]
  else
    as.bitwhich(do.call("c", lapply(l, as.bit)))
}


#' Reversing bit and bitwhich vectors
#'
#' Creating new bit or bitwhich by reversing such vectors
#'
#' @name rev.booltype
#' @param x bit or bitwhich object
#' @return An object of class 'bit' or 'bitwhich'
#' @author Jens Oehlschlägel
#' @seealso [rev()], [bit()] , [bitwhich()]
#' @keywords classes logic
#' @examples
#'
#'  rev(as.bit(c(FALSE, TRUE)))
#'  rev(as.bitwhich(c(FALSE, TRUE)))
NULL

#' @rdname rev.booltype
#' @export
rev.bit <- function(x) {
  if (length(x)) {
    x <- .Call(C_R_bit_reverse, x, bit(length(x)))
  }
  x
}

#' @rdname rev.booltype
#' @export
rev.bitwhich <- function(x) {
  n <- length(x)
  if (is.logical(x)) {
    ret <- bitwhich(n, copy_vector(x), poslength=sum(x))
  } else {
    y <- bitwhich_representation(x)
    if (n < .Machine$integer.max) {
      if (y[1] < 0)
        ret <- bitwhich(n, -(n + 1L) - reverse_vector(x), poslength=sum(x))
      else
        ret <- bitwhich(n, (n + 1L) - reverse_vector(x), poslength=sum(x))
    } else {
      # nolint next: unnecessary_nesting_linter. Good parallelism.
      if (y[1] < 0)
        ret <- bitwhich(n, -n - reverse_vector(x) - 1L, poslength=sum(x))
      else
        ret <- bitwhich(n, n - reverse_vector(x) + 1L, poslength=sum(x))
    }
  }
  ret
}


#' Replicating bit and bitwhich vectors
#'
#' Creating new bit or bitwhich by recycling such vectors
#'
#' @name rep.booltype
#' @param x bit or bitwhich object
#' @param times number of replications
#' @param length.out final length of replicated vector (dominates times)
#' @param ... not used
#' @return An object of class 'bit' or 'bitwhich'
#' @author Jens Oehlschlägel
#' @seealso [rep()], [bit()] , [bitwhich()]
#' @keywords classes logic
#' @examples
#'
#'  rep(as.bit(c(FALSE, TRUE)), 2)
#'  rep(as.bit(c(FALSE, TRUE)), length.out=7)
#'  rep(as.bitwhich(c(FALSE, TRUE)), 2)
#'  rep(as.bitwhich(c(FALSE, TRUE)), length.out=1)
NULL

#' @rdname rep.booltype
#' @export
rep.bit <- function(x, times = 1L, length.out = NA, ...) {
  if (length(times) > 1L)
    stop("only scalar times supported")
  if (is.na(length.out))
    length.out <- length(x) * as.integer(times)
  else
    length.out <- as.integer(length.out)
  ret <- bit(length.out)
  .Call(C_R_bit_recycle, ret, x)
}

#' @rdname rep.booltype
#' @export
rep.bitwhich <- function(x, times = 1L, length.out = NA, ...) {
  as.bitwhich(rep(as.bit(x), times=times, length.out=as.integer(length.out), ...))
}

#' @describeIn as.bit method to coerce to [bit()] (zero length) from [`NULL`][NULL]
#' @export
as.bit.NULL <- function(x, ...) bit(0L)

#' @describeIn as.bit method to coerce to [bit()] from [bit()]
#' @export
as.bit.bit <- function(x, ...) x

#' @describeIn as.bit method to coerce to [bit()] from [logical()]
#' @export
as.bit.logical <- function(x, ...) {
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_logical, b, x, c(1L, n))
}

#' @describeIn as.bit method to coerce to [bit()] from
#'   [integer()] (`0L` and `NA` become `FALSE`,
#'   everthing else becomes `TRUE`)
#' @examples as.bit(c(0L, 1L, 2L, -2L, NA))
#' @export
as.bit.integer <- function(x, ...) {
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_integer, b, x, c(1L, n))
}

#' @describeIn as.bit method to coerce to [bit()] from
#'   [double()] (`0` and `NA` become `FALSE`, everthing
#'   else becomes `TRUE`)
#' @examples as.bit(c(0, 1, 2, -2, NA))
#' @export
as.bit.double <- function(x, ...) {
  n <- length(x)
  b <- bit(n)
  .Call(C_R_bit_set_integer, b, as.integer(x), c(1L, n))
}

#' @describeIn as.bit method to coerce to [bit()] from [bitwhich()]
#' @export
as.bit.bitwhich <- function(x, ...) {
  if (length(x)) {
    b <- bit(length(x))
    if (is.logical(x)) {
      if (unclass(x))
        !b
      else
        b
    } else {
      .Call(C_R_bit_replace, b, x, TRUE)
    }
  } else {
    bit()
  }
}

#' @describeIn as.bit method to coerce to [bit()] from [`which()`][as.which]
#' @export
as.bit.which <- function(x, length=attr(x, "maxindex"), ...) {
  if (is.na(length))
    stop("cannot coerce to bit from which object with unknown maxindex")
  b <- bit(length)
  .Call(C_R_bit_replace, b, x, TRUE)
}

#' @describeIn as.bit method to coerce to [bit()] from [ri()]
#' @export
as.bit.ri <- function(x, ...) {
  if (is.na(x[3]))
    stop("cannot coerce to bit from ri object with unknown maxindex")
  b <- bit(x[3])
  .Call(C_R_bit_set_logical, b, TRUE, x)
}



#' Coercion from bit, bitwhich, which and ri to logical, integer, double
#'
#' Coercion from bit is quite fast because we use a double loop that fixes each
#' word in a processor register.
#'
#' @name CoercionToStandard
#' @param x an object of class [bit()], [bitwhich()] or
#' [ri()]
#' @param length length of the boolean vector (required for `as.logical.which`)
#' @param ... ignored
#' @return [as.logical()] returns a vector of `FALSE, TRUE`,
#' [as.integer()] and [as.double()] return a vector of
#' `0,1`.
#' @author Jens Oehlschlägel
#' @seealso [`CoercionToStandard`][CoercionToStandard], [as.booltype()], [as.bit()],
#'   [as.bitwhich()] , [as.which()], [as.ri()], [ff::as.hi()],  [ff::as.ff()]
#' @keywords classes logic
#' @examples
#'
#'   x <- ri(2, 5, 10)
#'   y <- as.logical(x)
#'   y
#'   stopifnot(identical(y, as.logical(as.bit(x))))
#'   stopifnot(identical(y, as.logical(as.bitwhich(x))))
#'
#'   y <- as.integer(x)
#'   y
#'   stopifnot(identical(y, as.integer(as.logical(x))))
#'   stopifnot(identical(y, as.integer(as.bit(x))))
#'   stopifnot(identical(y, as.integer(as.bitwhich(x))))
#'
#'   y <- as.double(x)
#'   y
#'   stopifnot(identical(y, as.double(as.logical(x))))
#'   stopifnot(identical(y, as.double(as.bit(x))))
#'   stopifnot(identical(y, as.double(as.bitwhich(x))))
NULL

#' @rdname CoercionToStandard
#' @export
as.logical.bit <- function(x, ...) {
  l <- logical(length(x))
  .Call(C_R_bit_get_logical, x, l, c(1L, length(x)))
}

#' @rdname CoercionToStandard
#' @export
as.integer.bit <- function(x, ...) {
  l <- integer(length(x))
  .Call(C_R_bit_get_integer, x, l, c(1L, length(x)))
}

#' @rdname CoercionToStandard
#' @export
as.double.bit <- function(x, ...) {
  l <- integer(length(x))
  as.double(.Call(C_R_bit_get_integer, x, l, c(1L, length(x))))
}

#' @rdname CoercionToStandard
#' @export
as.integer.bitwhich <- function(x, ...) {
  n <- length(x)
  if (is.logical(x)) {
    if (sum(x) == n)
      rep(1L, n)
    else
      rep(0L, n)
  } else {
    ret <- integer(n)
    ret[x] <- 1L
    ret
  }
}

#' @rdname CoercionToStandard
#' @export
as.double.bitwhich <- function(x, ...) {
  n <- length(x)
  if (is.logical(x)) {
    if (sum(x) == n)
      rep(1, n)
    else
      rep(0, n)
  } else {
    ret <- double(n)
    ret[x] <- 1
    ret
  }
}


#' @rdname CoercionToStandard
#' @export
as.logical.bitwhich <- function(x, ...) {
  n <- length(x)
  p <- sum(x)
  if (p == 0) {
    rep(FALSE, length(x))
  } else if (p == n) {
    rep(TRUE, length(x))
  } else {
    ret <- logical(length(x))
    ret[x] <- TRUE
    ret
  }
}

#' @rdname CoercionToStandard
#' @export
as.logical.ri <- function(x, ...) {
  if (is.na(x[3]))
    stop("cannot coerce to logical from ri object with unknown maxindex")
  ret <- logical(x[3])
  ret[x[1]:x[2]] <- TRUE
  ret
}

#' @rdname CoercionToStandard
#' @export
as.integer.ri <- function(x, ...) {
  if (is.na(x[3]))
    stop("cannot coerce to integer from ri object with unknown maxindex")
  ret <- integer(x[3])
  ret[x[1]:x[2]] <- 1L
  ret
}

#' @rdname CoercionToStandard
#' @export
as.double.ri <- function(x, ...) {
  if (is.na(x[3]))
    stop("cannot coerce to integer from ri object with unknown maxindex")
  ret <- double(x[3])
  ret[x[1]:x[2]] <- 1
  ret
}


#' @rdname CoercionToStandard
#' @export
as.logical.which <- function(x, length=attr(x, "maxindex"), ...) {
  if (is.na(length))
    stop("cannot coerce to logical from which object with unknown maxindex")
  l <- logical(length)
  l[x] <- TRUE
  l
}


#' @describeIn as.which method to coerce to [`which()`][as.which] from
#'   [`which()`][as.which]
#' @export
as.which.which <- function(x, maxindex=NA_integer_, ...) x

#' @describeIn as.which method to coerce to zero length [`which()`][as.which] from
#'   [`NULL`][NULL]
#' @export
as.which.NULL <- function(x, ...) {
  out = integer()
  attr(out, "maxindex") = 0L
  class(out) = c("booltype", "which")
  out
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [numeric()]
#' @export
as.which.numeric <- function(x, maxindex=NA_integer_, ...) {
  as.which(as.integer(x), maxindex=maxindex, ...)
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [integer()]
#' @export
as.which.integer <- function(x, maxindex=NA_integer_, is.unsorted=TRUE, has.dup=TRUE, ...) {
  ret <- range_nanozero(as.integer(x))
  r <- getsetattr(ret, "range_na", NULL)
  if (length(ret)) {
    if (r[3] > 0L)
      stop("NA positions not allowed (neither positive nor negative)")
    if (r[1] < 0L) {
      if (r[2] > 0L)
        stop("mixed negative and positive subscripts not allowed")
      if (is.na(maxindex))
        stop("need maxindex with negative subscripts")
      else if (-r[1] > maxindex)
        stop("index value outside -maxindex..-1")
    } else if (!is.na(maxindex) && r[2] > maxindex) {
      stop("index value outside 1..maxindex")
    }
    if (is.unsorted)
      ret <- bit_sort_unique(ret, na.last=NA, range_na=r, has.dup=has.dup)
    else if (has.dup)
      ret <- bit_unique(ret, na.rm = FALSE, range_na=r)
    if (r[1] < 0L)
      ret <- merge_rangediff(c(1L, maxindex), ret, revx=FALSE, revy=TRUE)
  }
  setattributes(ret, list(maxindex = maxindex, class = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [logical()]
#' @export
as.which.logical <- function(x, ...) {
  ret <- which(x)
  setattributes(ret, list(maxindex = as.integer(length(x)), class = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [ri()]
#' @export
as.which.ri <- function(x, ...) {
  ret <- x[1]:x[2]
  setattributes(ret, list(maxindex = as.integer(x[3]), class = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [bit()]
#' @export
as.which.bit <- function(x, range=NULL, ...) {
  maxindex <- length(x)
  if (is.null(range)) {
    range <- c(1L, maxindex)
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > maxindex)
      stop("illegal range")
  }
  s <- sum(x, range=range)
  n <- range[2] - range[1] + 1L
  if (s == 0L) {
    ret <- integer()
  } else if (s == n) {
    #ret <- as.integer(seq.int(from=range[1], to=range[2], by=1))
    ret <- merge_rangediff(range, integer())
  } else {
    ret <- .Call(C_R_bit_which, x, s, range, negative=FALSE)
  }
  setattributes(ret, list(maxindex = as.integer(maxindex), class = c("booltype", "which")))
  ret
}

#' @describeIn as.which method to coerce to [`which()`][as.which] from [bitwhich()]
#' @export
as.which.bitwhich <- function(x, ...) {
  maxindex <- length(x)
  if (is.logical(x)) {
    if (maxindex && unclass(x))
      ret <- seq_len(maxindex)
    else
      ret <- integer()
  } else if (unclass(x)[[1]] < 0) {
    ret <- merge_rangediff(c(1L, maxindex), x, revx=FALSE, revy=TRUE)
  } else {
    ret <- copy_vector(x)
  }
  setattributes(ret, list(maxindex = as.integer(maxindex), class = c("booltype", "which")))
  ret
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] (zero length) from
#'   [`NULL`][NULL]
#' @export
as.bitwhich.NULL <- function(x, ...) {
  bitwhich()
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from [bitwhich()]
#' @export
as.bitwhich.bitwhich <- function(x, ...) {
  x
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from [`which()`][as.which]
#' @export
as.bitwhich.which <- function(x, maxindex=attr(x, "maxindex"), ...) {
  if (is.na(maxindex))
    stop("need maxindex")
  if (maxindex == 0) return(bitwhich())
  poslength <- length(x)
  if (poslength == 0)
    bitwhich(maxindex, FALSE, poslength)
  else if (poslength == maxindex)
    bitwhich(maxindex, TRUE, poslength)
  else if (poslength > maxindex %/% 2L) {
    y = merge_rangediff(c(1L, maxindex), x, revx=TRUE, revy=TRUE)
    bitwhich(maxindex, y, poslength=poslength)
  } else {
    bitwhich(maxindex, x, poslength=poslength)
  }
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from [ri()]
#' @export
as.bitwhich.ri <- function(x, ...) {
  maxindex <- length(x)
  if (is.na(maxindex))
    stop("you must provide maxindex with ri() in as.bitwhich.ri()")
  # ri selects at least one element,
  # hence maxindex>0 and poslength>0
  poslength <- sum(x)
  if (poslength == maxindex)
    bitwhich(maxindex, TRUE, poslength=poslength)
  else if (poslength > maxindex %/% 2L) {
    if (x[1] > 1L) b <- (-x[1] + 1L):(-1) else b <- integer()
    if (x[2] < maxindex) a <- (-maxindex):(-x[2] - 1L) else a <- integer()
    bitwhich(maxindex, c(a, b), poslength=poslength)
  } else {
    bitwhich(maxindex, x[1]:x[2], poslength=poslength)
  }
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from
#'   [integer()] (`0` and `NA` become `FALSE`, everthing
#'   else becomes `TRUE`)
#' @examples as.bitwhich(c(0L, 1L, 2L, -2L, NA))
#' @export
as.bitwhich.integer <- function(x, poslength=NULL, ...) {
  as.bitwhich(as.logical(x), poslength=poslength, ...)
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from
#'   [double()] (`0` and `NA` become `FALSE`, everthing
#'   else becomes `TRUE`)
#' @examples as.bitwhich(c(0, 1, 2, -2, NA))
#' @export
as.bitwhich.double <- as.bitwhich.integer

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from [logical()]
#' @export
as.bitwhich.logical <- function(x, poslength=NULL, ...) {
  maxindex <- length(x)
  if (maxindex == 0) return(bitwhich())
  if (is.null(poslength))
    poslength <- sum(x, na.rm=TRUE)
  else if (poslength > maxindex)
    stop("poslength > maxindex")
  if (poslength == 0)
    bitwhich(maxindex, FALSE, poslength=poslength)
  else if (poslength == maxindex)
    bitwhich(maxindex, TRUE, poslength=poslength)
  else if (poslength > maxindex %/% 2L)
    as.bitwhich(as.bit(x), poslength=poslength)
  else
    bitwhich(maxindex, which(x), poslength=poslength)
}

#' @describeIn as.bitwhich method to coerce to [bitwhich()] from [bit()]
#' @export
as.bitwhich.bit <- function(x, range=NULL, poslength=NULL, ...) {
  maxindex <- length(x)
  if (!maxindex) return(bitwhich())
  if (is.null(range)) {
    range <- c(1L, maxindex)
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > maxindex)
      stop("illegal range")
  }
  if (is.null(poslength))
    poslength <- sum(x, range=range, na.rm=TRUE)
  else
    if (poslength > maxindex)
      stop("poslength > maxindex")
  if (poslength == 0)
    bitwhich(maxindex, FALSE, poslength=poslength)
  else if (poslength == maxindex)
    bitwhich(maxindex, TRUE, poslength=poslength)
  else if (poslength > maxindex %/% 2L) {
    bitwhich(maxindex,
      .Call(C_R_bit_which, x, maxindex - poslength, range=range, negative=TRUE),
      poslength=poslength
    )
  } else {
    bitwhich(maxindex,
      .Call(C_R_bit_which, x, poslength, range=range, negative=FALSE),
      poslength=poslength
    )
  }
}


#' Test for NA in bit and bitwhich
#'
#' @name is.na.bit
#' @param x a [bit()] or  [bitwhich()] vector
#'
#' @return vector of same type with all elements `FALSE`
#' @seealso [is.na()]
#'
#' @examples
#' is.na(bit(6))
#' is.na(bitwhich(6))
is.na.bit <- function(x) bit(length(x))

#' @describeIn is.na.bit method for [is.na()] from [bitwhich()]
#' @export
is.na.bitwhich <- function(x) bitwhich(length(x))


#' @describeIn xor default method for [xor()]
#' @export
xor.default <- function(x, y) {
  cat("default\n")
  (x | y) & !(x & y)
}

#' @describeIn xor [logical()] method for [xor()]
#' @export
xor.logical <- function(x, y) {
  as.logical(x) != as.logical(y)
}

#' @describeIn xor [bit()] method for [`!`][Logic]
#' @export
`!.bit` <- function(x) {
  if (length(x)) {
    ret <- copy_vector(x)
    setattributes(ret, attributes(x))
    .Call(C_R_bit_not, ret)
  } else {
    x
  }
}

#' @describeIn xor [bit()] method for [`&`][Logic]
#' @export
`&.bit` <- function(e1, e2) {
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bit())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_and, e1, e2, ret)
}

#' @describeIn xor [bit()] method for [`|`][Logic]
#' @export
`|.bit` <- function(e1, e2) {
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bit())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_or, e1, e2, ret)
}

#' @describeIn xor [bit()] method for [` == `][Comparison]
#' @export
`==.bit` <- function(e1, e2) {
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bit())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_equal, e1, e2, ret)
}

#' @describeIn xor [bit()] method for [` != `][Comparison]
#' @export
`!=.bit` <- function(e1, e2) {
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bit())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  e1 <- as.bit(e1)
  e2 <- as.bit(e2)
  ret <- bit(n[1])
  .Call(C_R_bit_xor, e1, e2, ret)
}

#' @describeIn xor [bit()] method for [xor()]
#' @export
xor.bit <- function(x, y) {
  n <- c(length(x), length(y))
  if (any(n == 0L))
    return(bit())
  if (n[1] != n[2])
    stop("length(x) != length()")
  x <- as.bit(x)
  y <- as.bit(y)
  ret <- bit(n[1])
  .Call(C_R_bit_xor, x, y, ret)
}


#' @describeIn xor [bitwhich()] method for [`!`][Logic]
#' @export
`!.bitwhich` <- function(x) {
  n <- length(x)
  p <- sum(x)
  if (is.logical(x)) {
    if (n == 0)
      bitwhich()
    else if (p == n) {
      bitwhich(maxindex=n, FALSE, poslength=0L)
    } else {
      bitwhich(maxindex=n, TRUE, poslength=n)
    }
  } else {
    bitwhich(maxindex=n,
      copy_vector(x, revx=TRUE),
      poslength=n - p,
      is.unsorted = FALSE,
      has.dup=FALSE
    )
  }
}

#' @describeIn xor [bitwhich()] method for [`&`][Logic]
#' @export
`&.bitwhich` <- function(e1, e2) {
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bitwhich())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1] == 0 || p[2] == 0)
    return(bitwhich(n[1], FALSE, 0L))
  if (p[1] == n[1])
    return(e2)
  if (p[2] == n[2])
    return(e1)
  #negative <- p>(n %/% 2L)
  negative <- c(unclass(e1)[1] < 0, unclass(e2)[1] < 0)
  if (negative[1]) {
    if (negative[2]) {
      ret <- merge_union(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    } else {
      ret <- merge_setdiff(e2, e1, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (negative[2]) {
      ret <- merge_setdiff(e1, e2, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    } else {
      ret <- merge_intersect(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }
}


#' @describeIn xor [bitwhich()] method for [`|`][Logic]
#' @export
`|.bitwhich` <- function(e1, e2) {
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bitwhich())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1] == n[1] || p[2] == n[2])
    return(bitwhich(n[1], TRUE, n[1]))
  if (p[1] == 0)
    return(e2)
  if (p[2] == 0)
    return(e1)
  #negative <- p>(n %/% 2L)
  negative <- c(unclass(e1)[1] < 0, unclass(e2)[1] < 0)
  if (negative[1]) {
    if (negative[2]) {
      ret <- merge_intersect(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    } else {
      ret <- merge_setdiff(e1, e2, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    }
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (negative[2]) {
      ret <- merge_setdiff(e2, e1, revy=TRUE, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    } else {
      ret <- merge_union(e1, e2, method="exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }
}

#' @describeIn xor [bitwhich()] method for [` == `][Comparison]
#' @export
`==.bitwhich` <- function(e1, e2) {
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bitwhich())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1] == 0)
    return(!e2)
  if (p[1] == n[1])
    return(e2)
  if (p[2] == 0)
    return(!e1)
  if (p[2] == n[2])
    return(e1)
  #negative <- p>(n %/% 2L)
  negative <- c(unclass(e1)[1] < 0, unclass(e2)[1] < 0)
  if (negative[1]) {
    if (negative[2]) {
      ret <- merge_symdiff(e1, e2, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    } else {
      ret <- merge_symdiff(e1, e2, revx=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (negative[2]) {
      ret <- merge_symdiff(e1, e2, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    } else {
      ret <- merge_symdiff(e1, e2, revx=TRUE, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    }
  }
}

#' @describeIn xor [bitwhich()] method for [` != `][Comparison]
#' @export
`!=.bitwhich` <- function(e1, e2) {
  e1 <- as.bitwhich(e1)
  e2 <- as.bitwhich(e2)
  n <- c(length(e1), length(e2))
  if (any(n == 0L))
    return(bitwhich())
  if (n[1] != n[2])
    stop("length(e1) != length(e2)")
  p <- c(sum(e1), sum(e2))
  if (p[1] == 0)
    return(e2)
  if (p[1] == n[1])
    return(!e2)
  if (p[2] == 0)
    return(e1)
  if (p[2] == n[2])
    return(!e1)
  #negative <- p>(n %/% 2L)
  negative <- c(unclass(e1)[1] < 0, unclass(e2)[1] < 0)
  if (negative[1]) {
    if (negative[2]) {
      ret <- merge_symdiff(e1, e2, revx=TRUE, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    } else {
      ret <- merge_symdiff(e1, e2, revy=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    }
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (negative[2]) {
      ret <- merge_symdiff(e1, e2, revx=TRUE, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=n[1] - length(ret))
    } else {
      ret <- merge_symdiff(e1, e2, method = "exact")
      bitwhich(maxindex=n[1], ret, poslength=length(ret))
    }
  }
}

#' @describeIn xor [bitwhich()] method for [xor()]
#' @export
xor.bitwhich <- function(x, y) x != y

#' @describeIn xor [booltype()] method for [`&`][Logic]
#' @export &.booltype
#' @export
`&.booltype` <- function(e1, e2) {
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1, b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2) {
    if (n1 < n2) {
      if (n2 %% n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    } else if (n2 < n1) {
      if (n1 %% n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(as.character(b),
    logical = e1 & e2,
    bit = "&.bit"(e1, e2),
    bitwhich = "&.bitwhich"(e1, e2)
  )
}

#' @describeIn xor [booltype()] method for [`|`][Logic]
#' @export |.booltype
#' @export
`|.booltype` <- function(e1, e2) {
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1, b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2) {
    if (n1 < n2) {
      if (n2 %% n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    } else if (n2 < n1) {
      if (n1 %% n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(as.character(b),
    logical = e1 | e2,
    bit = "|.bit"(e1, e2),
    bitwhich = "|.bitwhich"(e1, e2)
  )
}

#' @describeIn xor [booltype()] method for [` == `][Comparison]
#' @export ==.booltype
#' @export
`==.booltype` <- function(e1, e2) {
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1, b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2) {
    if (n1 < n2) {
      if (n2 %% n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    } else if (n2 < n1) {
      if (n1 %% n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(as.character(b),
    logical = e1 == e2,
    bit = "==.bit"(e1, e2),
    bitwhich = "==.bitwhich"(e1, e2)
  )
}

#' @describeIn xor [booltype()] method for [` != `][Comparison]
#' @export !=.booltype
#' @export
`!=.booltype` <- function(e1, e2) {
  # align booltype between logical and bitwhich
  b1 <- booltype(e1)
  b2 <- booltype(e2)
  b <- min(max(booltypes[["logical"]], min(b1, b2)), booltypes[["bitwhich"]])
  e1 <- as.booltype(e1, b)
  e2 <- as.booltype(e2, b)
  # align length
  n1 <- length(e1)
  n2 <- length(e2)
  if (n1 && n2) {
    if (n1 < n2) {
      if (n2 %% n1)
        warning("longer object length is not a multiple of shorter object length")
      e1 <- rep(e1, length.out=n2)
      n1 <- n2
    } else if (n2 < n1) {
      if (n1 %% n2)
        warning("longer object length is not a multiple of shorter object length")
      e2 <- rep(e2, length.out=n1)
    }
  }
  # do the operation
  switch(as.character(b),
    logical = e1 != e2,
    bit = "!=.bit"(e1, e2),
    bitwhich = "!=.bitwhich"(e1, e2)
  )
}

#' @describeIn xor [booltype()] method for [xor()]
#' @export xor.booltype
#' @export
xor.booltype <- function(x, y) {
  x != y
}



#' Summaries of boolean vectors
#'
#' Fast aggregation functions for [booltype()] vectors. namely [bit()], [all()], [any()],
#'   [anyNA()], [min()], [max()], [range()], [sum()] and [summary()].
#' Now all boolean summaries (except for `anyNA` because the generic does not allow it)
#'   have an optional `range` argument to restrict the range of evalution.
#' Note that the boolean summaries have meaning and return values differing from logical
#'   aggregation functions: they treat `NA` as `FALSE`, `min`, `max` and `range` give the
#'   minimum and maximum positions of `TRUE`, `summary` returns counts of `FALSE`,  `TRUE`
#'   and the `range`.
#' Note that you can force the boolean interpretation by calling the booltype method
#'   explicitly on any [`booltypes`][booltypes] input, e.g. `min.booltype()`, see the
#'   examples.
#'
#' Summaries of [bit()] vectors are quite fast because we use a double loop that fixes
#'   each word in a processor register.  Furthermore we break out of looping as soon
#'   as possible. Summaries of [bitwhich()] vectors are even faster, if the selection is
#'   very skewed.
#'
#' @name Summaries
#' @param x an object of class bit or bitwhich
#' @param object an object of class bit
#' @param range a [ri()] or an integer vector of length == 2 giving a
#' range restriction for chunked processing
#' @param recursive formally required but not used
#' @param ... formally required but not used
#' @return as expected
#' @author Jens Oehlschlägel
#' @seealso [length()]
#' @keywords classes logic
#' @examples
#'
#'   l <- c(NA, FALSE, TRUE)
#'   b <- as.bit(l)
#'
#'   all(l)
#'   all(b)
#'   all(b, range=c(3, 3))
#'   all.booltype(l, range=c(3, 3))
#'
#'   min(l)
#'   min(b)
#'
#'   sum(l)
#'   sum(b)
#'
#'   summary(l)
#'   summary(b)
#'   summary.booltype(l)
NULL

# xx MEMO: R CMD check --no-tests  --no-manual --no-vignettes bit

#' @rdname Summaries
#' @export
all.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_all, x, range)
}

#' @rdname Summaries
#' @export
any.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_any, x, range)
}

#' @rdname Summaries
#' @export
anyNA.bit <- function(x, recursive = FALSE) FALSE

#' @rdname Summaries
#' @export
sum.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_sum, x, range)
}

#' @rdname Summaries
#' @export
min.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_min, x, range)
}

#' @rdname Summaries
#' @export
max.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  .Call(C_R_bit_max, x, range)
}

#' @rdname Summaries
#' @export
range.bit <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(x))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(x))
      stop("illegal range")
  }
  ret <- integer(2)
  ret[1] <- .Call(C_R_bit_min, x, range)
  if (is.na(ret[1]))
    ret[2] <- NA_integer_
  else
    ret[2] <- .Call(C_R_bit_max, x, range)
  ret
}

#' @rdname Summaries
#' @export
summary.bit <- function(object, range=NULL, ...) {
  if (is.null(range)) {
    range <- c(1L, length(object))
  } else {
    range <- as.integer(range[1:2])
    if (range[1] < 1L || range[2] > length(object))
      stop("illegal range")
  }
  s <- sum(object, range=range)
  r <- range(object, range=range)
  c(`FALSE`=range[2] - range[1] + 1L - s, `TRUE`=s, Min.=r[1], Max.=r[2])
}

#' @rdname Summaries
#' @export
all.bitwhich <- function(x, range=NULL, ...) {
  if (is.null(range)) return(attr(x, "poslength") == attr(x, "maxindex"))
  y <- bitwhich_representation(x)
  range <- as.integer(range)
  if (is.logical(y)) {
    if (y)
      TRUE
    else
      FALSE
  } else if (y < 0) {
    all(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
  } else {
    all(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
any.bitwhich <- function(x, range=NULL, ...) {
  if (is.null(range)) return(attr(x, "poslength") > 0L)
  y <- bitwhich_representation(x)
  range <- as.integer(range)
  if (is.logical(y)) {
    if (y)
      TRUE
    else
      FALSE
  } else if (y < 0) {
    any(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
  } else {
    any(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
anyNA.bitwhich <- function(x, recursive = FALSE) FALSE

#' @rdname Summaries
#' @export
sum.bitwhich <- function(x, range=NULL, ...) {
  if (is.null(range)) return(attr(x, "poslength"))
  y <- bitwhich_representation(x)
  range <- as.integer(range)
  if (is.logical(y)) {
    if (y)
      range[2] - range[1] + 1L
    else
      0L
  } else if (y < 0) {
    sum(merge_rangenotin(rx=range, y=x, revx=FALSE, revy=TRUE))
  } else {
    sum(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
  }
}

#' @rdname Summaries
#' @export
min.bitwhich <- function(x, range=NULL, ...) {
  y <- bitwhich_representation(x)
  if (is.logical(y)) {
    if (length(y) && y)
      1L
    else
      NA_integer_
  } else if (is.null(range)) {
    if (y < 0L) {
      merge_firstnotin(c(1L, length(x)), x, revy=TRUE)
    } else {
      merge_first(x)
    }
  } else {
    range <- as.integer(range)
    if (y < 0L) {
      merge_firstnotin(range, x, revy=TRUE)
    } else {
      merge_firstin(range, x)
    }
  }
}

#' @rdname Summaries
#' @export
max.bitwhich <- function(x, range=NULL, ...) {
  y <- bitwhich_representation(x)
  if (is.logical(y)) {
    if (length(y) && y)
      length(x)
    else
      NA_integer_
  } else if (is.null(range)) {
    if (y < 0L) {
      merge_lastnotin(c(1L, length(x)), x, revy=TRUE)
    } else {
      merge_last(x)
    }
  } else {
    range <- as.integer(range)
    if (y < 0L) {
      merge_lastnotin(range, x, revy=TRUE)
    } else {
      merge_lastin(range, x)
    }
  }
}

#' @rdname Summaries
#' @export
range.bitwhich <- function(x, range=NULL, ...) {
  c(min(x, range=range, ...), max(x, range=range, ...))
}

#' @rdname Summaries
#' @export
summary.bitwhich <- function(object, range=NULL, ...) {
  n <- attr(object, "maxindex")
  p <- attr(object, "poslength")
  r <- range(object)
  c(`FALSE`=n - p, `TRUE`=p, Min.=r[1], Max.=r[2])
}

#' @rdname Summaries
#' @export
all.which <- function(x, range=NULL, ...) {
  if (is.null(range)) return(length(x) == attr(x, "maxindex"))
  range <- as.integer(range)
  all(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
}

#' @rdname Summaries
#' @export
any.which <- function(x, range=NULL, ...) {
  if (is.null(range)) return(length(x) > 0L)
  range <- as.integer(range)
  any(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
}

#' @rdname Summaries
#' @export
anyNA.which <- function(x, recursive = FALSE) FALSE

#' @rdname Summaries
#' @export
sum.which <- function(x, range=NULL, ...) {
  if (is.null(range))
    length(x)
  else
    sum(merge_rangein(rx=range, y=x, revx=FALSE, revy=FALSE))
}

#' @rdname Summaries
#' @export
min.which <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    merge_first(x)
  } else {
    range <- as.integer(range)
    merge_firstin(range, x)
  }
}

#' @rdname Summaries
#' @export
max.which <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    merge_last(x)
  } else {
    range <- as.integer(range)
    merge_lastin(range, x)
  }
}

#' @rdname Summaries
#' @export
range.which <- function(x, range=NULL, ...) {
  c(min(x, range=range, ...), max(x, range=range, ...))
}

#' @rdname Summaries
#' @export
summary.which <- function(object, range=NULL, ...) {
  n <- attr(object, "maxindex")
  p <- attr(object, "poslength")
  r <- range(object)
  c(`FALSE`=n - p, `TRUE`=p, Min.=r[1], Max.=r[2])
}

#' @rdname Summaries
#' @export all.booltype
#' @export
all.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=all.bit(as.bit(x), range=range, ...),
    logical=all.bit(as.bit(x), range=range, ...),
    bit=all.bit(x, range=range, ...),
    bitwhich=all.bitwhich(x, range=range, ...),
    which=all.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=all.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export any.booltype
#' @export
any.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=any.bit(as.bit(x), range=range, ...),
    logical=any.bit(as.bit(x), range=range, ...),
    bit=any.bit(x, range=range, ...),
    bitwhich=any.bitwhich(x, range=range, ...),
    which=any.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=any.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export anyNA.booltype
#' @export
anyNA.booltype <- function(x, ...) {
  switch(as.character(booltype(x)),
    nobool=anyNA.bit(as.bit(x), ...),
    logical=anyNA.bit(as.bit(x), ...),
    bit=anyNA.bit(x, ...),
    bitwhich=anyNA.bitwhich(x, ...),
    which=anyNA.bit(as.bit(x), ...),
    hi=stop("not implemented"),
    ri=anyNA.ri(x, ...)
  )
}


#' @rdname Summaries
#' @export sum.booltype
#' @export
sum.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=sum.bit(as.bit(x), range=range, ...),
    logical=sum.bit(as.bit(x), range=range, ...),
    bit=sum.bit(x, range=range, ...),
    bitwhich=sum.bitwhich(x, range=range, ...),
    which=sum.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=sum.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export min.booltype
#' @export
min.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=min.bit(as.bit(x), range=range, ...),
    logical=min.bit(as.bit(x), range=range, ...),
    bit=min.bit(x, range=range, ...),
    bitwhich=min.bitwhich(x, range=range, ...),
    which=min.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=min.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export max.booltype
#' @export
max.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=max.bit(as.bit(x), range=range, ...),
    logical=max.bit(as.bit(x), range=range, ...),
    bit=max.bit(x, range=range, ...),
    bitwhich=max.bitwhich(x, range=range, ...),
    which=max.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=max.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export range.booltype
#' @export
range.booltype <- function(x, range=NULL, ...) {
  switch(as.character(booltype(x)),
    nobool=range.bit(as.bit(x), range=range, ...),
    logical=range.bit(as.bit(x), range=range, ...),
    bit=range.bit(x, range=range, ...),
    bitwhich=range.bitwhich(x, range=range, ...),
    which=range.bit(as.bit(x), range=range, ...),
    hi=stop("not implemented"),
    ri=range.ri(x, range=range, ...)
  )
}

#' @rdname Summaries
#' @export summary.booltype
#' @export
summary.booltype <- function(object, range=NULL, ...) {
  switch(as.character(booltype(object)),
    nobool=summary.bit(as.bit(object), range=range, ...),
    logical=summary.bit(as.bit(object), range=range, ...),
    bit=summary.bit(object, range=range, ...),
    bitwhich=summary.bitwhich(object, range=range, ...),
    which=summary.bit(as.bit(object), range=range, ...),
    hi=stop("not implemented"),
    ri=summary.ri(object, range=range, ...)
  )
}


#' Extract or replace part of an boolean vector
#'
#' Operators acting on [bit()] or [bitwhich()] objects to extract or replace parts.
#'
#' The typical usecase for '[' and '[<-' is subscripting with positive integers,
#' negative integers are allowed but slower,
#' as logical subscripts only scalars are allowed.
#' The subscript can be given as a [bitwhich()] object.
#' Also [ri()] can be used as subscript.
#'
#' Extracting from [bit()] and [bitwhich()] is faster than from  [logical()] if positive
#'   subscripts are used. Unteger subscripts make sense.  Negative subscripts are
#'   converted to positive ones, beware the RAM consumption.
#'
#' @name Extract
#' @param x a [bit()] or [bitwhich()] object
#' @param i preferrably a positive integer subscript or a [ri()], see text
#' @param value new logical or integer values
#' @return The extractors `[[` and `[` return a logical scalar or
#' vector.  The replacment functions return an object of `class(x)`.
#' @author Jens Oehlschlägel
#' @seealso [bit()], [`Extract``][base::Extract]
#' @keywords classes logic
#' @examples
#'
#'   x <- as.bit(c(FALSE, NA, TRUE))
#'   x[] <- c(FALSE, NA, TRUE)
#'   x[1:2]
#'   x[-3]
#'   x[ri(1, 2)]
#'   x[as.bitwhich(c(TRUE, TRUE, FALSE))]
#'   x[[1]]
#'   x[] <- TRUE
#'   x[1:2] <- FALSE
#'   x[[1]] <- TRUE
#'
NULL

#' @rdname Extract
#' @export
`[[.bit` <- function(x, i) {
  if (length(i) != 1L)
    stop("subscript length not 1")
  if (!is.numeric(i))
    stop("subscript must be positive integer (or double) within length")
  i <- as.integer(i)
  if (is.na(i) || i < 1L || i > length(x))
    stop("subscript must be positive integer (or double) within length")
  ret <- .Call(C_R_bit_extract, x, i)
  setattr(ret, "vmode", "boolean")
  ret
}

#' @rdname Extract
#' @export
`[[<-.bit` <- function(x, i, value) {
  if (length(i) != 1L)
    stop("subscript length not 1")
  if (length(value) != 1)
    stop("value length not 1")
  if (!is.numeric(i))
    stop("subscript must be positive integer (or double) within length")
  i <- as.integer(i)
  if (is.na(i) || i < 1L)
    stop("subscript must be positive integer (or double)")
  if ((mi <- max(i)) > length(x))
    length(x) <- mi
  value <- as.logical(value)
  .Call(C_R_bit_replace, x, i, value)
}

#' @rdname Extract
#' @export
`[.bit` <- function(x, i) {
  nx <- length(x)
  if (missing(i)) {
    ret <- logical(nx)
    .Call(C_R_bit_get_logical, x, ret, range=c(1L, nx))
  } else {
    if (inherits(i, "bit")) {
      i <- as.bitwhich(i)
    }
    if (inherits(i, "bitwhich")) {
      i <- unclass(i)
    }
    if (is.numeric(i)) {
      if (inherits(i, "ri")) {
        if (i[1] < 1L || i[2] > nx)
          stop("illegal range index 'ri'")
        ret <- logical(i[2] - i[1] + 1L)
        .Call(C_R_bit_get_logical, x, ret, range=i)
      } else {
        i <- as.integer(i)
        r <- range_na(i)
        if (is.na(r[1])) {
          ret <- logical()
        } else if (r[1] < 0L) {
          # check for positive or NA mixed with negative
          if (r[2] > 0L || r[3] > 0L)
            stop("only 0's may be mixed with negative subscripts")
          isasc <- intisasc(i, "none") # NAs checked already, early terminate on FALSE
          if (!isasc) {
            if ((length(i) / (r[2] - r[1])) < 0.05)
              i <- sort.int(i, method="quick")
            else
              i <- bit_sort_unique(i)
          }
        } # is positive, hence no sorting
        ret <- .Call(C_R_bit_extract, x, i)
      }
    } else if (is.logical(i)) {
      if (poslength(i) == 0L) {
        ret <- logical()
      } else {
        if (inherits(i, "bitwhich")) {
          i <- unclass(i)
        } else if (length(i) != 1 || is.na(i)) {
          stop("only scalar TRUE or FALSE allowed")
        }
        if (i) {
          ret <- logical(nx)
          .Call(C_R_bit_get_logical, x, ret, range=c(1L, nx))
        } else {
          ret <- logical()
        }
      }
    } else {
      stop("subscript must be ri or integer (or double) or  TRUE (or missing) or FALSE")
    }
  }
  setattr(ret, "vmode", "boolean")
  ret
}


#' @rdname Extract
#' @export
`[<-.bit` <- function(x, i, value) {
  nx <- length(x)
  value <- as.logical(value)
  nv <- length(value)
  if (missing(i))
    i <- TRUE
  if (inherits(i, "bit")) {
    i <- as.bitwhich(i)
  }
  if (inherits(i, "bitwhich")) {
    i <- unclass(i)
  }
  if (is.logical(i)) {
    if (length(i) != 1L || is.na(i))
      stop("logical only scalar TRUE or FALSE allowed")
    if (i) {
      if (nv == 0L)
        stop("replacement has length zero")
      if (nx %% nv)
        warning("number of items to replace is not a multiple of replacement length")
      .Call(C_R_bit_set_logical, x, value, range=c(1L, nx))
    } else {
      x
    }
  } else if (is.numeric(i)) {
    if (inherits(i, "ri")) {
      if (i[1] < 1L)
        stop("illegal range index 'ri'")
      if (i[2] > nx)
        length(x) <- i[2]
      ni <- i[2] - i[1] + 1L
      if (nv == 0L)
        stop("replacement has length zero")
      if (ni %% nv)
        warning("number of items to replace is not a multiple of replacement length")
      .Call(C_R_bit_set_logical, x, value, range=i)
    } else {
      if (inherits(i, "which")) {
        ni <- length(i)
        if (ni && i[ni] > nx)
          length(x) <- i[ni]
      } else {
        i <- range_nanozero(as.integer(i))
        r <- getsetattr(i, "range_na", NULL)
        ni <- length(i)
        if (ni) {
          if (r[3] > 0L)
            stop("NAs are not allowed in subscripted assignments")
          if (r[1] > 0L) {
            if (r[2] > nx)
              length(x) <- r[2]
          } else {
            if (r[2] > 0L)
              stop("only 0's may be mixed with negative subscripts")
            # R_bit_replace expects sorted i if i is negative
            i <- bit_sort_unique(i, range_na = r)
            ni <- nx - length(i)
          }
        }
      }
      if (nv != ni) {
        if (nv == 0L)
          stop("replacement has length zero")
        if (ni %% nv)
          warning("number of items to replace is not a multiple of replacement length")

      }
      .Call(C_R_bit_replace, x, i, value)
    }
  } else {
    stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
  }
}


#' Check existence of integers in table
#'
#' If the table is sorted, this can be much faster than [`%in%`][match]
#'
#' @param x a vector of integer
#' @param table a [bitwhich()] object or a vector of integer
#' @param is.unsorted logical telling the function whether the table is (un)sorted. With
#'   the default `NULL` `FALSE` is assumed for [bitwhich()] tables, otherwise `TRUE`
#'
#' @return logical vector
#' @seealso [`%in%`][match]
#'
#' @examples
#' x <- bitwhich(100)
#' x[3] <- TRUE
#' in.bitwhich(c(NA, 2, 3), x)
#' @export
in.bitwhich <- function(x, table, is.unsorted=NULL) {
  x <- as.integer(x)
  if (is.null(is.unsorted))
    is.unsorted <- !is.bitwhich(table)
  if (is.logical(table)) {
    if (length(table) && table) {
      1L <= x & x <= length(table)
    } else {
      rep(FALSE, length(x))
    }
  } else {
    y <- bitwhich_representation(table)
    if (length(x) > 1L && is.unsorted) {
      if (y[1] > 0L)
        !is.na(match(x, table))
      else
        is.na(match(-x, table))

    } else {
      # nolint next: unnecessary_nesting_linter. Good parallelism.
      if (y[1] > 0L)
        merge_in(x, table)
      else
        merge_notin(x, table, revy=TRUE)
    }
  }
}


#' @rdname Extract
#' @export
`[[.bitwhich` <- function(x, i) {
  if (length(i) != 1L)
    stop("subscript length not 1")
  if (!is.numeric(i))
    stop("subscript must be positive integer (or double) within length")
  i <- as.integer(i)
  if (is.na(i) || i < 1L || i > length(x))
    stop("subscript must be positive integer (or double) within length")
  y <- bitwhich_representation(x)
  if (is.logical(y))
    ret <- as.vector(x)
  else
    ret <- in.bitwhich(i, x)
  setattr(ret, "vmode", "boolean")
  ret
}

#' @rdname Extract
#' @export
`[[<-.bitwhich` <- function(x, i, value) {
  if (length(i) != 1L)
    stop("subscript length not 1")
  if (length(value) != 1L)
    stop("value length not 1")
  value <- as.logical(value)
  if (is.na(value))
    value <- FALSE
  n <- length(x)
  if (i > n)
    warning("increasing length of bitwhich, which has non-standard semantics")
  if (!is.numeric(i))
    stop("subscript must be positive integer (or double) within length")
  i <- as.integer(i)
  if (is.na(i) || i < 1L || i > .Machine$integer.max)
    stop("subscript must be positive integer (or double)")
  y <- bitwhich_representation(x)
  if (is.logical(y)) {
    if (length(y)) {
      if (value == y) {
        if (i > n)
          length(x) <- i
        return(x)
      } else if (value) {
        ret <- bitwhich(max(n, i), i, poslength=1L)
      } else {
        ret <- bitwhich(max(n, i), -i, poslength=n - 1L)
      }
    } else if (value) {
      ret <- bitwhich(i, i, poslength=1L)
    } else {
      ret <- bitwhich(i, -i, poslength=n - 1L)
    }
  } else {
    if (i > n) {
      n <- i
      length(x) <- i
    }
    oldvalue <- in.bitwhich(i, x)
    if (value == oldvalue)
      return(x)
    if (value == (y > 0)) {
      ret <-
        bitwhich(n, merge_union(x, y * i, method = "all"), poslength=attr(x, "poslength") + y)
    } else {
      ret <-
        bitwhich(n, merge_setdiff(x, y * i, method = "exact"), poslength=attr(x, "poslength") - y)
    }
  }
  a <- attributes(x)
  a$poslength <- attr(ret, "poslength")
  setattributes(ret, a)
  ret
}


#' @rdname Extract
#' @export
`[.bitwhich` <- function(x, i) {
  nx <- length(x)
  if (missing(i)) {
    ret <- as.logical(x)
  } else {
    if (inherits(i, "bit"))
      stop("please use as.which(bit) for subscripting with bit")
    if (inherits(i, "bitwhich"))
      stop("please use unclass(bitwhich) or as.which(bitwhich) to clarify what you want")
    if (length(i) == 0) {
      ret <- logical()
    } else if (is.logical(i)) {
      if (length(i) != 1L || is.na(i))
        stop("only scalar TRUE or FALSE allowed")
      if (i) {
        ret <- as.logical(x)
      } else {
        ret <- logical()
      }
    } else if (is.numeric(i)) {
      if (inherits(i, "ri")) {
        if (i[1] < 1L || i[2] > nx)
          stop("illegal range index 'ri'")
        if (is.logical(x)) {
          if (length(x))
            ret <- rep(copy_vector(x), i[2] - i[1] + 1L)
          else
            ret <- rep(NA, i[2] - i[1] + 1L)
        } else {
          #y <- unclass(x)
          y <- bitwhich_representation(x)
          if (y[1] > 0L) {
            #ret <- rep(FALSE, i[2]-i[1] + 1L)
            #ret[y[i[1] <= y & y <= i[2]] - i[1] + 1L] <- TRUE
            ret <- merge_rangein(c(i[1], i[2]), x)
          } else {
            # ret <- rep(TRUE, i[2]-i[1] + 1L)
            # ret[-y[(-i[1]) >= y & y >= (-i[2])] - i[1] + 1L] <- FALSE
            ret <- merge_rangenotin(c(i[1], i[2]), x, revy=TRUE)
          }
        }
      } else {
        i <- range_nanozero(as.integer(i))
        r <- getsetattr(i, "range_na", NULL)
        n <- length(i)
        if (r[3] == n) { # if allNA
          ret <- rep(NA, n)
        } else {
          if (r[1] < 0L && r[2] > 0L)
            stop("only 0's may be mixed with negative subscripts")
          ret <- as.bit(x)[i]
        }
      }
    } else {
      stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
    }
  }
  setattr(ret, "vmode", "boolean")
  ret
}


#' @rdname Extract
#' @export
`[<-.bitwhich` <- function(x, i, value) {
  nx <- length(x)
  value <- as.logical(value)
  if (anyNA(value))
    value[is.na(value)] <- FALSE
  nv <- length(value)
  if (missing(i))
    i <- TRUE
  if (inherits(i, "bit"))
    stop("please use as.which(bit) for subscripting with bit")
  if (inherits(i, "bitwhich"))
    stop("please use unclass(bitwhich) or as.which(bitwhich) to clarify what you want")
  if (!length(i)) return(x)
  if (is.logical(i)) {
    if (length(i) != 1L || is.na(i))
      stop("logical only scalar TRUE or FALSE allowed")
    if (i) {
      if (nv == 1L) {
        ret <- bitwhich(nx, value)
      } else {
        b <- as.bit(value)
        if (nv == nx) {
          ret <- as.bitwhich(b)
        } else {
          if (nv == 0L)
            stop("replacement has length zero")
          if (nx %% nv)
            warning("number of items to replace is not a multiple of replacement length")
          ret <- as.bitwhich(rep(b, length.out=nx))
        }
      }
    } else {
      return(x)
    }
  } else if (is.numeric(i)) {
    if (nv > 1L) {
      b <- as.bit(x)
      b[i] <- value
      ret <- as.bitwhich(b)
    } else {
      if (inherits(i, "ri")) {
        if (i[1] < 1L)
          stop("illegal range index 'ri'")
        biggest_mentioned_index <- max(abs(i[1:2]))
        i <- i[1]:i[2]
        ni <- length(i)
      } else if (inherits(i, "which")) {
        ni <- length(i)
        biggest_mentioned_index <- i[length(i)]
      } else {
        i <- range_nanozero(as.integer(i))
        r <- getsetattr(i, "range_na", NULL)
        if (length(i)) {
          if (r[3] > 0L)
            stop("NAs are not allowed in subscripted assignments")
          if (r[1] > 0L) {
            # since value is a scalar removing duplicates does not harm and speeds up
            i <- bit_sort_unique(i, range_na = r)
            ni <- length(i)
          } else {
            if (r[2] > 0L)
              stop("only 0's may be mixed with negative subscripts")
            i <- bit_sort_unique(i, range_na = r)
            ni <- nx - length(i)
          }
          # since value is a scalar removing duplicates does not harm and speeds up
          biggest_mentioned_index <- max(abs(i[1:2]), na.rm=TRUE)
        } else {
          ni <- 0L
          biggest_mentioned_index <- 0L
        }
      }
      if (!ni) {
        return(x)
      }
      if (nv == 0L)
        stop("replacement has length zero")
      if (biggest_mentioned_index > nx) {
        length(x) <- biggest_mentioned_index
        nx <- biggest_mentioned_index
      }
      y <- bitwhich_representation(x)
      if (is.logical(y)) {
        if (value == y) {
          # assignment doesn't change anything
          return(x)
        } else if (value) {
          # assignment has first inclusions
          if (i[1] < 0) {
            # assignment enumerates those not included
            ret <- bitwhich(nx, i, poslength=nx - length(i))
          } else {
            # assignment enumerates those included
            ret <- bitwhich(nx, i, poslength=length(i))
          }
        } else {
          # assignment has first exclusions
          # nolint next: unnecessary_nesting_linter. Good parallelism.
          if (i[1] < 0) {
            # assignment enumerates those not excluded
            ret <- bitwhich(nx, copy_vector(i, revx=TRUE), poslength=length(i))
          } else {
            # assignment enumerates those excluded
            ret <- bitwhich(nx, copy_vector(i, revx=TRUE), poslength=nx - length(i))
          }
        }
      } else if (y < 0) {
        # object maintains exclusions
        if (value) {
          # assignment has inclusions
          if (i[1] < 0) {
            # assignment enumerates those not included
            # w2 = w = bitwhich(12, -(1:3)); w2[-(3:5)] = TRUE
            # cbind(as.logical(w), as.logical(w2))
            ret <- bitwhich(nx,
              merge_intersect(x, i, method='exact'),
              xempty=TRUE,
              is.unsorted = FALSE,
              has.dup = FALSE
            )
          } else {
            # assignment enumerates those included
            # w2 = w = bitwhich(12, -(1:3)); w2[3:5] = TRUE; cbind(as.logical(w), as.logical(w2))
            ret <- bitwhich(nx,
              merge_setdiff(x, i, revy=TRUE, method='exact'),
              xempty=TRUE,
              is.unsorted = FALSE,
              has.dup = FALSE
            )
          }
        } else {
          # assignment has exclusions
          # nolint next: unnecessary_nesting_linter. Good parallelism.
          if (i[1] < 0) {
            # assignment enumerates those not excluded
            # w2 = w = bitwhich(12, -(1:3)); w2[-(3:5)] = FALSE;
            # cbind(as.logical(w), as.logical(w2))
            ret <- bitwhich(nx,
              merge_setdiff(i, x, revx=TRUE, revy=TRUE, method='exact'),
              xempty=FALSE,
              is.unsorted = FALSE,
              has.dup = FALSE
            )
          } else {
            # assignment enumerates those excluded
            # w2 = w = bitwhich(12, -(1:3)); w2[3:5] = FALSE; cbind(as.logical(w), as.logical(w2))
            ret <- bitwhich(nx,
              merge_union(x, i, revy=TRUE, method='exact'),
              is.unsorted = FALSE,
              has.dup = FALSE
            )
          }
        }
        # object maintains inclusions
      } else if (value) {
        # assignment has inclusions
        if (i[1] < 0) {
          # assignment enumerates those not included
          # w2 = w = bitwhich(12, (1:3)); w2[-(3:5)] = TRUE; cbind(as.logical(w), as.logical(w2))
          ret <- bitwhich(nx,
            merge_setdiff(i, x, revy = TRUE, method='exact'),
            xempty=TRUE,
            is.unsorted = FALSE,
            has.dup = FALSE
          )
        } else {
          # assignment enumerates those included
          # w2 = w = bitwhich(12, (1:3)); w2[(3:5)] = TRUE; cbind(as.logical(w), as.logical(w2))
          ret <-
            bitwhich(nx, merge_union(x, i, method='exact'), is.unsorted = FALSE, has.dup = FALSE)
        }
      } else {
        # assignment has exclusions
        # nolint next: unnecessary_nesting_linter. Good parallelism.
        if (i[1] < 0) {
          # assignment enumerates those not excluded
          # w2 = w = bitwhich(12, (1:3)); w2[-(3:5)] = FALSE; cbind(as.logical(w), as.logical(w2))
          ret <- bitwhich(nx,
            merge_intersect(x, i, revy=TRUE, method='exact'),
            xempty=FALSE,
            is.unsorted = FALSE,
            has.dup = FALSE
          )
        } else {
          # assignment enumerates those excluded
          # w2 = w = bitwhich(12, (1:3)); w2[(3:5)] = FALSE; cbind(as.logical(w), as.logical(w2))
          ret <- bitwhich(nx,
            merge_setdiff(x, i, method='exact'),
            xempty=FALSE,
            is.unsorted = FALSE,
            has.dup = FALSE
          )
        }
      }
    }
  } else {
    stop("subscript must be integer (or double) or ri or bitwhich or TRUE or FALSE or missing")
  }
  a <- attributes(x)
  a$poslength <- sum(ret)
  setattributes(ret, a)
  ret
}



#' Range index
#'
#' A range index can be used to extract or replace a continuous ascending part
#' of the data
#'
#' @param from first position
#' @param to last posistion
#' @param x an object of class 'ri'
#' @param maxindex the maximal length of the object-to-be-subscripted (if
#' known)
#' @param ... further arguments
#' @return A two element integer vector with class 'ri'
#' @author Jens Oehlschlägel
#' @seealso [ff::as.hi()]
#' @keywords classes logic
#' @examples
#'
#'  bit(12)[ri(1, 6)]
#'
#' @export ri
ri <- function(from, to=NULL, maxindex=NA) {
  if (is.null(to)) {
    x <- as.integer(c(from, maxindex))
  } else {
    x <- as.integer(c(from, to, maxindex))
  }
  maxindex = maxindex
  if (length(x) != 3)
    stop("range must have exactly three elements")
  if (x[[1]] < 1L)
    stop("range must at least select one element")
  if (x[[1]] > x[[2]])
    stop("lower bound must be smaller or equal than upper bound")
  if (!is.na(x[[3]]) && x[[2]] > x[[3]])
    stop("lower and upper bound must be smaller or equal to maxindex")
  oldClass(x) <- c("booltype", "ri")
  x
}

#' @rdname ri
#' @export
print.ri <- function(x, ...) {
  cat("range index (ri) from", x[[1]], "to", x[[2]], "maxindex",  x[[3]], "\n")
  invisible(x)
}

#' @rdname length.bit
#' @export
length.ri <- function(x) x[[3]]

#' @rdname Summaries
#' @export
all.ri <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range[[1]] <- 1L
    range[[2]] <- x[[3]]
  }
  x[[1]] <= range[[1]] && x[[2]] >= range[[2]]
}

#' @rdname Summaries
#' @export
any.ri <- function(x, range=NULL, ...) {
  if (is.null(range)) {
    range[[1]] <- 1L
    range[[2]] <- x[[3]]
  }
  range[[1]] <= x[[1]] && range[[2]] >= x[[2]]
}

#' @rdname Summaries
#' @export
anyNA.ri <- function(x, recursive = FALSE) FALSE

#' @rdname Summaries
#' @export
sum.ri <- function(x, ...) {
  if (any(names(match.call(expand.dots = TRUE)) == "range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[2]] - x[[1]] + 1L
}

#' @rdname Summaries
#' @export
min.ri <- function(x, ...) {
  if (any(names(match.call(expand.dots = TRUE)) == "range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[1]]
}

#' @rdname Summaries
#' @export
max.ri <- function(x, ...) {
  if (any(names(match.call(expand.dots = TRUE)) == "range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[[2]]
}

#' @rdname Summaries
#' @export
range.ri <- function(x, ...) {
  if (any(names(match.call(expand.dots = TRUE)) == "range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  x[1:2]
}

#' @rdname Summaries
#' @export
summary.ri <- function(object, ...) {
  if (any(names(match.call(expand.dots = TRUE)) == "range") && !is.null(list(...)$range))
    stop("parameter 'range' allowed only for 'bit' but not for 'ri'")
  s <- object[[2]] - object[[1]] + 1L
  c(`FALSE` = object[[3]] - s, `TRUE` = s, Min. = object[[1]], Max. = object[[2]])
}

# this version without vmode() will be overwritte by the version in package ff
#' @rdname PhysVirt
#' @export
physical.default <- function(x) {
  p <- attributes(attr(x, "physical"))
  p <- p[is.na(match(names(p), "class"))]
  p
}
#' @rdname PhysVirt
#' @export
`physical<-.default` <- function(x, value) {
  attributes(attr(x, "physical")) <- c(value, list(class="physical"))
  x
}

#' @rdname PhysVirt
#' @export
virtual.default <- function(x) {
  v <- attributes(attr(x, "virtual"))
  v[is.na(match(names(v), "class"))]
}
#' @rdname PhysVirt
#' @export
`virtual<-.default` <- function(x, value) {
  attributes(attr(x, "virtual")) <- c(value, list(class="virtual"))
  x
}

#' @rdname PhysVirt
#' @export
print.physical <- function(x, ...) {
  cat(
    "(hidden, use physical(x) to access the physical attributes and vmode(x) for accessing vmode)\n"
  )
  invisible()
}

#' @rdname PhysVirt
#' @export
print.virtual <- function(x, ...) {
  cat("(hidden, use virtual(x) to access the virtual attributes)\n")
  invisible()
}

# not exported - just here to avoid cross calling the dll from ff
R_bit_as_hi <- function(x, range, offset) .Call(C_R_bit_as_hi, x, range, offset)

# Fast methods for sorted integers
# (c) 2016-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk


# Attention:
#   must not use as.integer() on x and y here, otherwise we cannot pass bitwhich objects
#   (as.integer) would turn positions into 0/1

#' Fast functions for sorted sets of integer
#'
#' The `merge_` functions allow unary and binary operations on (ascending) sorted vectors
#'    of [integer()].
#' `merge_rev(x)` will do in one scan what costs two scans in [`-rev(x)`][rev], see also
#'   [reverse_vector()].
#' Many of these `merge_` can optionally scan their input in reverse order (and switch the
#'   sign), which again saves extra scans for calling `merge_rev(x)` first.
#'
#' @details These are low-level functions and hence do not check whether the set is
#'   actually sorted.
#' Note that the `merge_*` and `merge_range*` functions have no special treatment for
#'   `NA`.
#' If vectors with `NA` are sorted ith `NA` in the first positions (`na.last=FALSE`) and
#'   arguments `revx=` or `revy=` have not been used, then `NAs` are treated like ordinary
#'   integers. `NA` sorted elsewhere or using `revx=` or `revy=` can cause unexpected
#'   results (note for example that `revx=` switches the sign on all integers but `NAs`).
#'
#' The *binary* `merge_*` functions have a `method="exact"`
#' which in both sets treats consecutive occurrences of the same value as if they were
#'   different values, more precisely they are handled as if the identity of ties were
#'   tuples of `ties, rank(ties)`. `method="exact"` delivers unique output if the input is
#'   unique, and in this case works faster than `method="unique"`.
#'
#' @note xx OPTIMIZATION OPPORTUNITY These are low-level functions could be optimized with
#'   initial binary search (not findInterval, which coerces to double).
#'
#' @param x a sorted set
#' @param rx range of integers given as [ri()] or as a two-element [integer()]
#' @param y a sorted set
#' @param revx default `FALSE`, set to `TRUE` to reverse scan parameter 'x'
#' @param revy default `FALSE`, set to `TRUE` to reverse scan parameter 'y'
#' @param nomatch integer value returned for non-matched elements, see [match()]
#' @param method one of "unique", "exact" (or "all") which governs how to treat ties, see
#'   the function descriptions
#'
#' @return `merge_rev(x)` returns [`-rev(x)`][rev] for [integer()] and [double()] and
#'   [`!rev(x)`][rev] for [logical()]
#'
#' @examples
#' merge_rev(1:9)
#'
#' merge_match(1:7, 3:9)
#' #' merge_match(merge_rev(1:7), 3:9)
#' merge_match(merge_rev(1:7), 3:9, revx=TRUE)
#' merge_match(merge_rev(1:7), 3:9, revy=TRUE)
#' merge_match(merge_rev(1:7), merge_rev(3:9))
#'
#' merge_in(1:7, 3:9)
#' merge_notin(1:7, 3:9)
#'
#' merge_anyDuplicated(c(1L, 1L, 2L, 3L))
#' merge_duplicated(c(1L, 1L, 2L, 3L))
#' merge_unique(c(1L, 1L, 2L, 3L))
#'
#' merge_union(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L))
#' merge_union(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L), method="exact")
#' merge_union(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L), method="all")
#'
#' merge_setdiff(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L))
#' merge_setdiff(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L), method="exact")
#' merge_setdiff(c(1L, 2L, 2L), c(2L, 2L, 2L, 3L), method="exact")
#'
#' merge_symdiff(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L))
#' merge_symdiff(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L), method="exact")
#' merge_symdiff(c(1L, 2L, 2L), c(2L, 2L, 2L, 3L), method="exact")
#'
#' merge_intersect(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L))
#' merge_intersect(c(1L, 2L, 2L, 2L), c(2L, 2L, 3L), method="exact")
#'
#' merge_setequal(c(1L, 2L, 2L), c(1L, 2L))
#' merge_setequal(c(1L, 2L, 2L), c(1L, 2L, 2L))
#' merge_setequal(c(1L, 2L, 2L), c(1L, 2L), method="exact")
#' merge_setequal(c(1L, 2L, 2L), c(1L, 2L, 2L), method="exact")
#'
#' @export

merge_rev <- function(x) {
  .Call(C_R_merge_rev, x)
}

#' @describeIn merge_rev returns integer positions of sorted set x in sorted set y, see
#'   [`match(x, y, ...)`][match]
#' @export
merge_match <- function(x, y, revx=FALSE, revy=FALSE, nomatch = NA_integer_) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_match, x, y, as.logical(revx), as.logical(revy), nomatch=as.integer(nomatch))
}

# xx OPTIMIZATION OPPORTUNITY: this could be optimized with proper binary search
#   (not findInterval, which coerces to double)
#' @describeIn merge_rev returns logical existence of sorted set x in sorted set y, see
#'   [`x %in% y`][match]
#' @export
merge_in <- function(x, y, revx=FALSE, revy=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_in, x, y, as.logical(revx), as.logical(revy))
}

# xx OPTIMIZATION OPPORTUNITY this could be optimized with proper binary search
#   (not findInterval, which coerces to double)
#' @describeIn merge_rev returns logical in-existence of sorted set x in sorted set y, see
#'   [`!(x %in% y)`][match]
#' @export
merge_notin <- function(x, y, revx=FALSE, revy=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_notin, x, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns the duplicated status of a sorted set x, see
#'   [duplicated()]
#' @export
merge_duplicated <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_duplicated, x, as.logical(revx))
}

#' @describeIn merge_rev returns the anyDuplicated status of a sorted set x, see
#'   [anyDuplicated()]
#' @export
merge_anyDuplicated <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_anyDuplicated, x, as.logical(revx))
}

#' @describeIn merge_rev returns the sumDuplicated status of a sorted set x, see
#'   [bit_sumDuplicated()]
#' @export
merge_sumDuplicated <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_sumDuplicated, x, as.logical(revx))
}
#' @describeIn merge_rev returns unique elements of sorted set x, see [unique()]
#' @export
merge_unique <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_unique, x, as.logical(revx))
}

#' @describeIn merge_rev returns union of two sorted sets.
#' Default `method='unique'` returns a unique sorted set, see [union()];
#' `method='exact'` returns a sorted set with the maximum number of ties in either
#'   input set; `method='all'` returns a sorted set with the sum of ties in both input
#'   sets.
#' @export
merge_union <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique", "exact", "all")) {
  method <- match.arg(method)
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_union, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns sorted set x minus sorted set y
#' Default `method='unique'` returns a unique sorted set, see [setdiff()];
#' `ethod='exact'` returns a sorted set with sum(x ties) minus sum(y ties);
#' @export
merge_setdiff <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique", "exact")) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_setdiff, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns those elements that are in sorted set `y` [xor()] in
#'   sorted set `y`
#' Default `method='unique'` returns the sorted unique set complement, see [symdiff()];
#'   `method='exact'` returns a sorted set set complement with
#'   `abs(sum(x ties) - sum(y ties))`.
#' @export
merge_symdiff <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique", "exact")) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_symdiff, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns the intersection of two sorted sets x and y
#' Default `method='unique'` returns the sorted unique intersect, see [intersect()];
#' `method='exact'` returns the intersect with the minium number of ties in either set;
#' @export
merge_intersect <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique", "exact")) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_intersect, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns `TRUE` for equal sorted sets and `FALSE` otherwise
#' Default `method='unique'` compares the sets after removing ties, see [setequal()];
#' `method='exact'` compares the sets without removing ties;
#' @export
merge_setequal <- function(x, y, revx=FALSE, revy=FALSE, method=c("unique", "exact")) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_setequal, x, y, as.logical(revx), as.logical(revy), method)
}

#' @describeIn merge_rev returns logical existence of range rx in sorted set y, see
#'   [merge_in()]
#' @export
merge_rangein <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangein, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns logical in-existence of range rx in sorted set y, see
#'   [merge_notin()]
#' @export
merge_rangenotin <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangenotin, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns the intersection of range rx and sorted set y, see
#'   [merge_intersect()]
#' @export
merge_rangesect <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangesect, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev returns range rx minus sorted set y, see [merge_setdiff()]
#' @export
merge_rangediff <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_rangediff, rx, y, as.logical(revx), as.logical(revy))
}

#' @describeIn merge_rev quickly returns the first element of a sorted set x (or `NA` if
#'   x is empty), hence `x[1]` or `merge_rev(x)[1]`
#' @export
merge_first <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_first, x, as.logical(revx))
}
#' @describeIn merge_rev quickly returns the last element of a sorted set x, (or `NA` if
#'   x is empty), hence `x[n]` or `merge_rev(x)[n]`
#' @export
merge_last <- function(x, revx=FALSE) {
  if (!(is.integer(x) || is.ordered(x)))
    stop("x must be integer (or ordered factor)")
  .Call(C_R_merge_last, x, as.logical(revx))
}

#' @describeIn merge_rev quickly returns the first common element of a range rx and a
#'   sorted set y, (or `NA` if the intersection is empty), hence
#'   `merge_first(merge_rangesect(rx, y))`
#' @export
merge_firstin <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_firstin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the last common element of a range rx and a
#'   sorted set y, (or `NA` if the intersection is empty), hence
#'   `merge_last(merge_rangesect(rx, y))`
#' @export
merge_lastin <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_lastin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the first element of a range rx which is not in a
#'   sorted set y (or `NA` if all rx are in y), hence `merge_first(merge_rangediff(rx, y))`
#' @export
merge_firstnotin <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_firstnotin, rx, y, as.logical(revx), as.logical(revy))
}
#' @describeIn merge_rev quickly returns the last element of a range rx which is not in a
#'   sorted set y (or `NA` if all rx are in y), hence `merge_last(merge_rangediff(rx, y))`
#' @export
merge_lastnotin <- function(rx, y, revx=FALSE, revy=FALSE) {
  if (!is.ri(rx)) {
    stopifnot(length(rx) == 2)
    rx <- as.integer(rx)
  }
  if (!(is.integer(y) || is.ordered(y)))
    stop("y must be integer (or ordered factor)")
  .Call(C_R_merge_lastnotin, rx, y, as.logical(revx), as.logical(revy))
}



#' Symmetric set complement
#'
#' @param x a vector
#' @param y a vector
#' @return `union(setdiff(x, y), setdiff(y, x))`
#' @seealso [merge_symdiff()] and [xor()]
#' @note that `symdiff(x, y)` is not [identical()]
#' as `symdiff(y, x)` without applying [sort()] to the result
#' @examples
#' symdiff(c(1L, 2L, 2L), c(2L, 3L))
#' symdiff(c(2L, 3L), c(1L, 2L, 2L))
#' @export
symdiff <- function(x, y) {
  union(setdiff(x, y), setdiff(y, x))
}

# Chunking utilities for bit and ff
# (c) 2007-2009 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2007-09-03
# Last changed: 2007-10-25

# source("D:/mwp/eanalysis/bit/R/chunkutil.R")




# non-vectorized
#bbatch <-
#function(N,B){
#  N <- as.integer(N)
#  B <- as.integer(B)
#  RB <- N %% B
#  NB <- N %/% B
#  if (RB){
#    cc <- min((B - RB) %/% NB, (B - RB) %/% (NB + 1L))
#    if (cc){
#      rb <- RB + cc * NB
#      b <- B - cc
#      if (rb==b){
#        return(list(b=b, nb=NB+1L, rb=0L))
#      }else{
#        return(list(b=b, nb=NB, rb=rb))
#      }
#    }else{
#      return(list(b=B, nb=NB, rb=RB))
#    }
#  }else{
#    return(list(b=B, nb=NB, rb=RB))
#  }
#}

#' Balanced Batch sizes
#' 
#' \command{bbatch} calculates batch sizes in 1..N so that they have rather balanced
#' sizes than very different sizes.
#' 
#' Tries to have \code{rb==0} or \code{rb} as close to \code{b} as possible
#' while guaranteeing that \code{rb < b && (b - rb) <= min(nb, b)}
#' 
#' @param N total size in 0..integer_max
#' @param B desired batch size in 1..integer_max
#' @return a list with components \item{ b }{ the batch size } \item{ nb }{ the
#' number of batches } \item{ rb }{ the size of the rest }
#' @author Jens Oehlschlägel
#' @seealso \code{\link{repfromto}}, \code{\link[ff:ffapply]{ffvecapply}}
#' @keywords IO data
#' @examples
#' 
#'   bbatch(100, 24)
#' 
#' @export
bbatch <- function(N,B){
  if (any(B<1))
    stop("B too small")
  if (any(N<0))
    stop("N too small")
  N <- as.integer(N)
  B <- pmin(pmax(1L,N), as.integer(B))
  RB <- N %% B
  NB <- N %/% B
  cc <- pmin((B - RB) %/% NB, (B - RB) %/% (NB + 1L))
  cc[RB==0 | NB == 0] <- 0L
  i <- cc > 0
  RB[i] <- RB[i] + cc[i] * NB[i]
  B[i] <- B[i] - cc[i]
  j <- i & (RB == B)
  NB[j] <- NB[j] + 1L
  RB[j] <- 0L
  i <- (RB>0) & (NB == 0)
  B[i] <- RB[i]
  NB[i] <- 1L
  RB[i] <- 0L
  return(list(b=B, nb=NB, rb=RB))
}



#' Virtual recycling
#' 
#' \command{repfromto} virtually recylcles object \code{x} and cuts out
#' positions \code{from .. to}
#' 
#' \code{repfromto} is a generalization of \code{\link{rep}}, where
#' \code{rep(x, n) == repfromto(x, 1, n)}.  You can see this as an R-side
#' (vector) solution of the \code{mod_iterate} macro in arithmetic.c
#' 
#' @param x an object from which to recycle
#' @param from first position to return
#' @param to last position to return
#' @param value value to assign
#' @return a vector of length \code{from - to + 1}
#' @author Jens Oehlschlägel
#' @seealso \code{\link{rep}}, \code{\link[ff:ffapply]{ffvecapply}}
#' @keywords IO data
#' @examples
#' 
#'   message("a simple example")
#'   repfromto(0:9, 11, 20)
#' 
#' @export
repfromto <- function(x, from, to){
  nx <- length(x)
  if (nx){
    from <- as.integer(from)
    to <- as.integer(to)
    if (to>nx){
      N <- to - from + 1L
      from <- (from-1L)%%nx + 1L
      to <- to%%nx
      # NOTE: fetch in sequence pre-main-post in case is.ff(x)
      if (from<=to && N<nx){
        ret <- x[from:to]
      }else{
        pre <- x[from:nx]
        nrep <- (N - length(pre) - to) %/%nx
        main <- if (nrep) rep(x[1:nx], nrep) else NULL
        post <- if (to) x[1:to] else NULL
        ret <- c(pre, main, post)
      }
    }else{
      ret <- x[from:to]
    }
    a <- attributes(x[1])
    a$names <- NULL
    attributes(ret) <- a
    return(ret)
  }else{
    return(NA[from:to])
  }
}

#' @rdname repfromto
#' @export
"repfromto<-" <- function(x, from, to, value){
  x[from:to] <- value
  x
}


if (FALSE){
  x <- 1:10
  for (n in 1:20)
  for (i1 in 1:30){
    i2 <- i1+n-1
    cat(i1,i2,"|",repfromto(x,i1,i2), "\n")
  }
}

if (FALSE){
  intseq <- function(from=NULL, to=NULL, by=NULL, length.out=NULL, along.with=NULL){
    if (is.null(from)){
      if (is.null(to))
        stop("need 'from' or 'to'")
      else
        to <- as.integer(to)
      if (is.null(by))
        by <- 1L
      else
        by <- as.integer(by)
    }else{
      from <- as.integer(from)
      if (is.null(to)){
        if (is.null(by))
          by <- 1L
        else
          by <- as.integer(by)
      }else{
        to <- as.integer(to)
        n <- to - from
        if (is.null(by)){
          if (to<from)
            by = -1L
          else
            by = 1L
        }else{
          by <- as.integer(by)
          if (n){
            if (sign(n) != sign(by))
              stop("wrong sign of by")
          }else
            return(from)  # to == from
        }
      }
    }

    if (is.null(length.out)){
      if (is.null(along.with)){
        if (is.null(to) || is.null(from))
          stop("not enough info to guess the length.out")
        else{
          length.out <- n %/% by + 1L
        }
      }else{
        length.out <- length(along.with)
      }
    }else{
      length.out <- as.integer(length.out)
    }
    if (length.out){
      if (length.out==1L)
        from
      else
        cumsum(c(from, rep(by, length.out-1L)))
    }else
      integer()
  }
}



#' Function for chunked range index
#' 
#' creates a sequence of range indexes using a syntax not completely unlike
#' 'seq'
#' 
#' @param from the starting value of the sequence.
#' @param to the (maximal) end value of the sequence.
#' @param by increment of the sequence
#' @param length.out desired length of the sequence.
#' @param along.with take the length from the length of this argument.
#' @param overlap number of values to overlap (will lower the starting value of
#' the sequence, the first range becomes smaller
#' @param method default 'bbatch' will try to balance the chunk size, see
#' \code{\link{bbatch}}, 'seq' will create chunks like \code{\link[base]{seq}}
#' @param maxindex passed to \code{\link{ri}}
#' @return returns a named list of \code{\link{ri}} objects
#' representing chunks of subscripts
#' @author Jens Oehlschlägel
#' @seealso generic \code{\link{chunk}}, \code{\link{ri}}, \code{\link[base]{seq}}, \code{\link{bbatch}}
#' @keywords data
#' @examples
#' 
#'   chunks(1, 100, by=30)
#'   chunks(1, 100, by=30, method="seq")
#'    \dontrun{
#' require(foreach)
#' m <- 10000
#' k <- 1000
#' n <- m*k
#' message("Four ways to loop from 1 to n. Slowest foreach to fastest chunk is 1700:1 
#' on a dual core notebook with 3GB RAM\n")
#' z <- 0L; 
#' print(k*system.time({it <- icount(m); foreach (i = it) %do% { z <- i; NULL }}))
#' z
#' 
#' z <- 0L
#' print(system.time({i <- 0L; while (i<n) {i <- i + 1L; z <- i}}))
#' z
#' 
#' z <- 0L
#' print(system.time(for (i in 1:n) z <- i))
#' z
#' 
#' z <- 0L; n <- m*k; 
#' print(system.time(for (ch in chunks(1, n, by=m)){for (i in ch[1]:ch[2])z <- i}))
#' z
#' 
#' message("Seven ways to calculate sum(1:n). 
#'  Slowest foreach to fastest chunk is 61000:1 on a dual core notebook with 3GB RAM\n")
#' print(k*system.time({it <- icount(m); foreach (i = it, .combine="+") %do% { i }}))
#' 
#' z <- 0; 
#' print(k*system.time({it <- icount(m); foreach (i = it) %do% { z <- z + i; NULL }}))
#' z
#' 
#' z <- 0; print(system.time({i <- 0L;while (i<n) {i <- i + 1L; z <- z + i}})); z
#' 
#' z <- 0; print(system.time(for (i in 1:n) z <- z + i)); z
#' 
#' print(system.time(sum(as.double(1:n))))
#' 
#' z <- 0; n <- m*k
#' print(system.time(for (ch in chunks(1, n, by=m)){for (i in ch[1]:ch[2])z <- z + i}))
#' z
#' 
#' z <- 0; n <- m*k
#' print(system.time(for (ch in chunks(1, n, by=m)){z <- z+sum(as.double(ch[1]:ch[2]))}))
#' z
#'    }
#' 
#' @export

chunks <- function(
  from = NULL
  , to = NULL
  , by = NULL
  , length.out = NULL
  , along.with = NULL
  , overlap = 0L
  , method=c("bbatch","seq")
  , maxindex = NA
)
{
  method <- match.arg(method)
  if (!is.null(along.with)){
    if (is.null(from))
      from <- 1L
    else{
      
      if (length(from)==1)
        from <- as.integer(from)
      else
        stop("'from' must be scalar")
    }
    if (is.null(to))
      to <- length(along.with)
    else{
      if (length(to)==1)
        to <- as.integer(to)
      else
        stop("'to' must be scalar")
    }
  }
  if (length(from)==1)
    from <- as.integer(from)
  else
    stop("'from' must be scalar")
  if (length(to)==1)
    to <- as.integer(to)
  else
    stop("'to' must be scalar")
  
  if (to<from)
    stop("to < from")
  N <- to - from + 1L
  
  if (is.null(by)){
    if (is.null(length.out))
      stop("need either 'by' or 'length.out'")
    else{
      if (length(length.out)==1){
        length.out <- as.integer(length.out)
        if (length.out>N)
          length.out <- N
        by <- N %/% length.out
      }else
        stop("'length.out' must be scalar")
    }
  }else{
    if (length(by)==1){
      by <- as.integer(by)
      if (by<1)
        stop("'by' must be > 0")
      length.out <- (N - 1L) %/% by + 1L
    }else
      stop("'by' must be scalar")
  }
  
  if (method=="bbatch")
    by <- as.integer(bbatch(N, by)$b)
  
  if (length.out>1L){
    from <- cumsum(c(from, rep(by, length.out - 1L)))
    to <- c(from[-1], from[1] + N) - 1L  # fixed by Edwin de Jonge, 18.1.2011
    if (overlap>0)
      from[-1] <- from[-1] - overlap
  }
  n <- length(from)
  s <- seq_len(n)
  ret <- vector("list", n)
  for (i in s){
    ret[[i]] <- ri(from[i], to[i], maxindex)
  }
  names(ret) <- paste(from, to, sep=":")
  ret
}



#' Methods for chunked range index
#' 
#' Calls \code{\link{chunks}} to create a sequence of range indexes along the object which causes the method dispatch.
#' 
#' \code{chunk} is generic, the default method is described here, other methods
#' that automatically consider RAM needs are provided with package 'ff', see
#' for example \code{\link[ff]{chunk.ffdf}}
#' 
#' @param x the object along we want chunks
#' @param RECORDBYTES integer scalar representing the bytes needed to process a single element of the boolean vector (default 4 bytes for logical)
#' @param BATCHBYTES  integer scalar limiting the number of bytes to be processed in one chunk, default from \code{getOption("ffbatchbytes")} if not null, otherwise 16777216
#' @param \dots further arguments passed to \code{\link{chunks}}
#' @return returns a named list of \code{\link{ri}} objects 
#' representing chunks of subscripts
#' @section available methods: \code{chunk.default}, \code{\link[ff:chunk.ffdf]{chunk.ff_vector}}, \code{\link[ff]{chunk.ffdf}}
#' @author Jens Oehlschlägel
#' @seealso \code{\link{chunks}}, \code{\link{ri}}, \code{\link[base]{seq}}, \code{\link{bbatch}}
#' @keywords data
#' @examples
#'   chunk(complex(1e7))
#'   chunk(raw(1e7))
#'   chunk(raw(1e7), length=3)
#'   
#'   chunks(1,10,3)
#'   # no longer do
#'   chunk(1,100,10)
#'   # but for bckward compatibility this works
#'   chunk(from=1,to=100,by=10)
#'   
#' @export
chunk <- function(x = NULL, ...){
  if (is.null(x))
    return(chunks(...))
  UseMethod("chunk")
}
#' @describeIn chunk default vector method 
#' @export
chunk.default <- function(x = NULL, ..., RECORDBYTES = NULL, BATCHBYTES = NULL){
  if (is.null(BATCHBYTES)){
    BATCHBYTES <- getOption("ffbatchbytes", 16777216L)
  }
  if (is.null(RECORDBYTES)){
    RECORDBYTES <- if ("ff" %in% loadedNamespaces())
      get(".rambytes")[get("vmode")(x)]
    else c(raw = 1L
    , character = 4L
    , logical = 4L
    , integer = 4L
    , double = 8L
    , complex = 16L
    )[typeof(x)]
  }
  RECORDBYTES <- sum(RECORDBYTES)
  if (is.na(RECORDBYTES) || RECORDBYTES == 0L)
    stop("RECORDBYTES not above zero, probably due to unknown type resp. vmode")
  n <- length(x)
  if (n){
    l <- list(...)
    if (is.null(l$from))
      l$from <- 1L
    if (is.null(l$to))
      l$to <- n
    if (is.null(l$by) && is.null(l$len)){
      b <- pmin(BATCHBYTES %/% RECORDBYTES, .Machine$integer.max)
      if (b==0L){
        b <- 1L
        warning("single record does not fit into BATCHBYTES")
      }
      l$by <- b
    }
    l$maxindex <- n
    ret <- do.call("chunks", l)
  }else{
    ret <- list()
  }
  ret
}




#' Vectorized Sequences
#' 
#' \command{vecseq} returns concatenated multiple sequences
#' 
#' This is a generalization of \code{\link{sequence}} in that you can choose
#' sequence starts other than 1 and also have options to no concat and/or
#' return a call instead of the evaluated sequence.
#' 
#' @param x vector of sequence start points
#' @param y vector of sequence end points (if \code{is.null(y)} then \code{x}
#' are taken as endpoints, all starting at 1)
#' @param concat vector of sequence end points (if \code{is.null(y)} then
#' \code{x} are taken as endpoints, all starting at 1)
#' @param eval vector of sequence end points (if \code{is.null(y)} then
#' \code{x} are taken as endpoints, all starting at 1)
#' @return if \code{concat==FALSE} and \code{eval==FALSE} a list with n calls
#' that generate sequences \cr if \code{concat==FALSE} and \code{eval==TRUE } a
#' list with n sequences \cr if \code{concat==TRUE } and \code{eval==FALSE} a
#' single call generating the concatenated sequences \cr if \code{concat==TRUE
#' } and \code{eval==TRUE } an integer vector of concatentated sequences
#' @author Angelo Canty, Jens Oehlschlägel
#' @seealso \code{\link{:}}, \code{\link{seq}}, \code{\link{sequence}}
#' @keywords manip
#' @examples
#' 
#'   sequence(c(3,4))
#'   vecseq(c(3,4))
#'   vecseq(c(1,11), c(5, 15))
#'   vecseq(c(1,11), c(5, 15), concat=FALSE, eval=FALSE)
#'   vecseq(c(1,11), c(5, 15), concat=FALSE, eval=TRUE)
#'   vecseq(c(1,11), c(5, 15), concat=TRUE, eval=FALSE)
#'   vecseq(c(1,11), c(5, 15), concat=TRUE, eval=TRUE)
#' 
#' @export

vecseq <- function(x, y=NULL, concat=TRUE, eval=TRUE){
        if (missing(y)){
          y <- x
          x <- 1L
				}
        if (concat){
          if (eval){
						# pure R version was: eval(parse(text=paste("c(",paste(x,y,sep=":",collapse=","),")")))
						# now calling C-code
						nx <- length(x)
						ny <- length(y)
						if (nx<ny)
							x <- rep(as.integer(x), length.out=ny)
						if (ny<nx)
							y <- rep(as.integer(y), length.out=nx)
            .Call(C_R_bit_vecseq, as.integer(x),  as.integer(y)) 
          }else
            parse(text=paste("c(",paste(x,y,sep=":",collapse=","),")"))[[1]]
        }else{
          if (eval)
            eval(parse(text=paste("list(",paste(x,y,sep=":",collapse=","),")")))
          else
            as.list(parse(text=paste(x,y,sep=":")))
        }
}

# attribute utilities for ff and bit
# (c) 2010 Jens Oehlschlägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2007-08-24
# Last changed: 2007-10-25

# WARNING: these functions are called for the side-effect of changing their arguments
# this can save RAM and avoid unnecessary copying
# but it can easily have unexpected effects
# Only use them if you know what you do - and even then think twice

#' Attribute setting by reference
#' 
#' Function \code{setattr} sets a singe attribute and function
#' \code{setattributes} sets a list of attributes.
#' 
#' The attributes of 'x' are changed in place without copying x. function
#' \code{setattributes} does only change the named attributes, it does not
#' delete the non-names attributes like \code{\link{attributes}} does.
#' 
#' @param x an R object
#' @param which name of the attribute
#' @param value value of the attribute, use NULL to remove this attribute
#' @param attributes a named list of attribute values
#' @return invisible(), we do not return the changed object to remind you of
#' the fact that this function is called for its side-effect of changing its
#' input object.
#' @author Jens Oehlschlägel
#' @seealso \code{\link{attr}} \code{\link{unattr}}
#' @references Writing R extensions -- System and foreign language interfaces
#' -- Handling R objects in C -- Attributes (Version 2.11.1 (2010-06-03 ) R
#' Development)
#' @keywords attributes
#' @examples
#' 
#'   x <- as.single(runif(10))
#'   attr(x, "Csingle")
#' 
#'   f <- function(x)attr(x, "Csingle") <- NULL
#'   g <- function(x)setattr(x, "Csingle", NULL)
#' 
#'   f(x)
#'   x
#'   g(x)
#'   x
#' 
#'  \dontrun{
#' 
#'   # restart R
#'   library(bit)
#' 
#'   mysingle <- function(length = 0){
#'     ret <- double(length)
#'     setattr(ret, "Csingle", TRUE)
#'     ret
#'   }
#' 
#'   # show that mysinge gives exactly the same result as single
#'   identical(single(10), mysingle(10))
#' 
#'   # look at the speedup and memory-savings of mysingle compared to single
#'   system.time(mysingle(1e7))
#'   memory.size(max=TRUE)
#'   system.time(single(1e7))
#'   memory.size(max=TRUE)
#' 
#'   # look at the memory limits
#'   # on my win32 machine the first line fails beause of not enough RAM, the second works
#'   x <- single(1e8)
#'   x <- mysingle(1e8)
#' 
#'   # .g. performance with factors
#'   x <- rep(factor(letters), length.out=1e7)
#'   x[1:10]
#'   # look how fast one can do this
#'   system.time(setattr(x, "levels", rev(letters)))
#'   x[1:10]
#'   # look at the performance loss in time caused by the non-needed copying
#'   system.time(levels(x) <- letters)
#'   x[1:10]
#' 
#' 
#'   # restart R
#'   library(bit)
#' 
#'   simplefactor <- function(n){
#'     factor(rep(1:2, length.out=n))
#'   }
#' 
#'   mysimplefactor <- function(n){
#'     ret <- rep(1:2, length.out=n)
#'     setattr(ret, "levels", as.character(1:2))
#'     setattr(ret, "class", "factor")
#'     ret
#'   }
#' 
#'   identical(simplefactor(10), mysimplefactor(10))
#' 
#'   system.time(x <- mysimplefactor(1e7))
#'   memory.size(max=TRUE)
#'   system.time(setattr(x, "levels", c("a","b")))
#'   memory.size(max=TRUE)
#'   x[1:4]
#'   memory.size(max=TRUE)
#'   rm(x)
#'   gc()
#' 
#'   system.time(x <- simplefactor(1e7))
#'   memory.size(max=TRUE)
#'   system.time(levels(x) <- c("x","y"))
#'   memory.size(max=TRUE)
#'   x[1:4]
#'   memory.size(max=TRUE)
#'   rm(x)
#'   gc()
#' 
#' }
#' 
#' 
#' @export
getsetattr <- function(x, which, value)
{
  ret <- copy_vector(attr(x, which))
  .Call(C_R_bit_set_attr, x, which, value)
  ret
}

#' @describeIn getsetattr  
#' @export
setattr <- function(x, which, value)
{
  .Call(C_R_bit_set_attr, x, which, value)
  invisible()
}

#' @describeIn getsetattr  
#' @export
setattributes <- function(x, attributes)
{
  nam <- names(attributes)
  for (i in seq_len(length(attributes))){
    .Call(C_R_bit_set_attr, x, nam[[i]], attributes[[i]])
  }
  invisible()
}


#' Attribute removal
#' 
#' Returns object with attributes removed
#' 
#' attribute removal copies the object as usual
#' 
#' @param x any R object
#' @return a similar object with attributes removed
#' @author Jens Oehlschlägel
#' @seealso \code{\link{attributes}}, \code{\link{setattributes}},
#' \code{\link{unclass}}
#' @keywords attribute
#' @examples
#' 
#'   bit(2)[]
#'   unattr(bit(2)[])
#' 
#' @export

unattr <- function(x){
  attributes(x) <- NULL
  x
}

# Chunking utilities for bit and ff
# (c) 2007-2009 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2007-09-03
# Last changed: 2007-10-25

# source("D:/mwp/eanalysis/bit/R/chunkutil.R")




# non-vectorized
#bbatch <-
#function(N, B) {
#  N <- as.integer(N)
#  B <- as.integer(B)
#  RB <- N %% B
#  NB <- N %/% B
#  if (RB) {
#    cc <- min((B - RB) %/ % NB, (B - RB) % /% (NB + 1L))
#    if (cc) {
#      rb <- RB + cc * NB
#      b <- B - cc
#      if (rb == b) {
#        return(list(b=b, nb=NB + 1L, rb=0L))
#      } else {
#        return(list(b=b, nb=NB, rb=rb))
#      }
#    } else {
#      return(list(b=B, nb=NB, rb=RB))
#    }
#  } else {
#    return(list(b=B, nb=NB, rb=RB))
#  }
#}

#' Balanced Batch sizes
#'
#' `bbatch` calculates batch sizes in 1..N so that they have rather balanced
#' sizes than very different sizes.
#'
#' Tries to have `rb == 0` or `rb` as close to `b` as possible
#' while guaranteeing that `rb < b && (b - rb) <= min(nb, b)`
#'
#' @param N total size in 0..integer_max
#' @param B desired batch size in 1..integer_max
#' @return a list with components:
#'  - b: the batch size
#'  - nb: the number of batches
#'  - rb: the size of the rest
#'
#' @author Jens Oehlschlägel
#' @seealso [repfromto()], [ff::ffvecapply()]
#' @keywords IO data
#' @examples
#'
#'   bbatch(100, 24)
#'
#' @export
bbatch <- function(N, B) {
  if (any(B < 1))
    stop("B too small")
  if (any(N < 0))
    stop("N too small")
  N <- as.integer(N)
  B <- pmin(pmax(1L, N), as.integer(B))
  RB <- N %% B
  NB <- N %/% B
  cc <- pmin((B - RB) %/% NB, (B - RB) %/% (NB + 1L))
  cc[RB == 0 | NB == 0] <- 0L
  i <- cc > 0
  RB[i] <- RB[i] + cc[i] * NB[i]
  B[i] <- B[i] - cc[i]
  j <- i & (RB == B)
  NB[j] <- NB[j] + 1L
  RB[j] <- 0L
  i <- (RB > 0) & (NB == 0)
  B[i] <- RB[i]
  NB[i] <- 1L
  RB[i] <- 0L
  list(b=B, nb=NB, rb=RB)
}



#' Virtual recycling
#'
#' `repfromto` virtually recylcles object `x` and cuts out
#' positions `from .. to`
#'
#' `repfromto` is a generalization of [rep()], where
#' `rep(x, n) == repfromto(x, 1, n)`.  You can see this as an R-side
#' (vector) solution of the `mod_iterate` macro in arithmetic.c
#'
#' @param x an object from which to recycle
#' @param from first position to return
#' @param to last position to return
#' @param value value to assign
#' @return a vector of length `from - to + 1`
#' @author Jens Oehlschlägel
#' @seealso [rep()], [ff::ffvecapply()]
#' @keywords IO data
#' @examples
#'
#'   message("a simple example")
#'   repfromto(0:9, 11, 20)
#'
#' @export
repfromto <- function(x, from, to) {
  nx <- length(x)
  if (!nx) {
    return(NA[from:to])
  }
  from <- as.integer(from)
  to <- as.integer(to)
  if (to > nx) {
    N <- to - from + 1L
    from <- (from - 1L) %% nx + 1L
    to <- to %% nx
    # NOTE: fetch in sequence pre-main-post in case is.ff(x)
    if (from <= to && N < nx) {
      ret <- x[from:to]
    } else {
      pre <- x[from:nx]
      nrep <- (N - length(pre) - to) %/% nx
      main <- if (nrep) rep(x[1:nx], nrep)
      post <- if (to) x[1:to]
      ret <- c(pre, main, post)
    }
  } else {
    ret <- x[from:to]
  }
  a <- attributes(x[1])
  a$names <- NULL
  attributes(ret) <- a
  ret
}

#' @rdname repfromto
#' @export
`repfromto<-` <- function(x, from, to, value) {
  x[from:to] <- value
  x
}


if (FALSE) {
  x <- 1:10
  for (n in 1:20)
  for (i1 in 1:30) {
    i2 <- i1 + n - 1
    cat(i1, i2, "|", repfromto(x, i1, i2), "\n")
  }
}

if (FALSE) {
  intseq <- function(from=NULL, to=NULL, by=NULL, length.out=NULL, along.with=NULL) {
    if (is.null(from)) {
      if (is.null(to))
        stop("need 'from' or 'to'")
      else
        to <- as.integer(to)
      if (is.null(by))
        by <- 1L
      else
        by <- as.integer(by)
    } else {
      from <- as.integer(from)
      if (is.null(to)) {
        if (is.null(by))
          by <- 1L
        else
          by <- as.integer(by)
      } else {
        to <- as.integer(to)
        n <- to - from
        if (is.null(by)) {
          if (to < from)
            by = -1L
          else
            by = 1L
        } else {
          by <- as.integer(by)
          if (!n) return(from) # to == from
          if (sign(n) != sign(by))
            stop("wrong sign of by")
        }
      }
    }

    if (is.null(length.out)) {
      if (is.null(along.with)) {
        if (is.null(to) || is.null(from))
          stop("not enough info to guess the length.out")
        length.out <- n %/% by + 1L
      } else {
        length.out <- length(along.with)
      }
    } else {
      length.out <- as.integer(length.out)
    }
    if (length.out) {
      if (length.out == 1L)
        from
      else
        cumsum(c(from, rep(by, length.out - 1L)))
    } else {
      integer()
    }
  }
}



#' Function for chunked range index
#'
#' creates a sequence of range indexes using a syntax not completely unlike
#' 'seq'
#'
#' @param from the starting value of the sequence.
#' @param to the (maximal) end value of the sequence.
#' @param by increment of the sequence
#' @param length.out desired length of the sequence.
#' @param along.with take the length from the length of this argument.
#' @param overlap number of values to overlap (will lower the starting value of
#' the sequence, the first range becomes smaller
#' @param method default 'bbatch' will try to balance the chunk size, see
#' [bbatch()], 'seq' will create chunks like [`seq()`][base::seq]
#' @param maxindex passed to [ri()]
#' @return returns a named list of [ri()] objects
#' representing chunks of subscripts
#' @author Jens Oehlschlägel
#' @seealso generic [chunk()], [ri()], [`seq()`][base::seq], [bbatch()]
#' @keywords data
#' @examples
#'
#'   chunks(1, 100, by=30)
#'   chunks(1, 100, by=30, method="seq")
#'    \dontrun{
#' require(foreach)
#' m <- 10000
#' k <- 1000
#' n <- m*k
#' message("Four ways to loop from 1 to n. Slowest foreach to fastest chunk is 1700:1
#' on a dual core notebook with 3GB RAM\n")
#' z <- 0L;
#' print(k*system.time({it <- icount(m); foreach (i = it) %do% { z <- i; NULL }}))
#' z
#'
#' z <- 0L
#' print(system.time({i <- 0L; while (i < n) {i <- i + 1L; z <- i}}))
#' z
#'
#' z <- 0L
#' print(system.time(for (i in 1:n) z <- i))
#' z
#'
#' z <- 0L; n <- m*k;
#' print(system.time(for (ch in chunks(1, n, by=m)) {for (i in ch[1]:ch[2]) z <- i}))
#' z
#'
#' message("Seven ways to calculate sum(1:n).
#'  Slowest foreach to fastest chunk is 61000:1 on a dual core notebook with 3GB RAM\n")
#' print(k*system.time({it <- icount(m); foreach (i = it, .combine="+") %do% { i }}))
#'
#' z <- 0;
#' print(k*system.time({it <- icount(m); foreach (i = it) %do% { z <- z + i; NULL }}))
#' z
#'
#' z <- 0; print(system.time({i <- 0L;while (i < n) {i <- i + 1L; z <- z + i}})); z
#'
#' z <- 0; print(system.time(for (i in 1:n) z <- z + i)); z
#'
#' print(system.time(sum(as.double(1:n))))
#'
#' z <- 0; n <- m*k
#' print(system.time(for (ch in chunks(1, n, by=m)) {for (i in ch[1]:ch[2]) z <- z + i}))
#' z
#'
#' z <- 0; n <- m*k
#' print(system.time(for (ch in chunks(1, n, by=m)) {z <- z + sum(as.double(ch[1]:ch[2]))}))
#' z
#'    }
#'
#' @export
chunks <- function(
  from = NULL,
  to = NULL,
  by = NULL,
  length.out = NULL,
  along.with = NULL,
  overlap = 0L,
  method=c("bbatch", "seq"),
  maxindex = NA
) {
  method <- match.arg(method)
  if (!is.null(along.with)) {
    if (is.null(from))
      from <- 1L
    else if (length(from) == 1)
      from <- as.integer(from)
    else
      stop("'from' must be scalar")
    if (is.null(to))
      to <- length(along.with)
    else if (length(to) == 1)
      to <- as.integer(to)
    else
      stop("'to' must be scalar")
  }
  if (length(from) == 1)
    from <- as.integer(from)
  else
    stop("'from' must be scalar")
  if (length(to) == 1)
    to <- as.integer(to)
  else
    stop("'to' must be scalar")

  if (to < from)
    stop("to < from")
  N <- to - from + 1L

  if (is.null(by)) {
    if (is.null(length.out))
      stop("need either 'by' or 'length.out'")
    if (length(length.out) != 1L)
      stop("'length.out' must be scalar")
    length.out <- as.integer(length.out)
    if (length.out > N)
      length.out <- N
    by <- N %/% length.out
  } else if (length(by) == 1) {
    by <- as.integer(by)
    if (by < 1)
      stop("'by' must be > 0")
    length.out <- (N - 1L) %/% by + 1L
  } else {
    stop("'by' must be scalar")
  }

  if (method == "bbatch")
    by <- as.integer(bbatch(N, by)$b)

  if (length.out > 1L) {
    from <- cumsum(c(from, rep(by, length.out - 1L)))
    to <- c(from[-1], from[1] + N) - 1L  # fixed by Edwin de Jonge, 18.1.2011
    if (overlap > 0)
      from[-1] <- from[-1] - overlap
  }
  n <- length(from)
  s <- seq_len(n)
  ret <- vector("list", n)
  for (i in s) {
    ret[[i]] <- ri(from[i], to[i], maxindex)
  }
  names(ret) <- paste(from, to, sep=":")
  ret
}



#' Methods for chunked range index
#'
#' Calls [chunks()] to create a sequence of range indexes along the object which causes
#'   the method dispatch.
#'
#' `chunk` is generic, the default method is described here, other methods
#'   that automatically consider RAM needs are provided with package 'ff', see
#'   for example [ff::chunk.ffdf()]
#'
#' @param x the object along we want chunks
#' @param RECORDBYTES integer scalar representing the bytes needed to process a single
#'   element of the boolean vector (default 4 bytes for logical)
#' @param BATCHBYTES  integer scalar limiting the number of bytes to be processed in one
#'   chunk, default from `getOption("ffbatchbytes")` if not null, otherwise 16777216
#' @param ... further arguments passed to [chunks()]
#' @return returns a named list of [ri()] objects
#'   representing chunks of subscripts
#' @section available methods: `chunk.default`, [ff::chunk.ff_vector()],
#'   [ff::chunk.ffdf()]
#' @author Jens Oehlschlägel
#' @seealso [chunks()], [ri()], [`seq()`][base::seq], [bbatch()]
#' @keywords data
#' @examples
#'   chunk(complex(1e7))
#'   chunk(raw(1e7))
#'   chunk(raw(1e7), length=3)
#'
#'   chunks(1, 10, 3)
#'   # no longer do
#'   chunk(1, 100, 10)
#'   # but for bckward compatibility this works
#'   chunk(from=1, to=100, by=10)
#'
#' @export
chunk <- function(x = NULL, ...) {
  if (is.null(x))
    return(chunks(...))
  UseMethod("chunk")
}
#' @describeIn chunk default vector method
#' @export
chunk.default <- function(x = NULL, ..., RECORDBYTES = NULL, BATCHBYTES = NULL) {
  if (is.null(BATCHBYTES)) {
    BATCHBYTES <- getOption("ffbatchbytes", 16777216L)
  }
  if (is.null(RECORDBYTES)) {
    if (isNamespaceLoaded("ff"))
      RECORDBYTES = ff::.rambytes[ff::vmode(x)]
    else
      RECORDBYTES =
        switch(typeof(x), raw=1L, character=4L, logical=4L, integer=4L, double=8L, complex=16L)
  }
  RECORDBYTES <- sum(RECORDBYTES)
  if (is.na(RECORDBYTES) || RECORDBYTES == 0L)
    stop("RECORDBYTES not above zero, probably due to unknown type resp. vmode")
  n <- length(x)
  if (n) {
    l <- list(...)
    if (is.null(l$from))
      l$from <- 1L
    if (is.null(l$to))
      l$to <- n
    if (is.null(l$by) && is.null(l$len)) {
      b <- pmin(BATCHBYTES %/% RECORDBYTES, .Machine$integer.max)
      if (b == 0L) {
        b <- 1L
        warning("single record does not fit into BATCHBYTES")
      }
      l$by <- b
    }
    l$maxindex <- n
    ret <- do.call("chunks", l)
  } else {
    ret <- list()
  }
  ret
}

#' Vectorized Sequences
#'
#' `vecseq` returns concatenated multiple sequences
#'
#' This is a generalization of [sequence()] in that you can choose
#' sequence starts other than 1 and also have options to no concat and/or
#' return a call instead of the evaluated sequence.
#'
#' @param x vector of sequence start points
#' @param y vector of sequence end points (if `is.null(y)` then `x`
#' are taken as endpoints, all starting at 1)
#' @param concat vector of sequence end points (if `is.null(y)` then
#' `x` are taken as endpoints, all starting at 1)
#' @param eval vector of sequence end points (if `is.null(y)` then
#' `x` are taken as endpoints, all starting at 1)
#' @return
#'  - if `concat == FALSE` and `eval == FALSE` a list with n calls that generate sequences
#'  - if `concat == FALSE` and `eval == TRUE ` a list with n sequences
#'  - if `concat == TRUE ` and `eval == FALSE` a single call generating the concatenated
#'    sequences
#'  - if `concat == TRUE` and `eval == TRUE ` an integer vector of concatentated sequences
#' @author Angelo Canty, Jens Oehlschlägel
#' @seealso [`:`][:], [seq()], [sequence()]
#' @keywords manip
#' @examples
#'
#'   sequence(c(3, 4))
#'   vecseq(c(3, 4))
#'   vecseq(c(1, 11), c(5, 15))
#'   vecseq(c(1, 11), c(5, 15), concat=FALSE, eval=FALSE)
#'   vecseq(c(1, 11), c(5, 15), concat=FALSE, eval=TRUE)
#'   vecseq(c(1, 11), c(5, 15), concat=TRUE, eval=FALSE)
#'   vecseq(c(1, 11), c(5, 15), concat=TRUE, eval=TRUE)
#'
#' @export
vecseq <- function(x, y=NULL, concat=TRUE, eval=TRUE) {
  if (missing(y)) {
    y <- x
    x <- 1L
  }
  if (concat) {
    if (eval) {
      # pure R version was: eval(parse(text=paste("c(", paste(x, y, sep=":", collapse=", "), ")")))
      # now calling C-code
      nx <- length(x)
      ny <- length(y)
      if (nx < ny)
        x <- rep(as.integer(x), length.out=ny)
      if (ny < nx)
        y <- rep(as.integer(y), length.out=nx)
      .Call(C_R_bit_vecseq, as.integer(x),  as.integer(y))
    } else {
      parse(text=paste("c(", paste(x, y, sep=":", collapse=", "), ")"))[[1]]
    }
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (eval)
      eval(parse(text=paste("list(", paste(x, y, sep=":", collapse=", "), ")")))
    else
      as.list(parse(text=paste(x, y, sep=":")))
  }
}

# timing utilities for ff and bit
# (c) 2012 Jens Oehlschlägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2012-05-28

#' Adaptive timer
#' 
#' Repeats timing expr until minSec is reached
#' 
#' 
#' @param expr Valid expression to be timed.
#' @param gcFirst Logical - should a garbage collection be performed
#' immediately before the timing?  Default is \code{TRUE}.
#' @param minSec number of seconds to repeat at least
#' @param envir the environment in which to evaluate \code{expr} (by default
#' the calling frame)
#' @return A object of class \code{"proc_time"}: see \code{\link{proc.time}}
#' for details.
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso \code{\link{system.time}}
#' @keywords utilities
#' @examples
#' 
#'   system.time(1+1)
#'   repeat.time(1+1)
#'   system.time(sort(runif(1e6)))
#'   repeat.time(sort(runif(1e6)))
#' 
#' @export
repeat.time <- function (expr, gcFirst = TRUE, minSec=0.5, envir=parent.frame()) 
{
	ppt <- function(y) {
		if (!is.na(y[4L])) 
			y[1L] <- y[1L] + y[4L]
		if (!is.na(y[5L])) 
			y[2L] <- y[2L] + y[5L]
		y[1L:3L]
	}
	if (!exists("proc.time")) 
		return(rep(NA_real_, 5L))
	if (gcFirst) 
		gc(FALSE)
	time <- proc.time()
	on.exit(cat("Timing stopped at:", ppt(proc.time() - time), 
		"\n"))
	r <- 0L
	while((proc.time()[3]-time[3]) < minSec){
		r <- r + 1L
		eval(substitute(expr), envir=envir)
	}	
	new.time <- proc.time()
	on.exit()
	structure((new.time - time)/r, class = "proc_time")
}	

# clone utilities for bit,bit64,ff
# (c) 2014 Jens Oehlschlägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2014-03-02


#' Cloning ff and ram objects
#' 
#' \command{clone} physically duplicates objects and can additionally change
#' some features, e.g. length.
#' 
#' \command{clone} is generic.  \command{clone.default} handles ram objects. 
#' Further methods are provided in package 'ff'. 
#' \command{still.identical} returns TRUE if the two atomic arguments still
#' point to the same memory.
#' 
#' @param x \code{x} an R object
#' @param \dots further arguments to the generic
#' @return an object that is a deep copy of x
#' @author Jens Oehlschlägel
#' @seealso \code{clone.ff}, \code{\link{copy_vector}}
#' @keywords IO data
#' @examples
#' 
#'   x <- 1:12
#'   y <- x
#'   still.identical(x,y)
#'   y[1] <- y[1]
#'   still.identical(x,y)
#'   y <- clone(x)
#'   still.identical(x,y)
#'   rm(x,y); gc()
#' 
#' @export
clone  <- function(x, ...){
  UseMethod("clone")
}

#' @describeIn clone default method uses R's C-API 'duplicate()'
#' @export
clone.default <- function(
  x
, ... # ignored here
){
  .Call(C_R_duplicate, x)
}

# timing utilities for ff and bit
# (c) 2012 Jens Oehlschlägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2012-05-28

#' Adaptive timer
#'
#' Repeats timing expr until minSec is reached
#'
#'
#' @param expr Valid expression to be timed.
#' @param gcFirst Logical - should a garbage collection be performed
#' immediately before the timing?  Default is `TRUE`.
#' @param minSec number of seconds to repeat at least
#' @param envir the environment in which to evaluate `expr` (by default
#' the calling frame)
#' @return A object of class `"proc_time"`: see [proc.time()]
#' for details.
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso [system.time()]
#' @keywords utilities
#' @examples
#'
#'   system.time(1 + 1)
#'   repeat.time(1 + 1)
#'   system.time(sort(runif(1e6)))
#'   repeat.time(sort(runif(1e6)))
#'
#' @export
repeat.time <- function(expr, gcFirst = TRUE, minSec=0.5, envir=parent.frame()) {
  ppt <- function(y) {
    if (!is.na(y[4L]))
      y[1L] <- y[1L] + y[4L]
    if (!is.na(y[5L]))
      y[2L] <- y[2L] + y[5L]
    y[1L:3L]
  }
  if (!exists("proc.time"))
    return(rep(NA_real_, 5L))
  if (gcFirst)
    gc(FALSE)
  time0 <- proc.time()
  on.exit(cat("Timing stopped at:", ppt(proc.time() - time0), "\n"))
  r <- 0L
  while ((proc.time()[3L] - time0[3L]) < minSec) {
    r <- r + 1L
    eval(substitute(expr), envir=envir)
  }
  new.time <- proc.time()
  on.exit()
  out = (new.time - time0) / r
  class(out) = "proc_time"
  out
}

# rle utilities for bit and ff
# (c) 2007-2009 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2007-09-03
# Last changed: 2007-10-25

# source("D:/mwp/eanalysis/bit/R/rle.R")

#' Hybrid Index, C-coded utilities
#' 
#' These C-coded utilitites speed up index preprocessing considerably. 
#' 
#' \code{intrle} is by factor 50 faster and needs less RAM (2x its input
#' vector) compared to \code{\link{rle}} which needs 9x the RAM of its input
#' vector.  This is achieved because we allow the C-code of \code{intrle} to
#' break when it turns out, that rle-packing will not achieve a compression
#' factor of 3 or better.  \cr \code{intisasc} is a faster version of
#' \code{\link{is.unsorted}}: it checks whether \code{x} is sorted.\cr 
#' \code{intisdesc} checks for being sorted descending and 
#' by default default assumes that the input \code{x} contains no NAs. 
#' \code{na.method="none"} treats \code{NAs} (the smallest integer) like every other integer and hence returns either \code{TRUE} or \code{FALSE}
#' \code{na.method="break"} checks for \code{NAs} and returns either \code{NA} as soon as  \code{NA} is encountered. 
#' \code{na.method="skip"} checks for \code{NAs} and skips over them, hence decides the return value only on the basis of non-NA values. 
#' 
#' 
#' @param x an integer vector
#' @param na.method one of "none","break","skip", see details. The strange defaults stem from the initial usage. 
#' @return \code{intrle} returns an object of class \code{\link{rle}} or NULL,
#' if rle-compression is not efficient (compression factor <3 or length(x)<3).
#' \cr \code{intisasc} returns one of \code{FALSE, NA, TRUE} \cr
#' \code{intisdesc} returns one of \code{FALSE, TRUE} (if the input contains
#' NAs, the output is undefined)
#' @author Jens Oehlschlägel
#' @seealso \code{\link[ff]{hi}}, \code{\link{rle}}, \code{\link{is.unsorted}},
#' \code{\link[ff]{is.sorted}}
#' @keywords IO data
#' @examples
#' 
#'   intrle(sample(1:10))
#'   intrle(diff(1:10))
#'   intisasc(1:10)
#'   intisasc(10:1)
#'   intisasc(c(NA, 1:10))
#'   intisdesc(1:10)
#'   intisdesc(c(10:1, NA))
#'   intisdesc(c(10:6, NA, 5:1))
#'   intisdesc(c(10:6, NA, 5:1), na.method="skip")
#'   intisdesc(c(10:6, NA, 5:1), na.method="break")
#' 
#' @export
# -- fast and efficient rle ------------------

# integer only
# returns rle object only if n>2 && rle is efficient (length(values)+lengths(lengths))<=length(x)
# returns NULL if n<3 || rle is inefficient
intrle <- function(x){
  stopifnot(is.integer(x))
  .Call(C_R_int_rle, x)
}


# -- check for sorting and NAs, 0s can be checked later when sorted ------------------

#' @describeIn intrle check whether integer vector is ascending
#' @export
intisasc <- function(x, na.method=c("none","break","skip")[2]){
  stopifnot(is.integer(x))
  if (na.method=="break")
    .Call(C_R_int_is_asc_break, x)
  else if (na.method=="none")
    .Call(C_R_int_is_asc_none, x)
  else
    .Call(C_R_int_is_asc_skip, x)
}

#' @describeIn intrle check whether integer vector is descending
#' @export
intisdesc <- function(x, na.method=c("none","break","skip")[1]){
  stopifnot(is.integer(x))
  if (na.method=="none")
    .Call(C_R_int_is_desc_none, x)
  else if (na.method=="break")
    .Call(C_R_int_is_desc_break, x)
  else
    .Call(C_R_int_is_desc_skip, x)
}




# -- basic sequence packing and unpacking ---------------------------------------------------

#' Hybrid Index, rle-pack utilities
#' 
#' Basic utilities for rle packing and unpacking and apropriate methods for
#' \code{\link{rev}} and \code{\link{unique}}.
#' 
#' 
#' @param x in 'rlepack' an integer vector, in the other functions an object of
#' class 'rlepack'
#' @param pack FALSE to suppress packing
#' @param incomparables just to keep R CMD CHECK quiet (not used)
#' @param \dots just to keep R CMD CHECK quiet (not used)
#' @return A list with components \item{ first }{ the first element of the
#' packed sequence } \item{ dat }{ either an object of class \code{\link{rle}}
#' or the complete input vector \code{x} if rle-packing is not efficient }
#' \item{ last }{ the last element of the packed sequence }
#' @author Jens Oehlschlägel
#' @seealso \code{\link[ff]{hi}}, \code{\link{intrle}}, \code{\link{rle}},
#' \code{\link{rev}}, \code{\link{unique}}
#' @keywords IO data
#' @examples
#' 
#'   x <- rlepack(rep(0L, 10))
#' \dontshow{
#'  for (x in list(10:1, 1:10, c(10:1,1:10), c(1:10,10:1), sample(100), sample(100, 100, TRUE), sample(10, 100, TRUE))){
#'    stopifnot(identical(rleunpack(rlepack(x)), x))
#'    stopifnot(identical(rleunpack(unique(rlepack(x))), unique(x)))
#'    stopifnot(identical(anyDuplicated(rlepack(x)), anyDuplicated(x)))
#'  }
#' }
#' 
#' @export
rlepack <- function(x, ...) 
UseMethod("rlepack")

#' @rdname rlepack
#' @export
rlepack.integer <- function(
  x
, pack = TRUE   # TRUE / FALSE
, ... # dummy to keep R CMD check quiet
){
  stopifnot(is.integer(x))
  n <- length(x)
  if (n>1){
    if (pack)
      r <- intrle(diff(x))  # returns NULL if rle is inefficient, old condition was 2*length(r$lengths)<n
    else
      r <- NULL
    structure(list(first=x[1], dat=if (is.null(r)) x else r, last=x[n]), class="rlepack")
  }else if (n==1){
    structure(list(first=x[1], dat=x, last=x[1]), class="rlepack")
  }else{
    structure(list(first=NA_integer_, dat=x, last=NA_integer_), class="rlepack")
  }
}

#' @rdname rlepack
#' @export
rleunpack <- function(x) 
UseMethod("rleunpack")

#' @rdname rlepack
#' @export
rleunpack.rlepack <- function(x){
  if (inherits(x$dat, "rle"))
    as.integer(cumsum(c(x$first, rep(x$dat$values, x$dat$lengths))))
  else
    x$dat
}


#' @rdname rlepack
#' @export
rev.rlepack <- function(x){
  if (inherits(x$dat,"rle")){
    x$dat$values <- -rev(x$dat$values)
    x$dat$lengths <- rev(x$dat$lengths)
  }else{
    x$dat <- rev(x$dat)
  }
  buf <- x$first
  x$first <- x$last
  x$last <- buf
  x
}


# beware: only for sorted input identical with unique()
# beware: rlepack(unique(x)) is faster than unique(rlepack(x))
# we use this only in hi() and as.hi.default()
#' @rdname rlepack
#' @export
unique.rlepack <- function(x
, incomparables = FALSE # dummy to keep R CMD check quiet
, ... # dummy to keep R CMD check quiet
){
  if (inherits(x$dat,"rle")){
    tab <- tabulate(sign(x$dat$values)+2L, nbins=3L)
    if (tab[1] && tab[3])
      x <- rlepack(unique(rleunpack(x)))
    else if (tab[2]){
      x$dat$lengths <- x$dat$lengths[x$dat$values != 0]
      x$dat$values <- x$dat$values[x$dat$values != 0]
    }
    # else nothing to do: no repeated values
  }else{
    x$dat <- unique(x$dat)
  }
  x
}


# beware: only for sorted input identical with unique()
# beware: returns TRUE/FALSE, not position of first duplicate
#' @rdname rlepack
#' @export
anyDuplicated.rlepack <- function(x
, incomparables = FALSE # dummy to keep R CMD check quiet
, ... # dummy to keep R CMD check quiet
){
  if (inherits(x$dat,"rle")){
    tab <- tabulate(sign(x$dat$values)+2L, nbins=3L)
    if (tab[1] && tab[3])
      anyDuplicated(rleunpack(x))
    else if (tab[2]){
      w <- .Call(C_R_first_zero, x$dat$values)
      if (w)
        if(w>1L)
          sum(x$dat$lengths[1:(w-1L)]) + 2L
        else 
          2L
      else
        0L
    }
    else
      0L
  }else{
    anyDuplicated(x$dat)
  }
}

test_that("intisasc is correct", {
  expect_true(intisasc(c(as.integer(NA, NA)), "none"))
  expect_true(intisasc(c(NA, 1L), "none"))
  expect_true(intisasc(c(1L, 1L), "none"))
  expect_false(intisasc(c(1L, NA), "none"))
  expect_true(intisasc(c(NA, 1L, NA, 2L, NA), "skip"))
  expect_true(intisasc(c(NA, 1L, NA, 1L, NA), "skip"))
  expect_false(intisasc(c(NA, 2L, NA, 1L, NA), "skip"))
  expect_identical(intisasc(c(1L, NA, 2L), "break"), NA)
  expect_true(intisasc(c(1L, 2L), "break"))
  expect_true(intisasc(c(1L, 1L), "break"))
  expect_false(intisasc(c(2L, 1L), "break"))
  expect_true(intisasc(c(1L, 2L)))
  expect_true(intisasc(c(1L, 1L)))
  expect_false(intisasc(c(2L, 1L)))
})

test_that("intisdesc is correct", {
  expect_true(intisdesc(c(as.integer(NA, NA)), "none"))
  expect_false(intisdesc(c(NA, 1L), "none"))
  expect_true(intisdesc(c(1L, 1L), "none"))
  expect_true(intisdesc(c(1L, NA), "none"))
  expect_false(intisdesc(c(NA, 1L, NA, 2L, NA), "skip"))
  expect_true(intisdesc(c(NA, 1L, NA, 1L, NA), "skip"))
  expect_true(intisdesc(c(NA, 2L, NA, 1L, NA), "skip"))
  expect_identical(intisdesc(c(1L, NA, 2L), "break"), NA)
  expect_false(intisdesc(c(1L, 2L), "break"))
  expect_true(intisdesc(c(1L, 1L), "break"))
  expect_true(intisdesc(c(2L, 1L), "break"))
  expect_true(intisdesc(c(as.integer(NA, NA))))
  expect_false(intisdesc(c(NA, 1L)))
  expect_true(intisdesc(c(1L, 1L)))
  expect_true(intisdesc(c(1L, NA)))
})

test_that("intrle is correct", {
  x = c(rep(1L, 60), 1:30)
  expect_identical(intrle(x), rle(x))
  expect_null(intrle(c(rep(1L, 60), 1:31)))
})

test_that("rlepack, rleunpack, and rlepack methods for rev, uniqu and anyDuplicated work", {
  inputs = list(
    10:1,
    1:10,
    c(10:1, 1:10),
    c(1:10, 10:1),
    sample(100),
    sample(100, 100, TRUE),
    sample(10, 100, TRUE)
  )
  for (x in inputs) {
    expect_identical(rleunpack(rlepack(x)), x)
    expect_identical(rleunpack(rev(rlepack(x))), rev(x))
    expect_identical(rleunpack(unique(rlepack(x))), unique(x))
    expect_identical(anyDuplicated(rlepack(x)), anyDuplicated(x))
  }
})

library("testthat")
library("bit")

context("rle")

test_that("intisasc is correct", {
  expect_identical(intisasc(c(as.integer(NA,NA)), "none"), TRUE)
  expect_identical(intisasc(c(NA,1L), "none"), TRUE)
  expect_identical(intisasc(c(1L,1L), "none"), TRUE)
  expect_identical(intisasc(c(1L,NA), "none"), FALSE)
  expect_identical(intisasc(c(NA,1L,NA,2L,NA), "skip"), TRUE)
  expect_identical(intisasc(c(NA,1L,NA,1L,NA), "skip"), TRUE)
  expect_identical(intisasc(c(NA,2L,NA,1L,NA), "skip"), FALSE)
  expect_identical(intisasc(c(1L,NA,2L), "break"), NA)
  expect_identical(intisasc(c(1L,2L), "break"), TRUE)
  expect_identical(intisasc(c(1L,1L), "break"), TRUE)
  expect_identical(intisasc(c(2L,1L), "break"), FALSE)
  expect_identical(intisasc(c(1L,2L)), TRUE)
  expect_identical(intisasc(c(1L,1L)), TRUE)
  expect_identical(intisasc(c(2L,1L)), FALSE)
})

test_that("intisdesc is correct", {
  expect_identical(intisdesc(c(as.integer(NA,NA)), "none"), TRUE)
  expect_identical(intisdesc(c(NA,1L), "none"), FALSE)
  expect_identical(intisdesc(c(1L,1L), "none"), TRUE)
  expect_identical(intisdesc(c(1L,NA), "none"), TRUE)
  expect_identical(intisdesc(c(NA,1L,NA,2L,NA), "skip"), FALSE)
  expect_identical(intisdesc(c(NA,1L,NA,1L,NA), "skip"), TRUE)
  expect_identical(intisdesc(c(NA,2L,NA,1L,NA), "skip"), TRUE)
  expect_identical(intisdesc(c(1L,NA,2L), "break"), NA)
  expect_identical(intisdesc(c(1L,2L), "break"), FALSE)
  expect_identical(intisdesc(c(1L,1L), "break"), TRUE)
  expect_identical(intisdesc(c(2L,1L), "break"), TRUE)
  expect_identical(intisdesc(c(as.integer(NA,NA))), TRUE)
  expect_identical(intisdesc(c(NA,1L)), FALSE)
  expect_identical(intisdesc(c(1L,1L)), TRUE)
  expect_identical(intisdesc(c(1L,NA)), TRUE)
})

# takes_less_than has been deprecated
# test_that("intisasc and intisdesc early terminate correctly", {
#   x <- y <- integer(1e7)
#   x[1] <- NA
#   expect_that(intisasc(x, "break"), takes_less_than(max(0.01,system.time(intisasc(y, "break"))[3])))
#   expect_that(intisdesc(x, "break"), takes_less_than(max(0.01,system.time(intisasc(y, "break"))[3])))
#   x[1] <- 1L
#   expect_that(intisasc(x, "skip"), takes_less_than(max(0.01,system.time(intisasc(y, "skip"))[3])))
#   expect_that(intisasc(x, "none"), takes_less_than(max(0.01,system.time(intisasc(y, "none"))[3])))
#   x[1] <- -1L
#   expect_that(intisdesc(x, "skip"), takes_less_than(max(0.01,system.time(intisasc(y, "skip"))[3])))
#   expect_that(intisdesc(x, "none"), takes_less_than(max(0.01,system.time(intisasc(y, "none"))[3])))
# })

test_that("intrle is correct", {
  expect_identical(intrle(c(rep(1L,60), 1:30))
  , structure(list(lengths = c(61L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L), values = 1:30)
  , .Names = c("lengths", "values"), class = "rle"))
  expect_identical(intrle(c(rep(1L,60), 1:31)), NULL)
})

test_that("rlepack, rleunpack rev.rlepack, unique.rlepack and anyDuplicated.rlepack work correctly", {
  for (x in list(10:1, 1:10, c(10:1,1:10), c(1:10,10:1), sample(100), sample(100, 100, TRUE), sample(10, 100, TRUE))){
    expect_identical(rleunpack(rlepack(x)), x)
    expect_identical(rleunpack(rev(rlepack(x))), rev(x))
    expect_identical(rleunpack(unique(rlepack(x))), unique(x))
    expect_identical(anyDuplicated(rlepack(x)), anyDuplicated(x))
  }
})

# clone utilities for bit,bit64,ff
# (c) 2014 Jens Oehlschlägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2014-03-02


#' Cloning ff and ram objects
#'
#' `clone` physically duplicates objects and can additionally change
#' some features, e.g. length.
#'
#' `clone` is generic.  `clone.default` handles ram objects.
#' Further methods are provided in package 'ff'.
#' `still.identical` returns TRUE if the two atomic arguments still
#' point to the same memory.
#'
#' @param x `x` an R object
#' @param ... further arguments to the generic
#' @return an object that is a deep copy of x
#' @author Jens Oehlschlägel
#' @seealso `clone.ff`, [copy_vector()]
#' @keywords IO data
#' @examples
#'
#'   x <- 1:12
#'   y <- x
#'   still.identical(x, y)
#'   y[1] <- y[1]
#'   still.identical(x, y)
#'   y <- clone(x)
#'   still.identical(x, y)
#'   rm(x, y); gc()
#'
#' @export
clone  <- function(x, ...) UseMethod("clone")

#' @describeIn clone default method uses R's C-API 'duplicate()'
#' @export
clone.default <- function(x, ...) .Call(C_R_duplicate, x)

# Integer utilities
# (c) 2016-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

## Get C reference count of a vector
##
## Gets C reference count of a vector
## 
## Queries the vector reference count using C-macro \code{NAMED}
## 
## @param x a vector
##   
## @return integer scalar
## @seealso \code{\link{get_parent_refcnt}}, \code{\link{set_refcnt}}
##
## @examples
## x <- integer()
## get_refcnt(x)
## y <- x
## get_refcnt(x)
## z <- x
## get_refcnt(x)
## 
#get_refcnt <- function(x){
#  y <- substitute(x)
#  if (is.name(y)){
#    .Call(C_R_get_refcnt, get(as.character(y), envir=parent.frame()))
#  } else 
#    stop("x must be the name of an object")
#}

## Set C reference count of a vector
##
## Sets C reference count of a vector
## 
## Manipulates the vector reference count using C-macro \code{SET_NAMED}
## 
## @note Do use this only if you know what you do! Otherwise unexpected behaviour can result!
## 
## @param x a vector
## @param refcnt integer scalar to be assigned (must be 1L or 2L) 
##   
## @return integer scalar
## @seealso @seealso \code{\link{get_refcnt}}, \code{\link{get_parent_refcnt}}
##
## @examples
## x <- integer()
## get_refcnt(x)
## y <- x
## get_refcnt(x)
## z <- x
## get_refcnt(x)
#set_refcnt <- function(x, refcnt=1L){
#  invisible(.Call(C_R_set_refcnt, x, as.integer(refcnt)))
#}

## Get C pareet reference count of a vector
##
## Gets C reference count of a vector in parent frame
## 
## Queries the vector reference count using C-macro \code{NAMED}
## 
## @param x a vector
##   
## @return integer scalar
## @seealso \code{\link{get_refcnt}}, \code{\link{set_refcnt}}
##
## @examples
## f <- function(x)c(refcnt=get_refcnt(x), parent_refcnt=get_parent_refcnt(x))
## x <- integer()
## f(x)
## y <- x
## f(x)
## z <- x
## f(x)
## f(0)
## try(get_parent_refcnt(0))
#get_parent_refcnt <- function(x){
#  y <- substitute(x)
#  if (is.name(y)){
#    z <- eval(call("substitute", y), parent.frame())
#    if (is.name(z)){
#      .Call(C_R_get_refcnt, get(as.character(z), envir=parent.frame(2)))
#    }else
#      0L
#  } else 
#    stop("x must be the name of an object")
#}

# named <- function(x)
# .Call(C_R_bit_named, x)



#' Get C length of a vector
#' 
#' Gets C length of a vector ignoring any length-methods dispatched by classes
#' 
#' Queries the vector length using C-macro \code{LENGTH}, this can be substantially faster than \code{length(unclass(x))}
#' 
#' @param x a vector
#' @return integer scalar
#' @examples
#' length(bit(12))
#' get_length(bit(12))
#' @export
get_length <- function(x){
  .Call(C_R_get_length, x)
}


#' Test for C-level identity of two atomic vectors
#'
#' @param x an atomic vector
#' @param y an atomic vector
#' @return logical scalar
#' @examples
#' x <- 1:2
#' y <- x
#' z <- copy_vector(x)
#' still.identical(y,x)
#' still.identical(z,x)
#' @export
still.identical <- function(x, y){
  .Call(C_R_still_identical, x = x, y = y)
}

#' Copy atomic R vector
#' 
#' Creates a true copy of the underlying C-vector -- dropping all attributes -- and optionally reverses the direction of the elements. 
#' 
#' This can be substantially faster than \code{duplicate(as.vector(unclass(x)))}
#'
#' @param x an R vector
#' @param revx default \code{FALSE}, set to \code{TRUE} to reverse the elements in 'x'
#' @return copied R vector
#' @seealso \code{\link{clone}}, \code{\link{still.identical}},  \code{\link{reverse_vector}}
#' @examples
#' x <- factor(letters)
#' y <- x
#' z <- copy_vector(x)
#' still.identical(x,y)
#' still.identical(x,z)
#' str(x)
#' str(y)
#' str(z)
#' @export
copy_vector <- function(x, revx=FALSE){
  .Call(C_R_copy_vector, x, as.logical(revx))
}


#' Reverse atomic vector 
#' 
#' Returns a reversed copy -- with attributes retained. 
#' 
#' This is substantially faster than \code{\link{rev}}
#'
#' @param x an R vector
#' @return a reversed vector
#' @seealso \code{\link{rev}}, \code{\link{copy_vector}}
#' @examples
#' x <- factor(letters)
#' rev(x)
#' reverse_vector(x)
#' \dontrun{
#' x <- 1:1e7
#' system.time(rev(x))
#' system.time(reverse_vector(x))
#' }
#' @export
reverse_vector <- function(x){
  .Call(C_R_reverse_vector, x)
}

#' Position of first NA
#' 
#' This is substantially faster than \code{which.max(is.na(x))}
#'
#' @param x an R vector
#' @return a reversed vector
#' @seealso \code{\link{which.max}}, \code{\link{is.na}}, \code{\link{anyNA}}, \code{\link{anyDuplicated}}, \code{\link{bit_anyDuplicated}}
#' @examples
#' x <- c(FALSE,NA,TRUE)
#' firstNA(x)
#' reverse_vector(x)
#' \dontrun{
#' x <- 1:1e7
#' system.time(rev(x))
#' system.time(reverse_vector(x))
#' }
#' @export
firstNA <- function(x){
  .Call(C_R_firstNA, x)
}

# Package documentation
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

#' bit: Classes and methods for fast memory-efficient boolean selections
#' 
#' Provided are classes for boolean and skewed boolean vectors, fast boolean 
#' methods, fast unique and non-unique integer sorting, fast set operations on 
#' sorted and unsorted sets of integers, and foundations for ff (range indices, 
#' compression, chunked processing).
#' 
#' For details view the vignettes \url{../doc/bit-usage.pdf} and
#' \url{../doc/bit-performance.pdf}
#'
#'@name bit-package
NULL

# devtools::use_vignette("bit-usage")
# devtools::use_vignette("bit-performance")

# require(rhub)
# rhub_bit_4.0.5 <- check_for_cran(
#   path = "../bit_4.0.5.tar.gz"
# , email = "Jens.Oehlschlaegel@truecluster.com"
# , check_args = "--as-cran"
# , env_vars = c('_R_CHECK_FORCE_SUGGESTS_'= "false",'_R_CHECK_CRAN_INCOMING_USE_ASPELL_'= "true", '_R_CHECK_XREFS_MIND_SUSPECT_ANCHORS_'="true")
# , platforms = NULL
# , show_status = FALSE
# )
# 
# ─  Uploading package
# ─  Preparing build, see status at
# https://builder.r-hub.io/status/bit_4.0.5.tar.gz-b6a1c13ef92c42d9b68257139666fb46
# https://builder.r-hub.io/status/bit_4.0.5.tar.gz-abc80afc9741404c9e86511dee4585c5
# https://builder.r-hub.io/status/bit_4.0.5.tar.gz-93409d19ee544acab83be7de1b380740
# https://builder.r-hub.io/status/bit_4.0.5.tar.gz-d4f141780d004fa388c82d027a6e40e2

# olddir <- "../revdepold"
# newdir <- "../revdepnew"
# tools::check_packages_in_dir(olddir,
#                              check_args = c("--as-cran", ""),
#                              reverse = list(repos = getOption("repos")["CRAN"]))
# tools::check_packages_in_dir(newdir, old=olddir
#                              check_args = c("--as-cran", ""),
#                              reverse = list(repos = getOption("repos")["CRAN"]))
# tools::summarize_check_packages_in_dir_results(newdir, all = FALSE, full = TRUE)
# tools::check_packages_in_dir_changes(newdir, olddir, outputs = TRUE, sources = FALSE)

# rle utilities for bit and ff
# (c) 2007-2009 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2007-09-03
# Last changed: 2007-10-25

# source("D:/mwp/eanalysis/bit/R/rle.R")

#' Hybrid Index, C-coded utilities
#'
#' These C-coded utilitites speed up index preprocessing considerably.
#'
#' `intrle` is by factor 50 faster and needs less RAM (2x its input
#' vector) compared to [rle()] which needs 9x the RAM of its input
#' vector.  This is achieved because we allow the C-code of `intrle` to
#' break when it turns out, that rle-packing will not achieve a compression
#' factor of 3 or better.
#'
#' `intisasc` is a faster version of [is.unsorted()]: it checks whether `x` is sorted.
#'
#' `intisdesc` checks for being sorted descending and by default default assumes that the
#'    input `x` contains no NAs.
#'
#' `na.method="none"` treats `NAs` (the smallest integer) like every other integer and
#'   hence returns either `TRUE` or `FALSE` `na.method="break"` checks for `NAs` and
#'   returns either `NA` as soon as  `NA` is encountered. `na.method="skip"` checks for
#'   `NAs` and skips over them, hence decides the return value only on the basis of
#'   non-NA values.
#'
#'
#' @param x an integer vector
#' @param na.method one of "none", "break", "skip", see details. The strange defaults stem
#'   from the initial usage.
#' @return
#'  - `intrle` returns an object of class [rle()] or NULL, if rle-compression is not
#'    efficient (compression factor <3 or `length(x) < 3`).
#'  - `intisasc` returns one of `FALSE, NA, TRUE`
#'  - `intisdesc` returns one of `FALSE, TRUE` (if the input contains NAs, the output is
#'    undefined)
#' @author Jens Oehlschlägel
#' @seealso [ff::hi()], [rle()], [is.unsorted()],
#' [ff::is.sorted.default()]
#' @keywords IO data
#' @examples
#'
#'   intrle(sample(1:10))
#'   intrle(diff(1:10))
#'   intisasc(1:10)
#'   intisasc(10:1)
#'   intisasc(c(NA, 1:10))
#'   intisdesc(1:10)
#'   intisdesc(c(10:1, NA))
#'   intisdesc(c(10:6, NA, 5:1))
#'   intisdesc(c(10:6, NA, 5:1), na.method="skip")
#'   intisdesc(c(10:6, NA, 5:1), na.method="break")
#'
#' @export
# -- fast and efficient rle ------------------

# integer only
# returns rle object only if n>2 && rle is efficient
#   (length(values) + lengths(lengths)) <= length(x)
# returns NULL if n<3 || rle is inefficient
intrle <- function(x) {
  stopifnot(is.integer(x))
  .Call(C_R_int_rle, x)
}


# -- check for sorting and NAs, 0s can be checked later when sorted ------------------

#' @describeIn intrle check whether integer vector is ascending
#' @export
intisasc <- function(x, na.method=c("none", "break", "skip")[2]) {
  stopifnot(is.integer(x))
  if (na.method == "break")
    .Call(C_R_int_is_asc_break, x)
  else if (na.method == "none")
    .Call(C_R_int_is_asc_none, x)
  else
    .Call(C_R_int_is_asc_skip, x)
}

#' @describeIn intrle check whether integer vector is descending
#' @export
intisdesc <- function(x, na.method=c("none", "break", "skip")[1]) {
  stopifnot(is.integer(x))
  if (na.method == "none")
    .Call(C_R_int_is_desc_none, x)
  else if (na.method == "break")
    .Call(C_R_int_is_desc_break, x)
  else
    .Call(C_R_int_is_desc_skip, x)
}




# -- basic sequence packing and unpacking ---------------------------------------------------

#' Hybrid Index, rle-pack utilities
#'
#' Basic utilities for rle packing and unpacking and apropriate methods for
#' [rev()] and [unique()].
#'
#'
#' @param x in 'rlepack' an integer vector, in the other functions an object of
#' class 'rlepack'
#' @param pack FALSE to suppress packing
#' @param incomparables just to keep R CMD CHECK quiet (not used)
#' @param ... just to keep R CMD CHECK quiet (not used)
#' @return A list with components:
#'  - first: the first element of the packed sequence
#'  - dat: either an object of class [rle()] or the complete input vector `x` if
#'    rle-packing is not efficient
#'  - last: the last element of the packed sequence
#'
#' @author Jens Oehlschlägel
#' @seealso [ff::hi()], [intrle()], [rle()], [rev()], [unique()]
#' @keywords IO data
#' @examples
#'
#'   x <- rlepack(rep(0L, 10))
#'
#' @export
rlepack <- function(x, ...) UseMethod("rlepack")

#' @rdname rlepack
#' @export
rlepack.integer <- function(x, pack = TRUE, ...) {
  stopifnot(is.integer(x))
  n <- length(x)
  if (n > 1L) {
    if (pack)
      # returns NULL if rle is inefficient, old condition was 2*length(r$lengths)<n
      r <- intrle(diff(x))
    else
      r <- NULL
    out = list(first=x[1], dat=if (is.null(r)) x else r, last=x[n])
  } else if (n == 1) {
    out = list(first=x[1], dat=x, last=x[1])
  } else {
    out = list(first=NA_integer_, dat=x, last=NA_integer_)
  }
  class(out) <- "rlepack"
  out
}

#' @rdname rlepack
#' @export
rleunpack <- function(x) UseMethod("rleunpack")

#' @rdname rlepack
#' @export
rleunpack.rlepack <- function(x) {
  if (inherits(x$dat, "rle"))
    as.integer(cumsum(c(x$first, rep(x$dat$values, x$dat$lengths))))
  else
    x$dat
}


#' @rdname rlepack
#' @export
rev.rlepack <- function(x) {
  if (inherits(x$dat, "rle")) {
    x$dat$values <- -rev(x$dat$values)
    x$dat$lengths <- rev(x$dat$lengths)
  } else {
    x$dat <- rev(x$dat)
  }
  buf <- x$first
  x$first <- x$last
  x$last <- buf
  x
}


# beware: only for sorted input identical with unique()
# beware: rlepack(unique(x)) is faster than unique(rlepack(x))
# we use this only in hi() and as.hi.default()
#' @rdname rlepack
#' @export
unique.rlepack <- function(x, incomparables = FALSE, ...) {
  if (inherits(x$dat, "rle")) {
    tab <- tabulate(sign(x$dat$values) + 2L, nbins=3L)
    if (tab[1] && tab[3])
      x <- rlepack(unique(rleunpack(x)))
    else if (tab[2]) {
      x$dat$lengths <- x$dat$lengths[x$dat$values != 0]
      x$dat$values <- x$dat$values[x$dat$values != 0]
    }
    # else nothing to do: no repeated values
  } else {
    x$dat <- unique(x$dat)
  }
  x
}


# beware: only for sorted input identical with unique()
# beware: returns TRUE/FALSE, not position of first duplicate
#' @rdname rlepack
#' @export
anyDuplicated.rlepack <- function(x, incomparables = FALSE, ...) {
  if (!inherits(x$dat, "rle"))
    return(anyDuplicated(x$dat))
  tab <- tabulate(sign(x$dat$values) + 2L, nbins=3L)
  if (tab[1] && tab[3])
    return(anyDuplicated(rleunpack(x)))
  if (!tab[2])
    return(0L)
  w <- .Call(C_R_first_zero, x$dat$values)
  if (!w)
    return(0L)
  if (w <= 1L)
    return(2L)
  sum(x$dat$lengths[1:(w - 1L)]) + 2L
}

# Integer utilities
# (c) 2016-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

## Get C reference count of a vector
##
## Gets C reference count of a vector
##
## Queries the vector reference count using C-macro `NAMED`
##
## @param x a vector
##
## @return integer scalar
## @seealso [get_parent_refcnt()], [set_refcnt()]
##
## @examples
## x <- integer()
## get_refcnt(x)
## y <- x
## get_refcnt(x)
## z <- x
## get_refcnt(x)
##
#get_refcnt <- function(x) {
#  y <- substitute(x)
#  if (is.name(y)) {
#    .Call(C_R_get_refcnt, get(as.character(y), envir=parent.frame()))
#  } else
#    stop("x must be the name of an object")
#}

## Set C reference count of a vector
##
## Sets C reference count of a vector
##
## Manipulates the vector reference count using C-macro `SET_NAMED`
##
## @note Do use this only if you know what you do! Otherwise unexpected behaviour can result!
##
## @param x a vector
## @param refcnt integer scalar to be assigned (must be 1L or 2L)
##
## @return integer scalar
## @seealso @seealso [get_refcnt()], [get_parent_refcnt()]
##
## @examples
## x <- integer()
## get_refcnt(x)
## y <- x
## get_refcnt(x)
## z <- x
## get_refcnt(x)
#set_refcnt <- function(x, refcnt=1L) {
#  invisible(.Call(C_R_set_refcnt, x, as.integer(refcnt)))
#}

## Get C pareet reference count of a vector
##
## Gets C reference count of a vector in parent frame
##
## Queries the vector reference count using C-macro `NAMED`
##
## @param x a vector
##
## @return integer scalar
## @seealso [get_refcnt()], [set_refcnt()]
##
## @examples
## f <- function(x) c(refcnt=get_refcnt(x), parent_refcnt=get_parent_refcnt(x))
## x <- integer()
## f(x)
## y <- x
## f(x)
## z <- x
## f(x)
## f(0)
## try(get_parent_refcnt(0))
#get_parent_refcnt <- function(x) {
#  y <- substitute(x)
#  if (is.name(y)) {
#    z <- eval(call("substitute", y), parent.frame())
#    if (is.name(z)) {
#      .Call(C_R_get_refcnt, get(as.character(z), envir=parent.frame(2)))
#    } else
#      0L
#  } else
#    stop("x must be the name of an object")
#}

# named <- function(x)
# .Call(C_R_bit_named, x)



#' Get C length of a vector
#'
#' Gets C length of a vector ignoring any length-methods dispatched by classes
#'
#' Queries the vector length using C-macro `LENGTH`, this can be substantially faster than
#'   `length(unclass(x))`
#'
#' @param x a vector
#' @return integer scalar
#' @examples
#' length(bit(12))
#' get_length(bit(12))
#' @export
get_length <- function(x) {
  .Call(C_R_get_length, x)
}


#' Test for C-level identity of two atomic vectors
#'
#' @param x an atomic vector
#' @param y an atomic vector
#' @return logical scalar
#' @examples
#' x <- 1:2
#' y <- x
#' z <- copy_vector(x)
#' still.identical(y, x)
#' still.identical(z, x)
#' @export
still.identical <- function(x, y) {
  .Call(C_R_still_identical, x = x, y = y)
}

#' Copy atomic R vector
#'
#' Creates a true copy of the underlying C-vector -- dropping all attributes -- and
#'   optionally reverses the direction of the elements.
#'
#' This can be substantially faster than `duplicate(as.vector(unclass(x)))`
#'
#' @param x an R vector
#' @param revx default `FALSE`, set to `TRUE` to reverse the elements in 'x'
#' @return copied R vector
#' @seealso [clone()], [still.identical()],  [reverse_vector()]
#' @examples
#' x <- factor(letters)
#' y <- x
#' z <- copy_vector(x)
#' still.identical(x, y)
#' still.identical(x, z)
#' str(x)
#' str(y)
#' str(z)
#' @export
copy_vector <- function(x, revx=FALSE) {
  .Call(C_R_copy_vector, x, as.logical(revx))
}


#' Reverse atomic vector
#'
#' Returns a reversed copy -- with attributes retained.
#'
#' This is substantially faster than [rev()]
#'
#' @param x an R vector
#' @return a reversed vector
#' @seealso [rev()], [copy_vector()]
#' @examples
#' x <- factor(letters)
#' rev(x)
#' reverse_vector(x)
#' \dontrun{
#' x <- 1:1e7
#' system.time(rev(x))
#' system.time(reverse_vector(x))
#' }
#' @export
reverse_vector <- function(x) {
  .Call(C_R_reverse_vector, x)
}

#' Position of first NA
#'
#' This is substantially faster than `which.max(is.na(x))`
#'
#' @param x an R vector
#' @return a reversed vector
#' @seealso [which.max()], [is.na()], [anyNA()], [anyDuplicated()], [bit_anyDuplicated()]
#' @examples
#' x <- c(FALSE, NA, TRUE)
#' firstNA(x)
#' reverse_vector(x)
#' \dontrun{
#' x <- 1:1e7
#' system.time(rev(x))
#' system.time(reverse_vector(x))
#' }
#' @export
firstNA <- function(x) {
  .Call(C_R_firstNA, x)
}

# Package documentation
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

#' bit: Classes and methods for fast memory-efficient boolean selections
#'
#' Provided are classes for boolean and skewed boolean vectors, fast boolean
#' methods, fast unique and non-unique integer sorting, fast set operations on
#' sorted and unsorted sets of integers, and foundations for ff (range indices,
#' compression, chunked processing).
#'
#' For details view the `vignette("bit-usage")` and `vignette("bit-performance")`.
#'
#'@name bit-package
NULL

# devtools::use_vignette("bit-usage")
# devtools::use_vignette("bit-performance")

# olddir <- "../revdepold"
# newdir <- "../revdepnew"
# tools::check_packages_in_dir(olddir,
#                              check_args = c("--as-cran", ""),
#                              reverse = list(repos = getOption("repos")["CRAN"]))
# tools::check_packages_in_dir(newdir, old=olddir
#                              check_args = c("--as-cran", ""),
#                              reverse = list(repos = getOption("repos")["CRAN"]))
# tools::summarize_check_packages_in_dir_results(newdir, all = FALSE, full = TRUE)
# tools::check_packages_in_dir_changes(newdir, olddir, outputs = TRUE, sources = FALSE)

# Package generics
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

# source("D:/mwp/eanalysis/bit/R/generics.R")


#' Get maxindex (length of boolean vector) and poslength (number of 'selected' elements)
#' 
#' For \code{\link{is.booltype}} objects the term \code{\link{length}} is ambiguous. 
#' For example the length of \code{\link{which}} corresponds to the sum of \code{\link{logical}}.
#' The generic \code{maxindex} gives \code{length(logical)} for all \code{\link{booltype}s}.
#' The generic \code{poslength} gives the number of positively selected elements, i.e. \code{sum(logical)} for all \code{\link{booltype}s} 
#' (and gives \code{NA} if \code{NAs} are present).
#' 
#' @name maxindex
#' @param x an R object, typically a \code{\link{is.booltype}} object.
#' @param ... further arguments (ignored)
#' @return an integer scalar
#' @examples
#' r <- ri(1,2,12)
#' i <- as.which(r)
#' w <- as.bitwhich(r)
#' b <- as.bit(r)
#' l <- as.logical(r)
#' u <- which(l)      # unclassed which
#' 
#' sapply(list(r=r,u=u,i=i,w=w,b=b,l=l), function(x){
#'   c(length=length(x), sum=sum(x), maxindex=maxindex(x), poslength=poslength(x))
#' })
#' @export
maxindex <- function(x, ...)
{
  UseMethod("maxindex", x)
}

#' @name maxindex
#' @export
poslength <- function(x, ...)
{
  UseMethod("poslength")
}



#' Coerce to booltype (generic)
#'
#' @param x object to coerce
#' @param booltype target \code{\link{booltype}} given as integer or as character
#' @param ... further arguments
#'
#' @return \code{x} coerced to \code{booltype}
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{booltypes}}, \code{\link{booltype}}, \code{\link{is.booltype}}
#'
#' @examples
#' as.booltype(0:1)
#' as.booltype(0:1, "logical")
#' as.booltype(0:1, "bit")
#' as.booltype(0:1, "bitwhich")
#' as.booltype(0:1, "which", maxindex=2)
#' as.booltype(0:1, "ri")
#' @export
as.booltype <- function (x, booltype, ...){
  UseMethod("as.booltype")
}


#' Coercing to bit
#' 
#' Coercing to bit vector
#' 
#' Coercing to bit is quite fast because we use a double loop that fixes each
#' word in a processor register
#' 
#' @param x an object of class \code{\link{bit}}, \code{\link{logical}},
#' \code{\link{integer}}, \code{\link{bitwhich}} or an integer from
#' \code{\link{as.which}} or a boolean \code{\link[ff:vmode]{ff}}
#' @param length the length of the new bit vector
#' @param \dots further arguments
#' @return \code{is.bit} returns FALSE or TRUE, \code{as.bit} returns a vector
#' of class 'bit'
#' @note Zero is coerced to FALSE, all other numbers including NA are coerced
#' to TRUE.  This differs from the NA-to-FALSE coercion in package ff and may
#' change in the future.
#' @author Jens Oehlschlägel
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
#' , \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
#' @keywords classes logic
#' @examples
#' 
#' as.bit(c(FALSE, NA, TRUE))
#' 
#' @export
as.bit <- function(x=NULL, ...)
  UseMethod("as.bit", x)


#' Coercing to bitwhich
#' 
#' Functions to coerce to bitwhich
#' 
#' @param x An object of class 'bitwhich', 'integer', 'logical' or 'bit' or an
#' integer vector as resulting from 'which'
#' @param maxindex the length of the new bitwhich vector
#' @param poslength the number of selected elements
#' @param range a \code{\link{ri}} or an integer vector of length==2 giving a
#' range restriction for chunked processing
#' @param \dots further arguments
#' @return a value of class \code{\link{bitwhich}}
#' @author Jens Oehlschlägel
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
#' , \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
#' @keywords classes logic
#' @examples
#' 
#'  as.bitwhich(c(NA,NA,NA))
#'  as.bitwhich(c(FALSE, FALSE, FALSE))
#'  as.bitwhich(c(FALSE, FALSE, TRUE))
#'  as.bitwhich(c(FALSE, TRUE, TRUE))
#'  as.bitwhich(c(TRUE, TRUE, TRUE))
#' 
#' @export
as.bitwhich <- function(x=NULL, ...)
  UseMethod("as.bitwhich")


#' Coercion to (positive) integer positions
#' 
#' Coercing to something like the result of which \code{\link{which}}
#' 
#' \code{as.which.bit} returns a vector of subscripts with class 'which'
#' 
#' @param x an object of classes \code{\link{bit}}, \code{\link{bitwhich}},
#' \code{\link{ri}} or something on which \code{\link{which}} works
#' @param range a \code{\link{ri}} or an integer vector of length==2 giving a
#' range restriction for chunked processing
#' @param maxindex the length of the boolean vector which is represented 
#' @param is.unsorted a logical scalar indicating whether the data may be unsorted 
#' @param has.dup a logical scalar indicating whether the data may have duplicates
#' @param \dots further arguments (passed to \code{\link{which}} for the
#' default method, ignored otherwise)
#' @return a vector of class 'logical' or 'integer'
#' @author Jens Oehlschlägel
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
#' , \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
#' @keywords classes logic
#' @examples
#' 
#'   r <- ri(5, 20, 100)
#'   x <- as.which(r)
#'   x
#' 
#'   stopifnot(identical(x, as.which(as.logical(r))))
#'   stopifnot(identical(x, as.which(as.bitwhich(r))))
#'   stopifnot(identical(x, as.which(as.bit(r))))
#' 
#' @export
as.which <- function (x, ...)
  UseMethod("as.which")


#' Coerce to ri
#'
#' @param x object to coerce
#' @param ... further arguments
#'
#' @return an \code{\link{ri}} object
#' @author Jens Oehlschlägel
#' @seealso \code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
#' , \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
#' @keywords classes logic
#'
#' @examples
#' as.ri(c(FALSE, TRUE, FALSE, TRUE))
#' @export
as.ri <- function (x, ...)
{
  UseMethod("as.ri")
}



#' Boolean operators and functions
#' 
#' Boolean NEGATION '!', AND '&', OR '|' and EXCLUSIVE OR xor', see \code{\link[base]{Logic}}.
#' 
#' The binary operators and function \code{xor} can now combine any \code{\link{is.booltype}} vectors. 
#' They now recycle if vectors have different length. If the two arguments have different \code{\link{booltypes}} the return value corresponds to the lower \code{\link{booltype}} of the two. \cr
#'
#' Boolean operations on \code{\link{bit}} vectors are extremely fast because they are
#' implemented using C's bitwise operators. Boolean operations on or \code{\link{bitwhich}}  
#' vectors are even faster, if they represent very skewed selections.  \cr
#' 
#' The \code{xor} function has been made generic and \code{xor.default} has
#' been implemented much faster than R's standard \code{\link[base:Logic]{xor}}.
#' This was possible because actually boolean function \code{xor} and
#' comparison operator \code{!=} do the same (even with NAs), and \code{!=} is
#' much faster than the multiple calls in \code{(x | y) & !(x & y)}
#' 
#' @param x a \code{\link{is.booltype}} vector
#' @param y a \code{\link{is.booltype}} vector
#' @param e1 a \code{\link{is.booltype}} vector
#' @param e2 a \code{\link{is.booltype}} vector
#' @return An object of class \code{\link{booltype}} or \code{\link{logical}}
#' @author Jens Oehlschlägel
#' @seealso \code{\link{booltypes}}, \code{\link{Logic}}
#' @keywords classes logic
#' @examples
#' 
#'   x <- c(FALSE, FALSE, FALSE, NA, NA, NA, TRUE, TRUE, TRUE)
#'   y <- c(FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE)
#'   
#'   x|y
#'   x|as.bit(y)
#'   x|as.bitwhich(y)
#'   x|as.which(y)
#'   x|ri(1,1,9)
#'   
#'   
#' @export
xor <- function(x, y)
  UseMethod("xor", x)



#' Physical and virtual attributes
#' 
#' Compatibility functions (to package ff) for getting and setting physical and
#' virtual attributes.
#' 
#' ff objects have physical and virtual attributes, which have different
#' copying semantics: physical attributes are shared between copies of ff
#' objects while virtual attributes might differ between copies.
#' \code{\link[ff:as.ff]{as.ram}} will retain some physical and virtual atrributes in
#' the ram clone, such that \code{\link[ff]{as.ff}} can restore an ff object
#' with the same attributes.
#' 
#' @name PhysVirt
#' @param x a ff or ram object
#' @param value a list with named elements
#' @param \dots further arguments
#' @return \command{physical} and \command{virtual} returns a list with named
#' elements
#' @author Jens Oehlschlägel
#' @seealso \code{\link[ff]{physical.ff}}, \code{\link[ff]{physical.ffdf}}
#' @keywords IO data attribute
#' @examples
#' 
#'   physical(bit(12))
#'   virtual(bit(12))
NULL

#' @rdname PhysVirt
#' @export
physical <- function(x)UseMethod("physical")

#' @rdname PhysVirt
#' @export
"physical<-" <- function(x, value)UseMethod("physical<-")

#' @rdname PhysVirt
#' @export
virtual <- function(x)UseMethod("virtual")

#' @rdname PhysVirt
#' @export
"virtual<-" <- function(x, value)UseMethod("virtual<-")


#' Generics for in-RAM sorting and ordering
#' 
#' These are generic stubs for low-level sorting and ordering methods
#' implemented in packages 'bit64' and 'ff'.  The \code{..sortorder} methods do
#' sorting and ordering at once, which requires more RAM than ordering but is
#' (almost) as fast as as sorting.
#' 
#' The \code{sort} generics do sort their argument 'x', some methods need
#' temporary RAM of the same size as 'x'.  The \code{order} generics do order
#' their argument 'i' leaving 'x' as it was, some methods need temporary RAM of
#' the same size as 'i'.  The \code{sortorder} generics do sort their argument
#' 'x' and order their argument 'i', this way of ordering is much faster at the
#' price of requiring temporary RAM for both, 'x' and 'i', if the method
#' requires temporary RAM.  The \code{ram} generics are high-level functions
#' containing an optimizer that chooses the 'best' algorithms given some
#' context.
#' 
#' @name Sorting
#' @param x a vector to be sorted by \code{\link{ramsort}} and
#' \code{\link{ramsortorder}}, i.e. the output of \code{\link{sort}}
#' @param i integer positions to be modified by \code{\link{ramorder}} and
#' \code{\link{ramsortorder}}, default is 1:n, in this case the output is
#' similar to \code{\link{order}}
#' @param \dots further arguments to the sorting methods
#' @return These functions return the number of \code{NAs} found or assumed
#' during sorting
#' @note Note that these methods purposely violate the functional programming
#' paradigm: they are called for the side-effect of changing some of their
#' arguments.  The rationale behind this is that sorting is very RAM-intensive
#' and in certain situations we might not want to allocate additional memory if
#' not necessary to do so.  The \code{sort}-methods change \code{x}, the
#' \code{order}-methods change \code{i}, and the \code{sortoder}-methods change
#' both \code{x} and \code{i} You as the user are responsible to create copies
#' of the input data 'x' and 'i' if you need non-modified versions.
#' @section Index of implemented methods: \tabular{rrl}{ \bold{generic} \tab
#' \bold{ff} \tab \bold{bit64} \cr \code{ramsort} \tab
#' \code{\link[ff]{ramsort.default}} \tab
#' \code{\link[bit64]{ramsort.integer64}} \cr \code{shellsort} \tab
#' \code{\link[ff:ramsort.default]{shellsort.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{shellsort.integer64}} \cr \code{quicksort} \tab \tab
#' \code{\link[bit64:ramsort.integer64]{quicksort.integer64}} \cr \code{mergesort} \tab
#' \code{\link[ff:ramsort.default]{mergesort.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{mergesort.integer64}} \cr \code{radixsort} \tab
#' \code{\link[ff:ramsort.default]{radixsort.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{radixsort.integer64}} \cr \code{keysort} \tab
#' \code{\link[ff:ramsort.default]{keysort.default}} \tab \cr \cr \bold{generic} \tab \bold{ff}
#' \tab \bold{bit64} \cr \code{ramorder} \tab
#' \code{\link[ff]{ramorder.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{ramorder.integer64}} \cr \code{shellorder} \tab
#' \code{\link[ff:ramorder.default]{shellorder.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{shellorder.integer64}} \cr \code{quickorder} \tab \tab
#' \code{\link[bit64:ramsort.integer64]{quickorder.integer64}} \cr \code{mergeorder} \tab
#' \code{\link[ff:ramorder.default]{mergeorder.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{mergeorder.integer64}} \cr \code{radixorder} \tab
#' \code{\link[ff:ramorder.default]{radixorder.default}} \tab
#' \code{\link[bit64:ramsort.integer64]{radixorder.integer64}} \cr \code{keyorder} \tab
#' \code{\link[ff:ramorder.default]{keyorder.default}} \tab \cr \cr \bold{generic} \tab
#' \bold{ff} \tab \bold{bit64} \cr \code{ramsortorder} \tab \tab
#' \code{\link[bit64:ramsort.integer64]{ramsortorder.integer64}} \cr \code{shellsortorder} \tab
#' \tab \code{\link[bit64:ramsort.integer64]{shellsortorder.integer64}} \cr \code{quicksortorder}
#' \tab \tab \code{\link[bit64:ramsort.integer64]{quicksortorder.integer64}} \cr
#' \code{mergesortorder} \tab \tab
#' \code{\link[bit64:ramsort.integer64]{mergesortorder.integer64}} \cr \code{radixsortorder} \tab
#' \tab \code{\link[bit64:ramsort.integer64]{radixsortorder.integer64}} \cr \code{keysortorder}
#' \tab \tab \cr }
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso \code{\link{sort}} and \code{\link{order}} in base R, \code{\link{bitsort}} for faster inteer sorting
#' @keywords univar manip arith
NULL

#' @rdname Sorting
#' @export
ramsort <- function(x, ...)UseMethod("ramsort")
#' @rdname Sorting
#' @export
ramorder <- function(x, i, ...)UseMethod("ramorder")
#' @rdname Sorting
#' @export
ramsortorder <- function(x, i, ...)UseMethod("ramsortorder")
#' @rdname Sorting
#' @export
mergesort <- function(x, ...)UseMethod("mergesort")
#' @rdname Sorting
#' @export
mergeorder <- function(x, i, ...)UseMethod("mergeorder")
#' @rdname Sorting
#' @export
mergesortorder <- function(x, i, ...)UseMethod("mergesortorder")
#' @rdname Sorting
#' @export
quicksort <- function(x, ...)UseMethod("quicksort")
#' @rdname Sorting
#' @export
quickorder <- function(x, i, ...)UseMethod("quickorder")
#' @rdname Sorting
#' @export
quicksortorder <- function(x, i, ...)UseMethod("quicksortorder")
#' @rdname Sorting
#' @export
shellsort <- function(x, ...)UseMethod("shellsort")
#' @rdname Sorting
#' @export
shellorder <- function(x, i, ...)UseMethod("shellorder")
#' @rdname Sorting
#' @export
shellsortorder <- function(x, i, ...)UseMethod("shellsortorder")
#' @rdname Sorting
#' @export
radixsort <- function(x, ...)UseMethod("radixsort")
#' @rdname Sorting
#' @export
radixorder <- function(x, i, ...)UseMethod("radixorder")
#' @rdname Sorting
#' @export
radixsortorder <- function(x, i, ...)UseMethod("radixsortorder")
#' @rdname Sorting
#' @export
keysort <- function(x, ...)UseMethod("keysort")
#' @rdname Sorting
#' @export
keyorder <- function(x, i, ...)UseMethod("keyorder")
#' @rdname Sorting
#' @export
keysortorder <- function(x, i, ...)UseMethod("keysortorder")


#' Generics related to cache access
#' 
#' These generics are packaged here for methods in packages \code{bit64} and
#' \code{ff}.
#' 
#' see help of the available methods
#' 
#' @name Metadata
#' @param x some object
#' @param value value assigned on responsibility of the user
#' @param \dots ignored
#' @return see help of the available methods
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso \code{\link[bit64]{is.sorted.integer64}},
#' \code{\link[bit64:is.sorted.integer64]{na.count.integer64}},
#' \code{\link[bit64:is.sorted.integer64]{nvalid.integer64}},
#' \code{\link[bit64:is.sorted.integer64]{nunique.integer64}}, \code{\link[bit64:is.sorted.integer64]{nties.integer64}}
#' \cr
#' @keywords environment methods
#' @examples
#' 	methods("na.count")
NULL

#' @rdname Metadata
#' @export
is.sorted <- function(x, ...)UseMethod("is.sorted")

#' @rdname Metadata
#' @export
"is.sorted<-" <- function(x, ..., value)UseMethod("is.sorted<-")

#' @rdname Metadata
#' @export
na.count <- function(x, ...)UseMethod("na.count")

#' @rdname Metadata
#' @export
"na.count<-" <- function(x, ..., value)UseMethod("na.count<-")

#' @rdname Metadata
#' @export
nvalid <- function(x, ...)UseMethod("nvalid")

#' @rdname Metadata
#' @export
nunique <- function(x, ...)UseMethod("nunique")

#' @rdname Metadata
#' @export
"nunique<-" <- function(x, ..., value)UseMethod("nunique<-")

#' @rdname Metadata
#' @export
nties <- function(x, ...)UseMethod("nties")

#' @rdname Metadata
#' @export
"nties<-" <- function(x, ..., value)UseMethod("nties<-")

#' @param N number of repetitions for random regression tests
regtest.bit <- function(N = 50) {
  OK <- TRUE
  pool <- c(FALSE, TRUE)

  if (!identical(unattr(as.bit(c(FALSE, NA, TRUE))[]), c(FALSE, FALSE, TRUE))) {
    message("bit error: wrong coercion of triboolean to (bi)boolean")
    OK <- FALSE
  }

  l <- TRUE
  b <- as.bit(l)

  i <- -c(1, 0, 1, NA)
  if (!inherits(try(b[i], silent=TRUE), "try-error")) {
    message("bit error: did not throw on mixing zero with negative subscripts")
    OK <- FALSE
  }

  i <- c(2, 1, 0, 1, NA)
  if (!identical(l[i], unattr(b[i]))) {
    message("\nregression test difference between b[i] and l[i]")
    message(l[i])
    message(unattr(b[i]))
    OK <- FALSE
  }

  l[0] <- TRUE
  b[0] <- TRUE
  if (!identical(l, unattr(b[]))) {
    message("\nregression test difference after assigning at R position zero")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }

  l[2] <- TRUE
  b[2] <- TRUE
  if (!identical(ifelse(is.na(l), FALSE, l), unattr(b[]))) {
    message("\nregression test difference after assigning after vector length (at 2)")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }

  l[.BITS + 1] <- FALSE
  b[.BITS + 1] <- NA
  if (!identical(ifelse(is.na(l), FALSE, l), unattr(b[]))) {
    message("\nregression test difference after assigning after vector length (at .BITS + 1)")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }

  if (!identical(ifelse(is.na(l[TRUE]), FALSE, l[TRUE]), unattr(b[TRUE]))) {
    message("\nregression test difference after subscripting with scalar TRUE")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }

  if (!identical(ifelse(is.na(l[FALSE]), FALSE, l[FALSE]), unattr(b[FALSE]))) {
    message("\nregression test difference after subscripting with scalar FALSE")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }

  for (i in 1:N) {
    n <- sample(1:(2 * .BITS), 1)
    l <- sample(pool, n, TRUE)
    # check direct coercion
    b <- as.bit(l)
    l2 <- as.logical(b)
    if (!identical(l, l2)) {
      message("\nregression test difference between logical")
      message(l)
      message("and as.logical(as.bit(logical))")
      message(l2)
      OK <- FALSE
    }


    # summary functions with logical return
    s <- c(all=all(l), any=any(l))
    s2 <- c(all=all(b), any=any(b))
    if (!identical(s, s2)) {
      message("\nregression test difference between logical summaries")
      message(s)
      message("and bit summaries")
      message(s2)
      OK <- FALSE
    }
    # summary functions with integer return
    if (any(l)) {
      s <- c(
        min=min(as.which(l)),
        max=max(as.which(l)),
        range=range(as.which(l)),
        sum=sum(l),
        summary=c(
          `FALSE`=length(l) - sum(l),
          `TRUE`=sum(l),
          Min.=min(as.which(l)),
          Max.=max(as.which(l))
        )
      )
    } else {
      s <- c(
        min=NA_integer_,
        max=NA_integer_,
        range=c(NA_integer_, NA_integer_),
        sum=sum(l),
        summary=c(
          `FALSE`=length(l) - sum(l),
          `TRUE`=sum(l),
          Min.=NA_integer_,
          Max.=NA_integer_
        )
      )
    }
    s2 <- c(min=min(b), max=max(b), range=range(b), sum=sum(b), summary=summary(b))
    if (!identical(s, s2)) {
      message("\nregression test difference between logical summaries")
      message(s)
      message("and bit summaries")
      message(s2)
      OK <- FALSE
    }
    # check positive whichs
    w <- as.which(l)
    w2 <- as.which(as.bit(w, n))
    if (!identical(w, w2)) {
      message("\nregression test difference between which")
      message(w)
      message("and as.which(as.bit.which(which))")
      message(w2)
      OK <- FALSE
    }
    # check automatic whichs (pos or neg whatever shorter)
    s <- sum(l)
    if (s == 0) {
      w <- FALSE
    } else if (s == n) {
      w <- TRUE
    } else if (s > n %/% 2L) {
      w <- -rev(which(!l))
    } else {
      w <- which(l)
    }
    w2 <- as.vector(as.bitwhich(as.bit(l)))
    if (!identical(w, w2)) {
      message("\nregression test difference between which")
      message(w)
      message("and as.which(as.bit.which(which))")
      message(w2)
      OK <- FALSE
    }
    # check boolean operators
    l2 <- sample(c(FALSE, TRUE), n, TRUE)
    b2 <- as.bit(l2)
    ops <- c(
      NOT = identical(!l, as.logical(!b))
      , AND = identical(l & l2, as.logical(b & b2))
      , OR = identical(l | l2, as.logical(b | b2))
      , XOR = identical(xor(l, l2), as.logical(xor(b, b2)))
      , NEQ = identical(l != l2, as.logical(b != b2))
      , EQ = identical(l == l2, as.logical(b == b2))
    )
    if (!all(ops)) {
      message("\nbit differs for boolean operators(s)")
      message(ops)
      message(cbind(l=l, l2=l))
      OK <- FALSE
    }
    w <- as.bitwhich(l)
    w2 <- as.bitwhich(l2)
    ops <- c(
      NOT = identical(!l, as.logical(!w))
      , AND = identical(l & l2, as.logical(w & w2))
      , OR = identical(l | l2, as.logical(w | w2))
      , XOR = identical(xor(l, l2), as.logical(xor(w, w2)))
      , NEQ = identical(l != l2, as.logical(w != w2))
      , EQ = identical(l == l2, as.logical(w == w2))
    )
    if (!all(ops)) {
      message("\nbitwhich differs for boolean operators(s)")
      message(ops)
      message(cbind(l=l, l2=l))
      OK <- FALSE
    }
    rm(l2, b2, w2)
    # check extractors
    n2 <- sample(1:n, 1)
    j <- sample(1:n, n2)
    if (!identical(l[j], unattr(b[j]))) {
      message("\nregression test difference when extracting")
      OK <- FALSE
    }
    # check replacement (index)
    new_value <- sample(pool, n2, TRUE)
    l[j] <- new_value
    b[j] <- new_value
    if (!identical(l, unattr(b[]))) {
      message("\nregression test difference when replacing with index")
      OK <- FALSE
    }
    # check replacement (recycle)
    if (n %% 2) {
      new_value <- sample(pool, 1)
      l[] <- new_value
      b[] <- new_value
    } else {
      l[] <- pool
      b[] <- pool
    }
    if (!identical(l, as.logical(b))) {
      message("\nregression test difference when replacing with recylcling")
      OK <- FALSE
    }
  }

  l0 <- c(FALSE, FALSE, FALSE)
  l1 <- c(FALSE, FALSE, TRUE)
  l2 <- c(FALSE, TRUE, TRUE)
  l3 <- c(TRUE, TRUE, TRUE)

  bw0 <- as.bitwhich(l0)
  bw1 <- as.bitwhich(l1)
  bw2 <- as.bitwhich(l2)
  bw3 <- as.bitwhich(l3)

  OK <- OK && identical(l0, as.logical(bw0))
  OK <- OK && identical(l1, as.logical(bw1))
  OK <- OK && identical(l2, as.logical(bw2))
  OK <- OK && identical(l3, as.logical(bw3))

  OK <- OK && identical(l0 & l0, as.logical(bw0 & bw0))
  OK <- OK && identical(l0 & l1, as.logical(bw0 & bw1))
  OK <- OK && identical(l0 & l2, as.logical(bw0 & bw2))
  OK <- OK && identical(l0 & l3, as.logical(bw0 & bw3))

  OK <- OK && identical(l1 & l0, as.logical(bw1 & bw0))
  OK <- OK && identical(l1 & l1, as.logical(bw1 & bw1))
  OK <- OK && identical(l1 & l2, as.logical(bw1 & bw2))
  OK <- OK && identical(l1 & l3, as.logical(bw1 & bw3))

  OK <- OK && identical(l2 & l0, as.logical(bw2 & bw0))
  OK <- OK && identical(l2 & l1, as.logical(bw2 & bw1))
  OK <- OK && identical(l2 & l2, as.logical(bw2 & bw2))
  OK <- OK && identical(l2 & l3, as.logical(bw2 & bw3))

  OK <- OK && identical(l3 & l0, as.logical(bw3 & bw0))
  OK <- OK && identical(l3 & l1, as.logical(bw3 & bw1))
  OK <- OK && identical(l3 & l2, as.logical(bw3 & bw2))
  OK <- OK && identical(l3 & l3, as.logical(bw3 & bw3))


  OK <- OK && identical(l0 | l0, as.logical(bw0 | bw0))
  OK <- OK && identical(l0 | l1, as.logical(bw0 | bw1))
  OK <- OK && identical(l0 | l2, as.logical(bw0 | bw2))
  OK <- OK && identical(l0 | l3, as.logical(bw0 | bw3))

  OK <- OK && identical(l1 | l0, as.logical(bw1 | bw0))
  OK <- OK && identical(l1 | l1, as.logical(bw1 | bw1))
  OK <- OK && identical(l1 | l2, as.logical(bw1 | bw2))
  OK <- OK && identical(l1 | l3, as.logical(bw1 | bw3))

  OK <- OK && identical(l2 | l0, as.logical(bw2 | bw0))
  OK <- OK && identical(l2 | l1, as.logical(bw2 | bw1))
  OK <- OK && identical(l2 | l2, as.logical(bw2 | bw2))
  OK <- OK && identical(l2 | l3, as.logical(bw2 | bw3))

  OK <- OK && identical(l3 | l0, as.logical(bw3 | bw0))
  OK <- OK && identical(l3 | l1, as.logical(bw3 | bw1))
  OK <- OK && identical(l3 | l2, as.logical(bw3 | bw2))
  OK <- OK && identical(l3 | l3, as.logical(bw3 | bw3))


  OK <- OK && identical(xor(l0, l0), as.logical(xor(bw0, bw0)))
  OK <- OK && identical(xor(l0, l1), as.logical(xor(bw0, bw1)))
  OK <- OK && identical(xor(l0, l2), as.logical(xor(bw0, bw2)))
  OK <- OK && identical(xor(l0, l3), as.logical(xor(bw0, bw3)))

  OK <- OK && identical(xor(l1, l0), as.logical(xor(bw1, bw0)))
  OK <- OK && identical(xor(l1, l1), as.logical(xor(bw1, bw1)))
  OK <- OK && identical(xor(l1, l2), as.logical(xor(bw1, bw2)))
  OK <- OK && identical(xor(l1, l3), as.logical(xor(bw1, bw3)))

  OK <- OK && identical(xor(l2, l0), as.logical(xor(bw2, bw0)))
  OK <- OK && identical(xor(l2, l1), as.logical(xor(bw2, bw1)))
  OK <- OK && identical(xor(l2, l2), as.logical(xor(bw2, bw2)))
  OK <- OK && identical(xor(l2, l3), as.logical(xor(bw2, bw3)))

  OK <- OK && identical(xor(l3, l0), as.logical(xor(bw3, bw0)))
  OK <- OK && identical(xor(l3, l1), as.logical(xor(bw3, bw1)))
  OK <- OK && identical(xor(l3, l2), as.logical(xor(bw3, bw2)))
  OK <- OK && identical(xor(l3, l3), as.logical(xor(bw3, bw3)))


  OK <- OK && identical(c(l0, l0), as.logical(c(bw0, bw0)))
  OK <- OK && identical(c(l0, l1), as.logical(c(bw0, bw1)))
  OK <- OK && identical(c(l0, l2), as.logical(c(bw0, bw2)))
  OK <- OK && identical(c(l0, l3), as.logical(c(bw0, bw3)))

  OK <- OK && identical(c(l1, l0), as.logical(c(bw1, bw0)))
  OK <- OK && identical(c(l1, l1), as.logical(c(bw1, bw1)))
  OK <- OK && identical(c(l1, l2), as.logical(c(bw1, bw2)))
  OK <- OK && identical(c(l1, l3), as.logical(c(bw1, bw3)))

  OK <- OK && identical(c(l2, l0), as.logical(c(bw2, bw0)))
  OK <- OK && identical(c(l2, l1), as.logical(c(bw2, bw1)))
  OK <- OK && identical(c(l2, l2), as.logical(c(bw2, bw2)))
  OK <- OK && identical(c(l2, l3), as.logical(c(bw2, bw3)))

  OK <- OK && identical(c(l3, l0), as.logical(c(bw3, bw0)))
  OK <- OK && identical(c(l3, l1), as.logical(c(bw3, bw1)))
  OK <- OK && identical(c(l3, l2), as.logical(c(bw3, bw2)))
  OK <- OK && identical(c(l3, l3), as.logical(c(bw3, bw3)))

  N <- 2L * .BITS
  l <- logical(N)
  b <- bit(N)
  for (i in 1:N) {
    l[i] <- TRUE
    b[i] <- TRUE
    if (!identical(l, as.logical(b))) {
      message("\nregression test difference when replacing at position", i, "")
      OK <- FALSE
    }
  }

  OK
}


test_that("old regtest is still OK", {
  expect_true(regtest.bit())
})

test_that("some old regression tests are also OK for bitwhich", {
  expect_error(TRUE[c(-1, 1)], label="positive mixed with zeros")
  expect_error(as.bit(TRUE)[c(-1, 1)], label="positive mixed with zeros")
  expect_error(as.bitwhich(TRUE)[c(-1, 1)], label="positive mixed with zeros")

  expect_error(TRUE[c(-1, NA)], label="NA mixed with zeros")
  expect_error(as.bit(TRUE)[c(-1, NA)], label="NA mixed with zeros")
  expect_error(as.bitwhich(TRUE)[c(-1, NA)], label="NA mixed with zeros")

  expect_identical(
    as.bit(TRUE)[c(2, 1, 0, 1, NA)],
    TRUE[c(2, 1, 0, 1, NA)],
    ignore_attr="vmode"
  )
  expect_identical(
    as.bitwhich(TRUE)[c(2, 1, 0, 1, NA)],
    TRUE[c(2, 1, 0, 1, NA)],
    ignore_attr="vmode"
  )

  l = FALSE
  l[0L] = TRUE

  b = as.bit(FALSE)
  b[0L] = TRUE
  expect_identical(as.logical(b), l)
  w = as.bitwhich(FALSE)
  w[0L] = TRUE
  expect_identical(as.logical(w), l)

  l = FALSE
  l[2L] = TRUE

  b = as.bit(FALSE)
  b[2L] = TRUE
  expect_identical(as.logical(b), l)
  w = as.bitwhich(FALSE)
  w[2L] = TRUE
  expect_identical(as.logical(w), l)

  l = FALSE
  l[.BITS + 1L] = FALSE
  l[is.na(l)] = FALSE

  b = as.bit(FALSE)
  b[.BITS + 1L] = NA
  expect_identical(as.logical(b), l)
  w = as.bitwhich(FALSE)
  w[.BITS + 1L] = NA
  expect_identical(as.logical(w), l)

  expect_identical(
    as.bit(rep(c(FALSE, TRUE), .BITS))[TRUE],
    rep(c(FALSE, TRUE), .BITS)[TRUE],
    ignore_attr="vmode",
    label="subscripting with scalar TRUE"
  )
  expect_identical(
    as.bitwhich(rep(c(FALSE, TRUE), .BITS))[TRUE],
    rep(c(FALSE, TRUE), .BITS)[TRUE],
    ignore_attr="vmode",
    label="subscripting with scalar TRUE"
  )

  expect_identical(
    as.bit(rep(c(FALSE, TRUE), .BITS))[FALSE],
    rep(c(FALSE, TRUE), .BITS)[FALSE],
    ignore_attr="vmode",
    label="subscripting with scalar FALSE"
  )
  expect_identical(
    as.bitwhich(rep(c(FALSE, TRUE), .BITS))[FALSE],
    rep(c(FALSE, TRUE), .BITS)[FALSE],
    ignore_attr="vmode",
    label="subscripting with scalar FALSE"
  )
})

library("testthat")
library("bit")

regtest.bit <- function(
  N = 50  # number of repetitions for random regression tests
)
{
  #.BITS <- bit:::.BITS  # available in package namespace
  OK <- TRUE
  pool <- c(FALSE, TRUE)
  
  if (!identical(unattr(as.bit(c(FALSE,NA,TRUE))[]), c(FALSE,FALSE,TRUE))){
    message("bit error: wrong coercion of triboolean to (bi)boolean")
    OK <- FALSE
  }
  
  l <- TRUE
  b <- as.bit(l)
  
  i <- -c(1, 0, 1, NA)
  if (!inherits(try(b[i], silent=TRUE), "try-error")){
    message("bit error: did not throw on mixing zero with negative subscripts")
    OK <- FALSE
  }
  
  i <- c(2, 1, 0, 1, NA)
  if (!identical(l[i],unattr(b[i]))){
    message("\nregression test difference between b[i] and l[i]")
    message(l[i])
    message(unattr(b[i]))
    OK <- FALSE
  }
  
  l[0] <- TRUE
  b[0] <- TRUE
  if (!identical(l,unattr(b[]))){
    message("\nregression test difference after assigning at R position zero")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }
  
  l[2] <- TRUE
  b[2] <- TRUE
  if (!identical(ifelse(is.na(l), FALSE, l),unattr(b[]))){
    message("\nregression test difference after assigning after vector length (at 2)")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }
  
  l[.BITS+1] <- FALSE
  b[.BITS+1] <- NA
  if (!identical(ifelse(is.na(l), FALSE, l),unattr(b[]))){
    message("\nregression test difference after assigning after vector length (at .BITS+1)")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }
  
  if (!identical(ifelse(is.na(l[TRUE]), FALSE, l[TRUE]),unattr(b[TRUE]))){
    message("\nregression test difference after subscripting with scalar TRUE")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }
  
  if (!identical(ifelse(is.na(l[FALSE]), FALSE, l[FALSE]),unattr(b[FALSE]))){
    message("\nregression test difference after subscripting with scalar FALSE")
    message(l)
    message(unattr(b[]))
    OK <- FALSE
  }
  
  
  
  for (i in 1:N){
    n <- sample(1:(2*.BITS), 1)
    l <- sample(pool, n, TRUE)
    # check direct coercion
    b <- as.bit(l)
    l2 <- as.logical(b)
    if (!identical(l,l2)){
      message("\nregression test difference between logical")
      message(l)
      message("and as.logical(as.bit(logical))")
      message(l2)
      OK <- FALSE
    }
    
    
    # summary functions with logical return
    s <- c(all=all(l), any=any(l))
    s2 <- c(all=all(b), any=any(b))
    if (!identical(s,s2)){
      message("\nregression test difference between logical summaries")
      message(s)
      message("and bit summaries")
      message(s2)
      OK <- FALSE
    }
    # summary functions with integer return
    if (any(l)){
      s <- c(min=min(as.which(l)), max=max(as.which(l)), range=range(as.which(l)), sum=sum(l), summary=c("FALSE"=length(l)-sum(l), "TRUE"=sum(l), "Min."=min(as.which(l)), "Max."=max(as.which(l))))
    }else{
      s <- c( min=NA_integer_, max=NA_integer_, range=c(NA_integer_, NA_integer_), sum=sum(l), summary=c("FALSE"=length(l)-sum(l), "TRUE"=sum(l), "Min."=as.integer(NA), "Max."=as.integer(NA)) )
    }
    s2 <- c(min=min(b), max=max(b), range=range(b), sum=sum(b), summary=summary(b))
    if (!identical(s,s2)){
      message("\nregression test difference between logical summaries")
      message(s)
      message("and bit summaries")
      message(s2)
      OK <- FALSE
    }
    # check positive whichs
    w <- as.which(l)
    w2 <- as.which(as.bit(w, n))
    if (!identical(w,w2)){
      message("\nregression test difference between which")
      message(w)
      message("and as.which(as.bit.which(which))")
      message(w2)
      OK <- FALSE
    }
    # check automatic whichs (pos or neg whatever shorter)
    s <- sum(l)
    if (s==0){
      w <- FALSE
    }else if (s==n){
      w <- TRUE
    }else if (s>(n%/%2L)){
      w <- -rev(which(!l))
    }else{
      w <- which(l)
    }
    w2 <- as.vector(as.bitwhich(as.bit(l)))
    if (!identical(w,w2)){
      message("\nregression test difference between which")
      message(w)
      message("and as.which(as.bit.which(which))")
      message(w2)
      OK <- FALSE
    }
    # check boolean operators
    l2 <- sample(c(FALSE, TRUE), n, TRUE)
    b2 <- as.bit(l2)
    ops <- c(
      NOT = identical(!l, as.logical(!b))
      , AND = identical(l&l2, as.logical(b&b2))
      , OR = identical(l|l2, as.logical(b|b2))
      , XOR = identical(xor(l,l2), as.logical(xor(b,b2)))
      , NEQ = identical(l!=l2, as.logical(b!=b2))
      , EQ = identical(l==l2, as.logical(b==b2))
    )
    if (!all(ops)){
      message("\nbit differs for boolean operators(s)")
      message(ops)
      message(cbind(l=l, l2=l))
      OK <- FALSE
    }
    w <- as.bitwhich(l)
    w2 <- as.bitwhich(l2)
    ops <- c(
      NOT = identical(!l, as.logical(!w))
      , AND = identical(l&l2, as.logical(w&w2))
      , OR = identical(l|l2, as.logical(w|w2))
      , XOR = identical(xor(l,l2), as.logical(xor(w,w2)))
      , NEQ = identical(l!=l2, as.logical(w!=w2))
      , EQ = identical(l==l2, as.logical(w==w2))
    )
    if (!all(ops)){
      message("\nbitwhich differs for boolean operators(s)")
      message(ops)
      message(cbind(l=l, l2=l))
      OK <- FALSE
    }
    rm(l2,b2,w2)
    # check extractors
    n2 <- sample(1:n, 1)
    j <- sample(1:n, n2)
    if (!identical(l[j], unattr(b[j]))){
      message("\nregression test difference when extracting")
      OK <- FALSE
    }
    # check replacement (index)
    new <- sample(pool, n2, TRUE)
    l[j] <- new
    b[j] <- new
    if (!identical(l, unattr(b[]))){
      message("\nregression test difference when replacing with index")
      OK <- FALSE
    }
    # check replacement (recycle)
    if (n%%2){
      new <- sample(pool, 1)
      l[] <- new
      b[] <- new
    }else{
      l[] <- pool
      b[] <- pool
    }
    if (!identical(l, as.logical(b))){
      message("\nregression test difference when replacing with recylcling")
      OK <- FALSE
    }
  }
  
  l0 <- c(FALSE, FALSE, FALSE)
  l1 <- c(FALSE, FALSE, TRUE)
  l2 <- c(FALSE, TRUE, TRUE)
  l3 <- c(TRUE, TRUE, TRUE)
  
  bw0 <- as.bitwhich(l0)
  bw1 <- as.bitwhich(l1)
  bw2 <- as.bitwhich(l2)
  bw3 <- as.bitwhich(l3)
  
  OK <- OK && identical(l0, as.logical(bw0))
  OK <- OK && identical(l1, as.logical(bw1))
  OK <- OK && identical(l2, as.logical(bw2))
  OK <- OK && identical(l3, as.logical(bw3))
  
  OK <- OK && identical(l0 & l0, as.logical(bw0 & bw0))
  OK <- OK && identical(l0 & l1, as.logical(bw0 & bw1))
  OK <- OK && identical(l0 & l2, as.logical(bw0 & bw2))
  OK <- OK && identical(l0 & l3, as.logical(bw0 & bw3))
  
  OK <- OK && identical(l1 & l0, as.logical(bw1 & bw0))
  OK <- OK && identical(l1 & l1, as.logical(bw1 & bw1))
  OK <- OK && identical(l1 & l2, as.logical(bw1 & bw2))
  OK <- OK && identical(l1 & l3, as.logical(bw1 & bw3))
  
  OK <- OK && identical(l2 & l0, as.logical(bw2 & bw0))
  OK <- OK && identical(l2 & l1, as.logical(bw2 & bw1))
  OK <- OK && identical(l2 & l2, as.logical(bw2 & bw2))
  OK <- OK && identical(l2 & l3, as.logical(bw2 & bw3))
  
  OK <- OK && identical(l3 & l0, as.logical(bw3 & bw0))
  OK <- OK && identical(l3 & l1, as.logical(bw3 & bw1))
  OK <- OK && identical(l3 & l2, as.logical(bw3 & bw2))
  OK <- OK && identical(l3 & l3, as.logical(bw3 & bw3))
  
  
  OK <- OK && identical(l0 | l0, as.logical(bw0 | bw0))
  OK <- OK && identical(l0 | l1, as.logical(bw0 | bw1))
  OK <- OK && identical(l0 | l2, as.logical(bw0 | bw2))
  OK <- OK && identical(l0 | l3, as.logical(bw0 | bw3))
  
  OK <- OK && identical(l1 | l0, as.logical(bw1 | bw0))
  OK <- OK && identical(l1 | l1, as.logical(bw1 | bw1))
  OK <- OK && identical(l1 | l2, as.logical(bw1 | bw2))
  OK <- OK && identical(l1 | l3, as.logical(bw1 | bw3))
  
  OK <- OK && identical(l2 | l0, as.logical(bw2 | bw0))
  OK <- OK && identical(l2 | l1, as.logical(bw2 | bw1))
  OK <- OK && identical(l2 | l2, as.logical(bw2 | bw2))
  OK <- OK && identical(l2 | l3, as.logical(bw2 | bw3))
  
  OK <- OK && identical(l3 | l0, as.logical(bw3 | bw0))
  OK <- OK && identical(l3 | l1, as.logical(bw3 | bw1))
  OK <- OK && identical(l3 | l2, as.logical(bw3 | bw2))
  OK <- OK && identical(l3 | l3, as.logical(bw3 | bw3))
  
  
  OK <- OK && identical(xor(l0,l0), as.logical(xor(bw0,bw0)))
  OK <- OK && identical(xor(l0,l1), as.logical(xor(bw0,bw1)))
  OK <- OK && identical(xor(l0,l2), as.logical(xor(bw0,bw2)))
  OK <- OK && identical(xor(l0,l3), as.logical(xor(bw0,bw3)))
  
  OK <- OK && identical(xor(l1,l0), as.logical(xor(bw1,bw0)))
  OK <- OK && identical(xor(l1,l1), as.logical(xor(bw1,bw1)))
  OK <- OK && identical(xor(l1,l2), as.logical(xor(bw1,bw2)))
  OK <- OK && identical(xor(l1,l3), as.logical(xor(bw1,bw3)))
  
  OK <- OK && identical(xor(l2,l0), as.logical(xor(bw2,bw0)))
  OK <- OK && identical(xor(l2,l1), as.logical(xor(bw2,bw1)))
  OK <- OK && identical(xor(l2,l2), as.logical(xor(bw2,bw2)))
  OK <- OK && identical(xor(l2,l3), as.logical(xor(bw2,bw3)))
  
  OK <- OK && identical(xor(l3,l0), as.logical(xor(bw3,bw0)))
  OK <- OK && identical(xor(l3,l1), as.logical(xor(bw3,bw1)))
  OK <- OK && identical(xor(l3,l2), as.logical(xor(bw3,bw2)))
  OK <- OK && identical(xor(l3,l3), as.logical(xor(bw3,bw3)))
  
  
  OK <- OK && identical(c(l0,l0), as.logical(c(bw0,bw0)))
  OK <- OK && identical(c(l0,l1), as.logical(c(bw0,bw1)))
  OK <- OK && identical(c(l0,l2), as.logical(c(bw0,bw2)))
  OK <- OK && identical(c(l0,l3), as.logical(c(bw0,bw3)))
  
  OK <- OK && identical(c(l1,l0), as.logical(c(bw1,bw0)))
  OK <- OK && identical(c(l1,l1), as.logical(c(bw1,bw1)))
  OK <- OK && identical(c(l1,l2), as.logical(c(bw1,bw2)))
  OK <- OK && identical(c(l1,l3), as.logical(c(bw1,bw3)))
  
  OK <- OK && identical(c(l2,l0), as.logical(c(bw2,bw0)))
  OK <- OK && identical(c(l2,l1), as.logical(c(bw2,bw1)))
  OK <- OK && identical(c(l2,l2), as.logical(c(bw2,bw2)))
  OK <- OK && identical(c(l2,l3), as.logical(c(bw2,bw3)))
  
  OK <- OK && identical(c(l3,l0), as.logical(c(bw3,bw0)))
  OK <- OK && identical(c(l3,l1), as.logical(c(bw3,bw1)))
  OK <- OK && identical(c(l3,l2), as.logical(c(bw3,bw2)))
  OK <- OK && identical(c(l3,l3), as.logical(c(bw3,bw3)))
  
  N <- 2L*.BITS
  l <- logical(N)
  b <- bit(N)
  for (i in 1:N){
    l[i] <- TRUE
    b[i] <- TRUE
    if (!identical(l,as.logical(b))){
      message("\nregression test difference when replacing at position", i, "")
      OK <- FALSE
    }
  }
  
  OK
}


context("old regression test")

test_that("old regtest is still OK", {
  expect_identical(regtest.bit(), TRUE)
})

test_that("some old regression tests are also OK for bitwhich", {
  expect_error(TRUE[c(-1, 1)], label="positive mixed with zeros")
  expect_error(as.bit(TRUE)[c(-1, 1)], label="positive mixed with zeros")
  expect_error(as.bitwhich(TRUE)[c(-1, 1)], label="positive mixed with zeros")
  
  expect_error(TRUE[c(-1, NA)], label="NA mixed with zeros")
  expect_error(as.bit(TRUE)[c(-1, NA)], label="NA mixed with zeros")
  expect_error(as.bitwhich(TRUE)[c(-1, NA)], label="NA mixed with zeros")
  
  expect_equivalent(as.bit(TRUE)[c(2, 1, 0, 1, NA)], TRUE[c(2, 1, 0, 1, NA)])
  expect_equivalent(as.bitwhich(TRUE)[c(2, 1, 0, 1, NA)], TRUE[c(2, 1, 0, 1, NA)])
  
  expect_identical({b <- as.bit(FALSE); b[0] <- TRUE; as.logical(b)}, {l <- FALSE; l[0] <- TRUE; l})
  expect_identical({w <- as.bitwhich(FALSE); w[0] <- TRUE; as.logical(w)}, {l <- FALSE; l[0] <- TRUE; l})
  
  expect_identical({b <- as.bit(FALSE); b[2] <- TRUE; as.logical(b)}, {l <- FALSE; l[2] <- TRUE; l})
  expect_identical({w <- as.bitwhich(FALSE); w[2] <- TRUE; as.logical(w)}, {l <- FALSE; l[2] <- TRUE; l})
  
  expect_identical({b <- as.bit(FALSE); b[bit:::.BITS+1] <- NA; as.logical(b)}, {l <- FALSE; l[bit:::.BITS+1] <- FALSE; l[is.na(l)]<- FALSE; l})
  expect_identical({w <- as.bitwhich(FALSE); w[bit:::.BITS+1] <- NA; as.logical(w)}, {l <- FALSE; l[bit:::.BITS+1] <- FALSE; l[is.na(l)]<- FALSE; l})
  
  expect_equivalent(as.bit(rep(c(FALSE,TRUE), bit:::.BITS))[TRUE], rep(c(FALSE,TRUE), bit:::.BITS)[TRUE], label="subscripting with scalar TRUE")  
  expect_equivalent(as.bitwhich(rep(c(FALSE,TRUE), bit:::.BITS))[TRUE], rep(c(FALSE,TRUE), bit:::.BITS)[TRUE], label="subscripting with scalar TRUE")  
  
  expect_equivalent(as.bit(rep(c(FALSE,TRUE), bit:::.BITS))[FALSE], rep(c(FALSE,TRUE), bit:::.BITS)[FALSE], label="subscripting with scalar FALSE")  
  expect_equivalent(as.bitwhich(rep(c(FALSE,TRUE), bit:::.BITS))[FALSE], rep(c(FALSE,TRUE), bit:::.BITS)[FALSE], label="subscripting with scalar FALSE")  
})  

# Package generics
# (c) 2008-2017 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk

# source("D:/mwp/eanalysis/bit/R/generics.R")


#' Get maxindex (length of boolean vector) and poslength (number of 'selected' elements)
#'
#' For [is.booltype()] objects the term [length()] is ambiguous.
#' For example the length of [which()] corresponds to the sum of [logical()].
#' The generic `maxindex` gives `length(logical)` for all [booltype()]s.
#' The generic `poslength` gives the number of positively selected elements, i.e.
#'   `sum(logical)` for all [booltype()]s (and gives `NA` if `NAs` are present).
#'
#' @name maxindex
#' @param x an R object, typically a [is.booltype()] object.
#' @param ... further arguments (ignored)
#' @return an integer scalar
#' @examples
#' r <- ri(1, 2, 12)
#' i <- as.which(r)
#' w <- as.bitwhich(r)
#' b <- as.bit(r)
#' l <- as.logical(r)
#' u <- which(l)      # unclassed which
#'
#' sapply(list(r=r, u=u, i=i, w=w, b=b, l=l), function(x) {
#'   c(length=length(x), sum=sum(x), maxindex=maxindex(x), poslength=poslength(x))
#' })
#' @export
maxindex <- function(x, ...) UseMethod("maxindex", x)

#' @name maxindex
#' @export
poslength <- function(x, ...) UseMethod("poslength")

#' Coerce to booltype (generic)
#'
#' @param x object to coerce
#' @param booltype target [booltype()] given as integer or as character
#' @param ... further arguments
#'
#' @return `x` coerced to `booltype`
#' @seealso [`CoercionToStandard`][CoercionToStandard], [booltypes()], [booltype()],
#'   [is.booltype()]
#'
#' @examples
#' as.booltype(0:1)
#' as.booltype(0:1, "logical")
#' as.booltype(0:1, "bit")
#' as.booltype(0:1, "bitwhich")
#' as.booltype(0:1, "which", maxindex=2)
#' as.booltype(0:1, "ri")
#' @export
as.booltype <- function(x, booltype, ...) UseMethod("as.booltype")

#' Coercing to bit
#'
#' Coercing to bit vector
#'
#' Coercing to bit is quite fast because we use a double loop that fixes each
#' word in a processor register
#'
#' @param x an object of class [bit()], [logical()],
#' [integer()], [bitwhich()] or an integer from
#' [as.which()] or a boolean [`ff`][ff::vmode]
#' @param length the length of the new bit vector
#' @param ... further arguments
#' @return `is.bit` returns FALSE or TRUE, `as.bit` returns a vector
#' of class 'bit'
#' @note Zero is coerced to FALSE, all other numbers including NA are coerced
#' to TRUE.  This differs from the NA-to-FALSE coercion in package ff and may
#' change in the future.
#' @author Jens Oehlschlägel
#' @seealso [`CoercionToStandard`][CoercionToStandard], [as.booltype()], [as.bit()],
#'   [as.bitwhich()] , [as.which()], [as.ri()], [ff::as.hi()],  [ff::as.ff()]
#' @keywords classes logic
#' @examples
#'
#' as.bit(c(FALSE, NA, TRUE))
#'
#' @export
as.bit <- function(x=NULL, ...) UseMethod("as.bit", x)

#' Coercing to bitwhich
#'
#' Functions to coerce to bitwhich
#'
#' @param x An object of class 'bitwhich', 'integer', 'logical' or 'bit' or an
#' integer vector as resulting from 'which'
#' @param maxindex the length of the new bitwhich vector
#' @param poslength the number of selected elements
#' @param range a [ri()] or an integer vector of length == 2 giving a
#' range restriction for chunked processing
#' @param ... further arguments
#' @return a value of class [bitwhich()]
#' @author Jens Oehlschlägel
#' @seealso [`CoercionToStandard`][CoercionToStandard], [as.booltype()], [as.bit()],
#'   [as.bitwhich()] , [as.which()], [as.ri()], [ff::as.hi()],  [ff::as.ff()]
#' @keywords classes logic
#' @examples
#'
#'  as.bitwhich(c(NA, NA, NA))
#'  as.bitwhich(c(FALSE, FALSE, FALSE))
#'  as.bitwhich(c(FALSE, FALSE, TRUE))
#'  as.bitwhich(c(FALSE, TRUE, TRUE))
#'  as.bitwhich(c(TRUE, TRUE, TRUE))
#'
#' @export
as.bitwhich <- function(x=NULL, ...) UseMethod("as.bitwhich")

#' Coercion to (positive) integer positions
#'
#' Coercing to something like the result of which [which()]
#'
#' `as.which.bit` returns a vector of subscripts with class 'which'
#'
#' @param x an object of classes [bit()], [bitwhich()],
#' [ri()] or something on which [which()] works
#' @param range a [ri()] or an integer vector of length == 2 giving a
#' range restriction for chunked processing
#' @param maxindex the length of the boolean vector which is represented
#' @param is.unsorted a logical scalar indicating whether the data may be unsorted
#' @param has.dup a logical scalar indicating whether the data may have duplicates
#' @param ... further arguments (passed to [which()] for the
#' default method, ignored otherwise)
#' @return a vector of class 'logical' or 'integer'
#' @author Jens Oehlschlägel
#' @seealso [`CoercionToStandard`][CoercionToStandard], [as.booltype()], [as.bit()],
#'   [as.bitwhich()] , [as.which()], [as.ri()], [ff::as.hi()],  [ff::as.ff()]
#' @keywords classes logic
#' @examples
#'
#'   r <- ri(5, 20, 100)
#'   x <- as.which(r)
#'   x
#'
#'   stopifnot(identical(x, as.which(as.logical(r))))
#'   stopifnot(identical(x, as.which(as.bitwhich(r))))
#'   stopifnot(identical(x, as.which(as.bit(r))))
#'
#' @export
as.which <- function(x, ...) UseMethod("as.which")

#' Coerce to ri
#'
#' @param x object to coerce
#' @param ... further arguments
#'
#' @return an [ri()] object
#' @author Jens Oehlschlägel
#' @seealso [`CoercionToStandard`][CoercionToStandard], [as.booltype()], [as.bit()],
#'   [as.bitwhich()] , [as.which()], [as.ri()], [ff::as.hi()],  [ff::as.ff()]
#' @keywords classes logic
#'
#' @examples
#' as.ri(c(FALSE, TRUE, FALSE, TRUE))
#' @export
as.ri <- function(x, ...) UseMethod("as.ri")

#' Boolean operators and functions
#'
#' Boolean NEGATION '!', AND '&', OR '|' and EXCLUSIVE OR xor', see
#'   [`Logic`][base::Logic].
#'
#' The binary operators and function `xor` can now combine any [is.booltype()] vectors.
#' They now recycle if vectors have different length. If the two arguments have different
#'   [booltypes()] the return value corresponds to the lower [booltype()] of the two.
#'
#' Boolean operations on [bit()] vectors are extremely fast because they are
#' implemented using C's bitwise operators. Boolean operations on or [bitwhich()]
#' vectors are even faster, if they represent very skewed selections.
#'
#' The `xor` function has been made generic and `xor.default` has
#' been implemented much faster than R's standard [xor()].
#' This was possible because actually boolean function `xor` and
#' comparison operator ` != ` do the same (even with NAs), and ` != ` is
#' much faster than the multiple calls in `(x | y) & !(x & y)`
#'
#' @param x a [is.booltype()] vector
#' @param y a [is.booltype()] vector
#' @param e1 a [is.booltype()] vector
#' @param e2 a [is.booltype()] vector
#' @return An object of class [booltype()] or [logical()]
#' @author Jens Oehlschlägel
#' @seealso [booltypes()], [`Logic`][Logic]
#' @keywords classes logic
#' @examples
#'
#'   x <- c(FALSE, FALSE, FALSE, NA, NA, NA, TRUE, TRUE, TRUE)
#'   y <- c(FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE)
#'
#'   x | y
#'   x | as.bit(y)
#'   x | as.bitwhich(y)
#'   x | as.which(y)
#'   x | ri(1, 1, 9)
#'
#'
#' @export
xor <- function(x, y) UseMethod("xor", x)

#' Physical and virtual attributes
#'
#' Compatibility functions (to package ff) for getting and setting physical and
#' virtual attributes.
#'
#' ff objects have physical and virtual attributes, which have different
#' copying semantics: physical attributes are shared between copies of ff
#' objects while virtual attributes might differ between copies.
#' [ff::as.ram()] will retain some physical and virtual atrributes in
#' the ram clone, such that [ff::as.ff()] can restore an ff object
#' with the same attributes.
#'
#' @name PhysVirt
#' @param x a ff or ram object
#' @param value a list with named elements
#' @param ... further arguments
#' @return `physical` and `virtual` returns a list with named elements
#' @author Jens Oehlschlägel
#' @seealso [ff::physical.ff()], [ff::physical.ffdf()]
#' @keywords IO data attribute
#' @examples
#'
#'   physical(bit(12))
#'   virtual(bit(12))
NULL

#' @rdname PhysVirt
#' @export
physical <- function(x) UseMethod("physical")

#' @rdname PhysVirt
#' @export
`physical<-` <- function(x, value) UseMethod("physical<-")

#' @rdname PhysVirt
#' @export
virtual <- function(x) UseMethod("virtual")

#' @rdname PhysVirt
#' @export
`virtual<-` <- function(x, value) UseMethod("virtual<-")


#' Generics for in-RAM sorting and ordering
#'
#' These are generic stubs for low-level sorting and ordering methods
#' implemented in packages 'bit64' and 'ff'.  The `..sortorder` methods do
#' sorting and ordering at once, which requires more RAM than ordering but is
#' (almost) as fast as as sorting.
#'
#' The `sort` generics do sort their argument 'x', some methods need
#' temporary RAM of the same size as 'x'.  The `order` generics do order
#' their argument 'i' leaving 'x' as it was, some methods need temporary RAM of
#' the same size as 'i'.  The `sortorder` generics do sort their argument
#' 'x' and order their argument 'i', this way of ordering is much faster at the
#' price of requiring temporary RAM for both, 'x' and 'i', if the method
#' requires temporary RAM.  The `ram` generics are high-level functions
#' containing an optimizer that chooses the 'best' algorithms given some
#' context.
#'
#' @name Sorting
#' @param x a vector to be sorted by [ramsort()] and
#' [ramsortorder()], i.e. the output of [sort()]
#' @param i integer positions to be modified by [ramorder()] and
#' [ramsortorder()], default is 1:n, in this case the output is
#' similar to [order()]
#' @param ... further arguments to the sorting methods
#' @return These functions return the number of `NAs` found or assumed
#' during sorting
#' @note Note that these methods purposely violate the functional programming
#' paradigm: they are called for the side-effect of changing some of their
#' arguments.  The rationale behind this is that sorting is very RAM-intensive
#' and in certain situations we might not want to allocate additional memory if
#' not necessary to do so.  The `sort`-methods change `x`, the
#' `order`-methods change `i`, and the `sortoder`-methods change
#' both `x` and `i` You as the user are responsible to create copies
#' of the input data 'x' and 'i' if you need non-modified versions.
#' @section Index of implemented methods:
#' |      **generic** |                     **ff** | **bit64**                           |
#' |-----------------:|---------------------------:|:------------------------------------|
#' |        `ramsort` |    [ff::ramsort.default()] |        [bit64::ramsort.integer64()] |
#' |      `shellsort` |  [ff::shellsort.default()] |      [bit64::shellsort.integer64()] |
#' |      `quicksort` |                            |      [bit64::quicksort.integer64()] |
#' |      `mergesort` |  [ff::mergesort.default()] |      [bit64::mergesort.integer64()] |
#' |      `radixsort` |  [ff::radixsort.default()] |      [bit64::radixsort.integer64()] |
#' |        `keysort` |    [ff::keysort.default()] |                                     |
#' |       `ramorder` |   [ff::ramorder.default()] |       [bit64::ramorder.integer64()] |
#' |     `shellorder` | [ff::shellorder.default()] |     [bit64::shellorder.integer64()] |
#' |     `quickorder` |                            |     [bit64::quickorder.integer64()] |
#' |     `mergeorder` | [ff::mergeorder.default()] |     [bit64::mergeorder.integer64()] |
#' |     `radixorder` | [ff::radixorder.default()] |     [bit64::radixorder.integer64()] |
#' |       `keyorder` |   [ff::keyorder.default()] |                                     |
#' |   `ramsortorder` |                            |   [bit64::ramsortorder.integer64()] |
#' | `shellsortorder` |                            | [bit64::shellsortorder.integer64()] |
#' | `quicksortorder` |                            | [bit64::quicksortorder.integer64()] |
#' | `mergesortorder` |                            | [bit64::mergesortorder.integer64()] |
#' | `radixsortorder` |                            | [bit64::radixsortorder.integer64()] |
#' |   `keysortorder` |                            |                                     |
#'
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso [sort()] and [order()] in base R, [bitsort()] for faster inteer sorting
#' @keywords univar manip arith
NULL

#' @rdname Sorting
#' @export
ramsort <- function(x, ...) UseMethod("ramsort")
#' @rdname Sorting
#' @export
ramorder <- function(x, i, ...) UseMethod("ramorder")
#' @rdname Sorting
#' @export
ramsortorder <- function(x, i, ...) UseMethod("ramsortorder")
#' @rdname Sorting
#' @export
mergesort <- function(x, ...) UseMethod("mergesort")
#' @rdname Sorting
#' @export
mergeorder <- function(x, i, ...) UseMethod("mergeorder")
#' @rdname Sorting
#' @export
mergesortorder <- function(x, i, ...) UseMethod("mergesortorder")
#' @rdname Sorting
#' @export
quicksort <- function(x, ...) UseMethod("quicksort")
#' @rdname Sorting
#' @export
quickorder <- function(x, i, ...) UseMethod("quickorder")
#' @rdname Sorting
#' @export
quicksortorder <- function(x, i, ...) UseMethod("quicksortorder")
#' @rdname Sorting
#' @export
shellsort <- function(x, ...) UseMethod("shellsort")
#' @rdname Sorting
#' @export
shellorder <- function(x, i, ...) UseMethod("shellorder")
#' @rdname Sorting
#' @export
shellsortorder <- function(x, i, ...) UseMethod("shellsortorder")
#' @rdname Sorting
#' @export
radixsort <- function(x, ...) UseMethod("radixsort")
#' @rdname Sorting
#' @export
radixorder <- function(x, i, ...) UseMethod("radixorder")
#' @rdname Sorting
#' @export
radixsortorder <- function(x, i, ...) UseMethod("radixsortorder")
#' @rdname Sorting
#' @export
keysort <- function(x, ...) UseMethod("keysort")
#' @rdname Sorting
#' @export
keyorder <- function(x, i, ...) UseMethod("keyorder")
#' @rdname Sorting
#' @export
keysortorder <- function(x, i, ...) UseMethod("keysortorder")


#' Generics related to cache access
#'
#' These generics are packaged here for methods in packages `bit64` and
#' `ff`.
#'
#' see help of the available methods
#'
#' @name Metadata
#' @param x some object
#' @param value value assigned on responsibility of the user
#' @param ... ignored
#' @return see help of the available methods
#' @author Jens Oehlschlägel <Jens.Oehlschlaegel@@truecluster.com>
#' @seealso [bit64::is.sorted.integer64()], [bit64::na.count.integer64()],
#'   [bit64::nvalid.integer64()], [bit64::nunique.integer64()], [bit64::nties.integer64()]
#' @keywords environment methods
#' @examples
#'   methods("na.count")
NULL

#' @rdname Metadata
#' @export
is.sorted <- function(x, ...) UseMethod("is.sorted")

#' @rdname Metadata
#' @export
`is.sorted<-` <- function(x, ..., value) UseMethod("is.sorted<-")

#' @rdname Metadata
#' @export
na.count <- function(x, ...) UseMethod("na.count")

#' @rdname Metadata
#' @export
`na.count<-` <- function(x, ..., value) UseMethod("na.count<-")

#' @rdname Metadata
#' @export
nvalid <- function(x, ...) UseMethod("nvalid")

#' @rdname Metadata
#' @export
nunique <- function(x, ...) UseMethod("nunique")

#' @rdname Metadata
#' @export
`nunique<-` <- function(x, ..., value) UseMethod("nunique<-")

#' @rdname Metadata
#' @export
nties <- function(x, ...) UseMethod("nties")

#' @rdname Metadata
#' @export
`nties<-` <- function(x, ..., value) UseMethod("nties<-")

---
title: "Performance of the bit package"
author: "Dr. Jens Oehlschlägel"
date: '`r Sys.Date()`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Performance of the bit package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(bit)
library(microbenchmark)
# rmarkdown::render("vignettes/bit-performance.Rmd")
# these are the real settings for the performance vignette
times <- 5
Domain <- c(small=1e3, big=1e6)
Sample <- c(small=1e3, big=1e6)
# these are the settings to keep the cost of CRAN low
#times <- 5
#Domain <- c(small=1e1, big=1e3)
#Sample <- c(small=1e1, big=1e3)

pagebreak <- function() {
  if (knitr::is_latex_output())
    "\\newpage"
  else
    '<div style="page-break-before: always;" />'
}

```

---


## A performance example

Before we measure performance of the main functionality of the package, note that something simple as '(a:b)[-i]' can and has been accelerated in this package:

```{r, echo=TRUE, results='asis'}
a <- 1L
b <- 1e7L
i <- sample(a:b, 1e3)
x <- c(
  R = median(microbenchmark((a:b)[-i], times=times)$time),
  bit = median(microbenchmark(bit_rangediff(c(a, b), i), times=times)$time),
  merge = median(microbenchmark(merge_rangediff(c(a, b), bit_sort(i)), times=times)$time)
)
knitr::kable(as.data.frame(as.list(x / x["R"] * 100)), caption="% of time relative to R", digits=1)
```

The vignette is compiled with the following performance settings: `r times` replications with domain size small `r Domain["small"]` and big `r Domain["big"]`, sample size small `r Sample["small"]` and big `r Sample["big"]`.

## Boolean data types


> "A designer knows he has achieved perfection not when there is nothing left to add, but when there is nothing left to take away."
"Il semble que la perfection soit atteinte non quand il n'y a plus rien à ajouter, mais quand il n'y a plus rien à retrancher"
(Antoine de St. Exupery, Terre des Hommes (Gallimard, 1939), p. 60.)

We compare memory consumption (n=`r format(Sample[["big"]])`) and runtime (median of `r times` replications) of the different `booltype`s for the following filter scenarios:

```{r, echo=FALSE, results='asis', echo=3:5}
# TODO(r-lib/lintr#773): nolint as a chunk option.
# nolint start: strings_as_factors_linter.
knitr::kable(
  data.frame(
    coin="random 50%",
    often="random 99%",
    rare="random 1%",
    chunk="contiguous chunk of 5%"
  ),
  caption="selection characteristic"
)
# nolint end: strings_as_factors_linter.
```

```{r, echo=FALSE, results='asis'}
B <- booltypes[c("logical", "bit", "bitwhich", "which", "ri")]
M <-
  c("size", "[]", "[which]", "[which]<-TRUE", "[]<-logical", "!", "&", "|", "==", "!=", "summary")
G <- list(
  coin = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.5, 0.5)),
  often = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.01, 0.99)),
  rare = function(n) sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.99, 0.01)),
  chunk = function(n) ri(n %/% 20, 2L * n %/% 20, n)
)
X <- vector("list", length(B) * length(G))
dim(X) <- c(booltype=length(B), data=length(G))
dimnames(X) <- list(booltype=names(B), data=names(G))
tim <- array(NA,
  dim=c(booltype=length(B), metric=length(M), data=length(G)),
  dimnames=list(booltype=names(B), metric=M, data=names(G))
)
for (g in names(G)) {
  x <- G[[g]](Sample[["big"]])
  if (g %in% c("coin", "often", "rare"))
    w <- as.which(as.logical(x))
  for (b in B) {
    if (booltypes[[b]] < 'ri' || (b == 'ri' && g == 'chunk')) {
      X[[b, g]] <- as.booltype(x, b)
      if (g %in% c("coin", "often", "rare") && b %in% c("logical", "bit", "bitwhich")) {
        l <- as.booltype(logical(Sample[["big"]]), b)
        tim[b, "[which]", g] <- median(microbenchmark(l[w], times=times)$time)
        tim[b, "[which]<-TRUE", g] <- median(microbenchmark(l[w] <- TRUE, times=times)$time)
        tim[b, "[]", g] <- median(microbenchmark(l[], times=times)$time)
        tim[b, "[]<-logical", g] <- median(microbenchmark(l[] <- x, times=times)$time)
      }
      tim[b, "size", g] <- object.size(X[[b, g]])
    }
  }
}
for (g in names(G)) {
  for (b in c("logical", "bit", "bitwhich")) {
    x <- X[[b, g]]
    if (!is.null(x)) {
      tim[b, "!", g] <- median(microbenchmark(!x, times=times)$time)
      tim[b, "&", g] <- median(microbenchmark(x & x, times=times)$time)
      tim[b, "|", g] <- median(microbenchmark(x | x, times=times)$time)
      tim[b, "==", g] <- median(microbenchmark(x == x, times=times)$time)
      tim[b, "!=", g] <- median(microbenchmark(x != x, times=times)$time)
      tim[b, "summary", g] <- median(microbenchmark(summary.booltype(x), times=times)$time)
    }
  }
}
i <- match("size", M)
for (b in rev(names(B))) { # logical was in first position, so we do this last!
  tim[b, i, ] <- 100 * tim[b, i, ] / tim["logical", i, ]
  tim[b, -i, ] <- 100 * tim[b, -i, ] / max(tim["logical", -i, ], na.rm=TRUE)
}
#rm(X)
```

There are substantial savings in skewed filter situations:

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'rare' scenario"}
x <- tim[1:3, , "rare"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'rare' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)
```

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'often' scenario"}
x <- tim[1:3, , "often"]
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'often' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)
```

Even in non-skewed situations the new booltypes are competitive:

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'coin' scenario"}
x <- tim[1:3, , "coin"]
dotchart(x,
  xlim=c(0, max(100, max(x))),
  labels=m,
  pch=c("R", "b", "w"),
  col=c("black", "blue", "red"),
  main="% size and timings in 'coin' scenario",
  sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios"
)
```

Detailed tables follow.


`r pagebreak()`


### % memory consumption of filter

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "size", ], 1), caption="% bytes of logical")
```

### % time extracting

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "[]", ], 1), caption="% time of logical")
```

### % time assigning

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "[]<-logical", ], 1), caption="% time of logical")
```

### % time subscripting with 'which'

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "[which]", ], 1), caption="% time of logical")
```

### % time assigning with 'which'

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "[which]<-TRUE", ], 1), caption="% time of logical")
```

### % time Boolean NOT

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "!", ], 1), caption="% time for Boolean NOT")
```

### % time Boolean AND

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "&", ], 1), caption="% time for Boolean &")
```


### % time Boolean OR

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "|", ], 1), caption="% time for Boolean |")
```

### % time Boolean EQUALITY

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "==", ], 1), caption="% time for Boolean ==")
```

### % time Boolean XOR
```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "!=", ], 1), caption="% time for Boolean !=")
```

### % time Boolean SUMMARY

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[, "summary", ][1:2, 1:2], 1), caption="% time for Boolean summary")
```

---

## Fast methods for `integer` set operations

> "The space-efficient structure of bitmaps dramatically reduced the run time of sorting"
(Jon Bently, Programming Pearls, Cracking the oyster, p. 7)


```{r, echo=FALSE, results='asis'}
binaryDomain <- list(
  smallsmall = rep(Domain["small"], 2L),
  smallbig=Domain,
  bigsmall=rev(Domain),
  bigbig=rep(Domain["big"], 2L)
)
binarySample <- list(
  smallsmall = rep(Sample["small"], 2L),
  smallbig=Sample,
  bigsmall=rev(Sample),
  bigbig=rep(Sample["big"], 2L)
)

M <- c("R", "bit", "merge")
G <- c("sort", "sortunique")
D <- c("unsorted", "sorted")

sortM <- vector("list", length(M) * length(G))
dim(sortM) <- c(method=length(M), goal=length(G))
dimnames(sortM) <- list(method=M, goal=G)
sortM[["R", "sort"]] <- sort
sortM[["R", "sortunique"]] <- function(x) sort(unique(x))
sortM[["bit", "sort"]] <- bit_sort
sortM[["bit", "sortunique"]] <- bit_sort_unique

timsort <- array(NA_integer_,
  dim=c(M=2, G=length(G), D=length(D), N=length(Domain)),
  dimnames=list(M=M[1:2], G=G, D=D, N=names(Domain))
)
for (n in names(Domain)) {
  x <- sample(Domain[[n]], Sample[[n]], replace = TRUE)
  d <- "unsorted"
  for (m in c("R", "bit")) {
    for (g in G) {
      timsort[m, g, d, n] <- median(microbenchmark(sortM[[m, g]](x), times=times)$time)
    }
  }
  x <- bit_sort(x)
  d <- "sorted"
  for (m in 1:2) {
    for (g in G) {
      timsort[m, g, d, n] <- median(microbenchmark(sortM[[m, g]](x), times=times)$time)
    }
  }
}


binaryU <-
  c("match", "in", "notin", "union", "intersect", "setdiff", "symdiff", "setequal", "setearly")
binaryM <- vector("list", length(M) * length(binaryU))
dim(binaryM) <- c(method=length(M), task=length(binaryU))
dimnames(binaryM) <- list(method=M, task=binaryU)
binaryM[["R", "match"]] <- match
binaryM[["merge", "match"]] <- merge_match

binaryM[["R", "in"]] <- get("%in%")
binaryM[["bit", "in"]] <- bit_in
binaryM[["merge", "in"]] <- merge_in

binaryM[["R", "notin"]] <- function(x, y) !(x %in% y)
binaryM[["bit", "notin"]] <- function(x, y) !bit_in(x, y)
binaryM[["merge", "notin"]] <- merge_notin

binaryM[["R", "union"]] <- union
binaryM[["bit", "union"]] <- bit_union
binaryM[["merge", "union"]] <- merge_union

binaryM[["R", "intersect"]] <- intersect
binaryM[["bit", "intersect"]] <- bit_intersect
binaryM[["merge", "intersect"]] <- merge_intersect

binaryM[["R", "setdiff"]] <- setdiff
binaryM[["bit", "setdiff"]] <- bit_setdiff
binaryM[["merge", "setdiff"]] <- merge_setdiff

binaryM[["R", "symdiff"]] <- function(x, y) union(setdiff(x, y), setdiff(y, x))
binaryM[["bit", "symdiff"]] <- bit_symdiff
binaryM[["merge", "symdiff"]] <- merge_symdiff

binaryM[["R", "setequal"]] <-
  function(x, y) setequal(x, x) # we compare x to x which avoids early termination
binaryM[["bit", "setequal"]] <- function(x, y) bit_setequal(x, x)
binaryM[["merge", "setequal"]] <- function(x, y) merge_setequal(x, x)

binaryM[["R", "setearly"]] <-
  function(x, y) setequal(x, y) # we compare x to x which avoids early termination
binaryM[["bit", "setearly"]] <- function(x, y) bit_setequal(x, y)
binaryM[["merge", "setearly"]] <- function(x, y) merge_setequal(x, y)

unaryU <- c("unique", "duplicated", "anyDuplicated", "sumDuplicated")
unaryM <- vector("list", length(M) * length(unaryU))
dim(unaryM) <- c(method=length(M), task=length(unaryU))
dimnames(unaryM) <- list(method=M, task=unaryU)
unaryM[["R", "unique"]] <- unique
unaryM[["bit", "unique"]] <- bit_unique
unaryM[["merge", "unique"]] <- merge_unique
unaryM[["R", "duplicated"]] <- duplicated
unaryM[["bit", "duplicated"]] <- bit_duplicated
unaryM[["merge", "duplicated"]] <- merge_duplicated
unaryM[["R", "anyDuplicated"]] <- anyDuplicated
unaryM[["bit", "anyDuplicated"]] <- bit_anyDuplicated
unaryM[["merge", "anyDuplicated"]] <- merge_anyDuplicated
unaryM[["R", "sumDuplicated"]] <- function(x) sum(duplicated(x))
unaryM[["bit", "sumDuplicated"]] <- bit_sumDuplicated
unaryM[["merge", "sumDuplicated"]] <- merge_sumDuplicated

tim <- array(NA_integer_,
  dim=c(M=length(M), U=length(unaryU) + length(binaryU), N=length(binaryDomain), D=length(D)),
  dimnames=list(M=M, U=c(unaryU, binaryU), N=names(binaryDomain), D=D)
)

for (n in names(binaryDomain)) {
  xnam <- names(binaryDomain[[n]])[1]
  ynam <- names(binaryDomain[[n]])[2]
  x <- sample(binaryDomain[[n]][1], binarySample[[n]][1], replace = FALSE)
  y <- sample(binaryDomain[[n]][2], binarySample[[n]][2], replace = FALSE)
  d <- "unsorted"
  if (length(x) == length(y)) {
    for (u in unaryU) {
      for (m in setdiff(M, "merge")) {
        f <- unaryM[[m, u]]
        if (!is.null(f))
          tim[m, u, n, d] <- median(microbenchmark(f(x), times=times)$time)
      }
    }
  }
  for (u in binaryU) {
    for (m in setdiff(M, "merge")) {
      f <- binaryM[[m, u]]
      if (!is.null(f))
        tim[m, u, n, d] <- median(microbenchmark(f(x, y), times=times)$time)
    }
  }
  x <- bit_sort(x)
  y <- bit_sort(y)
  d <- "sorted"
  if (length(x) == length(y)) {
    for (u in unaryU) {
      for (m in M) {
        f <- unaryM[[m, u]]
        if (!is.null(f)) {
          tim[m, u, n, d] <- median(microbenchmark(f(x), times=times)$time)
          # now plug-in measures for unsorted merge
          if (m == "merge") {
            tim["merge", u, n, "unsorted"] <-
              timsort["bit", "sort", "unsorted", xnam] +  tim["merge", u, n, "sorted"]
          }
        }
      }
    }
  }
  for (u in binaryU) {
    for (m in M) {
      f <- binaryM[[m, u]]
      if (!is.null(f)) {
        tim[m, u, n, d] <- median(microbenchmark(f(x, y), times=times)$time)
        # now plug-in measures for unsorted merge
        if (m == "merge") {
          tim["merge", u, n, "unsorted"] =
            timsort["bit", "sort", "unsorted", xnam] +
            timsort["bit", "sort", "unsorted", ynam] +
            tim["merge", u, n, "sorted"]
        }
      }
    }
  }
}
```


```{r, echo=FALSE, fig.cap = "Execution time for R (R) and bit (b)"}
y <- timsort[, , , "big"]
y <- 100 * y / max(y["R", , ], na.rm=TRUE)
oldpar <- par(mfrow=c(2, 1), mar=c(5, 8, 2, 1))
x <- y[, , "unsorted"]
dotchart(x,
  xlim=c(0, max(100, max(y))),
  labels="",
  pch=c("R", "b"),
  xlab="execution time",
  main="unsorted",
  col=c("red", "blue")
)
x <- y[, , "sorted"]
dotchart(x,
  xlim=c(0, max(100, max(y))),
  labels="",
  pch=c("R", "b"),
  xlab="execution time",
  main="sorted",
  col=c("red", "blue")
)
par(oldpar)
```


```{r, echo=FALSE, results='hide'}
tim2 <- tim
for (n in names(binaryDomain)) {
  for (d in D) {
    tim2[, , n, d] <- 100 * tim[, , n, d] / max(tim["R", , n, d], na.rm=TRUE)
  }
}
```

`r pagebreak()`


```{r, echo=FALSE, fig.cap = "Execution time for R, bit and merge relative to most expensive R in 'unsorted bigbig' scenario"}
y <- tim2[, , "bigbig", ]
y <- 100 * y / max(y["R", , ], na.rm=TRUE)
x <- y[, , "unsorted"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x,
  xlim=c(0, max(100, max(y, na.rm=TRUE))),
  labels=m,
  pch=c("R", "b", "m"),
  col=c("red", "blue", "black"),
  main="Timings in 'unsorted bigbig' scenario",
  sub="R='hash'   b='bit'   m='merge'"
)
```

```{r, echo=FALSE, fig.cap = "Execution time for R, bit and merge in 'sorted bigbig' scenario"}
x <- y[, , "sorted"]
dotchart(x,
  xlim=c(0, max(y, na.rm=TRUE)),
  labels=m,
  pch=c("R", "b", "m"),
  col=c("red", "blue", "black"),
  main="Timings in 'sorted bigbig' scenario",
  sub="R='hash'   b='bit'   m='merge'"
)
```

`r pagebreak()`


### % time for sorting

```{r, echo=FALSE, results='asis'}
x <- 100 * timsort["bit", , , ] / timsort["R", , , ]
s <- "sorted"
knitr::kable(x[, s, ], caption=paste(s, "data relative to R's sort"), digits=1)
```


```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, s, ], caption=paste(s, "data relative to R's sort"), digits=1)
```

### % time for unique

```{r, echo=FALSE, results='asis'}
f <- function(u) {
  n <- c("smallsmall", "bigbig")
  x <- tim[c("bit", "merge", "merge"), u, n, ]
  dimnames(x)$M[3] <- "sort"
  dimnames(x)$N <- c("small", "big")
  x["sort", , "unsorted"] <- timsort["bit", "sort", "unsorted", ]
  x["sort", , "sorted"] <- 0
  for (m in dimnames(x)$M) {
    x[m, , ] <- x[m, , ] / tim["R", u, n, ] * 100
  }
  x
}
x <- f("unique")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```


### % time for duplicated

```{r, echo=FALSE, results='asis'}
x <- f("duplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```



### % time for anyDuplicated

```{r, echo=FALSE, results='asis'}
x <- f("anyDuplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for sumDuplicated

```{r, echo=FALSE, results='asis'}
x <- f("sumDuplicated")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```




### % time for match
```{r, echo=FALSE, results='asis'}
f <- function(u) {
  x <- tim[c("bit", "merge", "merge"), u, , ]
  dimnames(x)$M[3] <- "sort"
  s <- timsort["bit", "sort", "unsorted", ]
  x["sort", , "unsorted"] <- rep(s, c(2, 2)) + c(s, s)
  x["sort", , "sorted"] <- 0
  for (m in dimnames(x)$M) {
    x[m, , ] <- x[m, , ] / tim["R", u, , ] * 100
  }
  x
}
x <- f("match")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for in

```{r, echo=FALSE, results='asis'}
x <- f("in")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for notin

```{r, echo=FALSE, results='asis'}
x <- f("notin")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```


### % time for union


```{r, echo=FALSE, results='asis'}
x <- f("union")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for intersect


```{r, echo=FALSE, results='asis'}
x <- f("intersect")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for setdiff


```{r, echo=FALSE, results='asis'}
x <- f("setdiff")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for symdiff


```{r, echo=FALSE, results='asis'}
x <- f("symdiff")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```


### % time for setequal


```{r, echo=FALSE, results='asis'}
x <- f("setequal")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

### % time for setearly


```{r, echo=FALSE, results='asis'}
x <- f("setearly")
s <- "sorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[, , s], caption=paste(s, "data relative to R"), digits=1)
```

---
title: "Demo of the bit package"
author: "Dr. Jens Oehlschlägel"
date: '`r Sys.Date()`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Demo of the bit package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(bit)
.ff.is.available = requireNamespace("ff", quietly=TRUE) && packageVersion("ff") >= "4.0.0"
if (.ff.is.available) library(ff)
#tools::buildVignette("vignettes/bit-demo.Rmd")
#devtools::build_vignettes()
```

---

## bit type

Create a huge boolean vector (no NAs allowed)

```{r}
n <- 1e8
b1 <- bit(n)
b1
```

It costs only one bit per element

```{r}
object.size(b1) / n
```


A couple of standard methods work

```{r}
b1[10:30] <- TRUE
summary(b1)
```

Create a another boolean vector with TRUE in some different positions

```{r}
b2 <- bit(n)
b2[20:40] <- TRUE
b2
```

fast boolean operations

```{r}
b1 & b2
```

fast boolean operations

```{r}
summary(b1 & b2)
```


## bitwhich type

Since we have a very skewed distribution we may coerce to an even sparser representation

```{r}
w1 <- as.bitwhich(b1)
w2 <- as.bitwhich(b2)
object.size(w1) / n
```

and everything

```{r}
w1 & w2
```

works as expected

```{r}
summary(w1 & w2)
```


even mixing

```{r}
summary(b1 & w2)
```


## processing chunks

Many bit functions support a range restriction,

```{r}
summary(b1, range=c(1, 1000))
```

which is useful

```{r}
as.which(b1, range=c(1, 1000))
```

for filtered chunked looping

```{r}
lapply(chunk(from=1, to=n, length=10), function(i) as.which(b1, range=i))
```

over large ff vectors

```{r, eval=.ff.is.available}
options(ffbatchbytes=1024^3)
x <- ff(vmode="single", length=n)
x[1:1000] <- runif(1000)
lapply(chunk(x, length.out = 10), function(i) sum(x[as.hi(b1, range=i)]))
```

and wrap-up

```{r, eval=.ff.is.available}
delete(x)
rm(x, b1, b2, w1, w2, n)
```

for more info check the usage vignette

---
title: "Usage of the bit package"
author: "Dr. Jens Oehlschlägel"
date: '`r Sys.Date()`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Usage of the bit package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The bit package provides the following S3 functionality:

- alternatives to the `logical` data type which need less memory with methods that are often faster
    - the `bit` type for boolean data (without `NA`, factor 32 smaller)
    - the `bitwhich` type for very skewed boolean data (without `NA`, potentially even smaller)
- further alternatives for representing Boolean selections
    - the `which` type for positive selections maintaining the original vector length
    - the `ri` range index
- very fast methods for `integer`, particularly
    - methods for *unsorted* integers leveraging `bit` vectors rather than *hash* tables
    - methods for *sorting* integers leveraging $O(N)$ *synthetic* sorting rather than $O(N \log N)$ divide and conquer
    - methods for *sorted* integers leveraging *merging* rather than *hash* tables
- some foundations for package `ff`, particularly
    - `bit` and `bitwhich` vectors and `ri` range indices for filtering and subscripting `ff` objects
    - `rlepack` compressing sets of *sorted* integers for `hi` hybrid indexing
    - methods for *chunking*
- helper methods for avoiding unwanted data copies


```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(bit)
.ff.is.available = requireNamespace("ff", quietly=TRUE) && packageVersion("ff") >= "4.0.0"
if (.ff.is.available) library(ff)
# rmarkdown::render("vignettes/bit-usage.Rmd")
# devtools::build_vignettes()
```

---

## Boolean data types

R's `logical` vectors cost 32 bit per element in order to code three states `FALSE`, `TRUE` and `NA`. By contrast `bit` vectors cost only 1 bit per element and support only the two Boolean states `FALSE` and `TRUE`. Internally `bit` are stored as `integer`. `bitwhich` vectors manage very skewed Boolean data as a set of sorted unique integers denoting either *included* or *excluded* positions, whatever costs less memory. Internally `bitwhich` are stored as `integer` *or* as `logical` for the extreme cases *all included* (`TRUE`), *all excluded* (`FALSE`) or *zero length* (`logical()`). Function `bitwhich_representation` allows to distinguish these five cases without the copying-cost of `unclass(bitwhich)`. All three Boolean types `logical`, `bit`, and `bitwhich` are unified in a super-class `booltype` and have `is.booltype(x) == TRUE`. Classes `which` and `ri` can be somewhat considered as a fourth and fifth even more special Boolean types (for skewed data with mostly `FALSE` that represents the sorted positions of the few `TRUE`, and as a consecutive series of the latter), and hence have `booltype` `4` and `5` but coerce differently to `integer` and `double` and hence are not `is.booltype`.

### Available classes


The `booltype`s `bit` and `bitwhich` behave very much like `logical` with the following exceptions

- length is limited to `.Machine$integer.max`
- currently only `vector` methods are supported, not `matrix` or `array`
- subscripting from them does return `logical` rather than returning their own class (like `ff` `boolean` vectors do)
- values assigned to them are internally coerced to `logical` before being processed, hence `x[] <- x` is a potentially expensive operation
- they do not support `names` and `character` subscripts
- only scalar `logical` subscripts are allowed, i.e. `FALSE`, `TRUE`
- assigned or coerced `NA` are silently converted to `FALSE`
- increasing the `length` of `bit` has semantics differing from `logical`: new elements are consistently initialized with `FALSE` instead of `NA`
- increasing the `length` of `bitwhich` has semantics differing from `logical` and `bit`: new elements are initialized with the *more frequent value* of the skewed distribution (`FALSE` winning in the equally frequent case).
- aggregation methods `min`, `max` and `range` and `summary` have special meaning, for example `min` corresponds to `which.max`.


Note the following features

- aggregation functions support a `range` argument for *chunked processing*
- sorted positive subscripts marked as class `which` are processed faster than 'just positive subscripts', and unlike the result of the function `which`, class `which` retains the length of the Boolean vector as attribute `maxindex`, but its `length` is the number of positive positions
- `ri` range indices are allowed as subscripts which supports *chunked looping* over in-memory `bit` or `bitwhich` vectors not unlike *chunked looping* over on-disk `ff` objects.
- binary Boolean operations will promote differing data type to the data type implying fewer assumptions, i.e. `logical` wins over `bit`, `bit` wins over `bitwhich` and `which` wins over `bitwhich`.

Note the following warnings

- currently `bit` and `bitwhich` may answer `is.logical` and `is.integer` according to their internal representation. Do not rely on this, it may be subject to change. Do use `booltype`, `is.booltype`, `is.bit`, `is.bitwhich` and `bitwhich_representation` for reasoning about the data type.

---

### Available methods

Basic methods

`is as length length<- [ [<- [[ [[<- rev rep c print`

Boolean operations

`is.na ! | & == != xor`

Aggregation methods

`anyNA any all sum min max range summary`

---

### Creating and manipulating


`bit` and `bitwhich` vectors are created like `logical`, for example zero length vectors

```{r}
logical()
bit()
bitwhich()
```

or vectors of a certain length initialized to `FALSE`

```{r}
logical(3)
bit(3)
bitwhich(3)
```

`bitwhich` can be created initialized to all elements TRUE with

```{r}
bitwhich(3, TRUE)
```

or can be created initialized to a few included or excluded elements

```{r}
bitwhich(3, 2)
bitwhich(3, -2)
```
---

Note that `logical` behaves somewhat inconsistent, when creating it, the default is `FALSE`, when increasing the length, the default is `NA`:

```{r}
l <- logical(3)
length(l) <- 6
l
```

Note that the default in `bit` is always `FALSE`, for creating and increasing the length.

```{r}
b <- bit(3)
length(b) <- 6
b
```

Increasing the length of `bitwhich` initializes new elements to *the majority* of the old elements, hence a `bitwhich` with a few exclusions has majority `TRUE` and will have new elements initialized to `TRUE` (if both, `TRUE` and `FALSE` have equal frequency the default is `FALSE`).

```{r}
w <- bitwhich(3, 2)
length(w) <- 6
w
w <- bitwhich(3, -2)
length(w) <- 6
w
```

---

Vector subscripting non-existing elements returns `NA`

```{r}
l <- logical(3L)
b <- bit(3L)
w <- bitwhich(3L)
l[6L]
b[6L]
w[6L]
```

while assigned `NA` turn into `FALSE` and assigning to a non-existing element does increase vector length

```{r}
l[6L] <- NA
b[6L] <- NA
w[6L] <- NA
l
b
w
```

---

As usual list subscripting is only allowed for existing elements

```{r}
l[[6]]
b[[6]]
w[[6]]
```

while assignments to non-existing elements do increase length.

```{r}
l[[9]] <- TRUE
b[[9]] <- TRUE
w[[9]] <- TRUE
l
b
w
```

---

### Coercion

There are coercion functions between classes `logical`, `bit`, `bitwhich`, `which`, `integer` and `double`. However, only the first three of those represent Boolean vectors, whereas `which` represents subscript positions, and `integer` and `double` are ambiguous in that they can represent both, Booleans or positions.

Remember first that coercing `logical` to `integer` (or `double`) gives `0` and `1` and not `integer` subscript positions:

```{r}
l <- c(FALSE, TRUE, FALSE)
i <- as.integer(l)
as.logical(i)
```

To obtain `integer` subscript positions use `which` or better `as.which` because the latter S3 class remembers the original vector length and hence we can go coerce back to logical

```{r}
l <- c(FALSE, TRUE, FALSE)
w <- as.which(l)
w
as.logical(w)
```

coercing back to `logical` fails using just `which`

```{r}
l <- c(FALSE, TRUE, FALSE)
w <- which(l)
w
as.logical(w)  # does not coerce back
```

Furthermore from class `which` we can deduce that the positions are sorted which can be leveraged for performance. Note that `as.which.integer` is a core method for converting integer positions to class `which` and it enforces sorting

```{r}
i <- c(7, 3)
w <- as.which(i, maxindex=12)
w
```

and `as.integer` gives us back those positions (now sorted)

```{r}
as.integer(w)
```

You see that the `as.integer` generic is ambiguous as the `integer` data type in giving positions on some, and zeroes and ones on other inputs. The following set of Boolean types can be coerced without loss of information

```
logical bit bitwhich which
```

The same is true for this set

```
logical bit bitwhich integer double
```

Furthermore positions are retained in this set

```
which integer double
```

although the length of the Boolean vector is lost when coercing from `which` to `integer` or `double`, therefore coercing to them is not reversible. Let's first create all six types and compare their sizes:

```{r}
r <- ri(1, 2^16, 2^20) # sample(2^20, replace=TRUE, prob=c(.125, 875))
all.as <- list(
  double = as.double,
  integer= as.integer,
  logical = as.logical,
  bit = as.bit,
  bitwhich = as.bitwhich,
  which = as.which,
  ri = function(x) x
)
all.types <- lapply(all.as, function(f) f(r))
sapply(all.types, object.size)
```

Now let's create all combinations of coercion:

```{r}
all.comb <- vector('list', length(all.types)^2)
all.id <- rep(NA, length(all.types)^2)
dim(all.comb)      <- dim(all.id)      <-    c(from=length(all.types), to=length(all.types))
dimnames(all.comb) <- dimnames(all.id) <- list(from= names(all.types), to= names(all.types))
for (i in seq_along(all.types)) {
  for (j in seq_along(all.as)) {
    # coerce all types to all types (FROM -> TO)
    all.comb[[i, j]] <- all.as[[j]](all.types[[i]])
    # and test whether coercing back to the FROM type gives the orginal object
    all.id[i, j] <- identical(all.as[[i]](all.comb[[i, j]]),  all.types[[i]])
  }
}
all.id
```

Do understand the `FALSE` above!

---

The functions `booltype` and `is.booltype` diagnose the Boolean type as follows

```{r}
data.frame(
  booltype=sapply(all.types, booltype),
  is.boolean=sapply(all.types, is.booltype),
  row.names=names(all.types)
)
```

Class `which` and `ri` are currently not `is.boolean` (no subscript, assignment and Boolean operators), but since it is even more specialized than `bitwhich` (assuming skew towards `TRUE`), we have ranked it as the most specialized `booltype`.

---


### Boolean operations

The usual Boolean operators

```
! | & == != xor is.na
```
are implemented and the binary operators work for all combinations of `logical`, `bit` and `bitwhich`.

Technically this is achieved in S3 by giving `bit` and `bitwhich` another class `booltype`. Note that this is not inheritance where `booltype` implements common methods and `bit` and `bitwhich` overrule this with more specific methods. Instead the method dispatch to `booltype` dominates `bit` and `bitwhich` and coordinates more specific methods, this was the only way to realize binary logical operators that combine `bit` and `bitwhich` in S3, because in this case R dispatches to neither `bit` nor `bitwhich`: if both arguments are custom classes R does a non-helpful dispatch to `integer` or `logical`.

Anyhow, if a binary Boolean operator meets two different types, argument and result type is promoted to *less assumptions*, hence `bitwhich` is promoted to `bit` dropping the assumption of *strong skew* and `bit` is promoted to `logical` dropping the assumption that no `NA` are present.

Such promotion comes at the price of increased memory requirements, for example the following multiplies the memory requirement by factor 32

```{r}
x <- bit(1e6)
y <- x | c(FALSE, TRUE)
object.size(y) / object.size(x)
```
Better than lazily relying on automatic propagation is

```{r}
x <- bit(1e6)
y <- x | as.bit(c(FALSE, TRUE))
object.size(y) / object.size(x)
```
---

### Manipulation methods

Concatenation follows the same promotion rules as Boolean operators. Note that `c` dispatches on the first argument only, hence when concatenating multiple Boolean types the first must not be logical, otherwise we get corrupt results:

```{r}
l <- logical(6)
b <- bit(6)
c(l, b)
```
because `c.logical` treats the six bits as a single value. The following expressions work

```{r}
c(b, l)
c(l, as.logical(b))
```
and of course the most efficient is

```{r}
c(as.bit(l), b)
```

If you want your code to process any `is.booltype`, you can use `c.booltype` directly

```{r}
c.booltype(l, b)
```

Both, `bit` and `bitwhich` also have replication (`rep`) and reverse (`rev`) methods that work as expected:


```{r, echo=3:6}
# TODO(r-lib/lintr#773): nolint as a chunk option.
# nolint start: rep_len_linter.
b <- as.bit(c(FALSE, TRUE))
rev(b)
rep(b, 3)
rep(b, length.out=6)
# nolint end: rep_len_linter.
```

---

### Aggregation methods for `booltype`

The usual logical aggregation functions `length`, `all`, `any` and `anyNA` work as expected. Note the exception that `length(which)` does not give the length of the Boolean vector but the length of the vector of positive integers (like the result of function `which`). `sum` gives the number of `TRUE` for all Boolean types. For the `booltype > 1` `min` gives the first position of a `TRUE` (i.e. `which.max`), `max` gives the last position of a `TRUE`, `range` gives both, the range at which we find `TRUE`, and finally `summary` gives the the counts of `FALSE` and `TRUE` as well as `min` and `max`. For example

```{r}
l <- c(NA, NA, FALSE, TRUE, TRUE)
b <- as.bit(l)
length(b)
anyNA(b)
any(b)
all(b)
sum(b)
min(b)
max(b)
range(b)
summary(b)
```

These special interpretations of `min`, `max`, `range` and `summary` can be enforced for type `logical`, `integer`, and `double` by using the `booltype` methods directly as in

```{r}
# minimum after coercion to integer
min(c(FALSE, TRUE))
# minimum  position of first TRUE
min.booltype(c(FALSE, TRUE))
```

Except for `length` and `anyNA` the aggregation functions support an optional argument `range` which restricts evaluation the specified range of the Boolean vector. This is useful in the context of *chunked processing*. For example analyzing the first 30\% of a million Booleans

```{r}
b <- as.bit(sample(c(FALSE, TRUE), 1e6, TRUE))
summary(b, range=c(1, 3e5))
```
and analyzing all such chunks

```{r}
sapply(chunk(b, by=3e5, method="seq"), function(i) summary(b, range=i))
```

or better balanced

```{r}
sapply(chunk(b, by=3e5), function(i) summary(b, range=i))
```


The real use-case for chunking is ff objects, where instead of processing huge objects at once

```{r, eval=.ff.is.available, message=FALSE}
x <- ff(vmode="single", length=length(b))   # create a huge ff vector
x[as.hi(b)] <- runif(sum(b))      # replace some numbers at filtered positions
summary(x[])
```

we can process the ff vector in chunks

```{r, eval=.ff.is.available}
sapply(chunk(x, by=3e5), function(i) summary(x[i]))
```
and even can process a bit-filtered ff vector in chunks

```{r, eval=.ff.is.available}
sapply(chunk(x, by=3e5), function(i) summary(x[as.hi(b, range=i)]))
```


---

## Fast methods for `integer` set operations

R implements *set methods* using *hash tables*, namely `match`, `%in%`, `unique`, `duplicated`, `union`, `intersect`, `setdiff`, `setequal`. Hashing is a powerful method, but it is costly in terms of random access and memory consumption. For integers package 'bit' implements faster set methods using *bit vectors* (which can be an order of magnitude faster and saves up to factor 64 on temporary memory) and *merging* (which is about two orders of magnitude faster, needs no temporary memory but requires sorted input). While many set methods return *unique* sets, the `merge_*` methods can optionally preserve ties, the `range_*` methods below allow to specify one of the sets as a *range*.

|R hashing        | bit vectors       |merging           |range merging     |rlepack           |
|:----------------|:------------------|:-----------------|:-----------------|:-----------------|
|`match`          |                   |`merge_match`     |||
|`%in%`           |`bit_in`           |`merge_in`        |`merge_rangein`||
|`!(%in%)`        |`!bit_in`          |`merge_notin`     |`merge_rangenotin`||
|`duplicated`     |`bit_duplicated`   ||||
|`unique`         |`bit_unique`       |`merge_unique`    ||`unique(rlepack)`|
|`union`          |`bit_union`        |`merge_union`     |||
|`intersect`      |`bit_intersect`    |`merge_intersect` |`merge_rangesect`||
|`setdiff`        |`bit_setdiff`      |`merge_setdiff`   |`merge_rangediff`||
|`(a:b)[-i]`      |`bit_rangediff`    |`merge_rangediff` |`merge_rangediff`||
|                 |`bit_symdiff`      |`merge_symdiff`   |||
|`setequal`       |`bit_setequal`     |`merge_setequal`  |||
|`anyDuplicated`  |`bit_anyDuplicated`|  ||`anyDuplicated(rlepack)`||
|`sum(duplicated)`|`bit_sumDuplicated`|  |||

Furthermore there are very fast methods for sorting integers (unique or keeping ties), reversals that simultaneously change the sign to preserve ascending order and methods for finding min or max in sorted vectors or within ranges of sorted vectors.

|R              | bit vectors     |merging                  |range merging      |
|:--------------|:----------------|:------------------------|:------------------|
|`sort(unique)` |`bit_sort_unique`|                         ||
|`sort`         |`bit_sort`       |                         ||
|`rev`          |`rev(bit)`       |`reverse_vector`         ||
|`-rev`         |                 |`copy_vector(revx=TRUE)` ||
|`min`          |`min(bit)`       |`merge_first`            |`merge_firstin`   |
|`max`          |`max(bit)`       |`merge_last`             |`merge_lastin`    |
|               |`min(!bit)`      |                         |`merge_firstnotin`|
|               |`max(!bit)`      |                         |`merge_lastnotin` |

---

### Methods using random access to bit vectors

Set operations using hash tables incur costs for populating and querying the hash table: this is random access cost to a relative large table. In order to avoid *hash collisions* hash tables need more elements than those being hashed, typically `2*N` for `N` elements. That is hashing `N` integer elements using a int32 hash function costs *random access* to `64*N` bits of memory. If the `N` elements are within a range of `N` values, it is much (by factor 64) cheaper to register them in a bit-vector of `N` bits. The `bit_*` functions first determine the range of the values (and count of `NA`), and then use an appropriately sized bit vector. Like the original R functions the `bit_*` functions keep the original order of values (although some implementations could be faster by delivering an undefined order). If `N` is small relative to the range of the values the `bit_*` fall back to the standard R functions using hash tables. Where the `bit_*` functions return Boolean vectors they do so by default as `bit` vectors, but but you can give a different coercion function as argument `retFUN`.

Bit vectors cannot communicate positions, hence cannot replace `match`, however they can replace the `%in%` operator:

```{r}
set.seed(1)
n <- 9L
x <- sample(n, replace=TRUE)
y <- sample(n, replace=TRUE)
x
y
x %in% y
bit_in(x, y)
bit_in(x, y, retFUN=as.logical)
```

The `bit_in` function combines a bit vector optimization with reverse look-up, i.e. if the range of `x` is smaller than the range of `table`, we build the bit vector on `x` instead of the `table`.

---

The `bit_duplicated` function can handle `NA` in three different ways

```{r}
x <- c(NA, NA, 1L, 1L, 2L, 3L)
duplicated(x)
bit_duplicated(x, retFUN=as.logical)
bit_duplicated(x, na.rm=NA, retFUN=as.logical)

duplicated(x, incomparables = NA)
bit_duplicated(x, na.rm=FALSE, retFUN=as.logical)

bit_duplicated(x, na.rm=TRUE, retFUN=as.logical)
```

The `bit_unique` function can also handle `NA` in three different ways

```{r}
x <- c(NA, NA, 1L, 1L, 2L, 3L)
unique(x)
bit_unique(x)

unique(x, incomparables = NA)
bit_unique(x, na.rm=FALSE)

bit_unique(x, na.rm=TRUE)
```

The `bit_union` function build a bit vector spanning the united range of both input sets and filters all unites duplicates:

```{r}
x <- c(NA, NA, 1L, 1L, 3L)
y <- c(NA, NA, 2L, 2L, 3L)
union(x, y)
bit_union(x, y)
```

The `bit_intersect` function builds a bit vector spanning only the intersected range of both input sets and filters all elements outside and the duplicates inside:

```{r}
x <- c(0L, NA, NA, 1L, 1L, 3L)
y <- c(NA, NA, 2L, 2L, 3L, 4L)
intersect(x, y)
bit_intersect(x, y)
```

---

The `bit_setdiff` function builds a bit vector spanning the range of the first input set, marks elements of the second set within this range as tabooed, and then outputs the remaining elements of the first set unless they are duplicates:

```{r}
x <- c(0L, NA, NA, 1L, 1L, 3L)
y <- c(NA, NA, 2L, 2L, 3L, 4L)
setdiff(x, y)
bit_setdiff(x, y)
```
The `bit_symdiff` function implements symmetric set difference. It builds two bit vectors spanning the full range and then outputs those elements of both sets that are marked at exactly one of the bit vectors.

```{r}
x <- c(0L, NA, NA, 1L, 1L, 3L)
y <- c(NA, NA, 2L, 2L, 3L, 4L)
union(setdiff(x, y), setdiff(y, x))
bit_symdiff(x, y)
```

The `bit_setequal` function terminates early if the ranges of the two sets (or the presence of `NA`) differ. Otherwise it builds two bit vectors spanning the identical range; finally it checks the two vectors for being equal with early termination if two unequal integers are found.

```{r}
x <- c(0L, NA, NA, 1L, 1L, 3L)
y <- c(NA, NA, 2L, 2L, 3L, 4L)
setequal(y, x)
bit_setequal(x, y)
```

---

The `bit_rangediff` function works like `bit_setdiff` with two differences: the first set is specified as a *range* of integers, and it has two arguments `revx` and `revy` which allow to reverse *order* and *sign* of the two sets *before* the set-diff operation is done. The order of the range is significant, e.g. `c(1L, 7L)` is different from `c(7L, 1L)`, while the order of the second set has no influence:

```{r}
bit_rangediff(c(1L, 7L), (3:5))
bit_rangediff(c(7L, 1L), (3:5))
bit_rangediff(c(1L, 7L), -(3:5), revy=TRUE)
bit_rangediff(c(1L, 7L), -(3:5), revx=TRUE)
```

If the range and the second set don't overlap, for example due to different signs, the full range is returned:

```{r}
bit_rangediff(c(1L, 7L), (1:7))
bit_rangediff(c(1L, 7L), -(1:7))
bit_rangediff(c(1L, 7L), (1:7), revy=TRUE)
```

Note that `bit_rangediff` provides faster negative subscripting from a range of integers than the usual phrase `(1:n)[-i]`:

```{r}
(1:9)[-7]
bit_rangediff(c(1L, 9L), -7L, revy=TRUE)
```

---

Functions `bit_anyDuplicated` is a faster version of `anyDuplicated`

```{r, echo=3:7}
# TODO(r-lib/lintr#773): nolint as a chunk option.
# nolint start: any_duplicated_linter.
x <- c(NA, NA, 1L, 1L, 2L, 3L)
any(duplicated(x))      # full hash work, returns FALSE or TRUE
anyDuplicated(x)        # early termination of hash work, returns 0 or position of first duplicate
any(bit_duplicated(x))  # full bit work, returns FALSE or TRUE
bit_anyDuplicated(x)    # early termination of bit work, returns 0 or position of first duplicate
# nolint end: any_duplicated_linter.
```

For the meaning of the `na.rm` parameter see `bit_duplicated`. Function `bit_sumDuplicated` is a faster version of `sum(bit_duplicated)`

```{r}
x <- c(NA, NA, 1L, 1L, 2L, 3L)
sum(duplicated(x))      # full hash work, returns FALSE or TRUE
sum(bit_duplicated(x))  # full bit work, returns FALSE or TRUE
bit_sumDuplicated(x)    # early termination of bit work, returns 0 or position of first duplicated
```
---

### Methods using bit vectors for sorting integers

A bit vector cannot replace a hash table for all possible kind of tasks (when it comes to counting values or to their positions), but on the other hand a bit vector allows something impossible with a hash table: *sorting* keys (without payload). Sorting a large subset of unique integers in [1, N] using a bit vector has been described in *"Programming pearls -- cracking the oyster"* by Jon Bentley (where 'uniqueness' is not an input condition but an output feature). This is easily generalized to sorting a large subset of integers in a range [min, max]. A first scan over the data determines the range of the keys, arranges `NA`s according to the usual `na.last=` argument and checks for presortedness (see `range_sortna`), then the range is projected to a bit vector of size `max-min+1`, all keys are registered causing random access and a final scan over the bit vector sequentially writes out the sorted keys. This is a *synthetical sort* because unlike a *comparison sort* the elements are not moved towards their ordered positions, instead they are *synthesized* from the bit-representation.

For `N` consecutive permuted integers this is by an order of magnitude faster than *quicksort*. For a *density* $d=N/(max-min) \ge 1$ this sort beats quicksort, but for $d << 1$ the bit vector becomes too large relative to `N` and hence quicksort is faster. `bit_sort_unique` implements a hybrid algorithm automatically choosing the faster of both, hence for integers the following gives identical results

```{r}
x <- sample(9, 9, TRUE)
unique(sort(x))
sort(unique(x))
bit_sort_unique(x)
```

What if duplicates shall be kept?

- A: sort all non-duplicated occurrences using a bit vector
- B: sort the rest
- merge A and B

The crucial question is how to sort the rest? Recursively using a bit-vector can again be faster than quicksort, however it is non-trivial to determine an optimal recursion-depth before falling back to quicksort. Hence a safe bet is using the bit vector only once and sort the rest via quicksort, let's call that `bitsort`. Again, using `bitsort` is fast only at medium density in the data range. For low density quicksort is faster. For high density (duplicates) another synthetic sort is faster: *counting sort*. `bit_sort` implements a *sandwhich sort* algorithm which given density uses `bitsort` between two complementing algorithms:

```
low density    medium density    high density
quicksort   << bitsort        << countsort
```

```{r}
x <- sample(9, 9, TRUE)
sort(x)
bit_sort(x)
```

Both, `bit_sort_unique` and `bit_sort` can sort *decreasing*, however, currently this requires an extra pass over the data in `bit_sort` and in the quicksort fallback of `bit_sort_unique`. So far, `bit_sort` does not leverage *radix sort* for very large `N`.

---

### Methods for sets of sorted integers

Efficient handling of sorted sets is backbone of class `bitwhich`. The `merge_*` and `merge_range*` integer functions expect sorted input (non-decreasing sets or increasing ranges) and return sorted sets (some return Boolean vectors or scalars). Exploiting the sortedness makes them even faster than the `bit_` functions. Many of them have `revx` or `revy` arguments, which reverse the scanning direction of an input vector *and* the  interpreted *sign* of its elements, hence we can change signs of input vectors in an order-preserving way and without any extra pass over the data. By default these functions return *unique* sets which have each element not more than once. However, the *binary* `merge_*` functions have a `method="exact"` which in both sets treats consecutive occurrences of the same value as different. With `method="exact"` for example `merge_setdiff` behaves as if counting the values in the first set and subtraction the respective counts of the second set (and capping the lower end at zero). Assuming positive integers and equal `tabulate(, nbins=)` with `method="exact"` the following is identical:

|merging then counting            |counting then combining             |
|:--------------------------------|:-----------------------------------|
|`tabulate(merge_union(x, y))`    |`pmax(tabulate(x), tabulate(y))`    |
|`tabulate(merge_intersect(x, y))`|`pmin(tabulate(x), tabulate(y))`    |
|`tabulate(merge_setdiff(x, y))`  |`pmax(tabulate(x) - tabulate(y), 0)`|
|`tabulate(merge_symdiff(x, y))`  |` abs(tabulate(x) - tabulate(y))`   |
|`tabulate(merge_setequal(x, y))` |` all(tabulate(x) == tabulate(y))`    |


Note further that `method="exact"` delivers *unique* output if the input is unique, and in this case works faster than `method="unique"`.

Note further, that the `merge_*` and `merge_range*` functions have no special treatment for `NA`. If vectors with `NA` are sorted ith `NA` in the first positions (`na.last=FALSE`) and arguments `revx=` or `revy=` have not been used, then `NAs` are treated like ordinary integers. `NA` sorted elsewhere or using `revx=` or `revy=` can cause unexpected results (note for example that `revx=` switches the sign on all integers but `NAs`).

---

The unary `merge_unique` function transform a sorted set into a unique sorted set:

```{r}
x = sample(12)
bit_sort(x)
merge_unique(bit_sort(x))
bit_sort_unique(x)
```

---

For binary functions let's start with *set equality*:

```{r}
x = as.integer(c(3, 4, 4, 5))
y = as.integer(c(3, 4, 5))
setequal(x, y)
merge_setequal(x, y)
merge_setequal(x, y, method="exact")
```

For *set complement* there is also a `merge_range*` function:

```{r}
x = as.integer(c(0, 1, 2, 2, 3, 3, 3))
y = as.integer(c(1, 2, 3))
setdiff(x, y)
merge_setdiff(x, y)
merge_setdiff(x, y, method="exact")
merge_rangediff(c(0L, 4L), y)
merge_rangediff(c(0L, 4L), c(-3L, -2L)) # y has no effect due to different sign
merge_rangediff(c(0L, 4L), c(-3L, -2L), revy=TRUE)
merge_rangediff(c(0L, 4L), c(-3L, -2L), revx=TRUE)
```

`merge_symdiff` for *symmetric set complement* is used similar (without a `merge_range*` function), as is `merge_intersect`, where the latter is accompanied by a `merge_range*` function:

```{r}
x = -2:1
y = -1:2
setdiff(x, y)
union(setdiff(x, y), setdiff(y, x))
merge_symdiff(x, y)
merge_intersect(x, y)
merge_rangesect(c(-2L, 1L), y)
```

The `merge_union` function has a third method `all` which behaves like `c` but keeps the output sorted

```{r}
x = as.integer(c(1, 2, 2, 3, 3, 3))
y = 2:4
union(x, y)
merge_union(x, y, method="unique")
merge_union(x, y, method="exact")
merge_union(x, y, method="all")
sort(c(x, y))
c(x, y)
```

---

Unlike the `bit_*` functions the `merge_*` functions have a `merge_match` function:

```{r}
x = 2:4
y = as.integer(c(0, 1, 2, 2, 3, 3, 3))
match(x, y)
merge_match(x, y)
```
and unlike R's `%in%` operator the following functions are directly implemented, not on top of merge, and hence save extra passes over the data:

```{r}
x %in% y
merge_in(x, y)
merge_notin(x, y)
```

The range versions extract logical vectors from `y`, but only for the range in `rx`.

```{r}
x <- c(2L, 4L)
merge_rangein(x, y)
merge_rangenotin(x, y)
```
Compare this to `merge_rangesect` above. `merge_rangein` is useful in the context of chunked processing, see the `any`, `all`, `sum` and `[` methods of class `bitwhich`.

---

The functions `merge_first` and `merge_last` give first and last element of a sorted set. By default that is *min* and *max*, however these functions also have an `revx` argument. There are also functions that deliver the first resp. last elements *of* a certain range that are *in* or *not in* a certain set.

```{r}
x <- bit_sort(sample(1000, 10))
merge_first(x)
merge_last(x)
merge_firstnotin(c(300L, 600L), x)
merge_firstin(c(300L, 600L), x)
merge_lastin(c(300L, 600L), x)
merge_lastnotin(c(300L, 600L), x)
```

---

---
title: "Performance of the bit package"
author: "Dr. Jens Oehlschlägel"
date: '`r Sys.Date()`'
output:
  pdf_document:
    toc: yes
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Performance of the bit package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
require(bit)
require(microbenchmark)
# rmarkdown::render("vignettes/bit-performance.Rmd")
# these are the real settings for the performance vignette
times <- 5
Domain <- c(small=1e3, big=1e6)
Sample <- c(small=1e3, big=1e6)
# these are the settings to keep the cost of CRAN low
#times <- 5
#Domain <- c(small=1e1, big=1e3)
#Sample <- c(small=1e1, big=1e3)

pagebreak <- function() {
  if(knitr::is_latex_output())
    return("\\newpage")
  else
    return('<div style="page-break-before: always;" />')
}

```

---


## A performance example

Before we measure performance of the main functionality of the package, note that something simple as '(a:b)[-i]' can and has been accelerated in this package:

```{r, echo=TRUE, results='asis'}
a <- 1L
b <- 1e7L
i <- sample(a:b,1e3)
x <- c(
  R = median(microbenchmark((a:b)[-i], times=times)$time)
, bit = median(microbenchmark(bit_rangediff(c(a,b), i), times=times)$time)
, merge = median(microbenchmark(merge_rangediff(c(a,b), bit_sort(i)), times=times)$time)
)
knitr::kable(as.data.frame(as.list(x/x["R"]*100)), caption="% of time relative to R", digits=1)
```

The vignette is compiled with the following performance settings: `r times` replications with domain size small `r Domain["small"]` and big `r Domain["big"]`, sample size small `r Sample["small"]` and big `r Sample["big"]`. 

## Boolean data types


> "A designer knows he has achieved perfection not when there is nothing left to add, but when there is nothing left to take away."  
"Il semble que la perfection soit atteinte non quand il n'y a plus rien à ajouter, mais quand il n'y a plus rien à retrancher"  
(Antoine de St. Exupery, Terre des Hommes (Gallimard, 1939), p. 60.)

We compare memory consumption (n=`r format(Sample[["big"]])`) and runtime (median of `r times` replications) of the different `booltype`s for the following filter scenarios:

```{r, echo=FALSE, results='asis'}
knitr::kable(
  data.frame(coin="random 50%", often="random 99%", rare="random 1%", chunk="contiguous chunk of 5%")
  , caption="selection characteristic")
```

```{r, echo=FALSE, results='asis'}
B <- booltypes[c("logical","bit","bitwhich","which","ri")]
M <- c("size", "[]", "[which]", "[which]<-TRUE", "[]<-logical", "!", "&", "|", "==", "!=", "summary")
G <- list(
  coin = function(n)sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.5,0.5))
, often = function(n)sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.01,0.99))
, rare = function(n)sample(c(FALSE, TRUE), n, replace=TRUE, prob=c(0.99,0.01))
, chunk = function(n)ri(n%/%20,2L*n%/%20,n)
)
X <- vector("list", length(B)*length(G))
dim(X) <- c(booltype=length(B), data=length(G))
dimnames(X) <- list(booltype=names(B), data=names(G))
tim <- array(NA
             , dim=c(booltype=length(B), metric=length(M), data=length(G))
             , dimnames=list(booltype=names(B), metric=M, data=names(G))
             )
for (g in names(G)){
  x <- G[[g]](Sample[["big"]])
  if (g %in% c("coin","often","rare"))
    w <- as.which(as.logical(x))
  for (b in B){
    if (booltypes[[b]] < 'ri' || (b == 'ri' && g=='chunk')){
      X[[b,g]] <- as.booltype(x, b)
      if (g %in% c("coin","often","rare") && b %in% c("logical","bit","bitwhich")){
        l <- as.booltype(logical(Sample[["big"]]), b)
        tim[b,"[which]",g] <- median(microbenchmark(l[w], times=times)$time)
        tim[b,"[which]<-TRUE",g] <- median(microbenchmark(l[w]<-TRUE, times=times)$time)
        tim[b,"[]",g] <- median(microbenchmark(l[], times=times)$time)
        tim[b,"[]<-logical",g] <- median(microbenchmark(l[]<-x, times=times)$time)
      }
      tim[b,"size",g] <- object.size(X[[b,g]])
    }
  }
}
for (g in names(G)){
  for (b in c("logical","bit","bitwhich")){
    x <- X[[b,g]]
    if (!is.null(x)){
      tim[b,"!",g] <- median(microbenchmark(!x, times=times)$time)
      tim[b,"&",g] <- median(microbenchmark(x & x, times=times)$time)
      tim[b,"|",g] <- median(microbenchmark(x | x, times=times)$time)
      tim[b,"==",g] <- median(microbenchmark(x == x, times=times)$time)
      tim[b,"!=",g] <- median(microbenchmark(x != x, times=times)$time)
      tim[b,"summary",g] <- median(microbenchmark(summary.booltype(x), times=times)$time)
    }
  }
}
i <- match("size", M)
for(b in rev(names(B)))  # logical was in first position, so we do this last!
{
  tim[b,i,] <- 100 * tim[b,i,] / tim["logical",i,]
  tim[b,-i,] <- 100 * tim[b,-i,] / max(tim["logical",-i,], na.rm=TRUE)
}
#rm(X)
```

There are substantial savings in skewed filter situations:

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'rare' scenario"}
x <- tim[1:3,,"rare"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x, xlim=c(0,max(100, max(x))), labels=m, pch=c("R","b","w"), col=c("black","blue","red"), main="% size and timings in 'rare' scenario", sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios")
```

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'often' scenario"}
x <- tim[1:3,,"often"]
dotchart(x, xlim=c(0,max(100, max(x))), labels=m, pch=c("R","b","w"), col=c("black","blue","red"), main="% size and timings in 'often' scenario", sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios")
```

Even in non-skewed situations the new booltypes are competitive:

```{r, echo=FALSE, fig.cap = "% size and execution time for bit (b) and bitwhich (w) relative to logical (R) in the 'coin' scenario"}
x <- tim[1:3,,"coin"]
dotchart(x, xlim=c(0,max(100, max(x))), labels=m, pch=c("R","b","w"), col=c("black","blue","red"), main="% size and timings in 'coin' scenario", sub="l='logical'  b='bit'  w='bitwhich'           % of max(R) in all scenarios")
```

Detailed tables follow.


`r pagebreak()`


### % memory consumption of filter

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"size",], 1), caption="% bytes of logical")
```

### % time extracting

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"[]",], 1), caption="% time of logical")
```

### % time assigning

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"[]<-logical",], 1), caption="% time of logical")
```

### % time subscripting with 'which'

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"[which]",], 1), caption="% time of logical")
```

### % time assigning with 'which'

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"[which]<-TRUE",], 1), caption="% time of logical")
```

### % time Boolean NOT

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"!",], 1), caption="% time for Boolean NOT")
```

### % time Boolean AND

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"&",], 1), caption="% time for Boolean &")
```


### % time Boolean OR

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"|",], 1), caption="% time for Boolean |")
```

### % time Boolean EQUALITY

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"==",], 1), caption="% time for Boolean ==")
```

### % time Boolean XOR
```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"!=",], 1), caption="% time for Boolean !=")
```

### % time Boolean SUMMARY

```{r, echo=FALSE, results='asis'}
knitr::kable(round(tim[,"summary",][1:2,1:2], 1), caption="% time for Boolean summary")
```

---

## Fast methods for `integer` set operations

> "The space-efficient structure of bitmaps dramatically reduced the run time of sorting"  
(Jon Bently, Programming Pearls, Cracking the oyster, p. 7)


```{r, echo=FALSE, results='asis'}
binaryDomain <- list(
    smallsmall = rep(Domain["small"], 2)
  , smallbig=Domain
  , bigsmall=rev(Domain)
  , bigbig=rep(Domain["big"], 2)
)
binarySample <- list(
    smallsmall = rep(Sample["small"], 2)
  , smallbig=Sample
  , bigsmall=rev(Sample)
  , bigbig=rep(Sample["big"], 2)
)

M <- c("R","bit","merge")
G <- c("sort","sortunique")
D <- c("unsorted","sorted")

sortM <- vector("list", length(M)*length(G))
dim(sortM) <- c(method=length(M), goal=length(G))
dimnames(sortM) <- list(method=M, goal=G)
sortM[["R","sort"]] <- sort
sortM[["R", "sortunique"]] <- function(x)sort(unique(x))
sortM[["bit","sort"]] <- bit_sort
sortM[["bit","sortunique"]] <- bit_sort_unique

timsort <- array(NA_integer_
             , dim=c(M=2, G=length(G), D=length(D), N=length(Domain)) 
             , dimnames=list(M=M[1:2], G=G, D=D, N=names(Domain)) 
             )
for(n in names(Domain)){
  x <- sample(Domain[[n]], Sample[[n]], replace = TRUE)
  d <- "unsorted"
  for (m in c("R","bit")){
    for (g in G){
      timsort[m,g,d,n] <- median(microbenchmark(sortM[[m,g]](x), times=times)$time)
    }
  }
  x <- bit_sort(x)
  d <- "sorted"
  for (m in 1:2){
    for (g in G){
      timsort[m,g,d,n] <- median(microbenchmark(sortM[[m,g]](x), times=times)$time)
    }
  }
}


binaryU <- c("match","in","notin","union","intersect","setdiff","symdiff","setequal","setearly")
binaryM <- vector("list", length(M)*length(binaryU))
dim(binaryM) <- c(method=length(M), task=length(binaryU))
dimnames(binaryM) <- list(method=M, task=binaryU)
binaryM[["R","match"]] <- match
binaryM[["merge","match"]] <- merge_match

binaryM[["R","in"]] <- get("%in%")
binaryM[["bit","in"]] <- bit_in
binaryM[["merge","in"]] <- merge_in

binaryM[["R","notin"]] <- function(x, y)!(x %in% y)
binaryM[["bit","notin"]] <- function(x, y)!bit_in(x,y)
binaryM[["merge","notin"]] <- merge_notin

binaryM[["R","union"]] <- union
binaryM[["bit","union"]] <- bit_union
binaryM[["merge","union"]] <- merge_union

binaryM[["R","intersect"]] <- intersect
binaryM[["bit","intersect"]] <- bit_intersect
binaryM[["merge","intersect"]] <- merge_intersect

binaryM[["R","setdiff"]] <- setdiff
binaryM[["bit","setdiff"]] <- bit_setdiff
binaryM[["merge","setdiff"]] <- merge_setdiff

binaryM[["R","symdiff"]] <- function(x,y)union(setdiff(x,y), setdiff(y,x))
binaryM[["bit","symdiff"]] <- bit_symdiff
binaryM[["merge","symdiff"]] <- merge_symdiff

binaryM[["R","setequal"]] <- function(x,y)setequal(x,x)  # we compare x to x which avoids early termination and hence 
binaryM[["bit","setequal"]] <- function(x,y)bit_setequal(x,x)
binaryM[["merge","setequal"]] <- function(x,y)merge_setequal(x,x)
  
binaryM[["R","setearly"]] <- function(x,y)setequal(x,y)  # we compare x to x which avoids early termination and hence 
binaryM[["bit","setearly"]] <- function(x,y)bit_setequal(x,y)
binaryM[["merge","setearly"]] <- function(x,y)merge_setequal(x,y)

unaryU <- c("unique","duplicated","anyDuplicated","sumDuplicated")
unaryM <- vector("list", length(M)*length(unaryU))
dim(unaryM) <- c(method=length(M), task=length(unaryU))
dimnames(unaryM) <- list(method=M, task=unaryU)
unaryM[["R","unique"]] <- unique
unaryM[["bit","unique"]] <- bit_unique
unaryM[["merge","unique"]] <- merge_unique
unaryM[["R","duplicated"]] <- duplicated
unaryM[["bit","duplicated"]] <- bit_duplicated
unaryM[["merge","duplicated"]] <- merge_duplicated
unaryM[["R","anyDuplicated"]] <- anyDuplicated
unaryM[["bit","anyDuplicated"]] <- bit_anyDuplicated
unaryM[["merge","anyDuplicated"]] <- merge_anyDuplicated
unaryM[["R","sumDuplicated"]] <- function(x)sum(duplicated(x))
unaryM[["bit","sumDuplicated"]] <- bit_sumDuplicated
unaryM[["merge","sumDuplicated"]] <- merge_sumDuplicated

tim <- array(NA_integer_
             , dim=c(M=length(M), U=length(unaryU)+length(binaryU), N=length(binaryDomain), D=length(D))
             , dimnames=list(M=M, U=c(unaryU,binaryU), N=names(binaryDomain), D=D)
             )

for(n in names(binaryDomain)){
  xnam <- names(binaryDomain[[n]])[1]
  ynam <- names(binaryDomain[[n]])[2]
  x <- sample(binaryDomain[[n]][1], binarySample[[n]][1], replace = FALSE)
  y <- sample(binaryDomain[[n]][2], binarySample[[n]][2], replace = FALSE)
  d <- "unsorted"
  if (length(x)==length(y))
  for (u in unaryU){
    for (m in setdiff(M,"merge")){
      f <- unaryM[[m,u]]
      if (!is.null(f))
        tim[m,u,n,d] <- median(microbenchmark(f(x), times=times)$time)
    }
  }
  for (u in binaryU){
    for (m in setdiff(M,"merge")){
      f <- binaryM[[m,u]]
      if (!is.null(f))
        tim[m,u,n,d] <- median(microbenchmark(f(x,y), times=times)$time)
    }
  }
  x <- bit_sort(x)
  y <- bit_sort(y)
  d <- "sorted"
  if (length(x)==length(y))
  for (u in unaryU){
    for (m in M){
      f <- unaryM[[m,u]]
      if (!is.null(f)){
        tim[m,u,n,d] <- median(microbenchmark(f(x), times=times)$time)
        # now plug-in measures for unsorted merge
        if (m == "merge") 
          tim["merge",u,n,"unsorted"] <- timsort["bit","sort","unsorted",xnam] +  tim["merge",u,n,"sorted"]
      }
    }
  }
  for (u in binaryU){
    for (m in M){
      f <- binaryM[[m,u]]
      if (!is.null(f)){
        tim[m,u,n,d] <- median(microbenchmark(f(x,y), times=times)$time)
        # now plug-in measures for unsorted merge
        if (m == "merge")
          tim["merge",u,n,"unsorted"] <- timsort["bit","sort","unsorted",xnam] + timsort["bit","sort","unsorted",ynam] + tim["merge",u,n,"sorted"]
      }
    }
  }
}
```


```{r, echo=FALSE, fig.cap = "Execution time for R (R) and bit (b)"}
y <- timsort[,,,"big"]
y <- 100 * y / max(y["R",,], na.rm=TRUE)
oldpar <- par(mfrow=c(2,1), mar=c(5,8,2,1))
x <- y[,,"unsorted"]
dotchart(x, xlim=c(0, max(100, max(y))), labels="", pch=c("R","b"), xlab="execution time", main="unsorted", col=c("red","blue"))
x <- y[,,"sorted"]
dotchart(x, xlim=c(0, max(100, max(y))), labels="", pch=c("R","b"), xlab="execution time", main="sorted", col=c("red","blue"))
par(oldpar)
```


```{r, echo=FALSE, results='hide'}
tim2 <- tim
for (n in names(binaryDomain))
  for (d in D)
    tim2[,,n,d] <- 100*tim[,,n,d]/max(tim["R",,n,d], na.rm=TRUE)
```

`r pagebreak()`


```{r, echo=FALSE, fig.cap = "Execution time for R, bit and merge relative to most expensive R in 'unsorted bigbig' scenario"}
y <- tim2[,,"bigbig",]
y <- 100 * y / max(y["R",,], na.rm=TRUE)
x <- y[,,"unsorted"]
m <- rep("", ncol(x))
m <- as.vector(rbind(m, colnames(x), m))
dotchart(x, xlim=c(0, max(100,max(y, na.rm=TRUE))), labels=m, pch=c("R","b","m"), col=c("red","blue","black"), main="Timings in 'unsorted bigbig' scenario", sub="R='hash'   b='bit'   m='merge'")
```

```{r, echo=FALSE, fig.cap = "Execution time for R, bit and merge in 'sorted bigbig' scenario"}
x <- y[,,"sorted"]
dotchart(x, xlim=c(0, max(y, na.rm=TRUE)), labels=m, pch=c("R","b","m"), col=c("red","blue","black"), main="Timings in 'sorted bigbig' scenario", sub="R='hash'   b='bit'   m='merge'")
```

`r pagebreak()`


### % time for sorting

```{r, echo=FALSE, results='asis'}
x <- 100*timsort["bit",,,]/timsort["R",,,]
s <- "sorted"
knitr::kable(x[,s,], caption=paste(s,"data relative to R's sort"), digits=1)
```


```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,s,], caption=paste(s,"data relative to R's sort"), digits=1)
```

### % time for unique

```{r, echo=FALSE, results='asis'}
f <- function(u){
  n <- c("smallsmall","bigbig")
  x <- tim[c("bit","merge","merge"),u,n,]
  dimnames(x)$M[3] <- "sort"
  dimnames(x)$N <- c("small","big")
  x["sort",,"unsorted"] <- timsort["bit","sort","unsorted",]
  x["sort",,"sorted"] <- 0
  for (m in dimnames(x)$M)
    x[m,,] <- x[m,,] / tim["R",u,n,] * 100
  x
}
x <- f("unique")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```


### % time for duplicated

```{r, echo=FALSE, results='asis'}
x <- f("duplicated")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```



### % time for anyDuplicated

```{r, echo=FALSE, results='asis'}
x <- f("anyDuplicated")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for sumDuplicated

```{r, echo=FALSE, results='asis'}
x <- f("sumDuplicated")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```




### % time for match
```{r, echo=FALSE, results='asis'}
f <- function(u){
  x <- tim[c("bit","merge","merge"),u,,]
  dimnames(x)$M[3] <- "sort"
  s <- timsort["bit","sort","unsorted",]
  x["sort",,"unsorted"] <- rep(s, c(2,2)) + c(s,s)
  x["sort",,"sorted"] <- 0
  for (m in dimnames(x)$M)
    x[m,,] <- x[m,,] / tim["R",u,,] * 100
  x
}
x <- f("match")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for in

```{r, echo=FALSE, results='asis'}
x <- f("in")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for notin

```{r, echo=FALSE, results='asis'}
x <- f("notin")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```


### % time for union


```{r, echo=FALSE, results='asis'}
x <- f("union")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for intersect


```{r, echo=FALSE, results='asis'}
x <- f("intersect")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for setdiff


```{r, echo=FALSE, results='asis'}
x <- f("setdiff")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for symdiff


```{r, echo=FALSE, results='asis'}
x <- f("symdiff")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```


### % time for setequal


```{r, echo=FALSE, results='asis'}
x <- f("setequal")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

### % time for setearly


```{r, echo=FALSE, results='asis'}
x <- f("setearly")
s <- "sorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```
```{r, echo=FALSE, results='asis'}
s <- "unsorted"
knitr::kable(x[,,s], caption=paste(s,"data relative to R"), digits=1)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{print.bit}
\alias{print.bit}
\title{Print method for bit}
\usage{
\method{print}{bit}(x, ...)
}
\arguments{
\item{x}{a bit vector}

\item{...}{passed to print}
}
\value{
a character vector showing first and last elements of the bit vector
}
\description{
Print method for bit
}
\examples{
print(bit(120))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{Extract}
\alias{Extract}
\alias{[[.bit}
\alias{[[<-.bit}
\alias{[.bit}
\alias{[<-.bit}
\alias{[[.bitwhich}
\alias{[[<-.bitwhich}
\alias{[.bitwhich}
\alias{[<-.bitwhich}
\title{Extract or replace part of an boolean vector}
\usage{
\method{[[}{bit}(x, i)

\method{[[}{bit}(x, i) <- value

\method{[}{bit}(x, i)

\method{[}{bit}(x, i) <- value

\method{[[}{bitwhich}(x, i)

\method{[[}{bitwhich}(x, i) <- value

\method{[}{bitwhich}(x, i)

\method{[}{bitwhich}(x, i) <- value
}
\arguments{
\item{x}{a \code{\link{bit}} or \code{\link{bitwhich}} object}

\item{i}{preferrably a positive integer subscript or a \code{\link{ri}}, see text}

\item{value}{new logical or integer values}
}
\value{
The extractors \code{[[} and \code{[} return a logical scalar or
vector.  The replacment functions return an object of \code{class(x)}.
}
\description{
Operators acting on \code{\link{bit}} or \code{\link{bitwhich}} objects to extract or replace parts.
}
\details{
The typical usecase for for '[' and '[<-' is subscripting with positive integers, 
negative integers are allowed but slower, 
as logical subscripts only scalars are allowed. 
The subscript can be given as a \code{\link{bitwhich}} object.
Also \code{\link{ri}} can be used as subscript.

Extracting from \code{\link{bit}} and \code{\link{bitwhich}} is faster than from  \code{\link{logical}} if positive subscripts are used.
integer subscripts make sense.  Negative subscripts are converted to
positive ones, beware the RAM consumption.
}
\examples{

  x <- as.bit(c(FALSE, NA, TRUE))
  x[] <- c(FALSE, NA, TRUE)
  x[1:2]
  x[-3]
  x[ri(1,2)]
  x[as.bitwhich(c(TRUE,TRUE,FALSE))]
  x[[1]]
  x[] <- TRUE
  x[1:2] <- FALSE
  x[[1]] <- TRUE

}
\seealso{
\code{\link{bit}}, \code{\link[base]{Extract}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{bitwhich_representation}
\alias{bitwhich_representation}
\title{Diagnose representation of bitwhich}
\usage{
bitwhich_representation(x)
}
\arguments{
\item{x}{a \code{\link{bitwhich}} object}
}
\value{
a scalar, one of \code{logical()}, \code{FALSE}, \code{TRUE}, \code{-1} or \code{1}
}
\description{
Diagnose representation of bitwhich
}
\examples{
bitwhich_representation(bitwhich())
bitwhich_representation(bitwhich(12,FALSE))
bitwhich_representation(bitwhich(12,TRUE))
bitwhich_representation(bitwhich(12, -3))
bitwhich_representation(bitwhich(12, 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{length.bit}
\alias{length.bit}
\alias{length<-.bit}
\alias{length.bitwhich}
\alias{length<-.bitwhich}
\alias{length.ri}
\title{Getting and setting length of bit, bitwhich and ri objects}
\usage{
\method{length}{bit}(x)

\method{length}{bit}(x) <- value

\method{length}{bitwhich}(x)

\method{length}{bitwhich}(x) <- value

\method{length}{ri}(x)
}
\arguments{
\item{x}{a \code{\link[=bit]{bit()}}, \code{\link[=bitwhich]{bitwhich()}} or \code{\link[=ri]{ri()}}
object}

\item{value}{the new number of bits}
}
\value{
the length A bit vector with the new length
}
\description{
Query the number of bits in a \code{\link[=bit]{bit()}} vector or change the number
of bits in a bit vector.
Query the number of bits in a bitwhich()] vector or change the number of bits in a bit
vector.
}
\details{
NOTE that the length does NOT reflect the number of selected (\code{TRUE})
bits, it reflects the sum of both, \code{TRUE} and \code{FALSE} bits.
Increasing the length of a \code{\link[=bit]{bit()}} object will set new bits to
\code{FALSE}.  The behaviour of increasing the length of a
\code{\link[=bitwhich]{bitwhich()}} object is different and depends on the content of the
object:
\itemize{
\item TRUE -- all included, new bits are set to \code{TRUE}
\item positive integers -- some included, new bits are set to \code{FALSE}
\item negative integers -- some excluded, new bits are set to \code{TRUE}
\item FALSE -- all excluded:, new bits are set to \code{FALSE}
}

Decreasing the length of bit or bitwhich removes any previous information
about the status bits above the new length.
}
\examples{

  stopifnot(length(ri(1, 1, 32)) == 32)

  x <- as.bit(ri(32, 32, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 0)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 0)

  x <- as.bit(ri(1, 1, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 1)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)

  x <- as.bitwhich(bit(32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 0)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 0)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 0)

  x <- as.bitwhich(!bit(32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 32)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 16)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 32)

  x <- as.bitwhich(ri(32, 32, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 0)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 0)

  x <- as.bitwhich(ri(2, 32, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 31)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 15)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 31)

  x <- as.bitwhich(ri(1, 1, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 1)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 1)

  x <- as.bitwhich(ri(1, 31, 32))
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 31)
  message("NOTE the change from 'some excluded' to 'all excluded' here")
  length(x) <- 16
  stopifnot(length(x) == 16)
  stopifnot(sum(x) == 16)
  length(x) <- 32
  stopifnot(length(x) == 32)
  stopifnot(sum(x) == 32)

}
\seealso{
\code{\link[=length]{length()}}, \code{\link[=sum.bit]{sum()}},
\code{\link[=poslength]{poslength()}}, \code{\link[=maxindex]{maxindex()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{length.bit}
\alias{length.bit}
\alias{length<-.bit}
\alias{length.bitwhich}
\alias{length<-.bitwhich}
\alias{length.ri}
\title{Getting and setting length of bit, bitwhich and ri objects}
\usage{
\method{length}{bit}(x)

\method{length}{bit}(x) <- value

\method{length}{bitwhich}(x)

\method{length}{bitwhich}(x) <- value

\method{length}{ri}(x)
}
\arguments{
\item{x}{a \code{\link{bit}}, \code{\link{bitwhich}} or \code{\link{ri}}
object}

\item{value}{the new number of bits}
}
\value{
the length A bit vector with the new length
}
\description{
Query the number of bits in a \code{\link{bit}} vector or change the number
of bits in a bit vector. \cr Query the number of bits in a
\code{\link{bitwhich}} vector or change the number of bits in a bit vector.
\cr
}
\details{
NOTE that the length does NOT reflect the number of selected (\code{TRUE})
bits, it reflects the sum of both, \code{TRUE} and \code{FALSE} bits.
Increasing the length of a \code{\link{bit}} object will set new bits to
\code{FALSE}.  The behaviour of increasing the length of a
\code{\link{bitwhich}} object is different and depends on the content of the
object: \itemize{ 
\item TRUE -- all included, new bits are set to \code{TRUE}
\item positive integers -- some included, new bits are set to \code{FALSE}
\item negative integers -- some excluded, new bits are set to \code{TRUE}
\item FALSE -- all excluded:, new bits are set to \code{FALSE} } Decreasing the
length of bit or bitwhich removes any previous information about the status
bits above the new length.
}
\examples{

  stopifnot(length(ri(1, 1, 32))==32)

  x <- as.bit(ri(32, 32, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==0)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==0)

  x <- as.bit(ri(1, 1, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==1)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)

  x <- as.bitwhich(bit(32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==0)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==0)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==0)

  x <- as.bitwhich(!bit(32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==32)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==16)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==32)

  x <- as.bitwhich(ri(32, 32, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==0)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==0)

  x <- as.bitwhich(ri(2, 32, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==31)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==15)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==31)

  x <- as.bitwhich(ri(1, 1, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==1)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==1)

  x <- as.bitwhich(ri(1, 31, 32))
  stopifnot(length(x)==32)
  stopifnot(sum(x)==31)
  message("NOTE the change from 'some excluded' to 'all excluded' here")
  length(x) <- 16
  stopifnot(length(x)==16)
  stopifnot(sum(x)==16)
  length(x) <- 32
  stopifnot(length(x)==32)
  stopifnot(sum(x)==32)

}
\seealso{
\code{\link{length}}, \code{\link[=sum.bit]{sum}},
\code{\link{poslength}}, \code{\link{maxindex}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_sort_unique}
\alias{bit_sort_unique}
\title{bit sort unique}
\usage{
bit_sort_unique(
  x,
  decreasing = FALSE,
  na.last = NA,
  has.dup = TRUE,
  range_na = NULL
)
}
\arguments{
\item{x}{an integer vector}

\item{decreasing}{\code{FALSE} (ascending) or \code{TRUE} (descending)}

\item{na.last}{\code{NA} removes NAs, \code{FALSE} puts NAs at the beginning, \code{TRUE} puts NAs at the end}

\item{has.dup}{TRUE (the default) assumes that \code{x} might have duplicates, set to \code{FALSE} if duplicates are impossible}

\item{range_na}{\code{NULL} calls \code{\link{range_na}}, optionally the result of \code{\link{range_na}} can be given here to avoid calling it again}
}
\value{
a sorted unique integer vector
}
\description{
fast combination of \code{\link{sort}} and \code{\link{unique}} for integers
}
\details{
determines the range of the integers and checks if the density justifies use 
of a bit vector; if yes, creates the result using a bit vector; if no, falls back to \code{sort(unique())}
}
\examples{
bit_sort_unique(c(2L,1L,NA,NA,1L,2L))
bit_sort_unique(c(2L,1L,NA,NA,1L,2L), na.last=FALSE)
bit_sort_unique(c(2L,1L,NA,NA,1L,2L), na.last=TRUE)
bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE)
bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE, na.last=FALSE)
bit_sort_unique(c(2L,1L,NA,NA,1L,2L), decreasing = TRUE, na.last=TRUE)

\dontrun{
x <- sample(1e7, replace=TRUE)
system.time(bit_sort_unique(x))
system.time(sort(unique(x)))
x <- sample(1e7)
system.time(bit_sort_unique(x))
system.time(sort(x))
} 
}
\seealso{
\code{\link{sort}}, \code{\link{unique}}, 
  \code{\link{bit_sort}}, \code{\link{bit_unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{physical.default}
\alias{physical.default}
\alias{physical<-.default}
\alias{virtual.default}
\alias{virtual<-.default}
\alias{print.physical}
\alias{print.virtual}
\alias{PhysVirt}
\alias{physical}
\alias{physical<-}
\alias{virtual}
\alias{virtual<-}
\title{Physical and virtual attributes}
\usage{
\method{physical}{default}(x)

\method{physical}{default}(x) <- value

\method{virtual}{default}(x)

\method{virtual}{default}(x) <- value

\method{print}{physical}(x, ...)

\method{print}{virtual}(x, ...)

physical(x)

physical(x) <- value

virtual(x)

virtual(x) <- value
}
\arguments{
\item{x}{a ff or ram object}

\item{value}{a list with named elements}

\item{\dots}{further arguments}
}
\value{
\command{physical} and \command{virtual} returns a list with named
elements
}
\description{
Compatibility functions (to package ff) for getting and setting physical and
virtual attributes.
}
\details{
ff objects have physical and virtual attributes, which have different
copying semantics: physical attributes are shared between copies of ff
objects while virtual attributes might differ between copies.
\code{\link[ff:as.ff]{as.ram}} will retain some physical and virtual atrributes in
the ram clone, such that \code{\link[ff]{as.ff}} can restore an ff object
with the same attributes.
}
\examples{

  physical(bit(12))
  virtual(bit(12))
}
\seealso{
\code{\link[ff]{physical.ff}}, \code{\link[ff]{physical.ffdf}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{attribute}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{print.bitwhich}
\alias{print.bitwhich}
\title{Print method for bitwhich}
\usage{
\method{print}{bitwhich}(x, ...)
}
\arguments{
\item{x}{a \code{\link{bitwhich}} object}

\item{...}{ignored}
}
\description{
Print method for bitwhich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{xor.default}
\alias{xor.default}
\alias{xor.logical}
\alias{!.bit}
\alias{&.bit}
\alias{|.bit}
\alias{==.bit}
\alias{!=.bit}
\alias{xor.bit}
\alias{!.bitwhich}
\alias{&.bitwhich}
\alias{|.bitwhich}
\alias{==.bitwhich}
\alias{!=.bitwhich}
\alias{xor.bitwhich}
\alias{&.booltype}
\alias{|.booltype}
\alias{==.booltype}
\alias{!=.booltype}
\alias{xor.booltype}
\alias{xor}
\title{Boolean operators and functions}
\usage{
\method{xor}{default}(x, y)

\method{xor}{logical}(x, y)

\method{!}{bit}(x)

\method{&}{bit}(e1, e2)

\method{|}{bit}(e1, e2)

\method{==}{bit}(e1, e2)

\method{!=}{bit}(e1, e2)

\method{xor}{bit}(x, y)

\method{!}{bitwhich}(x)

\method{&}{bitwhich}(e1, e2)

\method{|}{bitwhich}(e1, e2)

\method{==}{bitwhich}(e1, e2)

\method{!=}{bitwhich}(e1, e2)

\method{xor}{bitwhich}(x, y)

\method{&}{booltype}(e1, e2)

\method{|}{booltype}(e1, e2)

\method{==}{booltype}(e1, e2)

\method{!=}{booltype}(e1, e2)

\method{xor}{booltype}(x, y)

xor(x, y)
}
\arguments{
\item{x}{a \code{\link{is.booltype}} vector}

\item{y}{a \code{\link{is.booltype}} vector}

\item{e1}{a \code{\link{is.booltype}} vector}

\item{e2}{a \code{\link{is.booltype}} vector}
}
\value{
An object of class \code{\link{booltype}} or \code{\link{logical}}
}
\description{
Boolean NEGATION '!', AND '&', OR '|' and EXCLUSIVE OR xor', see \code{\link[base]{Logic}}.
}
\details{
The binary operators and function \code{xor} can now combine any \code{\link{is.booltype}} vectors. 
They now recycle if vectors have different length. If the two arguments have different \code{\link{booltypes}} the return value corresponds to the lower \code{\link{booltype}} of the two. \cr

Boolean operations on \code{\link{bit}} vectors are extremely fast because they are
implemented using C's bitwise operators. Boolean operations on or \code{\link{bitwhich}}  
vectors are even faster, if they represent very skewed selections.  \cr

The \code{xor} function has been made generic and \code{xor.default} has
been implemented much faster than R's standard \code{\link[base:Logic]{xor}}.
This was possible because actually boolean function \code{xor} and
comparison operator \code{!=} do the same (even with NAs), and \code{!=} is
much faster than the multiple calls in \code{(x | y) & !(x & y)}
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: default method for \code{\link{xor}}

\item \code{logical}: \code{\link{logical}} method for \code{\link{xor}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{!}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{&}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{|}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{==}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{!=}}

\item \code{bit}: \code{\link{bit}} method for \code{\link{xor}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{!}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{&}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{|}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{==}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{!=}}

\item \code{bitwhich}: \code{\link{bitwhich}} method for \code{\link{xor}}

\item \code{booltype}: \code{\link{booltype}} method for \code{\link{&}}

\item \code{booltype}: \code{\link{booltype}} method for \code{\link{|}}

\item \code{booltype}: \code{\link{booltype}} method for \code{\link{==}}

\item \code{booltype}: \code{\link{booltype}} method for \code{\link{!=}}

\item \code{booltype}: \code{\link{booltype}} method for \code{\link{xor}}
}}

\examples{

  x <- c(FALSE, FALSE, FALSE, NA, NA, NA, TRUE, TRUE, TRUE)
  y <- c(FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE)
  
  x|y
  x|as.bit(y)
  x|as.bitwhich(y)
  x|as.which(y)
  x|ri(1,1,9)
  
  
}
\seealso{
\code{\link{booltypes}}, \code{\link{Logic}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attrutil.R
\name{getsetattr}
\alias{getsetattr}
\alias{setattr}
\alias{setattributes}
\title{Attribute setting by reference}
\usage{
getsetattr(x, which, value)

setattr(x, which, value)

setattributes(x, attributes)
}
\arguments{
\item{x}{an R object}

\item{which}{name of the attribute}

\item{value}{value of the attribute, use NULL to remove this attribute}

\item{attributes}{a named list of attribute values}
}
\value{
invisible(), we do not return the changed object to remind you of
the fact that this function is called for its side-effect of changing its
input object.
}
\description{
Function \code{setattr} sets a singe attribute and function
\code{setattributes} sets a list of attributes.
}
\details{
The attributes of 'x' are changed in place without copying x. function
\code{setattributes} does only change the named attributes, it does not
delete the non-names attributes like \code{\link{attributes}} does.
}
\section{Functions}{
\itemize{
\item \code{setattr}: 

\item \code{setattributes}: 
}}

\examples{

  x <- as.single(runif(10))
  attr(x, "Csingle")

  f <- function(x)attr(x, "Csingle") <- NULL
  g <- function(x)setattr(x, "Csingle", NULL)

  f(x)
  x
  g(x)
  x

 \dontrun{

  # restart R
  library(bit)

  mysingle <- function(length = 0){
    ret <- double(length)
    setattr(ret, "Csingle", TRUE)
    ret
  }

  # show that mysinge gives exactly the same result as single
  identical(single(10), mysingle(10))

  # look at the speedup and memory-savings of mysingle compared to single
  system.time(mysingle(1e7))
  memory.size(max=TRUE)
  system.time(single(1e7))
  memory.size(max=TRUE)

  # look at the memory limits
  # on my win32 machine the first line fails beause of not enough RAM, the second works
  x <- single(1e8)
  x <- mysingle(1e8)

  # .g. performance with factors
  x <- rep(factor(letters), length.out=1e7)
  x[1:10]
  # look how fast one can do this
  system.time(setattr(x, "levels", rev(letters)))
  x[1:10]
  # look at the performance loss in time caused by the non-needed copying
  system.time(levels(x) <- letters)
  x[1:10]


  # restart R
  library(bit)

  simplefactor <- function(n){
    factor(rep(1:2, length.out=n))
  }

  mysimplefactor <- function(n){
    ret <- rep(1:2, length.out=n)
    setattr(ret, "levels", as.character(1:2))
    setattr(ret, "class", "factor")
    ret
  }

  identical(simplefactor(10), mysimplefactor(10))

  system.time(x <- mysimplefactor(1e7))
  memory.size(max=TRUE)
  system.time(setattr(x, "levels", c("a","b")))
  memory.size(max=TRUE)
  x[1:4]
  memory.size(max=TRUE)
  rm(x)
  gc()

  system.time(x <- simplefactor(1e7))
  memory.size(max=TRUE)
  system.time(levels(x) <- c("x","y"))
  memory.size(max=TRUE)
  x[1:4]
  memory.size(max=TRUE)
  rm(x)
  gc()

}


}
\references{
Writing R extensions -- System and foreign language interfaces
-- Handling R objects in C -- Attributes (Version 2.11.1 (2010-06-03 ) R
Development)
}
\seealso{
\code{\link{attr}} \code{\link{unattr}}
}
\author{
Jens Oehlschlägel
}
\keyword{attributes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\docType{data}
\name{booltypes}
\alias{booltypes}
\title{Boolean types}
\format{
An object of class \code{ordered} (inherits from \code{factor}) of length 6.
}
\usage{
booltypes
}
\description{
The \code{\link[=ordered]{ordered()}} factor \code{booltypes} ranks the boolean types.
}
\details{
There are currently six boolean types, \code{booltypes} is an \code{\link[=ordered]{ordered()}} vector with the
following ordinal \code{\link[=levels]{levels()}}:
\itemize{
\item nobool: non-boolean type
\item \code{\link[=logical]{logical()}}: for representing any boolean data including \code{NA}
\item \code{\link[=bit]{bit()}}: for representing dense boolean data
\item \code{\link[=bitwhich]{bitwhich()}}: for representing sparse (skewed) boolean data
\item \code{\link[=which]{which()}}: for representing sparse boolean data with few `TRUE
\item \code{\link[=ri]{ri()}}: range-indexing, for representing sparse boolean data with a single range of
\code{TRUE}
}

\code{booltypes} has a \code{\link[=names]{names()}} attribute such that elements can be selected by name.
}
\note{
do not rely on the internal integer codes of these levels, we might add-in
\code{\link[ff:hi]{hi}} later
}
\seealso{
\code{\link[=booltype]{booltype()}}, \code{\link[=is.booltype]{is.booltype()}}, \code{\link[=as.booltype]{as.booltype()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\docType{data}
\name{booltypes}
\alias{booltypes}
\title{Boolean types}
\format{
An object of class \code{ordered} (inherits from \code{factor}) of length 6.
}
\usage{
booltypes
}
\description{
The \code{\link{ordered}} factor \code{booltypes} ranks the boolean types.
}
\details{
There are currently six boolean types, \code{booltypes} is an \code{\link{ordered}} vector with the following ordinal \code{\link{levels}} \describe{
\item{nobool}{non-boolean types}
\item{\code{\link{logical}}}{for representing any boolean data including \code{NA} }
\item{\code{\link{bit}}}{for representing dense boolean data }
\item{\code{\link{bitwhich}}}{for representing sparse (skewed) boolean data  }
\item{\code{\link{which}}}{for representing sparse boolean data with few \code{TRUE}}
\item{\code{\link{ri}}}{range-indexing, for representing sparse boolean data with a single range of \code{TRUE} }
}
\code{booltypes} has a \code{\link{names}} attribute such that elements can be selected by name.
}
\note{
do not rely on the internal integer codes of these levels, we might add-in \code{\link[ff]{hi}} later
}
\seealso{
\code{\link{booltype}}, \code{\link{is.booltype}}, \code{\link{as.booltype}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{quicksort3}
\alias{quicksort3}
\title{Low-level sorting: threeway quicksort}
\usage{
quicksort3(x, na.last = NA)
}
\arguments{
\item{x}{an integer vector}

\item{na.last}{\code{NA} removes NAs, \code{FALSE} puts NAs at the beginning,
\code{TRUE} puts NAs at the end}
}
\value{
a sorted vector
}
\description{
In one pass over the vector \code{NA}s are handled according to parameter
\code{na.last} by \code{\link{range_sortna}}, then, if the vector is unsorted, 
threeway quicksort is invoked.
}
\examples{
countsort(c(2L,0L,1L,NA,2L))
countsort(c(2L,0L,1L,NA,2L), na.last=TRUE)
countsort(c(2L,0L,1L,NA,2L), na.last=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_setops}
\alias{bit_setops}
\alias{bit_union}
\alias{bit_intersect}
\alias{bit_setdiff}
\alias{bit_symdiff}
\alias{bit_setequal}
\title{bit set operations}
\usage{
bit_union(x, y)

bit_intersect(x, y)

bit_setdiff(x, y)

bit_symdiff(x, y)

bit_setequal(x, y)
}
\arguments{
\item{x}{an integer vector}

\item{y}{an integer vector}
}
\value{
an integer vector
}
\description{
Fast versions of \code{\link{union}}, \code{\link{intersect}}, 
\code{\link{setdiff}}, symmetric difference and \code{\link{setequal}}
for integers.
}
\details{
determines the range of the integers and checks if the density justifies use 
of a bit vector; if yes, uses a bit vector for finding duplicates; if no, 
falls back to \code{\link{union}}, \code{\link{intersect}},
\code{\link{setdiff}}, \code{union(setdiff(x,y),setdiff(y,x))} and \code{\link{setequal}}
}
\section{Functions}{
\itemize{
\item \code{bit_union}: union

\item \code{bit_intersect}: intersection

\item \code{bit_setdiff}: asymmetric difference

\item \code{bit_symdiff}: symmetricx difference

\item \code{bit_setequal}: equality
}}

\examples{
bit_union(1:2, 2:3)
bit_intersect(1:2, 2:3)
bit_setdiff(1:2, 2:3)
bit_symdiff(1:2, 2:3)
bit_setequal(1:2, 2:3)
bit_setequal(1:2, 2:1)
}
\seealso{
\code{\link{bit_in}}, \code{\link{bit_rangediff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_setops}
\alias{bit_setops}
\alias{bit_union}
\alias{bit_intersect}
\alias{bit_setdiff}
\alias{bit_symdiff}
\alias{bit_setequal}
\title{bit set operations}
\usage{
bit_union(x, y)

bit_intersect(x, y)

bit_setdiff(x, y)

bit_symdiff(x, y)

bit_setequal(x, y)
}
\arguments{
\item{x}{an integer vector}

\item{y}{an integer vector}
}
\value{
an integer vector
}
\description{
Fast versions of \code{\link[=union]{union()}}, \code{\link[=intersect]{intersect()}},
\code{\link[=setdiff]{setdiff()}}, symmetric difference and \code{\link[=setequal]{setequal()}}
for integers.
}
\details{
determines the range of the integers and checks if the density justifies use
of a bit vector; if yes, uses a bit vector for finding duplicates; if no,
falls back to \code{\link[=union]{union()}}, \code{\link[=intersect]{intersect()}},
\code{\link[=setdiff]{setdiff()}}, \code{union(setdiff(x, y), setdiff(y, x))} and \code{\link[=setequal]{setequal()}}
}
\section{Functions}{
\itemize{
\item \code{bit_union()}: union

\item \code{bit_intersect()}: intersection

\item \code{bit_setdiff()}: asymmetric difference

\item \code{bit_symdiff()}: symmetricx difference

\item \code{bit_setequal()}: equality

}}
\examples{
bit_union(1:2, 2:3)
bit_intersect(1:2, 2:3)
bit_setdiff(1:2, 2:3)
bit_symdiff(1:2, 2:3)
bit_setequal(1:2, 2:3)
bit_setequal(1:2, 2:1)
}
\seealso{
\code{\link[=bit_in]{bit_in()}}, \code{\link[=bit_rangediff]{bit_rangediff()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.booltype.default}
\alias{as.booltype.default}
\alias{as.booltype}
\title{Coerce to booltype (generic)}
\usage{
\method{as.booltype}{default}(x, booltype = "logical", ...)

as.booltype(x, booltype, ...)
}
\arguments{
\item{x}{object to coerce}

\item{booltype}{target \code{\link{booltype}} given as integer or as character}

\item{...}{further arguments}
}
\value{
\code{x} coerced to \code{booltype}
}
\description{
Coerce to booltype (generic)
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: default method for as.booltype
}}

\examples{
as.booltype(0:1)
as.booltype(0:1, "logical")
as.booltype(0:1, "bit")
as.booltype(0:1, "bitwhich")
as.booltype(0:1, "which", maxindex=2)
as.booltype(0:1, "ri")
}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{booltypes}}, \code{\link{booltype}}, \code{\link{is.booltype}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{rep.booltype}
\alias{rep.booltype}
\alias{rep.bit}
\alias{rep.bitwhich}
\title{Replicating bit and bitwhich vectors}
\usage{
\method{rep}{bit}(x, times = 1L, length.out = NA, ...)

\method{rep}{bitwhich}(x, times = 1L, length.out = NA, ...)
}
\arguments{
\item{x}{bit or bitwhich object}

\item{times}{number of replications}

\item{length.out}{final length of replicated vector (dominates times)}

\item{\dots}{not used}
}
\value{
An object of class 'bit' or 'bitwhich'
}
\description{
Creating new bit or bitwhich by recycling such vectors
}
\examples{

 rep(as.bit(c(FALSE,TRUE)), 2)
 rep(as.bit(c(FALSE,TRUE)), length.out=7)
 rep(as.bitwhich(c(FALSE,TRUE)), 2)
 rep(as.bitwhich(c(FALSE,TRUE)), length.out=1)
}
\seealso{
\code{\link{rep}}, \code{\link{bit}} , \code{\link{bitwhich}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\docType{data}
\name{.BITS}
\alias{.BITS}
\alias{bit_init}
\alias{bit_done}
\title{Initializing bit masks}
\format{
An object of class \code{integer} of length 1.
}
\usage{
.BITS

bit_init()

bit_done()
}
\description{
Functions to allocate (and de-allocate) bit masks
}
\details{
The C-code operates with bit masks.  The memory for these is allocated
dynamically.  \code{bit_init} is called by \code{\link{.First.lib}} and
\code{bit_done} is called by \code{\link{.Last.lib}}.  You don't need to
care about these under normal circumstances.
}
\examples{

  bit_done()
  bit_init()

}
\seealso{
\code{\link{bit}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{datasets}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{in.bitwhich}
\alias{in.bitwhich}
\title{Check existence of integers in table}
\usage{
in.bitwhich(x, table, is.unsorted = NULL)
}
\arguments{
\item{x}{a vector of integer}

\item{table}{a \code{\link{bitwhich}} object or a vector of integer}

\item{is.unsorted}{logical telling the function whether the table is (un)sorted. With the defautl \code{NULL} \code{FALSE} is assumed for \code{\link{bitwhich}} tables, otherwise \code{TRUE}}
}
\value{
logical vector
}
\description{
If the table is sorted, this can be much faster than \code{\link{\%in\%}}
}
\examples{
x <- bitwhich(100)
x[3] <- TRUE
in.bitwhich(c(NA,2,3), x)
}
\seealso{
\code{\link{\%in\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.R
\name{symdiff}
\alias{symdiff}
\title{Symmetric set complement}
\usage{
symdiff(x, y)
}
\arguments{
\item{x}{a vector}

\item{y}{a vector}
}
\value{
\code{union(setdiff(x, y), setdiff(y, x))}
}
\description{
Symmetric set complement
}
\note{
that \code{symdiff(x, y)} is not \code{\link[=identical]{identical()}}
as \code{symdiff(y, x)} without applying \code{\link[=sort]{sort()}} to the result
}
\examples{
symdiff(c(1L, 2L, 2L), c(2L, 3L))
symdiff(c(2L, 3L), c(1L, 2L, 2L))
}
\seealso{
\code{\link[=merge_symdiff]{merge_symdiff()}} and \code{\link[=xor]{xor()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.R
\name{symdiff}
\alias{symdiff}
\title{Symmetric set complement}
\usage{
symdiff(x, y)
}
\arguments{
\item{x}{a vector}

\item{y}{a vector}
}
\value{
\code{union(setdiff(x,y), setdiff(y,x))}
}
\description{
Symmetric set complement
}
\note{
that \code{symdiff(x,y)} is not \code{\link{identical}} 
as \code{symdiff(y,x)} without applying \code{\link{sort}} to the result
}
\examples{
symdiff(c(1L,2L,2L), c(2L,3L))
symdiff(c(2L,3L), c(1L,2L,2L))
}
\seealso{
\code{\link{merge_symdiff}} and \code{\link{xor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attrutil.R
\name{unattr}
\alias{unattr}
\title{Attribute removal}
\usage{
unattr(x)
}
\arguments{
\item{x}{any R object}
}
\value{
a similar object with attributes removed
}
\description{
Returns object with attributes removed
}
\details{
attribute removal copies the object as usual
}
\examples{

  bit(2)[]
  unattr(bit(2)[])

}
\seealso{
\code{\link{attributes}}, \code{\link{setattributes}},
\code{\link{unclass}}
}
\author{
Jens Oehlschlägel
}
\keyword{attribute}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rle.R
\name{rlepack}
\alias{rlepack}
\alias{rlepack.integer}
\alias{rleunpack}
\alias{rleunpack.rlepack}
\alias{rev.rlepack}
\alias{unique.rlepack}
\alias{anyDuplicated.rlepack}
\title{Hybrid Index, rle-pack utilities}
\usage{
rlepack(x, ...)

\method{rlepack}{integer}(x, pack = TRUE, ...)

rleunpack(x)

\method{rleunpack}{rlepack}(x)

\method{rev}{rlepack}(x)

\method{unique}{rlepack}(x, incomparables = FALSE, ...)

\method{anyDuplicated}{rlepack}(x, incomparables = FALSE, ...)
}
\arguments{
\item{x}{in 'rlepack' an integer vector, in the other functions an object of
class 'rlepack'}

\item{\dots}{just to keep R CMD CHECK quiet (not used)}

\item{pack}{FALSE to suppress packing}

\item{incomparables}{just to keep R CMD CHECK quiet (not used)}
}
\value{
A list with components \item{ first }{ the first element of the
packed sequence } \item{ dat }{ either an object of class \code{\link{rle}}
or the complete input vector \code{x} if rle-packing is not efficient }
\item{ last }{ the last element of the packed sequence }
}
\description{
Basic utilities for rle packing and unpacking and apropriate methods for
\code{\link{rev}} and \code{\link{unique}}.
}
\examples{

  x <- rlepack(rep(0L, 10))
\dontshow{
 for (x in list(10:1, 1:10, c(10:1,1:10), c(1:10,10:1), sample(100), sample(100, 100, TRUE), sample(10, 100, TRUE))){
   stopifnot(identical(rleunpack(rlepack(x)), x))
   stopifnot(identical(rleunpack(unique(rlepack(x))), unique(x)))
   stopifnot(identical(anyDuplicated(rlepack(x)), anyDuplicated(x)))
 }
}

}
\seealso{
\code{\link[ff]{hi}}, \code{\link{intrle}}, \code{\link{rle}},
\code{\link{rev}}, \code{\link{unique}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rle.R
\name{rlepack}
\alias{rlepack}
\alias{rlepack.integer}
\alias{rleunpack}
\alias{rleunpack.rlepack}
\alias{rev.rlepack}
\alias{unique.rlepack}
\alias{anyDuplicated.rlepack}
\title{Hybrid Index, rle-pack utilities}
\usage{
rlepack(x, ...)

\method{rlepack}{integer}(x, pack = TRUE, ...)

rleunpack(x)

\method{rleunpack}{rlepack}(x)

\method{rev}{rlepack}(x)

\method{unique}{rlepack}(x, incomparables = FALSE, ...)

\method{anyDuplicated}{rlepack}(x, incomparables = FALSE, ...)
}
\arguments{
\item{x}{in 'rlepack' an integer vector, in the other functions an object of
class 'rlepack'}

\item{...}{just to keep R CMD CHECK quiet (not used)}

\item{pack}{FALSE to suppress packing}

\item{incomparables}{just to keep R CMD CHECK quiet (not used)}
}
\value{
A list with components:
\itemize{
\item first: the first element of the packed sequence
\item dat: either an object of class \code{\link[=rle]{rle()}} or the complete input vector \code{x} if
rle-packing is not efficient
\item last: the last element of the packed sequence
}
}
\description{
Basic utilities for rle packing and unpacking and apropriate methods for
\code{\link[=rev]{rev()}} and \code{\link[=unique]{unique()}}.
}
\examples{

  x <- rlepack(rep(0L, 10))

}
\seealso{
\code{\link[ff:hi]{ff::hi()}}, \code{\link[=intrle]{intrle()}}, \code{\link[=rle]{rle()}}, \code{\link[=rev]{rev()}}, \code{\link[=unique]{unique()}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{firstNA}
\alias{firstNA}
\title{Position of first NA}
\usage{
firstNA(x)
}
\arguments{
\item{x}{an R vector}
}
\value{
a reversed vector
}
\description{
This is substantially faster than \code{which.max(is.na(x))}
}
\examples{
x <- c(FALSE,NA,TRUE)
firstNA(x)
reverse_vector(x)
\dontrun{
x <- 1:1e7
system.time(rev(x))
system.time(reverse_vector(x))
}
}
\seealso{
\code{\link{which.max}}, \code{\link{is.na}}, \code{\link{anyNA}}, \code{\link{anyDuplicated}}, \code{\link{bit_anyDuplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{bitwhich}
\alias{bitwhich}
\title{Create bitwhich vector (skewed boolean)}
\usage{
bitwhich(
  maxindex = 0L,
  x = NULL,
  xempty = FALSE,
  poslength = NULL,
  is.unsorted = TRUE,
  has.dup = TRUE
)
}
\arguments{
\item{maxindex}{length of the vector}

\item{x}{Information about which positions are FALSE or TRUE: either \code{logical()} or \code{TRUE} or \code{FALSE} or a integer vector of positive or of negative subscripts.}

\item{xempty}{what to assume about parameter \code{x} if \code{x=integer(0)}, typically \code{TRUE} or \code{FALSE}.}

\item{poslength}{tuning: \code{poslength} is calculated automatically, you can give \code{poslength} explicitely, in this case it must be correct and \code{x} must be sorted and not have duplicates.}

\item{is.unsorted}{tuning: FALSE implies that \code{x} is already sorted and sorting is skipped}

\item{has.dup}{tuning: FALSE implies that \code{x} has no duplicates}
}
\value{
an object of class 'bitwhich' carrying two attributes
\describe{
  \item{maxindex}{ see above }
  \item{poslength}{ see above }
}
}
\description{
A bitwhich object represents a boolean filter like a \code{\link{bit}} object (NAs are not allowed)
but uses a sparse representation suitable for very skewed (asymmetric) selections. 
Three extreme cases are represented with logical values, no length via logical(), 
all TRUE with TRUE and all FALSE with FALSE. All other selections are represented with 
positive or negative integers, whatever is shorter. 
This needs less RAM compared to \code{\link{logical}} (and often less than \code{\link{bit}} or \code{\link[=as.which]{which}}).
Logical operations are fast if the selection is asymetric (only few or almost all selected).
}
\examples{
bitwhich()
bitwhich(12)
bitwhich(12, x=TRUE)
bitwhich(12, x=3)
bitwhich(12, x=-3)
bitwhich(12, x=integer())
bitwhich(12, x=integer(), xempty=TRUE)
}
\seealso{
\code{\link{bitwhich_representation}},  \code{\link{as.bitwhich}}, \code{\link{bit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{bitwhich}
\alias{bitwhich}
\title{Create bitwhich vector (skewed boolean)}
\usage{
bitwhich(
  maxindex = 0L,
  x = NULL,
  xempty = FALSE,
  poslength = NULL,
  is.unsorted = TRUE,
  has.dup = TRUE
)
}
\arguments{
\item{maxindex}{length of the vector}

\item{x}{Information about which positions are \code{FALSE} or \code{TRUE}: either \code{logical()} or
\code{TRUE} or \code{FALSE} or a integer vector of positive or of negative subscripts.}

\item{xempty}{what to assume about parameter \code{x} if \code{x=integer(0)}, typically \code{TRUE}
or \code{FALSE}.}

\item{poslength}{tuning: \code{poslength} is calculated automatically, you can give
\code{poslength} explicitly, in this case it must be correct and \code{x} must be sorted and
not have duplicates.}

\item{is.unsorted}{tuning: FALSE implies that \code{x} is already sorted and sorting
is skipped}

\item{has.dup}{tuning: FALSE implies that \code{x} has no duplicates}
}
\value{
an object of class 'bitwhich' carrying two attributes
\itemize{
\item maxindex: see above
\item poslength: see above
}
}
\description{
A bitwhich object represents a boolean filter like a \code{\link[=bit]{bit()}} object (NAs are not
allowed) but uses a sparse representation suitable for very skewed (asymmetric)
selections. Three extreme cases are represented with logical values, no length via
\code{logical()}, all \code{TRUE} with \code{TRUE} and all \code{FALSE} with \code{FALSE}. All other
selections are represented with positive or negative integers, whatever is shorter.
This needs less RAM compared to \code{\link[=logical]{logical()}} (and often less than \code{\link[=bit]{bit()}} or
\code{\link[=as.which]{which()}}). Logical operations are fast if the selection is asymmetric
(only few or almost all selected).
}
\examples{
bitwhich()
bitwhich(12)
bitwhich(12, x=TRUE)
bitwhich(12, x=3)
bitwhich(12, x=-3)
bitwhich(12, x=integer())
bitwhich(12, x=integer(), xempty=TRUE)
}
\seealso{
\code{\link[=bitwhich_representation]{bitwhich_representation()}},  \code{\link[=as.bitwhich]{as.bitwhich()}}, \code{\link[=bit]{bit()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_unidup}
\alias{bit_unidup}
\alias{bit_unique}
\alias{bit_duplicated}
\alias{bit_anyDuplicated}
\alias{bit_sumDuplicated}
\title{bit unique and duplicated}
\usage{
bit_unique(x, na.rm = NA, range_na = NULL)

bit_duplicated(x, na.rm = NA, range_na = NULL, retFUN = as.bit)

bit_anyDuplicated(x, na.rm = NA, range_na = NULL)

bit_sumDuplicated(x, na.rm = NA, range_na = NULL)
}
\arguments{
\item{x}{an integer vector}

\item{na.rm}{\code{NA} treats NAs like other integers, \code{TRUE} treats 
\emph{all} NAs as duplicates, \code{FALSE} treats \emph{no} NAs as 
duplicates}

\item{range_na}{\code{NULL} calls \code{\link{range_na}}, optionally the result of \code{\link{range_na}} can be given here to avoid calling it again}

\item{retFUN}{a function that coerces \code{\link{bit}} and \code{\link{logical}} vectors}
}
\value{
\code{bit_unique} returns a vector of unique integers, \cr
  \code{bit_duplicated} returns a boolean vector coerced to \code{retFUN}, \cr
  \code{bit_anyDuplicated} returns the position of the first duplicate (or zero if no duplicates) \cr
  \code{bit_sumDuplicated} returns the number of duplicated values (as.integer)
}
\description{
Fast versions of \code{\link{unique}}, \code{\link{duplicated}} ,
\code{\link{anyDuplicated}} and \code{sum(duplicated(x))} for integers.
}
\details{
determines the range of the integers and checks if the density justifies use 
of a bit vector; if yes, uses a bit vector for finding duplicates; if no, 
falls back to \code{\link{unique}}, \code{\link{duplicated}}, \code{\link{anyDuplicated}} and \code{sum(duplicated(x))}
}
\section{Functions}{
\itemize{
\item \code{bit_unique}: extracts unique elements

\item \code{bit_duplicated}: determines duplicate elements

\item \code{bit_anyDuplicated}: checks for existence of duplicate elements

\item \code{bit_sumDuplicated}: counts duplicate elements
}}

\examples{
bit_unique(c(2L,1L,NA,NA,1L,2L))
bit_unique(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
bit_unique(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)

bit_duplicated(c(2L,1L,NA,NA,1L,2L))
bit_duplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
bit_duplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)

bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L))
bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
bit_anyDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)

bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L))
bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=FALSE)
bit_sumDuplicated(c(2L,1L,NA,NA,1L,2L), na.rm=TRUE)
}
\seealso{
\code{\link{bit_sort_unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{range_nanozero}
\alias{range_nanozero}
\title{Remove zeros and get range and number of NAs}
\usage{
range_nanozero(x)
}
\arguments{
\item{x}{an integer vector}
}
\value{
an integer vector without zeros and with an attribute \code{\link[=range_na]{range_na()}}  with three
elements:
\enumerate{
\item min integer
\item max integer
\item number of NAs
}
}
\description{
Remove zeros and get range and number of NAs
}
\examples{
range_nanozero(c(0L, 1L, 2L, NA))
}
\seealso{
\code{\link[=range_na]{range_na()}} and \code{\link[=range_sortna]{range_sortna()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.R
\name{merge_rev}
\alias{merge_rev}
\alias{merge_match}
\alias{merge_in}
\alias{merge_notin}
\alias{merge_duplicated}
\alias{merge_anyDuplicated}
\alias{merge_sumDuplicated}
\alias{merge_unique}
\alias{merge_union}
\alias{merge_setdiff}
\alias{merge_symdiff}
\alias{merge_intersect}
\alias{merge_setequal}
\alias{merge_rangein}
\alias{merge_rangenotin}
\alias{merge_rangesect}
\alias{merge_rangediff}
\alias{merge_first}
\alias{merge_last}
\alias{merge_firstin}
\alias{merge_lastin}
\alias{merge_firstnotin}
\alias{merge_lastnotin}
\title{Fast functions for sorted sets of integer}
\usage{
merge_rev(x)

merge_match(x, y, revx = FALSE, revy = FALSE, nomatch = NA_integer_)

merge_in(x, y, revx = FALSE, revy = FALSE)

merge_notin(x, y, revx = FALSE, revy = FALSE)

merge_duplicated(x, revx = FALSE)

merge_anyDuplicated(x, revx = FALSE)

merge_sumDuplicated(x, revx = FALSE)

merge_unique(x, revx = FALSE)

merge_union(
  x,
  y,
  revx = FALSE,
  revy = FALSE,
  method = c("unique", "exact", "all")
)

merge_setdiff(x, y, revx = FALSE, revy = FALSE, method = c("unique", "exact"))

merge_symdiff(x, y, revx = FALSE, revy = FALSE, method = c("unique", "exact"))

merge_intersect(
  x,
  y,
  revx = FALSE,
  revy = FALSE,
  method = c("unique", "exact")
)

merge_setequal(x, y, revx = FALSE, revy = FALSE, method = c("unique", "exact"))

merge_rangein(rx, y, revx = FALSE, revy = FALSE)

merge_rangenotin(rx, y, revx = FALSE, revy = FALSE)

merge_rangesect(rx, y, revx = FALSE, revy = FALSE)

merge_rangediff(rx, y, revx = FALSE, revy = FALSE)

merge_first(x, revx = FALSE)

merge_last(x, revx = FALSE)

merge_firstin(rx, y, revx = FALSE, revy = FALSE)

merge_lastin(rx, y, revx = FALSE, revy = FALSE)

merge_firstnotin(rx, y, revx = FALSE, revy = FALSE)

merge_lastnotin(rx, y, revx = FALSE, revy = FALSE)
}
\arguments{
\item{x}{a sorted set}

\item{y}{a sorted set}

\item{revx}{default \code{FALSE}, set to \code{TRUE} to reverse scan parameter 'x'}

\item{revy}{default \code{FALSE}, set to \code{TRUE} to reverse scan parameter 'y'}

\item{nomatch}{integer value returned for non-matched elements, see \code{\link{match}}}

\item{method}{one of "unique", "exact" (or "all") which governs how to treat ties, see the function descriptions}

\item{rx}{range of integers given as \code{\link{ri}} or as a two-element \code{\link{integer}}}
}
\value{
\code{merge_rev(x)} returns \code{-\link{rev}(x)} for \code{\link{integer}} and \code{\link{double}} and \code{!\link{rev}(x)} for \code{\link{logical}}
}
\description{
The \code{merge_} functions allow unary and binary operations 
on (ascending) sorted vectors of \code{link{integer}}. 
\code{merge_rev(x)} will do in one scan what costs two scans in \code{-\link{rev}(x)}, see also \code{\link{reverse_vector}(x)}.
Many of these \code{merge_} can optionally scan their input in reverse order (and switch the sign), 
which again saves extra scans for calling \code{merge_rev(x)} first.
}
\details{
These are low-level functions and hence do not check whether the set is actually sorted. 
Note that the `merge_*` and `merge_range*` functions have no special treatment for `NA`. 
If vectors with `NA` are sorted ith `NA` in the first positions (`na.last=FALSE`) and arguments `revx=` or `revy=` have not been used, 
then `NAs` are treated like ordinary integers. 
`NA` sorted elsewhere or using `revx=` or `revy=` can cause unexpected results 
(note for example that `revx=` switches the sign on all integers but `NAs`). 
\cr
\cr
The *binary* `merge_*` functions have a `method="exact"` 
which in both sets treats consecutive occurrences of the same value as if they were different values,
more precisely they are handled as if the identity of ties were tuples of \code{ties, rank(ties)}. 
\code{method="exact"} delivers unique output if the input is unique, and in this case works faster than \code{method="unique"}.
}
\section{Functions}{
\itemize{
\item \code{merge_match}: returns integer positions of sorted set x in sorted set y, see \code{\link{match}(x, y, ...)}

\item \code{merge_in}: returns logical existence of sorted set x in sorted set y, see \code{x \link{\%in\%} y}

\item \code{merge_notin}: returns logical in-existence of sorted set x in sorted set y, see \code{!(x \link{\%in\%} y)}

\item \code{merge_duplicated}: returns the duplicated status of a sorted set x, see \code{\link{duplicated}}

\item \code{merge_anyDuplicated}: returns the anyDuplicated status of a sorted set x, see \code{\link{anyDuplicated}}

\item \code{merge_sumDuplicated}: returns the sumDuplicated status of a sorted set x, see \code{\link{bit_sumDuplicated}}

\item \code{merge_unique}: returns unique elements of sorted set x, see \code{\link{unique}}

\item \code{merge_union}: returns union of two sorted sets.
Default \code{method='unique'} returns a unique sorted set, see \code{\link{union}}; 
\code{method='exact'} returns a sorted set with the maximum number of ties in either input set;
\code{method='all'} returns a sorted set with the sum of ties in both input sets.

\item \code{merge_setdiff}: returns sorted set x minus sorted set y
Default \code{method='unique'} returns a unique sorted set, see \code{\link{setdiff}}; 
\code{ethod='exact'} returns a sorted set with sum(x ties) minus sum(y ties);

\item \code{merge_symdiff}: returns those elements that are in sorted set \code{y} \code{\link{xor}} in sorted set \code{y}
Default \code{method='unique'} returns the sorted unique set complement, see \code{\link{symdiff}}; 
\code{method='exact'} returns a sorted set set complement with abs(sum(x ties) minus sum(y ties));

\item \code{merge_intersect}: returns the intersection of two sorted sets x and y
Default \code{method='unique'} returns the sorted unique intersect, see \code{\link{intersect}}; 
\code{method='exact'} returns the intersect with the minium number of ties in either set;

\item \code{merge_setequal}: returns \code{TRUE} for equal sorted sets and \code{FALSE} otherwise
Default \code{method='unique'} compares the sets after removing ties, see \code{\link{setequal}};  
\code{method='exact'} compares the sets without removing ties;

\item \code{merge_rangein}: returns logical existence of range rx in sorted set y, see \code{\link{merge_in}}

\item \code{merge_rangenotin}: returns logical in-existence of range rx in sorted set y, see \code{\link{merge_notin}}

\item \code{merge_rangesect}: returns the intersection of range rx and sorted set y, see \code{\link{merge_intersect}}

\item \code{merge_rangediff}: returns range rx minus sorted set y, see \code{\link{merge_setdiff}}

\item \code{merge_first}: quickly returns the first element of a sorted set x (or \code{NA} if x is empty), hence \code{x[1]} or \code{merge_rev(x)[1]}

\item \code{merge_last}: quickly returns the last element of a sorted set x, (or \code{NA} if x is empty), hence \code{x[n]} or \code{merge_rev(x)[n]}

\item \code{merge_firstin}: quickly returns the first common element of a range rx and a sorted set y, (or \code{NA} if the intersection is empty), hence \code{merge_first(merge_rangesect(rx,y))}

\item \code{merge_lastin}: quickly returns the last common element of a range rx and a sorted set y, (or \code{NA} if the intersection is empty), hence \code{merge_last(merge_rangesect(rx,y))}

\item \code{merge_firstnotin}: quickly returns the first element of a range rx which is not in a sorted set y (or \code{NA} if all rx are in y), hence \code{merge_first(merge_rangediff(rx,y))}

\item \code{merge_lastnotin}: quickly returns the last element of a range rx which is not in a sorted set y (or \code{NA} if all rx are in y), hence \code{merge_last(merge_rangediff(rx,y))}
}}

\note{
xx OPTIMIZATION OPPORTUNITY These are low-level functions could be optimized with initial binary search (not findInterval, which coerces to double).
}
\examples{
merge_rev(1:9)

merge_match(1:7, 3:9)
#' merge_match(merge_rev(1:7), 3:9)
merge_match(merge_rev(1:7), 3:9, revx=TRUE)
merge_match(merge_rev(1:7), 3:9, revy=TRUE)
merge_match(merge_rev(1:7), merge_rev(3:9))

merge_in(1:7, 3:9)
merge_notin(1:7, 3:9)

merge_anyDuplicated(c(1L,1L,2L,3L))
merge_duplicated(c(1L,1L,2L,3L))
merge_unique(c(1L,1L,2L,3L))

merge_union(c(1L,2L,2L,2L), c(2L,2L,3L))
merge_union(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
merge_union(c(1L,2L,2L,2L), c(2L,2L,3L), method="all")

merge_setdiff(c(1L,2L,2L,2L), c(2L,2L,3L))
merge_setdiff(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
merge_setdiff(c(1L,2L,2L), c(2L,2L,2L,3L), method="exact")

merge_symdiff(c(1L,2L,2L,2L), c(2L,2L,3L))
merge_symdiff(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")
merge_symdiff(c(1L,2L,2L), c(2L,2L,2L,3L), method="exact")

merge_intersect(c(1L,2L,2L,2L), c(2L,2L,3L))
merge_intersect(c(1L,2L,2L,2L), c(2L,2L,3L), method="exact")

merge_setequal(c(1L,2L,2L), c(1L,2L))
merge_setequal(c(1L,2L,2L), c(1L,2L,2L))
merge_setequal(c(1L,2L,2L), c(1L,2L), method="exact")
merge_setequal(c(1L,2L,2L), c(1L,2L,2L), method="exact")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_rangediff}
\alias{bit_rangediff}
\title{bit range difference}
\usage{
bit_rangediff(rx, y, revx = FALSE, revy = FALSE)
}
\arguments{
\item{rx}{range of integers given as \code{\link{ri}} or as a two-element \code{\link{integer}}}

\item{y}{an integer vector of elements to exclude}

\item{revx}{\code{FALSE} as is, \code{TRUE} to reverse the direction and sign of \code{rx}}

\item{revy}{\code{FALSE} as is, \code{TRUE} to reverse the direction and sign of \code{y}}
}
\value{
an integer vector
}
\description{
Fast version of \code{setdiff(rx[1]:rx[2], y)}.
}
\details{
determines the range of the integers \code{y} and checks if the density justifies use 
of a bit vector; if yes, uses a bit vector for the set operation; if no, 
falls back to a quicksort and \code{\link{merge_rangediff}}
}
\examples{
bit_rangediff(c(1L,6L), c(3L,4L))
bit_rangediff(c(6L,1L), c(3L,4L))
bit_rangediff(c(6L,1L), c(3L,4L), revx=TRUE)
bit_rangediff(c(6L,1L), c(3L,4L), revx=TRUE, revy=TRUE)
}
\seealso{
\code{\link{bit_setdiff}}, \code{\link{merge_rangediff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clone.R
\name{clone}
\alias{clone}
\alias{clone.default}
\title{Cloning ff and ram objects}
\usage{
clone(x, ...)

\method{clone}{default}(x, ...)
}
\arguments{
\item{x}{\code{x} an R object}

\item{\dots}{further arguments to the generic}
}
\value{
an object that is a deep copy of x
}
\description{
\command{clone} physically duplicates objects and can additionally change
some features, e.g. length.
}
\details{
\command{clone} is generic.  \command{clone.default} handles ram objects. 
Further methods are provided in package 'ff'. 
\command{still.identical} returns TRUE if the two atomic arguments still
point to the same memory.
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: default method uses R's C-API 'duplicate()'
}}

\examples{

  x <- 1:12
  y <- x
  still.identical(x,y)
  y[1] <- y[1]
  still.identical(x,y)
  y <- clone(x)
  still.identical(x,y)
  rm(x,y); gc()

}
\seealso{
\code{clone.ff}, \code{\link{copy_vector}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{chunks}
\alias{chunks}
\title{Function for chunked range index}
\usage{
chunks(
  from = NULL,
  to = NULL,
  by = NULL,
  length.out = NULL,
  along.with = NULL,
  overlap = 0L,
  method = c("bbatch", "seq"),
  maxindex = NA
)
}
\arguments{
\item{from}{the starting value of the sequence.}

\item{to}{the (maximal) end value of the sequence.}

\item{by}{increment of the sequence}

\item{length.out}{desired length of the sequence.}

\item{along.with}{take the length from the length of this argument.}

\item{overlap}{number of values to overlap (will lower the starting value of
the sequence, the first range becomes smaller}

\item{method}{default 'bbatch' will try to balance the chunk size, see
\code{\link{bbatch}}, 'seq' will create chunks like \code{\link[base]{seq}}}

\item{maxindex}{passed to \code{\link{ri}}}
}
\value{
returns a named list of \code{\link{ri}} objects
representing chunks of subscripts
}
\description{
creates a sequence of range indexes using a syntax not completely unlike
'seq'
}
\examples{

  chunks(1, 100, by=30)
  chunks(1, 100, by=30, method="seq")
   \dontrun{
require(foreach)
m <- 10000
k <- 1000
n <- m*k
message("Four ways to loop from 1 to n. Slowest foreach to fastest chunk is 1700:1 
on a dual core notebook with 3GB RAM\n")
z <- 0L; 
print(k*system.time({it <- icount(m); foreach (i = it) \%do\% { z <- i; NULL }}))
z

z <- 0L
print(system.time({i <- 0L; while (i<n) {i <- i + 1L; z <- i}}))
z

z <- 0L
print(system.time(for (i in 1:n) z <- i))
z

z <- 0L; n <- m*k; 
print(system.time(for (ch in chunks(1, n, by=m)){for (i in ch[1]:ch[2])z <- i}))
z

message("Seven ways to calculate sum(1:n). 
 Slowest foreach to fastest chunk is 61000:1 on a dual core notebook with 3GB RAM\n")
print(k*system.time({it <- icount(m); foreach (i = it, .combine="+") \%do\% { i }}))

z <- 0; 
print(k*system.time({it <- icount(m); foreach (i = it) \%do\% { z <- z + i; NULL }}))
z

z <- 0; print(system.time({i <- 0L;while (i<n) {i <- i + 1L; z <- z + i}})); z

z <- 0; print(system.time(for (i in 1:n) z <- z + i)); z

print(system.time(sum(as.double(1:n))))

z <- 0; n <- m*k
print(system.time(for (ch in chunks(1, n, by=m)){for (i in ch[1]:ch[2])z <- z + i}))
z

z <- 0; n <- m*k
print(system.time(for (ch in chunks(1, n, by=m)){z <- z+sum(as.double(ch[1]:ch[2]))}))
z
   }

}
\seealso{
generic \code{\link{chunk}}, \code{\link{ri}}, \code{\link[base]{seq}}, \code{\link{bbatch}}
}
\author{
Jens Oehlschlägel
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{ri}
\alias{ri}
\alias{print.ri}
\title{Range index}
\usage{
ri(from, to = NULL, maxindex = NA)

\method{print}{ri}(x, ...)
}
\arguments{
\item{from}{first position}

\item{to}{last posistion}

\item{maxindex}{the maximal length of the object-to-be-subscripted (if
known)}

\item{x}{an object of class 'ri'}

\item{\dots}{further arguments}
}
\value{
A two element integer vector with class 'ri'
}
\description{
A range index can be used to extract or replace a continuous ascending part
of the data
}
\examples{

 bit(12)[ri(1,6)]

}
\seealso{
\code{\link[ff]{as.hi}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{CoercionToStandard}
\alias{CoercionToStandard}
\alias{as.logical.bit}
\alias{as.integer.bit}
\alias{as.double.bit}
\alias{as.integer.bitwhich}
\alias{as.double.bitwhich}
\alias{as.logical.bitwhich}
\alias{as.logical.ri}
\alias{as.integer.ri}
\alias{as.double.ri}
\alias{as.logical.which}
\title{Coercion from bit, bitwhich, which and ri to logical, integer, double}
\usage{
\method{as.logical}{bit}(x, ...)

\method{as.integer}{bit}(x, ...)

\method{as.double}{bit}(x, ...)

\method{as.integer}{bitwhich}(x, ...)

\method{as.double}{bitwhich}(x, ...)

\method{as.logical}{bitwhich}(x, ...)

\method{as.logical}{ri}(x, ...)

\method{as.integer}{ri}(x, ...)

\method{as.double}{ri}(x, ...)

\method{as.logical}{which}(x, length = attr(x, "maxindex"), ...)
}
\arguments{
\item{x}{an object of class \code{\link{bit}}, \code{\link{bitwhich}} or
\code{\link{ri}}}

\item{\dots}{ignored}

\item{length}{length of the boolean vector (required for \code{as.logical.which})}
}
\value{
\code{\link{as.logical}} returns a vector of \code{FALSE, TRUE},
\code{\link{as.integer}} and \code{\link{as.double}} return a vector of
\code{0,1}.
}
\description{
Coercion from bit is quite fast because we use a double loop that fixes each
word in a processor register.
}
\examples{

  x <- ri(2, 5, 10)
  y <- as.logical(x)
  y
  stopifnot(identical(y, as.logical(as.bit(x))))
  stopifnot(identical(y, as.logical(as.bitwhich(x))))

  y <- as.integer(x)
  y
  stopifnot(identical(y, as.integer(as.logical(x))))
  stopifnot(identical(y, as.integer(as.bit(x))))
  stopifnot(identical(y, as.integer(as.bitwhich(x))))

  y <- as.double(x)
  y
  stopifnot(identical(y, as.double(as.logical(x))))
  stopifnot(identical(y, as.double(as.bit(x))))
  stopifnot(identical(y, as.double(as.bitwhich(x))))
}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
, \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{Metadata}
\alias{Metadata}
\alias{is.sorted}
\alias{is.sorted<-}
\alias{na.count}
\alias{na.count<-}
\alias{nvalid}
\alias{nunique}
\alias{nunique<-}
\alias{nties}
\alias{nties<-}
\title{Generics related to cache access}
\usage{
is.sorted(x, ...)

is.sorted(x, ...) <- value

na.count(x, ...)

na.count(x, ...) <- value

nvalid(x, ...)

nunique(x, ...)

nunique(x, ...) <- value

nties(x, ...)

nties(x, ...) <- value
}
\arguments{
\item{x}{some object}

\item{\dots}{ignored}

\item{value}{value assigned on responsibility of the user}
}
\value{
see help of the available methods
}
\description{
These generics are packaged here for methods in packages \code{bit64} and
\code{ff}.
}
\details{
see help of the available methods
}
\examples{
	methods("na.count")
}
\seealso{
\code{\link[bit64]{is.sorted.integer64}},
\code{\link[bit64:is.sorted.integer64]{na.count.integer64}},
\code{\link[bit64:is.sorted.integer64]{nvalid.integer64}},
\code{\link[bit64:is.sorted.integer64]{nunique.integer64}}, \code{\link[bit64:is.sorted.integer64]{nties.integer64}}
\cr
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{environment}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{CoercionToStandard}
\alias{CoercionToStandard}
\alias{as.logical.bit}
\alias{as.integer.bit}
\alias{as.double.bit}
\alias{as.integer.bitwhich}
\alias{as.double.bitwhich}
\alias{as.logical.bitwhich}
\alias{as.logical.ri}
\alias{as.integer.ri}
\alias{as.double.ri}
\alias{as.logical.which}
\title{Coercion from bit, bitwhich, which and ri to logical, integer, double}
\usage{
\method{as.logical}{bit}(x, ...)

\method{as.integer}{bit}(x, ...)

\method{as.double}{bit}(x, ...)

\method{as.integer}{bitwhich}(x, ...)

\method{as.double}{bitwhich}(x, ...)

\method{as.logical}{bitwhich}(x, ...)

\method{as.logical}{ri}(x, ...)

\method{as.integer}{ri}(x, ...)

\method{as.double}{ri}(x, ...)

\method{as.logical}{which}(x, length = attr(x, "maxindex"), ...)
}
\arguments{
\item{x}{an object of class \code{\link[=bit]{bit()}}, \code{\link[=bitwhich]{bitwhich()}} or
\code{\link[=ri]{ri()}}}

\item{...}{ignored}

\item{length}{length of the boolean vector (required for \code{as.logical.which})}
}
\value{
\code{\link[=as.logical]{as.logical()}} returns a vector of \verb{FALSE, TRUE},
\code{\link[=as.integer]{as.integer()}} and \code{\link[=as.double]{as.double()}} return a vector of
\verb{0,1}.
}
\description{
Coercion from bit is quite fast because we use a double loop that fixes each
word in a processor register.
}
\examples{

  x <- ri(2, 5, 10)
  y <- as.logical(x)
  y
  stopifnot(identical(y, as.logical(as.bit(x))))
  stopifnot(identical(y, as.logical(as.bitwhich(x))))

  y <- as.integer(x)
  y
  stopifnot(identical(y, as.integer(as.logical(x))))
  stopifnot(identical(y, as.integer(as.bit(x))))
  stopifnot(identical(y, as.integer(as.bitwhich(x))))

  y <- as.double(x)
  y
  stopifnot(identical(y, as.double(as.logical(x))))
  stopifnot(identical(y, as.double(as.bit(x))))
  stopifnot(identical(y, as.double(as.bitwhich(x))))
}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link[=as.booltype]{as.booltype()}}, \code{\link[=as.bit]{as.bit()}},
\code{\link[=as.bitwhich]{as.bitwhich()}} , \code{\link[=as.which]{as.which()}}, \code{\link[=as.ri]{as.ri()}}, \code{\link[ff:as.hi]{ff::as.hi()}},  \code{\link[ff:as.ff]{ff::as.ff()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{range_na}
\alias{range_na}
\title{Get range and number of NAs}
\usage{
range_na(x)
}
\arguments{
\item{x}{an integer vector}
}
\value{
an integer vector with three elements \describe{
  \item{ 1 }{ min integer }
  \item{ 2 }{ max integer }
  \item{ 3 }{ number of NAs }
  }
}
\description{
Get range and number of NAs
}
\examples{
range_na(c(0L,1L,2L,NA))
}
\seealso{
\code{\link{range_nanozero}} and \code{\link{range_sortna}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{repfromto}
\alias{repfromto}
\alias{repfromto<-}
\title{Virtual recycling}
\usage{
repfromto(x, from, to)

repfromto(x, from, to) <- value
}
\arguments{
\item{x}{an object from which to recycle}

\item{from}{first position to return}

\item{to}{last position to return}

\item{value}{value to assign}
}
\value{
a vector of length \code{from - to + 1}
}
\description{
\command{repfromto} virtually recylcles object \code{x} and cuts out
positions \code{from .. to}
}
\details{
\code{repfromto} is a generalization of \code{\link{rep}}, where
\code{rep(x, n) == repfromto(x, 1, n)}.  You can see this as an R-side
(vector) solution of the \code{mod_iterate} macro in arithmetic.c
}
\examples{

  message("a simple example")
  repfromto(0:9, 11, 20)

}
\seealso{
\code{\link{rep}}, \code{\link[ff:ffapply]{ffvecapply}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{as.character.bitwhich}
\alias{as.character.bitwhich}
\title{Coerce bitwhich to character}
\usage{
\method{as.character}{bitwhich}(x, ...)
}
\arguments{
\item{x}{a \code{\link{bitwhich}} vector}

\item{...}{ignored}
}
\value{
a character vector of zeroes and ones
}
\description{
Coerce bitwhich to character
}
\examples{
as.character(bitwhich(12))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{Summaries}
\alias{Summaries}
\alias{all.bit}
\alias{any.bit}
\alias{anyNA.bit}
\alias{sum.bit}
\alias{min.bit}
\alias{max.bit}
\alias{range.bit}
\alias{summary.bit}
\alias{all.bitwhich}
\alias{any.bitwhich}
\alias{anyNA.bitwhich}
\alias{sum.bitwhich}
\alias{min.bitwhich}
\alias{max.bitwhich}
\alias{range.bitwhich}
\alias{summary.bitwhich}
\alias{all.which}
\alias{any.which}
\alias{anyNA.which}
\alias{sum.which}
\alias{min.which}
\alias{max.which}
\alias{range.which}
\alias{summary.which}
\alias{all.booltype}
\alias{any.booltype}
\alias{anyNA.booltype}
\alias{sum.booltype}
\alias{min.booltype}
\alias{max.booltype}
\alias{range.booltype}
\alias{summary.booltype}
\alias{all.ri}
\alias{any.ri}
\alias{anyNA.ri}
\alias{sum.ri}
\alias{min.ri}
\alias{max.ri}
\alias{range.ri}
\alias{summary.ri}
\title{Summaries of boolean vectors}
\usage{
\method{all}{bit}(x, range = NULL, ...)

\method{any}{bit}(x, range = NULL, ...)

\method{anyNA}{bit}(x, recursive = FALSE)

\method{sum}{bit}(x, range = NULL, ...)

\method{min}{bit}(x, range = NULL, ...)

\method{max}{bit}(x, range = NULL, ...)

\method{range}{bit}(x, range = NULL, ...)

\method{summary}{bit}(object, range = NULL, ...)

\method{all}{bitwhich}(x, range = NULL, ...)

\method{any}{bitwhich}(x, range = NULL, ...)

\method{anyNA}{bitwhich}(x, recursive = FALSE)

\method{sum}{bitwhich}(x, range = NULL, ...)

\method{min}{bitwhich}(x, range = NULL, ...)

\method{max}{bitwhich}(x, range = NULL, ...)

\method{range}{bitwhich}(x, range = NULL, ...)

\method{summary}{bitwhich}(object, range = NULL, ...)

\method{all}{which}(x, range = NULL, ...)

\method{any}{which}(x, range = NULL, ...)

\method{anyNA}{which}(x, recursive = FALSE)

\method{sum}{which}(x, range = NULL, ...)

\method{min}{which}(x, range = NULL, ...)

\method{max}{which}(x, range = NULL, ...)

\method{range}{which}(x, range = NULL, ...)

\method{summary}{which}(object, range = NULL, ...)

\method{all}{booltype}(x, range = NULL, ...)

\method{any}{booltype}(x, range = NULL, ...)

\method{anyNA}{booltype}(x, ...)

\method{sum}{booltype}(x, range = NULL, ...)

\method{min}{booltype}(x, range = NULL, ...)

\method{max}{booltype}(x, range = NULL, ...)

\method{range}{booltype}(x, range = NULL, ...)

\method{summary}{booltype}(object, range = NULL, ...)

\method{all}{ri}(x, range = NULL, ...)

\method{any}{ri}(x, range = NULL, ...)

\method{anyNA}{ri}(x, recursive = FALSE)

\method{sum}{ri}(x, ...)

\method{min}{ri}(x, ...)

\method{max}{ri}(x, ...)

\method{range}{ri}(x, ...)

\method{summary}{ri}(object, ...)
}
\arguments{
\item{x}{an object of class bit or bitwhich}

\item{range}{a \code{\link{ri}} or an integer vector of length==2 giving a
range restriction for chunked processing}

\item{\dots}{formally required but not used}

\item{recursive}{formally required but not used}

\item{object}{an object of class bit}
}
\value{
as expected
}
\description{
Fast aggregation functions for \code{\link{booltype}} vectors. namely \code{\link{bit}}, \code{\link{all}}, \code{\link{any}}, \code{\link{anyNA}},
\code{\link{min}}, \code{\link{max}}, \code{\link{range}},
\code{\link{sum}} and \code{\link{summary}}.
Now all boolean summaries (except for \code{anyNA} because the generic does not allow it) have an optional \code{range} argument to restrict the range of evalution.
Note that the boolean summaries have meaning and return values differing from logical aggregation functions: they treat \code{NA} as \code{FALSE}, 
\code{min}, \code{max} and \code{range} give the minimum and maximum positions of \code{TRUE}, \code{summary} returns counts of \code{FALSE},  \code{TRUE} and the \code{range}.
Note that you can force the boolean interpretation by calling the booltype method explicitely on any \code{\link{booltypes}} input, e.g. \code{min.booltype()}, see the examples.
}
\details{
Summaries of \code{\link{bit}} vectors are quite fast because we use a double loop that fixes each
word in a processor register.  Furthermore we break out of looping as soon
as possible. Summaries of \code{\link{bitwhich}} vectors are even faster, if the selection is very skewed.
}
\examples{

  l <- c(NA, FALSE, TRUE)
  b <- as.bit(l)
  
  all(l)
  all(b)
  all(b, range=c(3,3))
  all.booltype(l, range=c(3,3))
  
  min(l)
  min(b)
  
  sum(l)
  sum(b)
  
  summary(l)
  summary(b)
  summary.booltype(l)
}
\seealso{
\code{\link{length}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{reverse_vector}
\alias{reverse_vector}
\title{Reverse atomic vector}
\usage{
reverse_vector(x)
}
\arguments{
\item{x}{an R vector}
}
\value{
a reversed vector
}
\description{
Returns a reversed copy -- with attributes retained.
}
\details{
This is substantially faster than \code{\link{rev}}
}
\examples{
x <- factor(letters)
rev(x)
reverse_vector(x)
\dontrun{
x <- 1:1e7
system.time(rev(x))
system.time(reverse_vector(x))
}
}
\seealso{
\code{\link{rev}}, \code{\link{copy_vector}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{rev.booltype}
\alias{rev.booltype}
\alias{rev.bit}
\alias{rev.bitwhich}
\title{Reversing bit and bitwhich vectors}
\usage{
\method{rev}{bit}(x)

\method{rev}{bitwhich}(x)
}
\arguments{
\item{x}{bit or bitwhich object}
}
\value{
An object of class 'bit' or 'bitwhich'
}
\description{
Creating new bit or bitwhich by reversing such vectors
}
\examples{

 rev(as.bit(c(FALSE,TRUE)))
 rev(as.bitwhich(c(FALSE,TRUE)))
}
\seealso{
\code{\link{rev}}, \code{\link{bit}} , \code{\link{bitwhich}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{still.identical}
\alias{still.identical}
\title{Test for C-level identity of two atomic vectors}
\usage{
still.identical(x, y)
}
\arguments{
\item{x}{an atomic vector}

\item{y}{an atomic vector}
}
\value{
logical scalar
}
\description{
Test for C-level identity of two atomic vectors
}
\examples{
x <- 1:2
y <- x
z <- copy_vector(x)
still.identical(y,x)
still.identical(z,x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{range_sortna}
\alias{range_sortna}
\title{Prepare for sorting and get range, number of NAs and unsortedness}
\usage{
range_sortna(x, decreasing = FALSE, na.last = NA)
}
\arguments{
\item{x}{an integer vector}

\item{decreasing}{(currently only \code{FALSE} is supported)}

\item{na.last}{\code{NA} removes NAs, \code{FALSE} puts NAs at the beginning,
\code{TRUE} puts NAs at the end}
}
\value{
an integer vector with \code{NA}s are treated and an \code{attribute} \code{\link{range_na}}  with four elements \describe{
  \item{ 1 }{ min integer }
  \item{ 2 }{ max integer }
  \item{ 3 }{ number of NAs }
  \item{ 3 }{ 0 for sorted vector and 1 for \code{\link{is.unsorted}}  }
  }
}
\description{
In one pass over the vector \code{NA}s are treated according to parameter
\code{na.last} exactly like \code{\link{sort}} does, the \code{\link{range}},
number of \code{NA}s and unsortedness is determined.
}
\examples{
range_sortna(c(0L,1L,NA,2L))
range_sortna(c(2L,NA,1L,0L))
range_sortna(c(0L,1L,NA,2L), na.last=TRUE)
range_sortna(c(2L,NA,1L,0L), na.last=TRUE)
range_sortna(c(0L,1L,NA,2L), na.last=FALSE)
range_sortna(c(2L,NA,1L,0L), na.last=FALSE)
}
\seealso{
\code{\link{range_na}} and \code{\link{range_nanozero}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rle.R
\name{intrle}
\alias{intrle}
\alias{intisasc}
\alias{intisdesc}
\title{Hybrid Index, C-coded utilities}
\usage{
intrle(x)

intisasc(x, na.method = c("none", "break", "skip")[2])

intisdesc(x, na.method = c("none", "break", "skip")[1])
}
\arguments{
\item{x}{an integer vector}

\item{na.method}{one of "none","break","skip", see details. The strange defaults stem from the initial usage.}
}
\value{
\code{intrle} returns an object of class \code{\link{rle}} or NULL,
if rle-compression is not efficient (compression factor <3 or length(x)<3).
\cr \code{intisasc} returns one of \code{FALSE, NA, TRUE} \cr
\code{intisdesc} returns one of \code{FALSE, TRUE} (if the input contains
NAs, the output is undefined)
}
\description{
These C-coded utilitites speed up index preprocessing considerably.
}
\details{
\code{intrle} is by factor 50 faster and needs less RAM (2x its input
vector) compared to \code{\link{rle}} which needs 9x the RAM of its input
vector.  This is achieved because we allow the C-code of \code{intrle} to
break when it turns out, that rle-packing will not achieve a compression
factor of 3 or better.  \cr \code{intisasc} is a faster version of
\code{\link{is.unsorted}}: it checks whether \code{x} is sorted.\cr 
\code{intisdesc} checks for being sorted descending and 
by default default assumes that the input \code{x} contains no NAs. 
\code{na.method="none"} treats \code{NAs} (the smallest integer) like every other integer and hence returns either \code{TRUE} or \code{FALSE}
\code{na.method="break"} checks for \code{NAs} and returns either \code{NA} as soon as  \code{NA} is encountered. 
\code{na.method="skip"} checks for \code{NAs} and skips over them, hence decides the return value only on the basis of non-NA values.
}
\section{Functions}{
\itemize{
\item \code{intisasc}: check whether integer vector is ascending

\item \code{intisdesc}: check whether integer vector is descending
}}

\examples{

  intrle(sample(1:10))
  intrle(diff(1:10))
  intisasc(1:10)
  intisasc(10:1)
  intisasc(c(NA, 1:10))
  intisdesc(1:10)
  intisdesc(c(10:1, NA))
  intisdesc(c(10:6, NA, 5:1))
  intisdesc(c(10:6, NA, 5:1), na.method="skip")
  intisdesc(c(10:6, NA, 5:1), na.method="break")

}
\seealso{
\code{\link[ff]{hi}}, \code{\link{rle}}, \code{\link{is.unsorted}},
\code{\link[ff]{is.sorted}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{bit}
\alias{bit}
\title{Create empty bit vector}
\usage{
bit(length = 0L)
}
\arguments{
\item{length}{length in bits}
}
\value{
\code{bit} returns a vector of integer sufficiently long to store 'length' bits
}
\description{
Bit vectors are a boolean type wihout \code{NA} that requires by factor 32 less RAM than \code{\link{logical}}.
For details on usage see the \href{../doc/bit-usage.html}{usage-vignette} and for details on performance see \href{../doc/bit-performance.html}{performance-vignette}
}
\examples{
bit(12)
!bit(12)
str(bit(128))
}
\seealso{
\code{\link{booltype}}, \code{\link{bitwhich}}, \code{\link{logical}}
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{get_length}
\alias{get_length}
\title{Get C length of a vector}
\usage{
get_length(x)
}
\arguments{
\item{x}{a vector}
}
\value{
integer scalar
}
\description{
Gets C length of a vector ignoring any length-methods dispatched by classes
}
\details{
Queries the vector length using C-macro \code{LENGTH}, this can be substantially faster than \code{length(unclass(x))}
}
\examples{
length(bit(12))
get_length(bit(12))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{bit}
\alias{bit}
\title{Create empty bit vector}
\usage{
bit(length = 0L)
}
\arguments{
\item{length}{length in bits}
}
\value{
\code{bit} returns a vector of integer sufficiently long to store 'length' bits
}
\description{
Bit vectors are a boolean type wihout \code{NA} that requires by factor 32 less
RAM than \code{\link[=logical]{logical()}}.
For details on usage see \code{vignette("bit-usage")} and for details on
performance see \code{vignette("bit-performance")}.
}
\examples{
bit(12)
!bit(12)
str(bit(128))
}
\seealso{
\code{\link[=booltype]{booltype()}}, \code{\link[=bitwhich]{bitwhich()}}, \code{\link[=logical]{logical()}}
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{c.booltype}
\alias{c.booltype}
\alias{c.bit}
\alias{c.bitwhich}
\title{Concatenating booltype vectors}
\usage{
\method{c}{booltype}(...)

\method{c}{bit}(...)

\method{c}{bitwhich}(...)
}
\arguments{
\item{\dots}{\code{\link{booltype}} vectors}
}
\value{
a vector with the lowest input \code{\link{booltype}} (but not lower than\code{\link{logical}})
}
\description{
Creating new boolean vectors by concatenating boolean vectors
}
\examples{
 c(bit(4), !bit(4))
 c(bit(4), !bitwhich(4))
 c(bitwhich(4), !bit(4))
 c(ri(1,2,4), !bit(4))
 c(bit(4), !logical(4))
 message("logical in first argument does not dispatch: c(logical(4), bit(4))")
 c.booltype(logical(4), !bit(4))

}
\seealso{
\code{\link{c}}, \code{\link{bit}} , \code{\link{bitwhich}},  , \code{\link{which}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_sort}
\alias{bit_sort}
\title{bit sort}
\usage{
bit_sort(x, decreasing = FALSE, na.last = NA, has.dup = TRUE)
}
\arguments{
\item{x}{an integer vector}

\item{decreasing}{(currently only \code{FALSE} is supported)}

\item{na.last}{\code{NA} removes NAs, \code{FALSE} puts NAs at the beginning,
\code{TRUE} puts NAs at the end}

\item{has.dup}{TRUE (the default) assumes that \code{x} might have 
duplicates, set to \code{FALSE} if duplicates are impossible}
}
\value{
a sorted vector
}
\description{
fast sorting of integers
}
\details{
determines the range of the integers and checks if the density justifies use 
of a bit vector; if yes, sorts the first occurences of each integer in the
range using a bit vector, sorts the rest and merges; if no, falls back to quicksort.
}
\examples{
bit_sort(c(2L,1L,NA,NA,1L,2L))
bit_sort(c(2L,1L,NA,NA,1L,2L), na.last=FALSE)
bit_sort(c(2L,1L,NA,NA,1L,2L), na.last=TRUE)

\dontrun{
x <- sample(1e7, replace=TRUE)
system.time(bit_sort(x))
system.time(sort(x))
}
}
\seealso{
\code{\link{sort}}, \code{\link{ramsort}}, 
  \code{\link{bit_sort_unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{c.booltype}
\alias{c.booltype}
\alias{c.bit}
\alias{c.bitwhich}
\title{Concatenating booltype vectors}
\usage{
\method{c}{booltype}(...)

\method{c}{bit}(...)

\method{c}{bitwhich}(...)
}
\arguments{
\item{...}{\code{\link[=booltype]{booltype()}} vectors}
}
\value{
a vector with the lowest input \code{\link[=booltype]{booltype()}} (but not lower than\code{\link[=logical]{logical()}})
}
\description{
Creating new boolean vectors by concatenating boolean vectors
}
\examples{
 c(bit(4), !bit(4))
 c(bit(4), !bitwhich(4))
 c(bitwhich(4), !bit(4))
 c(ri(1, 2, 4), !bit(4))
 c(bit(4), !logical(4))
 message("logical in first argument does not dispatch: c(logical(4), bit(4))")
 c.booltype(logical(4), !bit(4))

}
\seealso{
\code{\link[=c]{c()}}, \code{\link[=bit]{bit()}} , \code{\link[=bitwhich]{bitwhich()}},  , \code{\link[=which]{which()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{maxindex.default}
\alias{maxindex.default}
\alias{poslength.default}
\alias{maxindex.logical}
\alias{poslength.logical}
\alias{maxindex.bit}
\alias{poslength.bit}
\alias{maxindex.bitwhich}
\alias{poslength.bitwhich}
\alias{maxindex.which}
\alias{poslength.which}
\alias{maxindex.ri}
\alias{poslength.ri}
\alias{maxindex}
\alias{poslength}
\title{Get maxindex (length of boolean vector) and poslength (number of 'selected' elements)}
\usage{
\method{maxindex}{default}(x, ...)

\method{poslength}{default}(x, ...)

\method{maxindex}{logical}(x, ...)

\method{poslength}{logical}(x, ...)

\method{maxindex}{bit}(x, ...)

\method{poslength}{bit}(x, ...)

\method{maxindex}{bitwhich}(x, ...)

\method{poslength}{bitwhich}(x, ...)

\method{maxindex}{which}(x, ...)

\method{poslength}{which}(x, ...)

\method{maxindex}{ri}(x, ...)

\method{poslength}{ri}(x, ...)

maxindex(x, ...)

poslength(x, ...)
}
\arguments{
\item{x}{an R object, typically a \code{\link{is.booltype}} object.}

\item{...}{further arguments (ignored)}
}
\value{
an integer scalar
}
\description{
For \code{\link{is.booltype}} objects the term \code{\link{length}} is ambiguous. 
For example the length of \code{\link{which}} corresponds to the sum of \code{\link{logical}}.
The generic \code{maxindex} gives \code{length(logical)} for all \code{\link{booltype}s}.
The generic \code{poslength} gives the number of positively selected elements, i.e. \code{sum(logical)} for all \code{\link{booltype}s} 
(and gives \code{NA} if \code{NAs} are present).
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: default method for \code{maxindex}

\item \code{default}: default method for \code{poslength}

\item \code{logical}: \code{maxindex} method for class \code{\link{logical}}

\item \code{logical}: \code{poslength} method for class \code{\link{logical}}

\item \code{bit}: \code{maxindex} method for class \code{\link{bit}}

\item \code{bit}: \code{poslength} method for class \code{\link{bit}}

\item \code{bitwhich}: \code{maxindex} method for class \code{\link{bitwhich}}

\item \code{bitwhich}: \code{poslength} method for class \code{\link{bitwhich}}

\item \code{which}: \code{maxindex} method for class \code{\link[=as.which]{which}}

\item \code{which}: \code{poslength} method for class \code{\link[=as.which]{which}}

\item \code{ri}: \code{maxindex} method for class \code{\link{ri}}

\item \code{ri}: \code{poslength} method for class \code{\link{ri}}
}}

\examples{
r <- ri(1,2,12)
i <- as.which(r)
w <- as.bitwhich(r)
b <- as.bit(r)
l <- as.logical(r)
u <- which(l)      # unclassed which

sapply(list(r=r,u=u,i=i,w=w,b=b,l=l), function(x){
  c(length=length(x), sum=sum(x), maxindex=maxindex(x), poslength=poslength(x))
})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bit_in}
\alias{bit_in}
\title{bit \%in\%}
\usage{
bit_in(x, table, retFUN = as.bit)
}
\arguments{
\item{x}{an integer vector of values to be looked-up}

\item{table}{an integer vector used as lookup-table}

\item{retFUN}{a function that coerces \code{\link{bit}} and \code{\link{logical}} vectors}
}
\value{
a boolean vector coerced to \code{retFUN}
}
\description{
fast \code{\link{\%in\%}} for integers
}
\details{
determines the range of the integers and checks if the density justifies use 
of a bit vector; if yes, maps \code{x} or \code{table} -- whatever is smaller
-- into a bit vector and searches the other of \code{table} or \code{x} in
the it vector; if no, falls back to \code{\link{\%in\%}}
}
\examples{
bit_in(1:2, 2:3)
bit_in(1:2, 2:3, retFUN=as.logical)
}
\seealso{
\code{\link{\%in\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{is.booltype}
\alias{is.booltype}
\alias{is.bit}
\alias{is.bitwhich}
\alias{is.which}
\alias{is.hi}
\alias{is.ri}
\title{Testing for boolean types}
\usage{
is.booltype(x)

is.bit(x)

is.bitwhich(x)

is.which(x)

is.hi(x)

is.ri(x)
}
\arguments{
\item{x}{an R object}
}
\value{
logical scalar
}
\description{
All \code{\link{booltypes}} including  \code{\link{logical}} except 'nobool' types are considered 'is.booltype'.
}
\section{Functions}{
\itemize{
\item \code{is.bit}: tests for \code{\link{bit}}

\item \code{is.bitwhich}: tests for \code{\link{bitwhich}}

\item \code{is.which}: tests for \code{\link[=as.which]{which}}

\item \code{is.hi}: tests for \code{\link[ff]{hi}}

\item \code{is.ri}: tests for \code{\link{ri}}
}}

\examples{
sapply(list(double(),integer(),logical(),bit(),bitwhich(),as.which(),ri(1,2,3)), is.booltype)
}
\seealso{
\code{\link{booltypes}}, \code{\link{booltype}}, \code{\link{as.booltype}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{str.bitwhich}
\alias{str.bitwhich}
\title{Str method for bitwhich}
\usage{
\method{str}{bitwhich}(
  object,
  vec.len = strO$vec.len,
  give.head = TRUE,
  give.length = give.head,
  ...
)
}
\arguments{
\item{object}{any \R object about which you want to have some
    information.}

\item{vec.len}{numeric (>= 0) indicating how many \sQuote{first few} elements
    are displayed of each vector.  The number is multiplied by different
    factors (from .5 to 3) depending on the kind of vector.  Defaults to
    the \code{vec.len} component of option \code{"str"} (see
    \code{\link{options}}) which defaults to 4.}

\item{give.head}{logical; if \code{TRUE} (default), give (possibly
    abbreviated) mode/class and length (as \code{<type>[1:\dots]}).}

\item{give.length}{logical; if \code{TRUE} (default), indicate
    length (as \code{[1:\dots]}).}

\item{...}{potential further arguments (required for Method/Generic
    reasons).}
}
\value{
\code{\link{invisible}}
}
\description{
To actually view the internal structure use \code{str(unclass(bitwhich))}
}
\examples{
str(bitwhich(120))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{is.booltype}
\alias{is.booltype}
\alias{is.bit}
\alias{is.bitwhich}
\alias{is.which}
\alias{is.hi}
\alias{is.ri}
\title{Testing for boolean types}
\usage{
is.booltype(x)

is.bit(x)

is.bitwhich(x)

is.which(x)

is.hi(x)

is.ri(x)
}
\arguments{
\item{x}{an R object}
}
\value{
logical scalar
}
\description{
All \code{\link[=booltypes]{booltypes()}} including  \code{\link[=logical]{logical()}} except 'nobool' types are considered
'is.booltype'.
}
\section{Functions}{
\itemize{
\item \code{is.bit()}: tests for \code{\link[=bit]{bit()}}

\item \code{is.bitwhich()}: tests for \code{\link[=bitwhich]{bitwhich()}}

\item \code{is.which()}: tests for \code{\link[=as.which]{which()}}

\item \code{is.hi()}: tests for \code{\link[ff:hi]{hi}}

\item \code{is.ri()}: tests for \code{\link[=ri]{ri()}}

}}
\examples{
sapply(
  list(double(), integer(), logical(), bit(), bitwhich(), as.which(), ri(1, 2, 3)),
  is.booltype
)
}
\seealso{
\code{\link[=booltypes]{booltypes()}}, \code{\link[=booltype]{booltype()}}, \code{\link[=as.booltype]{as.booltype()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit-package.R
\name{bit-package}
\alias{bit-package}
\title{bit: Classes and methods for fast memory-efficient boolean selections}
\description{
Provided are classes for boolean and skewed boolean vectors, fast boolean 
methods, fast unique and non-unique integer sorting, fast set operations on 
sorted and unsorted sets of integers, and foundations for ff (range indices, 
compression, chunked processing).
}
\details{
For details view the vignettes \url{../doc/bit-usage.pdf} and
\url{../doc/bit-performance.pdf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.bit.NULL}
\alias{as.bit.NULL}
\alias{as.bit.bit}
\alias{as.bit.logical}
\alias{as.bit.integer}
\alias{as.bit.double}
\alias{as.bit.bitwhich}
\alias{as.bit.which}
\alias{as.bit.ri}
\alias{as.bit}
\title{Coercing to bit}
\usage{
\method{as.bit}{`NULL`}(x, ...)

\method{as.bit}{bit}(x, ...)

\method{as.bit}{logical}(x, ...)

\method{as.bit}{integer}(x, ...)

\method{as.bit}{double}(x, ...)

\method{as.bit}{bitwhich}(x, ...)

\method{as.bit}{which}(x, length = attr(x, "maxindex"), ...)

\method{as.bit}{ri}(x, ...)

as.bit(x = NULL, ...)
}
\arguments{
\item{x}{an object of class \code{\link{bit}}, \code{\link{logical}},
\code{\link{integer}}, \code{\link{bitwhich}} or an integer from
\code{\link{as.which}} or a boolean \code{\link[ff:vmode]{ff}}}

\item{\dots}{further arguments}

\item{length}{the length of the new bit vector}
}
\value{
\code{is.bit} returns FALSE or TRUE, \code{as.bit} returns a vector
of class 'bit'
}
\description{
Coercing to bit vector
}
\details{
Coercing to bit is quite fast because we use a double loop that fixes each
word in a processor register
}
\section{Methods (by class)}{
\itemize{
\item \code{NULL}: method to coerce to \code{\link{bit}} (zero length) from \code{\link{NULL}}

\item \code{bit}: method to coerce to \code{\link{bit}} from \code{\link{bit}}

\item \code{logical}: method to coerce to \code{\link{bit}} from \code{\link{logical}}

\item \code{integer}: method to coerce to \code{\link{bit}} from
\code{\link{integer}} (\code{0L} and \code{NA} become \code{FALSE},
everthing else becomes \code{TRUE})

\item \code{double}: method to coerce to \code{\link{bit}} from
\code{\link{double}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{bitwhich}: method to coerce to \code{\link{bit}} from \code{\link{bitwhich}}

\item \code{which}: method to coerce to \code{\link{bit}} from \code{\link[=as.which]{which}}

\item \code{ri}: method to coerce to \code{\link{bit}} from \code{\link{ri}}
}}

\note{
Zero is coerced to FALSE, all other numbers including NA are coerced
to TRUE.  This differs from the NA-to-FALSE coercion in package ff and may
change in the future.
}
\examples{
as.bit(c(0L,1L,2L,-2L,NA))
as.bit(c(0,1,2,-2,NA))

as.bit(c(FALSE, NA, TRUE))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
, \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timeutil.R
\name{repeat.time}
\alias{repeat.time}
\title{Adaptive timer}
\usage{
repeat.time(expr, gcFirst = TRUE, minSec = 0.5, envir = parent.frame())
}
\arguments{
\item{expr}{Valid expression to be timed.}

\item{gcFirst}{Logical - should a garbage collection be performed
immediately before the timing?  Default is \code{TRUE}.}

\item{minSec}{number of seconds to repeat at least}

\item{envir}{the environment in which to evaluate \code{expr} (by default
the calling frame)}
}
\value{
A object of class \code{"proc_time"}: see \code{\link{proc.time}}
for details.
}
\description{
Repeats timing expr until minSec is reached
}
\examples{

  system.time(1+1)
  repeat.time(1+1)
  system.time(sort(runif(1e6)))
  repeat.time(sort(runif(1e6)))

}
\seealso{
\code{\link{system.time}}
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.bitwhich.NULL}
\alias{as.bitwhich.NULL}
\alias{as.bitwhich.bitwhich}
\alias{as.bitwhich.which}
\alias{as.bitwhich.ri}
\alias{as.bitwhich.integer}
\alias{as.bitwhich.double}
\alias{as.bitwhich.logical}
\alias{as.bitwhich.bit}
\alias{as.bitwhich}
\title{Coercing to bitwhich}
\usage{
\method{as.bitwhich}{`NULL`}(x, ...)

\method{as.bitwhich}{bitwhich}(x, ...)

\method{as.bitwhich}{which}(x, maxindex = attr(x, "maxindex"), ...)

\method{as.bitwhich}{ri}(x, ...)

\method{as.bitwhich}{integer}(x, poslength = NULL, ...)

\method{as.bitwhich}{double}(x, poslength = NULL, ...)

\method{as.bitwhich}{logical}(x, poslength = NULL, ...)

\method{as.bitwhich}{bit}(x, range = NULL, poslength = NULL, ...)

as.bitwhich(x = NULL, ...)
}
\arguments{
\item{x}{An object of class 'bitwhich', 'integer', 'logical' or 'bit' or an
integer vector as resulting from 'which'}

\item{\dots}{further arguments}

\item{maxindex}{the length of the new bitwhich vector}

\item{poslength}{the number of selected elements}

\item{range}{a \code{\link{ri}} or an integer vector of length==2 giving a
range restriction for chunked processing}
}
\value{
a value of class \code{\link{bitwhich}}
}
\description{
Functions to coerce to bitwhich
}
\section{Methods (by class)}{
\itemize{
\item \code{NULL}: method to coerce to \code{\link{bitwhich}} (zero length) from \code{\link{NULL}}

\item \code{bitwhich}: method to coerce to \code{\link{bitwhich}} from \code{\link{bitwhich}}

\item \code{which}: method to coerce to \code{\link{bitwhich}} from \code{\link[=as.which]{which}}

\item \code{ri}: method to coerce to \code{\link{bitwhich}} from \code{\link{ri}}

\item \code{integer}: method to coerce to \code{\link{bitwhich}} from
\code{\link{integer}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{double}: method to coerce to \code{\link{bitwhich}} from
\code{\link{double}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{logical}: method to coerce to \code{\link{bitwhich}} from \code{\link{logical}}

\item \code{bit}: method to coerce to \code{\link{bitwhich}} from \code{\link{bit}}
}}

\examples{
as.bitwhich(c(0L,1L,2L,-2L,NA))
as.bitwhich(c(0,1,2,-2,NA))

 as.bitwhich(c(NA,NA,NA))
 as.bitwhich(c(FALSE, FALSE, FALSE))
 as.bitwhich(c(FALSE, FALSE, TRUE))
 as.bitwhich(c(FALSE, TRUE, TRUE))
 as.bitwhich(c(TRUE, TRUE, TRUE))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
, \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.bit.NULL}
\alias{as.bit.NULL}
\alias{as.bit.bit}
\alias{as.bit.logical}
\alias{as.bit.integer}
\alias{as.bit.double}
\alias{as.bit.bitwhich}
\alias{as.bit.which}
\alias{as.bit.ri}
\alias{as.bit}
\title{Coercing to bit}
\usage{
\method{as.bit}{`NULL`}(x, ...)

\method{as.bit}{bit}(x, ...)

\method{as.bit}{logical}(x, ...)

\method{as.bit}{integer}(x, ...)

\method{as.bit}{double}(x, ...)

\method{as.bit}{bitwhich}(x, ...)

\method{as.bit}{which}(x, length = attr(x, "maxindex"), ...)

\method{as.bit}{ri}(x, ...)

as.bit(x = NULL, ...)
}
\arguments{
\item{x}{an object of class \code{\link[=bit]{bit()}}, \code{\link[=logical]{logical()}},
\code{\link[=integer]{integer()}}, \code{\link[=bitwhich]{bitwhich()}} or an integer from
\code{\link[=as.which]{as.which()}} or a boolean \code{\link[ff:vmode]{ff}}}

\item{...}{further arguments}

\item{length}{the length of the new bit vector}
}
\value{
\code{is.bit} returns FALSE or TRUE, \code{as.bit} returns a vector
of class 'bit'
}
\description{
Coercing to bit vector
}
\details{
Coercing to bit is quite fast because we use a double loop that fixes each
word in a processor register
}
\section{Methods (by class)}{
\itemize{
\item \code{as.bit(`NULL`)}: method to coerce to \code{\link[=bit]{bit()}} (zero length) from \code{\link{NULL}}

\item \code{as.bit(bit)}: method to coerce to \code{\link[=bit]{bit()}} from \code{\link[=bit]{bit()}}

\item \code{as.bit(logical)}: method to coerce to \code{\link[=bit]{bit()}} from \code{\link[=logical]{logical()}}

\item \code{as.bit(integer)}: method to coerce to \code{\link[=bit]{bit()}} from
\code{\link[=integer]{integer()}} (\code{0L} and \code{NA} become \code{FALSE},
everthing else becomes \code{TRUE})

\item \code{as.bit(double)}: method to coerce to \code{\link[=bit]{bit()}} from
\code{\link[=double]{double()}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{as.bit(bitwhich)}: method to coerce to \code{\link[=bit]{bit()}} from \code{\link[=bitwhich]{bitwhich()}}

\item \code{as.bit(which)}: method to coerce to \code{\link[=bit]{bit()}} from \code{\link[=as.which]{which()}}

\item \code{as.bit(ri)}: method to coerce to \code{\link[=bit]{bit()}} from \code{\link[=ri]{ri()}}

}}
\note{
Zero is coerced to FALSE, all other numbers including NA are coerced
to TRUE.  This differs from the NA-to-FALSE coercion in package ff and may
change in the future.
}
\examples{
as.bit(c(0L, 1L, 2L, -2L, NA))
as.bit(c(0, 1, 2, -2, NA))

as.bit(c(FALSE, NA, TRUE))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link[=as.booltype]{as.booltype()}}, \code{\link[=as.bit]{as.bit()}},
\code{\link[=as.bitwhich]{as.bitwhich()}} , \code{\link[=as.which]{as.which()}}, \code{\link[=as.ri]{as.ri()}}, \code{\link[ff:as.hi]{ff::as.hi()}},  \code{\link[ff:as.ff]{ff::as.ff()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{vecseq}
\alias{vecseq}
\title{Vectorized Sequences}
\usage{
vecseq(x, y = NULL, concat = TRUE, eval = TRUE)
}
\arguments{
\item{x}{vector of sequence start points}

\item{y}{vector of sequence end points (if \code{is.null(y)} then \code{x}
are taken as endpoints, all starting at 1)}

\item{concat}{vector of sequence end points (if \code{is.null(y)} then
\code{x} are taken as endpoints, all starting at 1)}

\item{eval}{vector of sequence end points (if \code{is.null(y)} then
\code{x} are taken as endpoints, all starting at 1)}
}
\value{
if \code{concat==FALSE} and \code{eval==FALSE} a list with n calls
that generate sequences \cr if \code{concat==FALSE} and \code{eval==TRUE } a
list with n sequences \cr if \code{concat==TRUE } and \code{eval==FALSE} a
single call generating the concatenated sequences \cr if \code{concat==TRUE
} and \code{eval==TRUE } an integer vector of concatentated sequences
}
\description{
\command{vecseq} returns concatenated multiple sequences
}
\details{
This is a generalization of \code{\link{sequence}} in that you can choose
sequence starts other than 1 and also have options to no concat and/or
return a call instead of the evaluated sequence.
}
\examples{

  sequence(c(3,4))
  vecseq(c(3,4))
  vecseq(c(1,11), c(5, 15))
  vecseq(c(1,11), c(5, 15), concat=FALSE, eval=FALSE)
  vecseq(c(1,11), c(5, 15), concat=FALSE, eval=TRUE)
  vecseq(c(1,11), c(5, 15), concat=TRUE, eval=FALSE)
  vecseq(c(1,11), c(5, 15), concat=TRUE, eval=TRUE)

}
\seealso{
\code{\link{:}}, \code{\link{seq}}, \code{\link{sequence}}
}
\author{
Angelo Canty, Jens Oehlschlägel
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.bitwhich.NULL}
\alias{as.bitwhich.NULL}
\alias{as.bitwhich.bitwhich}
\alias{as.bitwhich.which}
\alias{as.bitwhich.ri}
\alias{as.bitwhich.integer}
\alias{as.bitwhich.double}
\alias{as.bitwhich.logical}
\alias{as.bitwhich.bit}
\alias{as.bitwhich}
\title{Coercing to bitwhich}
\usage{
\method{as.bitwhich}{`NULL`}(x, ...)

\method{as.bitwhich}{bitwhich}(x, ...)

\method{as.bitwhich}{which}(x, maxindex = attr(x, "maxindex"), ...)

\method{as.bitwhich}{ri}(x, ...)

\method{as.bitwhich}{integer}(x, poslength = NULL, ...)

\method{as.bitwhich}{double}(x, poslength = NULL, ...)

\method{as.bitwhich}{logical}(x, poslength = NULL, ...)

\method{as.bitwhich}{bit}(x, range = NULL, poslength = NULL, ...)

as.bitwhich(x = NULL, ...)
}
\arguments{
\item{x}{An object of class 'bitwhich', 'integer', 'logical' or 'bit' or an
integer vector as resulting from 'which'}

\item{...}{further arguments}

\item{maxindex}{the length of the new bitwhich vector}

\item{poslength}{the number of selected elements}

\item{range}{a \code{\link[=ri]{ri()}} or an integer vector of length == 2 giving a
range restriction for chunked processing}
}
\value{
a value of class \code{\link[=bitwhich]{bitwhich()}}
}
\description{
Functions to coerce to bitwhich
}
\section{Methods (by class)}{
\itemize{
\item \code{as.bitwhich(`NULL`)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} (zero length) from
\code{\link{NULL}}

\item \code{as.bitwhich(bitwhich)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from \code{\link[=bitwhich]{bitwhich()}}

\item \code{as.bitwhich(which)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from \code{\link[=as.which]{which()}}

\item \code{as.bitwhich(ri)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from \code{\link[=ri]{ri()}}

\item \code{as.bitwhich(integer)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from
\code{\link[=integer]{integer()}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{as.bitwhich(double)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from
\code{\link[=double]{double()}} (\code{0} and \code{NA} become \code{FALSE}, everthing
else becomes \code{TRUE})

\item \code{as.bitwhich(logical)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from \code{\link[=logical]{logical()}}

\item \code{as.bitwhich(bit)}: method to coerce to \code{\link[=bitwhich]{bitwhich()}} from \code{\link[=bit]{bit()}}

}}
\examples{
as.bitwhich(c(0L, 1L, 2L, -2L, NA))
as.bitwhich(c(0, 1, 2, -2, NA))

 as.bitwhich(c(NA, NA, NA))
 as.bitwhich(c(FALSE, FALSE, FALSE))
 as.bitwhich(c(FALSE, FALSE, TRUE))
 as.bitwhich(c(FALSE, TRUE, TRUE))
 as.bitwhich(c(TRUE, TRUE, TRUE))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link[=as.booltype]{as.booltype()}}, \code{\link[=as.bit]{as.bit()}},
\code{\link[=as.bitwhich]{as.bitwhich()}} , \code{\link[=as.which]{as.which()}}, \code{\link[=as.ri]{as.ri()}}, \code{\link[ff:as.hi]{ff::as.hi()}},  \code{\link[ff:as.ff]{ff::as.ff()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{is.na.bit}
\alias{is.na.bit}
\alias{is.na.bitwhich}
\title{Test for NA in bit and bitwhich}
\usage{
\method{is.na}{bit}(x)

\method{is.na}{bitwhich}(x)
}
\arguments{
\item{x}{a \code{\link{bit}} or  \code{\link{bitwhich}} vector}
}
\value{
vector of same type with all elements \code{FALSE}
}
\description{
Test for NA in bit and bitwhich
}
\section{Functions}{
\itemize{
\item \code{is.na.bitwhich}: method for \code{\link{is.na}} from \code{\link{bitwhich}}
}}

\examples{
is.na(bit(6))
is.na(bitwhich(6))
}
\seealso{
\code{\link{is.na}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitsort.R
\name{bitsort}
\alias{bitsort}
\title{Low-level sorting: bit sort}
\usage{
bitsort(x, na.last = NA, depth = 1)
}
\arguments{
\item{x}{an integer vector}

\item{na.last}{\code{NA} removes NAs, \code{FALSE} puts NAs at the beginning,
\code{TRUE} puts NAs at the end}

\item{depth}{an integer scalar giving the number of bit-passed before switching to
quicksort}
}
\value{
a sorted vector
}
\description{
In one pass over the vector \code{NA}s are handled according to parameter
\code{na.last} by \code{\link[=range_sortna]{range_sortna()}}, then, if the vector is unsorted,
bit sort is invoked.
}
\examples{
bitsort(c(2L, 0L, 1L, NA, 2L))
bitsort(c(2L, 0L, 1L, NA, 2L), na.last=TRUE)
bitsort(c(2L, 0L, 1L, NA, 2L), na.last=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{bbatch}
\alias{bbatch}
\title{Balanced Batch sizes}
\usage{
bbatch(N, B)
}
\arguments{
\item{N}{total size in 0..integer_max}

\item{B}{desired batch size in 1..integer_max}
}
\value{
a list with components \item{ b }{ the batch size } \item{ nb }{ the
number of batches } \item{ rb }{ the size of the rest }
}
\description{
\command{bbatch} calculates batch sizes in 1..N so that they have rather balanced
sizes than very different sizes.
}
\details{
Tries to have \code{rb==0} or \code{rb} as close to \code{b} as possible
while guaranteeing that \code{rb < b && (b - rb) <= min(nb, b)}
}
\examples{

  bbatch(100, 24)

}
\seealso{
\code{\link{repfromto}}, \code{\link[ff:ffapply]{ffvecapply}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.which.which}
\alias{as.which.which}
\alias{as.which.NULL}
\alias{as.which.numeric}
\alias{as.which.integer}
\alias{as.which.logical}
\alias{as.which.ri}
\alias{as.which.bit}
\alias{as.which.bitwhich}
\alias{as.which}
\title{Coercion to (positive) integer positions}
\usage{
\method{as.which}{which}(x, maxindex = NA_integer_, ...)

\method{as.which}{`NULL`}(x, ...)

\method{as.which}{numeric}(x, maxindex = NA_integer_, ...)

\method{as.which}{integer}(x, maxindex = NA_integer_, is.unsorted = TRUE, has.dup = TRUE, ...)

\method{as.which}{logical}(x, ...)

\method{as.which}{ri}(x, ...)

\method{as.which}{bit}(x, range = NULL, ...)

\method{as.which}{bitwhich}(x, ...)

as.which(x, ...)
}
\arguments{
\item{x}{an object of classes \code{\link{bit}}, \code{\link{bitwhich}},
\code{\link{ri}} or something on which \code{\link{which}} works}

\item{maxindex}{the length of the boolean vector which is represented}

\item{\dots}{further arguments (passed to \code{\link{which}} for the
default method, ignored otherwise)}

\item{is.unsorted}{a logical scalar indicating whether the data may be unsorted}

\item{has.dup}{a logical scalar indicating whether the data may have duplicates}

\item{range}{a \code{\link{ri}} or an integer vector of length==2 giving a
range restriction for chunked processing}
}
\value{
a vector of class 'logical' or 'integer'
}
\description{
Coercing to something like the result of which \code{\link{which}}
}
\details{
\code{as.which.bit} returns a vector of subscripts with class 'which'
}
\section{Methods (by class)}{
\itemize{
\item \code{which}: method to coerce to \code{\link[=as.which]{which}} from \code{\link[=as.which]{which}}

\item \code{NULL}: method to coerce to zero length \code{\link[=as.which]{which}} from \code{\link{NULL}}

\item \code{numeric}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{numeric}}

\item \code{integer}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{integer}}

\item \code{logical}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{logical}}

\item \code{ri}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{ri}}

\item \code{bit}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{bit}}

\item \code{bitwhich}: method to coerce to \code{\link[=as.which]{which}} from \code{\link{bitwhich}}
}}

\examples{

  r <- ri(5, 20, 100)
  x <- as.which(r)
  x

  stopifnot(identical(x, as.which(as.logical(r))))
  stopifnot(identical(x, as.which(as.bitwhich(r))))
  stopifnot(identical(x, as.which(as.bit(r))))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
, \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{copy_vector}
\alias{copy_vector}
\title{Copy atomic R vector}
\usage{
copy_vector(x, revx = FALSE)
}
\arguments{
\item{x}{an R vector}

\item{revx}{default \code{FALSE}, set to \code{TRUE} to reverse the elements in 'x'}
}
\value{
copied R vector
}
\description{
Creates a true copy of the underlying C-vector -- dropping all attributes -- and optionally reverses the direction of the elements.
}
\details{
This can be substantially faster than \code{duplicate(as.vector(unclass(x)))}
}
\examples{
x <- factor(letters)
y <- x
z <- copy_vector(x)
still.identical(x,y)
still.identical(x,z)
str(x)
str(y)
str(z)
}
\seealso{
\code{\link{clone}}, \code{\link{still.identical}},  \code{\link{reverse_vector}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{bbatch}
\alias{bbatch}
\title{Balanced Batch sizes}
\usage{
bbatch(N, B)
}
\arguments{
\item{N}{total size in 0..integer_max}

\item{B}{desired batch size in 1..integer_max}
}
\value{
a list with components:
\itemize{
\item b: the batch size
\item nb: the number of batches
\item rb: the size of the rest
}
}
\description{
\code{bbatch} calculates batch sizes in 1..N so that they have rather balanced
sizes than very different sizes.
}
\details{
Tries to have \code{rb == 0} or \code{rb} as close to \code{b} as possible
while guaranteeing that \code{rb < b && (b - rb) <= min(nb, b)}
}
\examples{

  bbatch(100, 24)

}
\seealso{
\code{\link[=repfromto]{repfromto()}}, \code{\link[ff:ffapply]{ff::ffvecapply()}}
}
\author{
Jens Oehlschlägel
}
\keyword{IO}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.which.which}
\alias{as.which.which}
\alias{as.which.NULL}
\alias{as.which.numeric}
\alias{as.which.integer}
\alias{as.which.logical}
\alias{as.which.ri}
\alias{as.which.bit}
\alias{as.which.bitwhich}
\alias{as.which}
\title{Coercion to (positive) integer positions}
\usage{
\method{as.which}{which}(x, maxindex = NA_integer_, ...)

\method{as.which}{`NULL`}(x, ...)

\method{as.which}{numeric}(x, maxindex = NA_integer_, ...)

\method{as.which}{integer}(x, maxindex = NA_integer_, is.unsorted = TRUE, has.dup = TRUE, ...)

\method{as.which}{logical}(x, ...)

\method{as.which}{ri}(x, ...)

\method{as.which}{bit}(x, range = NULL, ...)

\method{as.which}{bitwhich}(x, ...)

as.which(x, ...)
}
\arguments{
\item{x}{an object of classes \code{\link[=bit]{bit()}}, \code{\link[=bitwhich]{bitwhich()}},
\code{\link[=ri]{ri()}} or something on which \code{\link[=which]{which()}} works}

\item{maxindex}{the length of the boolean vector which is represented}

\item{...}{further arguments (passed to \code{\link[=which]{which()}} for the
default method, ignored otherwise)}

\item{is.unsorted}{a logical scalar indicating whether the data may be unsorted}

\item{has.dup}{a logical scalar indicating whether the data may have duplicates}

\item{range}{a \code{\link[=ri]{ri()}} or an integer vector of length == 2 giving a
range restriction for chunked processing}
}
\value{
a vector of class 'logical' or 'integer'
}
\description{
Coercing to something like the result of which \code{\link[=which]{which()}}
}
\details{
\code{as.which.bit} returns a vector of subscripts with class 'which'
}
\section{Methods (by class)}{
\itemize{
\item \code{as.which(which)}: method to coerce to \code{\link[=as.which]{which()}} from
\code{\link[=as.which]{which()}}

\item \code{as.which(`NULL`)}: method to coerce to zero length \code{\link[=as.which]{which()}} from
\code{\link{NULL}}

\item \code{as.which(numeric)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=numeric]{numeric()}}

\item \code{as.which(integer)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=integer]{integer()}}

\item \code{as.which(logical)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=logical]{logical()}}

\item \code{as.which(ri)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=ri]{ri()}}

\item \code{as.which(bit)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=bit]{bit()}}

\item \code{as.which(bitwhich)}: method to coerce to \code{\link[=as.which]{which()}} from \code{\link[=bitwhich]{bitwhich()}}

}}
\examples{

  r <- ri(5, 20, 100)
  x <- as.which(r)
  x

  stopifnot(identical(x, as.which(as.logical(r))))
  stopifnot(identical(x, as.which(as.bitwhich(r))))
  stopifnot(identical(x, as.which(as.bit(r))))

}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link[=as.booltype]{as.booltype()}}, \code{\link[=as.bit]{as.bit()}},
\code{\link[=as.bitwhich]{as.bitwhich()}} , \code{\link[=as.which]{as.which()}}, \code{\link[=as.ri]{as.ri()}}, \code{\link[ff:as.hi]{ff::as.hi()}},  \code{\link[ff:as.ff]{ff::as.ff()}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{Sorting}
\alias{Sorting}
\alias{ramsort}
\alias{ramorder}
\alias{ramsortorder}
\alias{mergesort}
\alias{mergeorder}
\alias{mergesortorder}
\alias{quicksort}
\alias{quickorder}
\alias{quicksortorder}
\alias{shellsort}
\alias{shellorder}
\alias{shellsortorder}
\alias{radixsort}
\alias{radixorder}
\alias{radixsortorder}
\alias{keysort}
\alias{keyorder}
\alias{keysortorder}
\title{Generics for in-RAM sorting and ordering}
\usage{
ramsort(x, ...)

ramorder(x, i, ...)

ramsortorder(x, i, ...)

mergesort(x, ...)

mergeorder(x, i, ...)

mergesortorder(x, i, ...)

quicksort(x, ...)

quickorder(x, i, ...)

quicksortorder(x, i, ...)

shellsort(x, ...)

shellorder(x, i, ...)

shellsortorder(x, i, ...)

radixsort(x, ...)

radixorder(x, i, ...)

radixsortorder(x, i, ...)

keysort(x, ...)

keyorder(x, i, ...)

keysortorder(x, i, ...)
}
\arguments{
\item{x}{a vector to be sorted by \code{\link{ramsort}} and
\code{\link{ramsortorder}}, i.e. the output of \code{\link{sort}}}

\item{\dots}{further arguments to the sorting methods}

\item{i}{integer positions to be modified by \code{\link{ramorder}} and
\code{\link{ramsortorder}}, default is 1:n, in this case the output is
similar to \code{\link{order}}}
}
\value{
These functions return the number of \code{NAs} found or assumed
during sorting
}
\description{
These are generic stubs for low-level sorting and ordering methods
implemented in packages 'bit64' and 'ff'.  The \code{..sortorder} methods do
sorting and ordering at once, which requires more RAM than ordering but is
(almost) as fast as as sorting.
}
\details{
The \code{sort} generics do sort their argument 'x', some methods need
temporary RAM of the same size as 'x'.  The \code{order} generics do order
their argument 'i' leaving 'x' as it was, some methods need temporary RAM of
the same size as 'i'.  The \code{sortorder} generics do sort their argument
'x' and order their argument 'i', this way of ordering is much faster at the
price of requiring temporary RAM for both, 'x' and 'i', if the method
requires temporary RAM.  The \code{ram} generics are high-level functions
containing an optimizer that chooses the 'best' algorithms given some
context.
}
\note{
Note that these methods purposely violate the functional programming
paradigm: they are called for the side-effect of changing some of their
arguments.  The rationale behind this is that sorting is very RAM-intensive
and in certain situations we might not want to allocate additional memory if
not necessary to do so.  The \code{sort}-methods change \code{x}, the
\code{order}-methods change \code{i}, and the \code{sortoder}-methods change
both \code{x} and \code{i} You as the user are responsible to create copies
of the input data 'x' and 'i' if you need non-modified versions.
}
\section{Index of implemented methods}{
 \tabular{rrl}{ \bold{generic} \tab
\bold{ff} \tab \bold{bit64} \cr \code{ramsort} \tab
\code{\link[ff]{ramsort.default}} \tab
\code{\link[bit64]{ramsort.integer64}} \cr \code{shellsort} \tab
\code{\link[ff:ramsort.default]{shellsort.default}} \tab
\code{\link[bit64:ramsort.integer64]{shellsort.integer64}} \cr \code{quicksort} \tab \tab
\code{\link[bit64:ramsort.integer64]{quicksort.integer64}} \cr \code{mergesort} \tab
\code{\link[ff:ramsort.default]{mergesort.default}} \tab
\code{\link[bit64:ramsort.integer64]{mergesort.integer64}} \cr \code{radixsort} \tab
\code{\link[ff:ramsort.default]{radixsort.default}} \tab
\code{\link[bit64:ramsort.integer64]{radixsort.integer64}} \cr \code{keysort} \tab
\code{\link[ff:ramsort.default]{keysort.default}} \tab \cr \cr \bold{generic} \tab \bold{ff}
\tab \bold{bit64} \cr \code{ramorder} \tab
\code{\link[ff]{ramorder.default}} \tab
\code{\link[bit64:ramsort.integer64]{ramorder.integer64}} \cr \code{shellorder} \tab
\code{\link[ff:ramorder.default]{shellorder.default}} \tab
\code{\link[bit64:ramsort.integer64]{shellorder.integer64}} \cr \code{quickorder} \tab \tab
\code{\link[bit64:ramsort.integer64]{quickorder.integer64}} \cr \code{mergeorder} \tab
\code{\link[ff:ramorder.default]{mergeorder.default}} \tab
\code{\link[bit64:ramsort.integer64]{mergeorder.integer64}} \cr \code{radixorder} \tab
\code{\link[ff:ramorder.default]{radixorder.default}} \tab
\code{\link[bit64:ramsort.integer64]{radixorder.integer64}} \cr \code{keyorder} \tab
\code{\link[ff:ramorder.default]{keyorder.default}} \tab \cr \cr \bold{generic} \tab
\bold{ff} \tab \bold{bit64} \cr \code{ramsortorder} \tab \tab
\code{\link[bit64:ramsort.integer64]{ramsortorder.integer64}} \cr \code{shellsortorder} \tab
\tab \code{\link[bit64:ramsort.integer64]{shellsortorder.integer64}} \cr \code{quicksortorder}
\tab \tab \code{\link[bit64:ramsort.integer64]{quicksortorder.integer64}} \cr
\code{mergesortorder} \tab \tab
\code{\link[bit64:ramsort.integer64]{mergesortorder.integer64}} \cr \code{radixsortorder} \tab
\tab \code{\link[bit64:ramsort.integer64]{radixsortorder.integer64}} \cr \code{keysortorder}
\tab \tab \cr }
}

\seealso{
\code{\link{sort}} and \code{\link{order}} in base R, \code{\link{bitsort}} for faster inteer sorting
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{arith}
\keyword{manip}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{chunk}
\alias{chunk}
\alias{chunk.default}
\title{Methods for chunked range index}
\usage{
chunk(x = NULL, ...)

\method{chunk}{default}(x = NULL, ..., RECORDBYTES = NULL, BATCHBYTES = NULL)
}
\arguments{
\item{x}{the object along we want chunks}

\item{\dots}{further arguments passed to \code{\link{chunks}}}

\item{RECORDBYTES}{integer scalar representing the bytes needed to process a single element of the boolean vector (default 4 bytes for logical)}

\item{BATCHBYTES}{integer scalar limiting the number of bytes to be processed in one chunk, default from \code{getOption("ffbatchbytes")} if not null, otherwise 16777216}
}
\value{
returns a named list of \code{\link{ri}} objects 
representing chunks of subscripts
}
\description{
Calls \code{\link{chunks}} to create a sequence of range indexes along the object which causes the method dispatch.
}
\details{
\code{chunk} is generic, the default method is described here, other methods
that automatically consider RAM needs are provided with package 'ff', see
for example \code{\link[ff]{chunk.ffdf}}
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: default vector method
}}

\section{available methods}{
 \code{chunk.default}, \code{\link[ff:chunk.ffdf]{chunk.ff_vector}}, \code{\link[ff]{chunk.ffdf}}
}

\examples{
  chunk(complex(1e7))
  chunk(raw(1e7))
  chunk(raw(1e7), length=3)
  
  chunks(1,10,3)
  # no longer do
  chunk(1,100,10)
  # but for bckward compatibility this works
  chunk(from=1,to=100,by=10)
  
}
\seealso{
\code{\link{chunks}}, \code{\link{ri}}, \code{\link[base]{seq}}, \code{\link{bbatch}}
}
\author{
Jens Oehlschlägel
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{booltype}
\alias{booltype}
\title{Diagnosing boolean types}
\usage{
booltype(x)
}
\arguments{
\item{x}{an R object}
}
\value{
one scalar element of \code{\link{booltypes}} in case of 'nobool' it carries a name attribute with the data type.
}
\description{
Specific methods for \code{booltype} are required, where non-unary methods can combine multiple bollean types, particularly boolean binary operators.
}
\details{
Function \code{booltype} returns the boolean type of its argument.
There are currently six boolean types, \code{booltypes} is an \code{\link{ordered}} vector with the following ordinal \code{\link{levels}} \describe{
\item{nobool}{non-boolean types}
\item{\code{\link{logical}}}{for representing any boolean data including \code{NA} }
\item{\code{\link{bit}}}{for representing dense boolean data }
\item{\code{\link{bitwhich}}}{for representing sparse (skewed) boolean data  }
\item{\code{\link{which}}}{for representing sparse boolean data with few \code{TRUE}}
\item{\code{\link{ri}}}{range-indexing, for representing sparse boolean data with a single range of \code{TRUE} }
}
}
\note{
do not rely on the internal integer codes of these levels, we might add-in \code{\link[ff]{hi}} later
}
\examples{
unname(booltypes)
str(booltypes)
sapply(list(double(),integer(),logical(),bit(),bitwhich(),as.which(),ri(1,2,3)), booltype)
}
\seealso{
\code{\link{booltypes}}, \code{\link{is.booltype}}, \code{\link{as.booltype}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R, R/generics.R
\name{as.ri.ri}
\alias{as.ri.ri}
\alias{as.ri.default}
\alias{as.ri}
\title{Coerce to ri}
\usage{
\method{as.ri}{ri}(x, ...)

\method{as.ri}{default}(x, ...)

as.ri(x, ...)
}
\arguments{
\item{x}{object to coerce}

\item{...}{further arguments}
}
\value{
an \code{\link{ri}} object
}
\description{
Coerce to ri
}
\section{Methods (by class)}{
\itemize{
\item \code{ri}: method to coerce \code{\link{ri}} to \code{\link{ri}}

\item \code{default}: default method to coerce to \code{\link{ri}}
}}

\examples{
as.ri(c(FALSE, TRUE, FALSE, TRUE))
}
\seealso{
\code{\link{CoercionToStandard}}, \code{\link{as.booltype}}, \code{\link{as.bit}}, \code{\link{as.bitwhich}}
, \code{\link{as.which}}, \code{\link{as.ri}}, \code{\link[ff]{as.hi}},  \code{\link[ff]{as.ff}}
}
\author{
Jens Oehlschlägel
}
\keyword{classes}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunkutil.R
\name{chunk}
\alias{chunk}
\alias{chunk.default}
\title{Methods for chunked range index}
\usage{
chunk(x = NULL, ...)

\method{chunk}{default}(x = NULL, ..., RECORDBYTES = NULL, BATCHBYTES = NULL)
}
\arguments{
\item{x}{the object along we want chunks}

\item{...}{further arguments passed to \code{\link[=chunks]{chunks()}}}

\item{RECORDBYTES}{integer scalar representing the bytes needed to process a single
element of the boolean vector (default 4 bytes for logical)}

\item{BATCHBYTES}{integer scalar limiting the number of bytes to be processed in one
chunk, default from \code{getOption("ffbatchbytes")} if not null, otherwise 16777216}
}
\value{
returns a named list of \code{\link[=ri]{ri()}} objects
representing chunks of subscripts
}
\description{
Calls \code{\link[=chunks]{chunks()}} to create a sequence of range indexes along the object which causes
the method dispatch.
}
\details{
\code{chunk} is generic, the default method is described here, other methods
that automatically consider RAM needs are provided with package 'ff', see
for example \code{\link[ff:chunk.ffdf]{ff::chunk.ffdf()}}
}
\section{Methods (by class)}{
\itemize{
\item \code{chunk(default)}: default vector method

}}
\section{available methods}{
 \code{chunk.default}, \code{\link[ff:chunk.ffdf]{ff::chunk.ff_vector()}},
\code{\link[ff:chunk.ffdf]{ff::chunk.ffdf()}}
}

\examples{
  chunk(complex(1e7))
  chunk(raw(1e7))
  chunk(raw(1e7), length=3)

  chunks(1, 10, 3)
  # no longer do
  chunk(1, 100, 10)
  # but for bckward compatibility this works
  chunk(from=1, to=100, by=10)

}
\seealso{
\code{\link[=chunks]{chunks()}}, \code{\link[=ri]{ri()}}, \code{\link[base:seq]{seq()}}, \code{\link[=bbatch]{bbatch()}}
}
\author{
Jens Oehlschlägel
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit.R
\name{booltype}
\alias{booltype}
\title{Diagnosing boolean types}
\usage{
booltype(x)
}
\arguments{
\item{x}{an R object}
}
\value{
one scalar element of \code{\link[=booltypes]{booltypes()}} in case of 'nobool' it carries a name
attribute with the data type.
}
\description{
Specific methods for \code{booltype} are required, where non-unary methods can combine
multiple bollean types, particularly boolean binary operators.
}
\details{
Function \code{booltype} returns the boolean type of its argument.
There are currently six boolean types, \code{booltypes} is an \code{\link[=ordered]{ordered()}} vector with the
following ordinal \code{\link[=levels]{levels()}}:
\itemize{
\item nobool: non-boolean type
\item \code{\link[=logical]{logical()}}: for representing any boolean data including \code{NA}
\item \code{\link[=bit]{bit()}}: for representing dense boolean data
\item \code{\link[=bitwhich]{bitwhich()}}: for representing sparse (skewed) boolean data
\item \code{\link[=which]{which()}}: for representing sparse boolean data with few `TRUE
\item \code{\link[=ri]{ri()}}: range-indexing, for representing sparse boolean data with a single range of
\code{TRUE}
}
}
\note{
do not rely on the internal integer codes of these levels, we might add-in
\code{\link[ff:hi]{hi}} later
}
\examples{
unname(booltypes)
str(booltypes)
sapply(
  list(double(), integer(), logical(), bit(), bitwhich(), as.which(), ri(1, 2, 3)),
  booltype
)
}
\seealso{
\code{\link[=booltypes]{booltypes()}}, \code{\link[=is.booltype]{is.booltype()}}, \code{\link[=as.booltype]{as.booltype()}}
}
